Package: nlsr
Type: Package
Title: Functions for Nonlinear Least Squares Solutions - Updated 2022
Version: 2026.4.29
Date: 2026-04-29
Authors@R: c(person(given = c("John", "C"), family = "Nash", 
        role = c("aut", "cre"), email = "profjcnash@gmail.com"), 
    	person(given = "Duncan", family = "Murdoch", role = "aut", 
        email = "murdoch.duncan@gmail.com"),
        person(given= "Fernando", family = "Miguez", role="ctb", 
         email="femiguez@iastate.edu"), 
        person(given= "Arkajyoti", family = "Bhattacharjee", role="ctb", 
         email="arkastat98@gmail.com"))
Maintainer: John C Nash <profjcnash@gmail.com>
Description: Provides tools for working with nonlinear least squares problems.
      For the estimation of models reliable and robust tools than nls(), where the
      the Gauss-Newton method frequently stops with 'singular gradient' messages. 
      This is accomplished by using, where possible, analytic derivatives to compute
      the matrix of derivatives and a stabilization of the solution of the estimation
      equations. Tools for approximate or externally supplied derivative matrices
      are included. Bounds and masks on parameters are handled properly.
License: GPL-2
Encoding: UTF-8
Depends: R (>= 3.5)
Imports: digest
Suggests: minpack.lm, optimx, numDeriv, knitr, rmarkdown, markdown,
        Ryacas, Deriv, microbenchmark, MASS, ggplot2, nlraa
VignetteBuilder: knitr
RoxygenNote: 7.2.1
NeedsCompilation: no
Packaged: 2026-04-30 15:33:39 UTC; john
Author: John C Nash [aut, cre],
  Duncan Murdoch [aut],
  Fernando Miguez [ctb],
  Arkajyoti Bhattacharjee [ctb]
Repository: CRAN
Date/Publication: 2026-05-01 11:30:02 UTC
Built: R 4.5.2; ; 2026-05-01 15:30:39 UTC; unix

Package: nlsr
Type: Package
Title: Functions for Nonlinear Least Squares Solutions - Updated 2022
Version: 2026.4.29
Date: 2026-04-29
Authors@R: c(person(given = c("John", "C"), family = "Nash", 
        role = c("aut", "cre"), email = "profjcnash@gmail.com"), 
    	person(given = "Duncan", family = "Murdoch", role = "aut", 
        email = "murdoch.duncan@gmail.com"),
        person(given= "Fernando", family = "Miguez", role="ctb", 
         email="femiguez@iastate.edu"), 
        person(given= "Arkajyoti", family = "Bhattacharjee", role="ctb", 
         email="arkastat98@gmail.com"))
Maintainer: John C Nash <profjcnash@gmail.com>
Description: Provides tools for working with nonlinear least squares problems.
      For the estimation of models reliable and robust tools than nls(), where the
      the Gauss-Newton method frequently stops with 'singular gradient' messages. 
      This is accomplished by using, where possible, analytic derivatives to compute
      the matrix of derivatives and a stabilization of the solution of the estimation
      equations. Tools for approximate or externally supplied derivative matrices
      are included. Bounds and masks on parameters are handled properly.
License: GPL-2
Encoding: UTF-8
Depends: R (>= 3.5)
Imports: digest
Suggests: minpack.lm, optimx, numDeriv, knitr, rmarkdown, markdown,
        Ryacas, Deriv, microbenchmark, MASS, ggplot2, nlraa
VignetteBuilder: knitr
RoxygenNote: 7.2.1
NeedsCompilation: no
Packaged: 2026-04-30 15:33:39 UTC; john
Author: John C Nash [aut, cre],
  Duncan Murdoch [aut],
  Fernando Miguez [ctb],
  Arkajyoti Bhattacharjee [ctb]
Repository: CRAN
Date/Publication: 2026-05-01 11:30:02 UTC
Built: R 4.5.2; ; 2026-05-01 17:05:38 UTC; unix

S3method(coef,nlsr)
S3method(fitted,nlsr)
S3method(predict,nlsr)
S3method(print,nlsr)
S3method(residuals,nlsr)
S3method(summary,nlsr)
export(SSlogisJN)
export(SSmod2rjfun)
export(codeDeriv)
export(dex)
export(findSubexprs)
export(fnDeriv)
export(isCALL)
export(isMINUSONE)
export(isONE)
export(isZERO)
export(jaback)
export(jacentral)
export(jafwd)
export(jand)
export(model2rjfun)
export(model2ssgrfun)
export(modelexpr)
export(newDeriv)
export(newSimplification)
export(nlfb)
export(nlsDeriv)
export(nlsSimplify)
export(nlsr.control)
export(nlsr)
export(nlsrSS)
export(nlxb)
export(numericDerivR)
export(nvec)
export(pctrl)
export(pnlslm)
export(pnls)
export(prt)
export(pshort)
export(rawres)
export(resgr)
export(resss)
export(sysDerivs)
export(sysSimplifications)
export(wrapnlsr)
importFrom(digest,digest)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,deriv)
importFrom(stats,deviance)
importFrom(stats, getInitial)
importFrom(stats,nls)
importFrom(stats,pt)
importFrom(stats,setNames)
importFrom(utils,str)

This repo to allow access to others to working files in nlsr package by J C Nash as of 2022-9-1.

Attempts to alias wrapnlsr() as nlsr() have not been successful. However, explicit copy of code
from wrapnlsr() to nlsr() with name changes gives a sane package. Attempts with 

    nlsr <- wrapnlsr

gave check errors, mainly w.r.t. the .Rd manual files. 

JN 

## ----setup, echo=FALSE--------------------------------------------------------
rm(list=ls()) # clear workspace for each major section

## ----nlsbtsimple, echo=TRUE---------------------------------------------------
# Bounds Test nlsbtsimple.R (see BT.RES in Nash and Walker-Smith (1987))
rm(list=ls())
bt.res<-function(x){ x }
bt.jac<-function(x){nn <- length(x); JJ <- diag(nn); attr(JJ, "gradient") <- JJ;  JJ}
n <- 4
x<-rep(0,n) ; lower<-rep(NA,n); upper<-lower # to get arrays set
for (i in 1:n) { lower[i]<-1.0*(i-1)*(n-1)/n; upper[i]<-1.0*i*(n+1)/n }
x <-0.5*(lower+upper) # start on mean
xnames<-as.character(1:n) # name our parameters just in case
for (i in 1:n){ xnames[i]<-paste("p",xnames[i],sep='') }
names(x) <- xnames
require(minpack.lm)
require(nlsr)
bsnlf0<-nlfb(start=x, resfn=bt.res, jacfn=bt.jac) # unconstrained
bsnlf0
bsnlm0<-nls.lm(par=x, fn=bt.res, jac=bt.jac) # unconstrained
bsnlm0
bsnlf1<-nlfb(start=x, resfn=bt.res, jacfn=bt.jac, lower=lower, upper=upper)
bsnlf1
bsnlm1<-nls.lm(par=x, fn=bt.res, jac=bt.jac, lower=lower, upper=upper)
bsnlm1

# single value bounds
bsnlf2<-nlfb(start=x, resfn=bt.res, jacfn=bt.jac, lower=0.25, upper=4)
bsnlf2
# nls.lm will NOT expand single value bounds
bsnlm2<-try(nls.lm(par=x, fn=bt.res, jac=bt.jac, lower=0.25, upper=4))

## ----nlsLMoob, echo=TRUE------------------------------------------------------
x<-rep(0,n) # resetting to this puts x out of bounds
cat("lower:"); print(lower)
cat("upper:"); print(upper)
names(x) <- xnames # to ensure names set
obnlm<-nls.lm(par=x, fn=bt.res, jac=bt.jac, lower=lower, upper=upper)
obnlm

## ----Hobbssetup, echo=TRUE----------------------------------------------------
## Use the Hobbs Weed problem
weed <- c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443,
          38.558, 50.156, 62.948, 75.995, 91.972)
tt <- 1:12
weeddf <- data.frame(y=weed, tt=tt)
st <- c(b1=1, b2=1, b3=1) # a default starting vector (named!)
## Unscaled model
wmodu <- y ~ b1/(1+b2*exp(-b3*tt))
## Scaled model
wmods <- y ~ 100*b1/(1+10*b2*exp(-0.1*b3*tt))
## We can provide the residual and Jacobian as functions
# Unscaled Hobbs problem
hobbs.res  <-  function(x){ # scaled Hobbs weeds problem -- residual
  # This variant uses looping
  if(length(x) != 3) stop("hobbs.res -- parameter vector n!=3")
  y  <-  c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443, 
           38.558, 50.156, 62.948, 75.995, 91.972)
  tt  <-  1:12
  res  <-  x[1]/(1+x[2]*exp(-x[3]*tt)) - y
}

hobbs.jac  <-  function(x) { # scaled Hobbs weeds problem -- Jacobian
  jj  <-  matrix(0.0, 12, 3)
  tt  <-  1:12
  yy  <-  exp(-x[3]*tt)
  zz  <-  1.0/(1+x[2]*yy)
  jj[tt,1]   <-   zz
  jj[tt,2]   <-   -x[1]*zz*zz*yy
  jj[tt,3]   <-   x[1]*zz*zz*yy*x[2]*tt
  attr(jj, "gradient") <- jj
  jj
}
# Scaled Hobbs problem
shobbs.res  <-  function(x){ # scaled Hobbs weeds problem -- residual
  # This variant uses looping
  if(length(x) != 3) stop("shobbs.res -- parameter vector n!=3")
  y  <-  c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443, 
           38.558, 50.156, 62.948, 75.995, 91.972)
  tt  <-  1:12
  res  <-  100.0*x[1]/(1+x[2]*10.*exp(-0.1*x[3]*tt)) - y
}

shobbs.jac  <-  function(x) { # scaled Hobbs weeds problem -- Jacobian
  jj  <-  matrix(0.0, 12, 3)
  tt  <-  1:12
  yy  <-  exp(-0.1*x[3]*tt)
  zz  <-  100.0/(1+10.*x[2]*yy)
  jj[tt,1]   <-   zz
  jj[tt,2]   <-   -0.1*x[1]*zz*zz*yy
  jj[tt,3]   <-   0.01*x[1]*zz*zz*yy*x[2]*tt
  attr(jj, "gradient") <- jj
  jj
}

## ----exnlxb-------------------------------------------------------------------
require(nlsr)
# Use Hobbs scaled formula model
anlxb1  <-  try(nlxb(wmods, start=st, data=weeddf))
print(anlxb1)
# summary(anlxb1) # for different output
# Test without starting parameters
anlxb1nostart  <-  try(nlxb(wmodu, data=weeddf))
print(anlxb1nostart)

## ----nlxb1side1---------------------------------------------------------------
# One-sided unscaled Hobbs weed model formula
wmodu1  <-  ~ b1/(1+b2*exp(-b3*tt)) - y
anlxb11  <-  try(nlxb(wmodu1, start=st, data=weeddf))
print(anlxb11)

## ----c003---------------------------------------------------------------------
# st  <-  c(b1=1, b2=1, b3=1)
wrj <- model2rjfun(wmods, st, data=weeddf)
wrj
weedux <- nlxb(wmods, start=st, data=weeddf) 
print(weedux)

wss <- model2ssgrfun(wmods, st, data=weeddf)
print(wss)

# We can get expressions used to calculate these as follows:
wexpr.rj <- modelexpr(wrj) 
print(wexpr.rj)

wexpr.ss <- modelexpr(wss) 
print(wexpr.ss)

## ----c004---------------------------------------------------------------------
require(nlsr)

cat("try nlfb\n")
st  <-  c(b1=1, b2=1, b3=1)
ans1 <- nlfb(st, shobbs.res, shobbs.jac, trace=FALSE)
summary(ans1)

## No jacobian function -- use internal approximation
ans1n <- nlfb(st, shobbs.res, trace=FALSE, control=list(japprox="jafwd", watch=FALSE)) # NO jacfn -- tell it fwd approx
print(ans1n)
## difference
coef(ans1)-coef(ans1n)

## ----c001---------------------------------------------------------------------
coef(anlxb1) # this is solution of scaled problem from unit start

## ----c006---------------------------------------------------------------------
### From examples above
print(weedux)
print(ans1)

## ----c006a--------------------------------------------------------------------
### From examples above
summary(weedux)
summary(ans1)

## ----c009, echo=TRUE----------------------------------------------------------
## The following attempt at Hobbs unscaled with nls() fails!
rm(list=ls()) # clear before starting
weed <- c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443,
          38.558, 50.156, 62.948, 75.995, 91.972)
tt <- 1:12
weeddf <- data.frame(tt, weed)
st <- c(b1=1, b2=1, b3=1) # a default starting vector (named!)
wmodu<- weed ~ b1/(1 + b2 * exp(- b3 * tt))
anls1u  <-  try(nls(wmodu, start=st, trace=FALSE, data=weeddf)) # fails
## But we succeed by calling nlxb first.
library(nlsr)
anlxb1un  <-  try(nlxb(wmodu, start=st, trace=FALSE, data=weeddf))
print(anlxb1un)
st2 <- coef(anlxb1un) # We try to start nls from solution using nlxb
anls2u  <-  try(nls(wmodu, start=st2, trace=FALSE, data=weeddf))
print(anls2u)
## Or we can simply call wrapnlsr FROM THE ORIGINAL start
anls2a  <-  try(wrapnlsr(wmodu, start=st, trace=FALSE, data=weeddf))
summary(anls2a)
# # For comparison could run nlsLM
# require(minpack.lm)
# anlsLM1u  <-  try(nlsLM(wmodu, start=st, trace=FALSE, data=weeddf))
# print(anlsLM1u)

## ----c007, echo=TRUE----------------------------------------------------------
## Use shobbs example -- Scaled Hobbs problem
shobbs.res  <-  function(x){ # scaled Hobbs weeds problem -- residual
  if(length(x) != 3) stop("shobbs.res -- parameter vector n!=3")
  y  <-  c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443, 
           38.558, 50.156, 62.948, 75.995, 91.972)
  tt  <-  1:12
  res  <-  100.0*x[1]/(1+x[2]*10.*exp(-0.1*x[3]*tt)) - y
}

shobbs.jac  <-  function(x) { # scaled Hobbs weeds problem -- Jacobian
  jj  <-  matrix(0.0, 12, 3)
  tt  <-  1:12
  yy  <-  exp(-0.1*x[3]*tt)
  zz  <-  100.0/(1+10.*x[2]*yy)
  jj[tt,1]   <-   zz
  jj[tt,2]   <-   -0.1*x[1]*zz*zz*yy
  jj[tt,3]   <-   0.01*x[1]*zz*zz*yy*x[2]*tt
  attr(jj, "gradient") <- jj
  jj
}
RG <- resgr(st, shobbs.res, shobbs.jac)
RG
SS <- resss(st, shobbs.res)
SS

## ----c002---------------------------------------------------------------------
newDeriv() # a call with no arguments returns a list of available functions
           # for which derivatives are currently defined
newDeriv(sin(x)) # a call with a function that is in the list of available derivatives
                 # returns the derivative expression for that function
nlsDeriv(~ sin(x+y), "x") # partial derivative of this function w.r.t. "x"

## CAUTION !! ##
newDeriv(joe(x)) # but an undefined function returns NULL
newDeriv(joe(x), deriv=log(x^2)) # We can define derivatives, though joe() is meaningless.
nlsDeriv(~ joe(x+z), "x")

# Some examples of usage
f <- function(x) x^2
newDeriv(f(x), 2*x*D(x))
nlsDeriv(~ f(abs(x)), "x")
nlsDeriv(~ x^2, "x")
nlsDeriv(~ (abs(x)^2), "x")

# derivatives of distribution functions
nlsDeriv(~ pnorm(x, sd=2, log = TRUE), "x")
# get evaluation code from a formula
codeDeriv(~ pnorm(x, sd = sd, log = TRUE), "x")
# wrap it in a function call
fnDeriv(~ pnorm(x, sd = sd, log = TRUE), "x")
f <- fnDeriv(~ pnorm(x, sd = sd, log = TRUE), "x", args = alist(x =, sd = 2))
f
f(1)
100*(f(1.01) - f(1))  # Should be close to the gradient
                      # The attached gradient attribute (from f(1.01)) is
                      # meaningless after the subtraction.

## ----c008---------------------------------------------------------------------
## nlsSimplify
nlsSimplify(quote(a + 0))
nlsSimplify(quote(exp(1)), verbose = TRUE)

nlsSimplify(quote(sqrt(a + b)))  # standard rule
## sysSimplifications
# creates a new environment whose parent is emptyenv()  Why?
str(sysSimplifications)
myrules <- new.env(parent = sysSimplifications)
## newSimplification
newSimplification(sqrt(a), TRUE, a^0.5, simpEnv = myrules)
nlsSimplify(quote(sqrt(a + b)), simpEnv = myrules)

## isFALSE
print(isFALSE(1==2))
print(isFALSE(2==2))

## isZERO
print(isZERO(0))
x <- -0
print(isZERO(x))
x <- 0
print(isZERO(x))
print(isZERO(~(-1)))
print(isZERO("-1"))
print(isZERO(expression(-1)))


## isONE
print(isONE(1))
x <- 1
print(isONE(x))
print(isONE(~(1)))
print(isONE("1"))
print(isONE(expression(1)))


## isMINUSONE
print(isMINUSONE(-1))
x <- -1
print(isMINUSONE(x))
print(isMINUSONE(~(-1)))
print(isMINUSONE("-1"))
print(isMINUSONE(expression(-1)))

## isCALL  ?? don't have good understanding of this
x <- -1
print(isCALL(x,"isMINUSONE"))
print(isCALL(x, quote(isMINUSONE)))
      
## findSubexprs
findSubexprs(expression(x^2, x-y, y^2-x^2))

## sysDerivs
# creates a new environment whose parent is emptyenv()  Why? 
# Where are derivative definitions are stored?
str(sysDerivs)

## ----nlfbnumex, echo=TRUE-----------------------------------------------------
## Test with functional spec. of problem
## Default call WITH jacobian function
ans1 <- nlfb(st, resfn=shobbs.res, jacfn=shobbs.jac)
ans1
## No jacobian function -- and no japprox control setting
ans1n <- try(nlfb(st, shobbs.res)) # NO jacfn
ans1n
## Force jafwd approximation
ans1nf <- nlfb(st, shobbs.res, control=list(japprox="jafwd")) # NO jacfn, but specify fwd approx
ans1nf
## Alternative specification
ans1nfa <- nlfb(st, shobbs.res, jacfn="jafwd") # NO jacfn, but specify fwd approx in jacfn
ans1nfa
## Coeff differences from analytic: 
ans1nf$coefficients-ans1$coefficients
## Force jacentral approximation
ans1nc <- nlfb(st, shobbs.res, control=list(japprox="jacentral")) # NO jacfn
ans1nc
## Coeff differences from analytic:
ans1nc$coefficients-ans1$coefficients
## Force jaback approximation
ans1nb <- nlfb(st, shobbs.res, control=list(japprox="jaback")) # NO jacfn
ans1nb
## Coeff differences from analytic:
ans1nb$coefficients-ans1$coefficients
## Force jand approximation
ans1nn <- nlfb(st, shobbs.res, control=list(japprox="jand"), trace=FALSE) # NO jacfn
ans1nn
## Coeff differences from analytic:
ans1nc$coefficients-ans1$coefficients

## ----nlxbnumex1, echo=TRUE----------------------------------------------------
## rm(list=ls()) # clear workspace for each major section
## Use the Hobbs Weed problem
weed <- c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443,
          38.558, 50.156, 62.948, 75.995, 91.972)
tt <- 1:12
weeddf <- data.frame(y=weed, tt=tt)
st1 <- c(b1=1, b2=1, b3=1) # a default starting vector (named!)
wmods <- y ~ 100*b1/(1+10*b2*exp(-0.1*b3*tt)) ## Scaled model
print(wmods)
anlxb1  <-  try(nlxb(wmods, start=st1, data=weeddf))
print(anlxb1)

anlxb1fwd  <-  (nlxb(wmods, start=st1, data=weeddf,
    control=list(japprox="jafwd")))
print(anlxb1fwd)

## fwd - analytic
print(anlxb1fwd$coefficients - anlxb1$coefficients)

anlxb1bak  <- (nlxb(wmods, start=st1, data=weeddf,
    control=list(japprox="jaback")))
print(anlxb1bak)
## back - analytic
print(anlxb1bak$coefficients - anlxb1$coefficients)

anlxb1cen  <- (nlxb(wmods, start=st1, data=weeddf,
    control=list(japprox="jacentral")))
print(anlxb1cen)
## central - analytic
print(anlxb1cen$coefficients - anlxb1$coefficients)

anlxb1nd  <-  (nlxb(wmods, start=st1, data=weeddf,
    control=list(japprox="jand")))
print(anlxb1nd)
## numDeriv - analytic
print(anlxb1nd$coefficients - anlxb1$coefficients)

## ----wts1, echo=TRUE----------------------------------------------------------
## weighted nonlinear regression using inverse squared variance of the response
require(minpack.lm)
Treated <- Puromycin[Puromycin$state == "treated", ]
# We want the variance of each "group" of the rate variable 
# for which the conc variable is the same. We first find
# this variance by group using the tapply() function.
var.Treated <- tapply(Treated$rate, Treated$conc, var)
var.Treated <- rep(var.Treated, each = 2)
Pur.wt1nls <- nls(rate ~ (Vm * conc)/(K + conc), data = Treated, 
               start = list(Vm = 200, K = 0.1), weights = 1/var.Treated^2)
Pur.wt1nlm <- nlsLM(rate ~ (Vm * conc)/(K + conc), data = Treated, 
               start = list(Vm = 200, K = 0.1), weights = 1/var.Treated^2)
Pur.wt1nlx <- nlxb(rate ~ (Vm * conc)/(K + conc), data = Treated, 
               start = list(Vm = 200, K = 0.1), weights = 1/var.Treated^2)
rnls <- summary(Pur.wt1nls)$residuals
ssrnls<-as.numeric(crossprod(rnls))
rnlm <- summary(Pur.wt1nlm)$residuals
ssrnlm<-as.numeric(crossprod(rnlm))
rnlx <- Pur.wt1nlx$resid
ssrnlx<-as.numeric(crossprod(rnlx))
cat("Compare nls and nlsLM: ", all.equal(coef(Pur.wt1nls), coef(Pur.wt1nlm)),"\n")
cat("Compare nls and nlsLM: ", all.equal(coef(Pur.wt1nls), coef(Pur.wt1nlx)),"\n")
cat("Sumsquares nls - nlsLM: ", ssrnls-ssrnlm,"\n")
cat("Sumsquares nls - nlxb: ", ssrnls-ssrnlx,"\n")

## ----wtsfunction, eval=FALSE--------------------------------------------------
# ## minpack.lm::wfct
# ### Examples from 'nls' doc where wfct used ###
# ## Weighting by inverse of response 1/y_i:
# wtt1nlm<-nlsLM(rate ~ Vm * conc/(K + conc), data = Treated,
#               start = c(Vm = 200, K = 0.05), weights = wfct(1/rate))
# print(wtt1nlm)
# 
# wtt1nls<-nls(rate ~ Vm * conc/(K + conc), data = Treated,
#           start = c(Vm = 200, K = 0.05), weights = wfct(1/rate))
# print(wtt1nls)
# 
# wtt1nlx<-nlxb(rate ~ Vm * conc/(K + conc), data = Treated,
#           start = c(Vm = 200, K = 0.05), weights = wfct(1/rate))
# print(wtt1nlx)
# 
# ## Weighting by square root of predictor \sqrt{x_i}:
# # ?? why does try() not work
# wtt2nlm<-nlsLM(rate ~ Vm * conc/(K + conc), data = Treated,
#        start = c(Vm = 200, K = 0.05), weights = wfct(sqrt(conc)))
# print(wtt2nlm)
# 
# wtt2nls<-nls(rate ~ Vm * conc/(K + conc), data = Treated,
#        start = c(Vm = 200, K = 0.05), weights = wfct(sqrt(conc)))
# print(wtt2nls)
# 
# wtt2nlx<-nlxb(rate ~ Vm * conc/(K + conc), data = Treated,
#        start = c(Vm = 200, K = 0.05), weights = wfct(sqrt(conc)))
# print(wtt2nlx)
# 
# ## Weighting by inverse square of fitted values 1/\hat{y_i}^2:
# wtt3nlm<-nlsLM(rate ~ Vm * conc/(K + conc), data = Treated,
#        start = c(Vm = 200, K = 0.05), weights = wfct(1/fitted^2))
# print(wtt3nlm)
# 
# wtt3nls<-nls(rate ~ Vm * conc/(K + conc), data = Treated,
#        start = c(Vm = 200, K = 0.05), weights = wfct(1/fitted^2))
# print(wtt3nls)
# 
# # These don't work -- there is no fitted() function available in nlsr
# # wtt3nlx<-try(nlxb(rate ~ Vm * conc/(K + conc), data = Treated,
# #        start = c(Vm = 200, K = 0.05), weights = wfct(1/fitted^2)))
# ##  (nlxb(rate ~ Vm * conc/(K + conc), data = Treated,
# ##        start = c(Vm = 200, K = 0.05), weights = wfct(1/(resid+rate)^2)))
# ## (wrapnlsr(rate ~ Vm * conc/(K + conc), data = Treated,
# ##        start = c(Vm = 200, K = 0.05), weights = wfct(1/(resid+rate)^2)))
# 
# ## Weighting by inverse variance 1/\sigma{y_i}^2:
# wtt4nlm<-nlsLM(rate ~ Vm * conc/(K + conc), data = Treated,
#        start = c(Vm = 200, K = 0.05), weights = wfct(1/error^2))
# print(wtt4nlm)
# 
# wtt4nls<-nls(rate ~ Vm * conc/(K + conc), data = Treated,
#        start = c(Vm = 200, K = 0.05), weights = wfct(1/error^2))
# print(wtt4nls)
# 
# wtt4nlx<-nlxb(rate ~ Vm * conc/(K + conc), data = Treated,
#        start = c(Vm = 200, K = 0.05), weights = wfct(1/error^2))
# print(wtt4nlx)
# 
# wtt5nls<-nls(rate ~ Vm * conc/(K + conc), data = Treated,
#               start = c(Vm = 200, K = 0.05), weights = wfct(1/resid^2))
# print(wtt5nls)
# wtt5nlm<-nlsLM(rate ~ Vm * conc/(K + conc), data = Treated,
#               start = c(Vm = 200, K = 0.05), weights = wfct(1/resid^2))
# print(wtt5nlm)
# ## Won't work! No resid function available from nlxb.
# ## wtt5nlx<-nlxb(rate ~ Vm * conc/(K + conc), data = Treated,
# ##            start = c(Vm = 200, K = 0.05), weights = wfct(1/resid^2))
# ## print(wtt5nlx)

## ----nolhs, echo=TRUE---------------------------------------------------------
st<-c(b1=1, b2=1, b3=1)
frm <- ~ b1/(1+b2*exp(-b3*tt))-y
nolhs <- nlxb(formula=frm, data=weeddf, start=st)
nolhs

## ----wts1side-----------------------------------------------------------------
## weighted nonlinear regression using 1-sided model functions
Treated <- Puromycin[Puromycin$state == "treated", ]
weighted.MM <- function(resp, conc, Vm, K)
{
  ## Purpose: exactly as white book p. 451 -- RHS for nls()
  ##  Weighted version of Michaelis-Menten model
  ## ----------------------------------------------------------
  ## Arguments: 'y', 'x' and the two parameters (see book)
  ## ----------------------------------------------------------
  ## Author: Martin Maechler, Date: 23 Mar 2001
  
  pred <- (Vm * conc)/(K + conc)
  (resp - pred) / sqrt(pred)
}

Pur.wtMMnls <- nls( ~ weighted.MM(rate, conc, Vm, K), data = Treated,
               start = list(Vm = 200, K = 0.1))
print(Pur.wtMMnls)

Pur.wtMMnlm <- nlsLM( ~ weighted.MM(rate, conc, Vm, K), data = Treated,
                    start = list(Vm = 200, K = 0.1))
print(Pur.wtMMnlm)

Pur.wtMMnlx <- nlxb( ~ weighted.MM(rate, conc, Vm, K), data = Treated,
                    start = list(Vm = 200, K = 0.1), control=list(japprox="jafwd"))
print(Pur.wtMMnlx)

# Another approach
## Passing arguments using a list that can not be coerced to a data.frame
lisTreat <- with(Treated, list(conc1 = conc[1], conc.1 = conc[-1], rate = rate))
weighted.MM1 <- function(resp, conc1, conc.1, Vm, K){
  conc <- c(conc1, conc.1)
  pred <- (Vm * conc)/(K + conc)
  (resp - pred) / sqrt(pred)
}
Pur.wtMM1nls <- nls( ~ weighted.MM1(rate, conc1, conc.1, Vm, K),
                data = lisTreat, start = list(Vm = 200, K = 0.1))
print(Pur.wtMM1nls)
# Should we put in comparison of coeffs / sumsquares??
# stopifnot(all.equal(coef(Pur.wt), coef(Pur.wt1)))
Pur.wtMM1nlm <- nlsLM( ~ weighted.MM1(rate, conc1, conc.1, Vm, K),
                     data = lisTreat, start = list(Vm = 200, K = 0.1))
print(Pur.wtMM1nlm)
Pur.wtMM1nlx <- nlxb( ~ weighted.MM1(rate, conc1, conc.1, Vm, K),
                     data = lisTreat, start = list(Vm = 200, K = 0.1), control=list(japprox="jafwd"))
print(Pur.wtMM1nlx)
# yet another approach
## Chambers and Hastie (1992) Statistical Models in S  (p. 537):
## If the value of the right side [of formula] has an attribute called
## 'gradient' this should be a matrix with the number of rows equal
## to the length of the response and one column for each parameter.
weighted.MM.grad <- function(resp, conc1, conc.1, Vm, K) {
  conc <- c(conc1, conc.1)
  K.conc <- K+conc
  dy.dV <- conc/K.conc
  dy.dK <- -Vm*dy.dV/K.conc
  pred <- Vm*dy.dV
  pred.5 <- sqrt(pred)
  dev <- (resp - pred) / pred.5
  Ddev <- -0.5*(resp+pred)/(pred.5*pred)
  attr(dev, "gradient") <- Ddev * cbind(Vm = dy.dV, K = dy.dK)
  dev
}
Pur.wtMM.gradnls <- nls( ~ weighted.MM.grad(rate, conc1, conc.1, Vm, K),
                    data = lisTreat, start = list(Vm = 200, K = 0.1))
print(Pur.wtMM.gradnls)
Pur.wtMM.gradnlm <- nlsLM( ~ weighted.MM.grad(rate, conc1, conc.1, Vm, K),
                         data = lisTreat, start = list(Vm = 200, K = 0.1))
print(Pur.wtMM.gradnlm)
Pur.wtMM.gradnlx <- nlxb( ~ weighted.MM.grad(rate, conc1, conc.1, Vm, K),
                         data = lisTreat, start = list(Vm = 200, K = 0.1),
                         control=list(japprox="jafwd"))
print(Pur.wtMM.gradnlx)
#3 To display the coefficients for comparison:
## rbind(coef(Pur.wtMMnls), coef(Pur.wtMM1nls), coef(Pur.wtMM.gradnls))
## In this example, there seems no advantage to providing the gradient.
## In other cases, there might be.

## ----lls01--------------------------------------------------------------------
# QRsolveEx.R -- example of solving least squares with QR
# J C Nash 2020-12-06
# Enter matrix from Compact Numerical Methods for Computers, Ex04
text <- c(563, 262, 461, 221, 1, 305,
658, 291, 473, 222, 1, 342,
676, 294, 513, 221, 1, 331,
749, 302, 516, 218, 1, 339,
834, 320, 540, 217, 1, 354,
973, 350, 596, 218, 1, 369,
1079, 386, 650, 218, 1, 378,
1151, 401, 676, 225, 1, 368,
1324, 446, 769, 228, 1, 405,
1499, 492, 870, 230, 1, 438,
1690, 510, 907, 237, 1, 438,
1735, 534, 932, 235, 1, 451,
1778, 559, 956, 236, 1, 485)
mm<-matrix(text, ncol=6, byrow=TRUE) # byrow is critical!
A <- mm[,1:5] # select the matrix
A # display
b <- mm[,6] # rhs
b
oss<-as.numeric(crossprod(b))
cat("Overall sumsquares of b =",oss,"\n")
cat("(n-1)*variance=",(length(b)-1)*var(b),"= tss = ",as.numeric(crossprod(b-mean(b))),"\n")
Aqr<-qr(A) # compute the QR decomposition of A
QAqr<-qr.Q(Aqr) # extract the Q matrix of this decomposition
RAqr<-qr.R(Aqr) # This is how to get the R matrix
XAqr<-qr.X(Aqr) # And this is the reconstruction of A from the decomposition
cat("max reconstruction error = ", max(abs(XAqr-A)),"\n")
cat("check the orthogonality Q' * Q: ", max(abs(t(QAqr)%*%QAqr-diag(rep(1,dim(A)[2])))),"\n")
cat("note failure of orthogonality of Q * Q': ",max(abs(QAqr%*%t(QAqr)-diag(rep(1,dim(A)[1])))),"\n")
Absol<-qr.solve(A,b, tol=1000*.Machine$double.eps) # solve the linear ls problem
Absol # and display it 
Abres<-qr.resid(Aqr,b)# and get the residual
rss<-as.numeric(crossprod(Abres))
cat("residual sumsquares=",rss,"\n")
Qtb<-as.vector(t(QAqr)%*%b) # Q' * b -- turns out to be reduction in sumsquares
ssQtb<-as.numeric(crossprod(Qtb))
cat("oss-ssQtb = overall sumsquares minus sumsquares Q' * b = ",oss-ssQtb,"\n")
cat("Thus reduction in sumsquares is ", ssQtb,"\n")
Qtr<-as.vector(t(QAqr)%*%Abres) # projection of residuals onto range space of A
Qtr

## ----maskRvmmin---------------------------------------------------------------
require(optimx)
sq<-function(x){
   nn<-length(x)
   yy<-1:nn
   f<-sum((yy-x)^2)
#   cat("Fv=",f," at ")
#   print(x)
   f
}
sq.g <- function(x){
   nn<-length(x)
   yy<-1:nn
   gg<- 2*(x - yy)
}
xx <- c(.3, 4)
uncans <- Rvmmin(xx, sq, sq.g)
uncans
mybm <- c(0,1) # fix parameter 1
cans <- Rvmmin(xx, sq, sq.g, bdmsk=mybm)
cans

## ----masknlfb, echo=TRUE------------------------------------------------------
## rm(list=ls()) # clear workspace??
library(nlsr)
sqres<-function(x){
   nn<-length(x)
   yy<-1:nn
   res <- (yy-x)
}
sqjac <- function(x){
   nn<-length(x)
   yy<-1:nn
   JJ <- matrix(-1, nrow=nn, ncol=nn)
   attr(JJ, "gradient") <- JJ ## !! Critical statement
   JJ
}
xx <- c(.3, 4) # repeat for completeness
anlfbu<-nlfb(xx, sqres, sqjac)
anlfbu
anlfbc<-nlfb(xx, sqres, sqjac, lower=c(xx[1], -Inf), upper=c(xx[1], Inf), trace=FALSE)
anlfbc
# Following will warn All parameters are masked
anlfbe<-try(nlfb(xx, sqres, sqjac, lower=c(xx[1], xx[2]), upper=c(xx[1],xx[2]))) 

## ----masknlxb, echo=TRUE------------------------------------------------------
nn<-length(xx) # Also length(yy)
if (nn != 2) stop("This example has nn=2 only!")
yy<-1:2
v1 <- c(1,0); v2 <- c(0,1)
anlxbu <- nlxb(yy~v1*p1+v2*p2, start=c(p1=0.3, p2=4) )
anlxbu  
anlxbc <- nlxb(yy~v1*p1+v2*p2, start=c(p1=0.3, p2=4), masked=c("p1") )
anlxbc  

## ----masknlxb2, echo=TRUE-----------------------------------------------------
weed <- c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443,
   38.558, 50.156, 62.948, 75.995, 91.972)
ii <- 1:12
wdf <- data.frame(weed, ii)
weedux <- nlxb(weed~b1/(1+b2*exp(-b3*ii)), start=c(b1=200, b2=50, b3=0.3)) 
weedux
#- Old mechanism of 'nlsr' NO longer works
#- weedcx <- nlxb(weed~b1/(1+b2*exp(-b3*ii)), start=c(b1=200, b2=50, b3=0.3), masked=c("b1")) 
weedcx <- nlxb(weed~b1/(1+b2*exp(-b3*ii)), start=c(b1=200, b2=50, b3=0.3), 
               lower=c(b1=200, b2=0, b3=0), upper=c(b1=200, b2=100, b3=40)) 
weedcx
rfn <- function(bvec, weed=weed, ii=ii){
  res <- rep(NA, length(ii))
  for (i in ii){
    res[i]<- bvec[1]/(1+bvec[2]*exp(-bvec[3]*i))-weed[i]
  }
  res
}
weeduf <- nlfb(start=c(200, 50, 0.3),resfn=rfn,weed=weed, ii=ii, control=list(japprox="jacentral"))
weeduf
#- maskidx method to specify masks no longer works
#- weedcf <- nlfb(start=c(200, 50, 0.3),resfn=rfn,weed=weed, ii=ii, 
#-                maskidx=c(1), control=list(japprox="jacentral"))
weedcf <- nlfb(start=c(200, 50, 0.3),resfn=rfn,weed=weed, ii=ii, lower=c(200, 0, 0), 
               upper=c(200, 100, 40), control=list(japprox="jacentral"))
weedcf

## ----ssmicmen, echo=TRUE------------------------------------------------------
cat("=== SSmicmen ===\n")
dat <- PurTrt <- Puromycin[ Puromycin$state == "treated", ]
frm <- rate ~ SSmicmen(conc, Vm, K)
strt<-getInitial(frm, data = dat)
print(strt)
fm1x <- nlxb(frm, data = dat, start=strt,
             trace=FALSE, control=list(japprox="SSJac"))
fm1x

## ----partlin1-----------------------------------------------------------------
## Comment in example is "## using conditional linearity"
DNase1 <- subset(DNase, Run == 1) # data
## using nls with plinear
# Using a formula that explicitly includes the asymptote. (Default algorithm.)
fm2DNase1orig <- nls(density ~ Asym/(1 + exp((xmid - log(conc))/scal)),
                 data = DNase1,
                 start = list(Asym = 10, xmid = 0, scal = 1))
summary(fm2DNase1orig)
# Using conditional linearity. Note the linear paraemter does not appear explicitly.
#  And we must change the starting vector.
fm2DNase1 <- nls(density ~ 1/(1 + exp((xmid - log(conc))/scal)),
                 data = DNase1,
                 start = list(xmid = 0, scal = 1),
                 algorithm = "plinear")
summary(fm2DNase1)
# How to run with "port" algorithm -- NOT RUN
# fm2DNase1port <- nls(density ~ Asym/(1 + exp((xmid - log(conc))/scal)),
#                  data = DNase1,
#                  start = list(Asym = 10, xmid = 0, scal = 1),
#                  algorithm = "port")
# summary(fm2DNase1port)

# require(minpack.lm) # Does NOT offer VARPRO. First example is WRONG. NOT RUN.
# fm2DNase1mA <- nlsLM(density ~ 1/(1 + exp((xmid - log(conc))/scal)),
#                  data = DNase1,
#                  start = list(xmid = 0, scal = 1))
# summary(fm2DNase1mA)
# fm2DNase1mB <- nlsLM(density ~ Asym/(1 + exp((xmid - log(conc))/scal)),
#                  data = DNase1,
#                  start = list(Asym=10, xmid = 0, scal = 1))
# summary(fm2DNase1mB)
# 
# # This gets wrong answer. NOT RUN
# fm2DNase1xA <- nlxb(density ~ 1/(1 + exp((xmid - log(conc))/scal)),
#                  data = DNase1,
#                  start = list(xmid = 0, scal = 1))
# print(fm2DNase1xA)
# Original formula with nlsr::nlxb().
fm2DNase1xB <- nlxb(density ~ Asym/(1 + exp((xmid - log(conc))/scal)),
                 data = DNase1,
                 start = list(Asym=10, xmid = 0, scal = 1))
print(fm2DNase1xB)

## ----nls-indexed-ex-----------------------------------------------------------
## The muscle dataset in MASS is from an experiment on muscle
## contraction on 21 animals.  The observed variables are Strip
## (identifier of muscle), Conc (Cacl concentration) and Length
## (resulting length of muscle section).
utils::data(muscle, package = "MASS")

## The non linear model considered is
##       Length = alpha + beta*exp(-Conc/theta) + error
## where theta is constant but alpha and beta may vary with Strip.

with(muscle, table(Strip)) # 2, 3 or 4 obs per strip

## We first use the plinear algorithm to fit an overall model,
## ignoring that alpha and beta might vary with Strip.

musc.1 <- nls(Length ~ cbind(1, exp(-Conc/th)), muscle,
              start = list(th = 1), algorithm = "plinear")
summary(musc.1)

## Then we use nls' indexing feature for parameters in non-linear
## models to use the conventional algorithm to fit a model in which
## alpha and beta vary with Strip.  The starting values are provided
## by the previously fitted model.
## Note that with indexed parameters, the starting values must be
## given in a list (with names):
b <- coef(musc.1)
musc.2 <- nls(Length ~ a[Strip] + b[Strip]*exp(-Conc/th), muscle,
              start = list(a = rep(b[2], 21), b = rep(b[3], 21), th = b[1]))
## IGNORE_RDIFF_BEGIN
summary(musc.2)
## IGNORE_RDIFF_END

## ----nls-indexed-start--------------------------------------------------------
istart = list(a = rep(b[2], 21), b = rep(b[3], 21), th = b[1])
str(istart)

## ----appx1, cache=FALSE, echo=TRUE--------------------------------------------
# NOTE: This is OLD material and not consistent in usage with rest of vignette
library(knitr)
# try different ways of supplying data to R nls stuff
ydata <- c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443, 38.558,
           50.156, 62.948, 75.995, 91.972)
ttdata <- seq_along(ydata) # for testing
mydata <- data.frame(y = ydata, tt = ttdata)
hobsc <- y ~ 100*b1/(1 + 10*b2 * exp(-0.1 * b3 * tt))
ste <- c(b1 = 2, b2 = 5, b3 = 3)

# let's try finding the variables
findmainenv <- function(formula, prm) {
  vn <- all.vars(formula)
  pnames <- names(prm)
  ppos <- match(pnames, vn)
  datvar <- vn[-ppos]
  cat("Data variables:")
  print(datvar)
  cat("Are the variables present in the current working environment?\n")
  for (i in seq_along(datvar)){
    cat(datvar[[i]]," : present=",exists(datvar[[i]]),"\n")
  }
}

findmainenv(hobsc, ste)
y <- ydata
tt <- ttdata
findmainenv(hobsc, ste)
rm(y)
rm(tt)

# ===============================


# let's try finding the variables in dotargs
finddotargs <- function(formula, prm, ...) {
  dots <- list(...)
  cat("dots:")
  print(dots)
  cat("names in dots:")
  dtn <- names(dots)
  print(dtn)
  vn <- all.vars(formula)
  pnames <- names(prm)
  ppos <- match(pnames, vn)
  datvar <- vn[-ppos]
  cat("Data variables:")
  print(datvar)
  cat("Are the variables present in the dot args?\n")
  for (i in seq_along(datvar)){
    dname <- datvar[[i]]
    cat(dname," : present=",(dname %in% dtn),"\n")
  }
}

finddotargs(hobsc, ste, y=ydata, tt=ttdata)
# ===============================

y <- ydata
tt <- ttdata
tryq <- try(nlsquiet <- nls(formula=hobsc, start=ste))
if (class(tryq) != "try-error") {print(nlsquiet)} else {cat("try-error\n")}
#- OK
rm(y); rm(tt)

## this will fail
tdots1<-try(nlsdots <- nls(formula=hobsc, start=ste, y=ydata, tt=ttdata))
# But ...
y <- ydata # put data in globalenv
tt <- ttdata
tdots2<-try(nlsdots <- nls(formula=hobsc, start=ste))
tdots2
rm(y); rm(tt)
## but ...
mydata<-data.frame(y=ydata, tt=ttdata)
tframe <- try(nlsframe <- nls(formula=hobsc, start=ste, data=mydata))
if (class(tframe) != "try-error") {print(nlsframe)} else {cat("try-error\n")}
#- OK

y <- ydata
tt <- ttdata
tquiet <- try(nlsrquiet <- nlxb(formula=hobsc, start=ste))
if ( class(tquiet) != "try-error") {print(nlsrquiet)}
#- OK
rm(y); rm(tt)
test <- try(nlsrdots <- nlxb(formula=hobsc, start=ste, y=ydata, tt=ttdata))
#- Note -- does NOT work -- do we need to specify the present env. in nlfb for y, tt??
if (class(test) != "try-error") { print(nlsrdots) } else {cat("Try error\n") }
test2 <- try(nlsframe <- nls(formula=hobsc, start=ste, data=mydata))
if (class(test) != "try-error") {print(nlsframe) } else {cat("Try error\n") }
#- OK


library(minpack.lm)
y <- ydata
tt <- ttdata
nlsLMquiet <- nlsLM(formula=hobsc, start=ste)
print(nlsLMquiet)
#- OK
rm(y)
rm(tt)
## Dotargs
## Following fails
## tdots <- try(nlsLMdots <- nlsLM(formula=hobsc, start=ste, y=ydata, tt=ttdata))
## but ... 
tdots <- try(nlsLMdots <- nlsLM(formula=hobsc, start=ste, data=mydata))
if (class(tdots) != "try-error") { print(nlsLMdots) } else {cat("try-error\n") }
#-  Note -- does NOT work
## dataframe
tframe <- try(nlsLMframe <- nlsLM(formula=hobsc, start=ste, data=mydata) )
if (class(tdots) != "try-error") {print(nlsLMframe)} else {cat("try-error\n") }
#- does not work

## detach("package:nlsr", unload=TRUE)
## Uses nlmrt here for comparison
## library(nlmrt)
## txq <- try( nlxbquiet <- nlxb(formula=hobsc, start=ste))
## if (class(txq) != "try-error") {print(nlxbquiet)} else { cat("try-error\n")}
#- Note -- does NOT work
## txdots <- try( nlxbdots <- nlxb(formula=hobsc, start=ste, y=y, tt=tt) )
## if (class(txdots) != "try-error") {print(nlxbdots)} else {cat("try-error\n")}
#- Note -- does NOT work
## dataframe
## nlxbframe <- nlxb(formula=hobsc, start=ste, data=mydata)
## print(nlxbframe)
#- OK

## ----rndr, code=readLines("../R/numericDerivR.R"), echo=TRUE------------------
numericDerivR <- function(expr, theta, rho = parent.frame(), dir = 1,
                 eps = .Machine$double.eps ^ (1/if(central) 3 else 2), central = FALSE)
## Note: Must this expr must be set up as a call to work properly?
## We set eps conditional on central. But central set AFTER eps. Is this OK?
{   ## cat("numericDeriv-Alt\n")
    dir <- rep_len(dir, length(theta))
    stopifnot(is.finite(eps), eps > 0)
##    rho1 <- new.env(FALSE, rho, 0)
    if (!is.character(theta) ) {stop("'theta' should be of type character")}
    if (is.null(rho)) {
            stop("use of NULL environment is defunct")
            #        rho <- R_BaseEnv;
    } else {
          if(! is.environment(rho)) {stop("'rho' should be an environment")}
          #    int nprot = 3;
    }
    if( ! ((length(dir) == length(theta) ) & (is.numeric(dir) ) ) )
              {stop("'dir' is not a numeric vector of the correct length") }
    if(is.na(central)) { stop("'central' is NA, but must be TRUE or FALSE") }
    res0 <- eval(expr, rho) # the base residuals. C has a check for REAL ANS=res0
    nt <- length(theta) # number of parameters
    mr <- length(res0) # number of residuals
    JJ <- matrix(NA, nrow=mr, ncol=nt) # Initialize the Jacobian
    for (j in 1:nt){
       origPar<-get(theta[j],rho) # This is parameter by NAME, not index
       xx <- abs(origPar)
       delta <- if (xx == 0.0) {eps} else { xx*eps }
       ## JN: I prefer eps*(xx + eps)  which is simpler?
       prmx<-origPar+delta*dir[j]
       assign(theta[j],prmx,rho)
       res1 <- eval(expr, rho) # new residuals (forward step)
#       cat("res1:"); print(res1)
       if (central) { # compute backward step resids for central diff
          prmb <- origPar - dir[j]*delta
          assign(theta[j], prmb, envir=rho) # may be able to make more efficient later?
          resb <- eval(expr, rho)
          JJ[, j] <- dir[j]*(res1-resb)/(2*delta) # vectorized
       } else { ## forward diff
          JJ[, j] <- dir[j]*(res1-res0)/delta
       }  # end forward diff
       assign(theta[j],origPar, rho) # reset value 
    } # end loop over the parameters
    attr(res0, "gradient") <- JJ
    return(res0)
}

## ----ndx, echo=TRUE-----------------------------------------------------------
library(nlsr) # so we have numericDerivR code
# Data for Hobbs problem
weed  <-  c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443, 
            38.558, 50.156, 62.948, 75.995, 91.972) # for testing
tt  <-  seq_along(weed) # for testing
# A simple starting vector -- must have named parameters for nlxb, nls, wrapnlsr.
st  <-  c(b1=1, b2=1, b3=1)
wmodu  <-   weed ~ b1/(1+b2*exp(-b3*tt))
weeddf  <-  data.frame(weed=weed, tt=tt)
weedenv <- list2env(weeddf)
weedenv$b1 <- st[[1]]
weedenv$b2 <- st[[2]]
weedenv$b3 <- st[[3]]
rexpr<-call("-",wmodu[[3]], wmodu[[2]])
r0<-eval(rexpr, weedenv)
cat("Sumsquares at 1,1,1 is ",sum(r0^2),"\n")## Another way
expr <- wmodu
rho <- weedenv
rexpr<-call("-",wmodu[[3]], wmodu[[2]])
res0<-eval(rexpr, rho) # the base residuals
res0
## Try the numericDeriv option
theta<-names(st)
nDnls<-numericDeriv(rexpr, theta, weedenv)
nDnls
nDnlsR<-numericDerivR(rexpr, theta, weedenv)
nDnlsR

## ----derivexp, eval=TRUE------------------------------------------------------
partialderiv <- D(expression(a * (x ^ b)),"b")
print(partialderiv)

## ----timingshobbsmodel--------------------------------------------------------
require(microbenchmark)
## nls on Hobbs scaled model
wmods  <-   weed ~ 100*b1/(1+10*b2*exp(-0.1*b3*tt))
stx<-c(b1=2, b2=5, b3=3)
tnls<-microbenchmark((anls<-nls(wmods, start=stx, data=weeddf)), unit="us")
tnls
## nlsr::nlfb() on Hobbs scaled model
tnlxb<-microbenchmark((anlxb<-nlsr::nlxb(wmods, start=stx, data=weeddf)), unit="us")
tnlxb
## minpack.lm::nlsLM() on Hobbs scaled model
tnlsLM<-microbenchmark((anlsLM<-minpack.lm::nlsLM(start=stx, formula=wmods, data=weeddf)), 
                       unit="us")
tnlsLM

## ----timingnlsrminpackfn------------------------------------------------------
require(microbenchmark)
## nlsr::nlfb() on Hobbs scaled
tnlfb<-microbenchmark((anlfb<-nlsr::nlfb(start=st1, resfn=shobbs.res, jacfn=shobbs.jac)), unit="us")
tnlfb
## minpack.lm::nls.lm() on Hobbs scaled
tnls.lm<-microbenchmark((anls.lm<-minpack.lm::nls.lm(par=st1, fn=shobbs.res, jac=shobbs.jac)))
tnls.lm

## ----hiddenexprob1, echo=FALSE------------------------------------------------
# Here we set up an example problem with data
# Define independent variable
t0 <- 0:19
t0a<-t0
t0a[1]<-1e-20 # very small value
# Drop first value in vectors
t0t<-t0[-1]
y1 <- 4 * (t0^0.25)
y1t<-y1[-1]
n <- length(t0)
fuzz <- rnorm(n)
range <- max(y1)-min(y1)
## add some "error" to the dependent variable
y1q <- y1 + 0.2*range*fuzz
edta <- data.frame(t0=t0, t0a=t0a, y1=y1, y1q=y1q)
edtat <- data.frame(t0t=t0t, y1t=y1t)

start1 <- c(a=1, b=1)
try(nlsy0t0ax <- nls(formula=y1~a*(t0a^b), start=start1, data=edta, control=nls.control(maxiter=10000)))
nlsry1t0a <- nlxb(formula=y1~a*(t0a^b), start=start1, data=edta)
library(minpack.lm)
nlsLMy1t0a <- nlsLM(formula=y1~a*(t0a^b), start=start1, data=edta)

## ----nlsstr-------------------------------------------------------------------
str(nlsy0t0ax)

## ----nlsrstr------------------------------------------------------------------
str(nlsry1t0a)

## ----pred1--------------------------------------------------------------------
nudta <- colMeans(edta)
predict(nlsy0t0ax, newdata=nudta)
predict(nlsLMy1t0a, newdata=nudta)
predict(nlsry1t0a, newdata=nudta)

## ----exprob1------------------------------------------------------------------
# Here we set up an example problem with data
# Define independent variable
t0 <- 0:19
t0a<-t0
t0a[1]<-1e-20 # very small value
# Drop first value in vectors
t0t<-t0[-1]
y1 <- 4 * (t0^0.25)
y1t<-y1[-1]
n <- length(t0)
fuzz <- rnorm(n)
range <- max(y1)-min(y1)
## add some "error" to the dependent variable
y1q <- y1 + 0.2*range*fuzz
edta <- data.frame(t0=t0, t0a=t0a, y1=y1, y1q=y1q)
edtat <- data.frame(t0t=t0t, y1t=y1t)

## ----extrynls-----------------------------------------------------------------
cprint <- function(obj){
   # print object if it exists
  sobj<-deparse(substitute(obj))
  if (exists(sobj)) {
      print(obj)
  } else {
      cat(sobj," does not exist\n")
  }
#  return(NULL)
}
start1 <- c(a=1, b=1)
try(nlsy0t0 <- nls(formula=y1~a*(t0^b), start=start1, data=edta))
cprint(nlsy0t0)
# Since this fails to converge, let us increase the maximum iterations
try(nlsy0t0x <- nls(formula=y1~a*(t0^b), start=start1, data=edta,
                    control=nls.control(maxiter=10000)))
cprint(nlsy0t0x)
try(nlsy0t0ax <- nls(formula=y1~a*(t0a^b), start=start1, data=edta, 
                     control=nls.control(maxiter=10000)))
cprint(nlsy0t0ax)
try(nlsy0t0t <- nls(formula=y1t~a*(t0t^b), start=start1, data=edtat))
cprint(nlsy0t0t)

## ----extry1nlsr---------------------------------------------------------------
nlsry1t0 <- try(nlxb(formula=y1~a*(t0^b), start=start1, data=edta))
cprint(nlsry1t0)
nlsry1t0a <- nlxb(formula=y1~a*(t0a^b), start=start1, data=edta)
cprint(nlsry1t0a)
nlsry1t0t <- nlxb(formula=y1t~a*(t0t^b), start=start1, data=edtat)
cprint(nlsry1t0t)

## ----extry1minlm--------------------------------------------------------------
library(minpack.lm)
nlsLMy1t0 <- nlsLM(formula=y1~a*(t0^b), start=start1, data=edta)
nlsLMy1t0
nlsLMy1t0a <- nlsLM(formula=y1~a*(t0a^b), start=start1, data=edta)
nlsLMy1t0a
nlsLMy1t0t <- nlsLM(formula=y1t~a*(t0t^b), start=start1, data=edtat)
nlsLMy1t0t

## ----brownden-----------------------------------------------------------------
m <- 20
t <- seq(1, m) / 5
y <- rep(0,m)
library(nlsr)
library(minpack.lm)

bddata <- data.frame(t=t, y=y)
bdform <- y ~ ((x1 + t * x2 - exp(t))^2 + (x3 + x4 * sin(t) - cos(t))^2)
prm0 <- c(x1=25, x2=5, x3=-5, x4=-1)
fbd <-model2ssgrfun(bdform, prm0, bddata)
cat("initial sumsquares=",as.numeric(crossprod(fbd(prm0))),"\n")
nlsrbd <- nlxb(bdform, start=prm0, data=bddata, trace=FALSE)
nlsrbd

nlsbd10k <- nls(bdform, start=prm0, data=bddata, trace=FALSE, 
                control=nls.control(maxiter=10000))
nlsbd10k

nlsLMbd10k <- nlsLM(bdform, start=prm0, data=bddata, trace=FALSE, 
                    control=nls.lm.control(maxiter=10000, maxfev=10000))
nlsLMbd10k

## ----browndenpred-------------------------------------------------------------
ndata <- data.frame(t=c(5,6), y=c(0,0))
predict(nlsLMbd10k, newdata=ndata)

# now nls
predict(nlsbd10k, newdata=ndata)

# now nlsr
predict(nlsrbd, newdata=ndata)

## ----brownden2----------------------------------------------------------------
bdf2 <-  (x1 + t * x2 - exp(t))^2 ~ - (x3 + x4 * sin(t) - cos(t))^2

## ----bdcheck, eval=FALSE------------------------------------------------------
# #' Brown and Dennis Function
# #'
# #' Test function 16 from the More', Garbow and Hillstrom paper.
# #'
# #' The objective function is the sum of \code{m} functions, each of \code{n}
# #' parameters.
# #'
# #' \itemize{
# #'   \item Dimensions: Number of parameters \code{n = 4}, number of summand
# #'   functions \code{m >= n}.
# #'   \item Minima: \code{f = 85822.2} if \code{m = 20}.
# #' }
# #'
# #' @param m Number of summand functions in the objective function. Should be
# #'   equal to or greater than 4.
# #' @return A list containing:
# #' \itemize{
# #'   \item \code{fn} Objective function which calculates the value given input
# #'   parameter vector.
# #'   \item \code{gr} Gradient function which calculates the gradient vector
# #'   given input parameter vector.
# #'   \item \code{fg} A function which, given the parameter vector, calculates
# #'   both the objective value and gradient, returning a list with members
# #'   \code{fn} and \code{gr}, respectively.
# #'   \item \code{x0} Standard starting point.
# #' }
# #' @references
# #' More', J. J., Garbow, B. S., & Hillstrom, K. E. (1981).
# #' Testing unconstrained optimization software.
# #' \emph{ACM Transactions on Mathematical Software (TOMS)}, \emph{7}(1), 17-41.
# #' \url{https://doi.org/10.1145/355934.355936}
# #'
# #' Brown, K. M., & Dennis, J. E. (1971).
# #' \emph{New computational algorithms for minimizing a sum of squares of
# #' nonlinear functions} (Report No. 71-6).
# #' New Haven, CT: Department of Computer Science, Yale University.
# #'
# #' @examples
# #' # Use 10 summand functions
# #' fun <- brown_den(m = 10)
# #' # Optimize using the standard starting point
# #' x0 <- fun$x0
# #' res_x0 <- stats::optim(par = x0, fn = fun$fn, gr = fun$gr, method =
# #' "L-BFGS-B")
# #' # Use your own starting point
# #' res <- stats::optim(c(0.1, 0.2, 0.3, 0.4), fun$fn, fun$gr, method =
# #' "L-BFGS-B")
# #'
# #' # Use 20 summand functions
# #' fun20 <- brown_den(m = 20)
# #' res <- stats::optim(fun20$x0, fun20$fn, fun20$gr, method = "L-BFGS-B")
# #' @export
# #`
# brown_den <- function(m = 20) {
#   list(
#     fn = function(par) {
#       x1 <- par[1]
#       x2 <- par[2]
#       x3 <- par[3]
#       x4 <- par[4]
# 
#       ti <- (1:m) * 0.2
#       l <- x1 + ti * x2 - exp(ti)
#       r <- x3 + x4 * sin(ti) - cos(ti)
#       f <- l * l + r * r
#       sum(f * f)
#     },
#     gr = function(par) {
#       x1 <- par[1]
#       x2 <- par[2]
#       x3 <- par[3]
#       x4 <- par[4]
# 
#       ti <- (1:m) * 0.2
#       sinti <- sin(ti)
#       l <- x1 + ti * x2 - exp(ti)
#       r <- x3 + x4 * sinti - cos(ti)
#       f <- l * l + r * r
#       lf4 <- 4 * l * f
#       rf4 <- 4 * r * f
#       c(
#         sum(lf4),
#         sum(lf4 * ti),
#         sum(rf4),
#         sum(rf4 * sinti)
#       )
#     },
#     fg = function(par) {
#       x1 <- par[1]
#       x2 <- par[2]
#       x3 <- par[3]
#       x4 <- par[4]
# 
#       ti <- (1:m) * 0.2
#       sinti <- sin(ti)
#       l <- x1 + ti * x2 - exp(ti)
#       r <- x3 + x4 * sinti - cos(ti)
#       f <- l * l + r * r
#       lf4 <- 4 * l * f
#       rf4 <- 4 * r * f
# 
#       fsum <- sum(f * f)
#       grad <- c(
#         sum(lf4),
#         sum(lf4 * ti),
#         sum(rf4),
#         sum(rf4 * sinti)
#       )
# 
#       list(
#         fn = fsum,
#         gr = grad
#       )
#     },
#     x0 = c(25, 5, -5, 1)
#   )
# }
# mbd <- brown_den(m=20)
# mbd
# mbd$fg(mbd$x0)
# bdsolnm <- optim(mbd$x0, mbd$fn, control=list(trace=0))
# bdsolnm
# bdsolbfgs <- optim(mbd$x0, mbd$fn, method="BFGS", control=list(trace=0))
# bdsolbfgs
# 
# library(optimx)
# methlist <- c("Nelder-Mead","BFGS","Rvmmin","L-BFGS-B","Rcgmin","ucminf")
# 
# solo <- opm(mbd$x0, mbd$fn, mbd$gr, method=methlist, control=list(trace=0))
# summary(solo, order=value)
# 
# ## A failure above is generally because a package in the 'methlist' is not installed.

## ----setup, echo=FALSE--------------------------------------------------------
rm(list=ls()) # clear workspace for each major section

## ----chunk01------------------------------------------------------------------
dx2x <- deriv(~ x^2, "x") 
dx2x
mode(dx2x)
str(dx2x)
x <- -1:2
eval(dx2x) # This is evaluated at -1, 0, 1, 2, with the result in the "gradient" attribute
# Note that we cannot (easily) differentiate this again.
firstd <- attr(dx2x,"gradient")
str
#  ... and the following gives an error
d2x2x <- try(deriv(firstd, "x"))
str(d2x2x)
#- Build a function from the expression
fdx2x<-function(x){eval(dx2x)}
fdx2x(1)
fdx2x(3.21)
fdx2x(1:5)
#- # Now try D()
Dx2x <- D(expression(x^2), "x")
Dx2x
x <- -1:2
eval(Dx2x)
# We can differentiate aggain
D2x2x <- D(Dx2x,"x")
D2x2x
eval(D2x2x) #- But we don't get a vector -- could be an issue in gradients/Jacobians
#- Note how we handle an expression stored in a string via parse(text=  ))
sx2 <- "x^2"
sDx2x <- D(parse(text=sx2), "x")
sDx2x
#- But watch out! The following "seems" to work, but the answer is not as intended.  The problem is that the first
# argument is evaluated before being used.  Since 
# x exists, it fails
x
Dx2xx <- D(x^2, "x")
Dx2xx
eval(Dx2xx)
#-  Something 'tougher':
trig.exp <- expression(sin(cos(x + y^2)))
( D.sc <- D(trig.exp, "x") )
all.equal(D(trig.exp[[1]], "x"), D.sc)
( dxy <- deriv(trig.exp, c("x", "y")) )
y <- 1
eval(dxy)
eval(D.sc)
#-  function returned:
deriv((y ~ sin(cos(x) * y)), c("x","y"), func = TRUE)
#- ??#-  Surely there is an error, since documentation says no lhs! i.e.,
#- "expr: a 'expression' or 'call' or (except 'D') a formula with no lhs."
#-  function with defaulted arguments:
(fx <- deriv(y ~ b0 + b1 * 2^(-x/th), c("b0", "b1", "th"),
             function(b0, b1, th, x = 1:7){} ) )
fx(2, 3, 4)
#-  First derivative
D(expression(x^2), "x")
#-  stopifnot(D(as.name("x"), "x") == 1) #- A way of testing. 
#- This works by coercing "x" to name/symbol, and derivative should be 1.
#- Would fail only if "x" cannot be so coerced. How could this happen??
#-  Higher derivatives showing deriv3
myd3 <- deriv3(y ~ b0 + b1 * 2^(-x/th), c("b0", "b1", "th"),
     c("b0", "b1", "th", "x") )
myd3(2,3,4, x=1:7)
#- check against deriv()
myd3a <- deriv(y ~ b0 + b1 * 2^(-x/th), c("b0", "b1", "th"),
     c("b0", "b1", "th", "x"), hessian=TRUE )
myd3a(2,3,4, x=1:7)
identical(myd3a, myd3) #- Remember to check things!

#-  Higher derivatives:
DD <- function(expr, name, order = 1) {
   if(order < 1) stop("'order' must be >= 1")
   if(order == 1) D(expr, name)
   else DD(D(expr, name), name, order - 1)
}
DD(expression(sin(x^2)), "x", 3)
#-  showing the limits of the internal "simplify()" :
#-  -sin(x^2) * (2 * x) * 2 + ((cos(x^2) * (2 * x) * (2 * x) + sin(x^2) *
#-     2) * (2 * x) + sin(x^2) * (2 * x) * 2)

## ----chunk02------------------------------------------------------------------
library(nlsr)
dx2xn <- nlsDeriv(~ x^2, "x")
dx2xn
mode(dx2xn)
str(dx2xn)
x <- -1:2
eval(dx2xn) # This is evaluated at -1, 0, 1, 2, BUT result is returned directly,
#-  NOT in "gradient" attribute
firstdn <- dx2xn
str(firstdn)
d2x2xn <- nlsDeriv(firstdn, "x")
d2x2xn
d2x2xnF <- nlsDeriv(firstdn, "x", do_substitute=FALSE)
d2x2xnF # in this case we get the same result
d2x2xnT <- nlsDeriv(firstdn, "x", do_substitute=TRUE)
d2x2xnT # 0 ## WATCH OUT

#- ?? We can iterate the derivatives
nlsDeriv(d2x2xn, "x")

nlsDeriv(x^2, "x")# 0
nlsDeriv(x^2, "x", do_substitute=FALSE)# 0
nlsDeriv(x^2, "x", do_substitute=TRUE) # 2 * x
nlsDeriv(~ x^2, "x") # 2 * x
nlsDeriv(~ x^2, "x", do_substitute=FALSE) # 2 * x
nlsDeriv(~ x^2, "x", do_substitute=TRUE) # 2 * x

#?? firstde <- quote(firstd)
#?? firstde
#?? firstde <- bquote(firstd)
#?? firstde
#?? nlsDeriv(firstde, "x")
d2 <- nlsDeriv(2 * x, "x")
str(d2)
d2
#?? firstc <- as.call(firstd)
#?? nlsDeriv(firstc, "x")
#- Build a function from the expression
#?? fdx2xn<-function(x){eval(dx2xn)}
#?? fdx2xn(1)
#?? fdx2xn(3.21)
#?? fdx2xn(1:5)

## ----chunk03------------------------------------------------------------------
codeDeriv(parse(text="b0 + b1 * 2^(-x/th)"), c("b0", "b1", "th")) 
#- Include parameters as arguments
fj.1 <- fnDeriv(parse(text="b0 + b1 * 2^(-x/th)"), c("b0", "b1", "th")) 
head(fj.1)
fj.1(1,2,3,4)
#- Get all parameters from the calling environment
fj.2 <- fnDeriv(parse(text="b0 + b1 * 2^(-x/th)"), c("b0", "b1", "th"),
        args = character())
head(fj.2)
b0 <- 1
b1 <- 2
x <- 3
th <- 4 
fj.2()

#- Just use an expression
fje <- codeDeriv(parse(text="b0 + b1 * 2^(-x/th)"), c("b0", "b1", "th"))
eval(fje)

dx2xnf <- fnDeriv(~ x^2, "x") #- Use tilde
dx2xnf <- fnDeriv(expression(x^2), "x") #- or use expression()
dx2xnf
mode(dx2xnf)
str(dx2xnf)
x <- -1:2
#?? eval(dx2xnf) # This is evaluated at -1, 0, 1, 2, BUT result is returned directly,
#-  NOT in "gradient" attribute
# Note that we cannot (easily) differentiate this again.
# firstd <- dx2xnf
# str(firstd)
# d2x2xnf <- try(nlsDeriv(firstd, "x")) #- this APPEARS to work, but WRONG answer 
# str(d2x2xnf)
# d2x2xnf
# eval(d2x2xnf)

# dx2xnfh <- fnDeriv(expression(x^2), "x", hessian=TRUE) #- Try for second derivatives
# dx2xnfh
# mode(dx2xnfh)
# str(dx2xnfh)
# x <- -1
# eval(dx2xnfh) # This is evaluated at -1, 0, 1, 2, BUT result is returned directly,

## ----chunk04------------------------------------------------------------------
require(Deriv)

f <- function(x) x^2
Deriv(f)
#- Should see
#- function (x)
#- 2 * x
#- Now save the derivative
f1 <- Deriv(f)
f1 #- print it
f2 <- Deriv(f1) #- and take second derivative
f2 #- print it

f <- function(x, y) sin(x) * cos(y)
f_ <- Deriv(f)
f_ #- print it
#- Should see
#- function (x, y)
#- c(x = cos(x) * cos(y), y = -(sin(x) * sin(y)))
f_(3, 4)
#- Should see
#-              x         y
#- [1,] 0.6471023 0.1068000

f2 <- Deriv(~ f(x, y^2), "y") #- This has a tilde to render the 1st argument as a formula object
#- Also we are substituting in y^2 for y
f2 #- print it
#- -(2 * (y * sin(x) * sin(y^2)))
mode(f2) #- check what type of object it is
arg1 <- ~ f(x,y^2)
mode(arg1) #- check the type
f2a <- Deriv(arg1, "y")
f2a #- and print to see if same as before
#- try evaluation of f using current x and y
x
y
f(x,y^2)
eval(f2a) #- We need x and y defined to do this.

f3 <- Deriv(quote(f(x, y^2)), c("x", "y"), cache.exp=FALSE) #- check cache.exp operation
#- Note that we need to quote or will get evaluation at current x, y values (if they exist)
f3 #- print it
#- c(x = cos(x) * cos(y^2), y = -(2 * (y * sin(x) * sin(y^2))))
f3c <- Deriv(quote(f(x, y^2)), c("x", "y"), cache.exp=TRUE) #- check cache.exp operation
f3c #- print it
#- Now want to evaluate the results
#- First must provide some data
x <- 3
y <- 4
eval(f3c)
#- Should see
#- x         y 
#- 0.9480757 0.3250313 
eval(f3) #- check this also
#- or we can create functions
f3cf <- function(x, y){eval(f3c)}
f3cf(x=1, y=2)
#-          x          y 
#- -0.3531652  2.5473094 
f3f <- function(x,y){eval(f3)}
f3f(x=3, y=4)
#-         x         y 
#- 0.9480757 0.3250313 

#- try an expression
Deriv(expression(sin(x^2) * y), "x")
#- should see
#- expression(2 * (x * y * cos(x^2)))

#- quoted string
Deriv("sin(x^2) * y", "x") # differentiate only by x
#- Should see
#- "2 * (x * y * cos(x^2))"

Deriv("sin(x^2) * y", cache.exp=FALSE) #- differentiate by all variables (here by x and y)
#- Note that default is to differentiate by all variables.
#- Should see
#- "c(x = 2 * (x * y * cos(x^2)), y = sin(x^2))"

#- Compound function example (here abs(x) smoothed near 0)
#- Note that this introduces the possibilty of `if` statements in the code
#- BUT (JN) seems to give back quoted string, so we must parse.
fc <- function(x, h=0.1) if (abs(x) < h) 0.5*h*(x/h)**2 else abs(x)-0.5*h
efc1 <- Deriv("fc(x)", "x", cache.exp=FALSE) 
#- "if (abs(x) < h) x/h else sign(x)"
#- A few checks on the results
efc1

fc1 <- function(x,h=0.1){ eval(parse(text=efc1)) }
fc1
## h=0.1
fc1(1)
fc1(0.001)
fc1(-0.001)
fc1(-10)
fc1(0.001, 1)

#- Example of a first argument that cannot be evaluated in the current environment:
  try(suppressWarnings(rm("xx", "yy"))) #- Make sure there are no objects xx or yy
  Deriv(~ xx^2+yy^2)
#- Should show
#- c(xx = 2 * xx, yy = 2 * yy)
#- ?? What is the meaning / purpose of this construct?
  
#- ?? Is following really AD?  
#- Automatic differentiation (AD), note intermediate variable 'd' assignment
Deriv(~{d <- ((x-m)/s)^2; exp(-0.5*d)}, "x")
# Note that the result we see does NOT match what follows in the example(Deriv) (JN ??)
#{
#   d <- ((x - m)/s)^2
#   .d_x <- 2 * ((x - m)/s^2)
#   -(0.5 * (.d_x * exp(-(0.5 * d))))
#}
#- For some reason the intermediate variable d is NOT included.??


#- Custom derivative rule. Note that this needs explanations??
  myfun <- function(x, y=TRUE) NULL #- do something useful
  dmyfun <- function(x, y=TRUE) NULL #- myfun derivative by x.
  drule[["myfun"]] <- alist(x=dmyfun(x, y), y=NULL) #- y is just a logical
  Deriv(myfun(z^2, FALSE), "z")
  # 2 * (z * dmyfun(z^2, FALSE))

#- Differentiation by list components
  theta <- list(m=0.1, sd=2.) #- Why do we set values??
  x <- names(theta) #- and why these particular names??
  names(x)=rep("theta", length(theta))
  Deriv(~exp(-(x-theta$m)**2/(2*theta$sd)), x, cache.exp=FALSE)
#- Should show the following (but why??)
#- c(theta_m = exp(-((x - theta$m)^2/(2 * theta$sd))) *
#-  (x - theta$m)/theta$sd, theta_sd = 2 * (exp(-((x - theta$m)^2/
#-  (2 * theta$sd))) * (x - theta$m)^2/(2 * theta$sd)^2))
lderiv <-  Deriv(~exp(-(x-theta$m)**2/(2*theta$sd)), x, cache.exp=FALSE)
fld <- function(x){ eval(lderiv)} #- put this in a function
fld(2) #- and evaluate at a value

## ----chunk05------------------------------------------------------------------
library(Deriv)
rm(x) # ensures x is undefined
Deriv(~ x, "x")  # returns [1] 1 -- clearly a bug!
Deriv(~ x^2, "x")   # returns 2 * x
x <- quote(x^2)
Deriv(x, "x") # returns 2 * x

## ----chunk06------------------------------------------------------------------
rm(x) # in case it is defined
try(nlsDeriv(x, "x")  ) # fails, not a formula
try(nlsDeriv(as.expression("x"), "x")  ) # expression(NULL)
try(nlsDeriv(~x, "x")  ) # 1
try(nlsDeriv(x^2, "x"))   # fails
try(nlsDeriv(~x^2, "x")) # 2 * x
x <- quote(x^2)
try(nlsDeriv(x, "x")) # returns 2 * x

## ----chunk07, eval = require(Ryacas)------------------------------------------
dnlsr <- nlsr::nlsDeriv(~ sin(x+y), "x")
print(dnlsr)
class(dnlsr)
detach("package:nlsr", unload=TRUE)
detach("package:Deriv", unload=TRUE)

## New Ryacas mechanism as of 2019-8-29 from mikl@math.aau.dk (Mikkel Meyer Andersen)

yac_str("D(x) Sin(x+y)") 

# or if an expression is needed:
ex <- yac_expr("D(x) Sin(x+y)")
ex
expression(cos(x + y))
eval(ex, list(x = pi, y = pi/2))

## Previous syntax for Ryacas was
## x <- Sym("x")
## y <- Sym("y")
## dryacas <- deriv(sin(x+y), x)
## print(dryacas)
## class(dryacas)

detach("package:Ryacas", unload=TRUE)

## ----chunk08------------------------------------------------------------------
ls(nlsr::sysDerivs)

## ----chunk09------------------------------------------------------------------
require(nlsr)
## Try different ways to supply the log function
aDeriv <- nlsDeriv(~ log(x), "x")
class(aDeriv)
aDeriv
aderiv <- try(deriv( ~ log(x), "x"))
class(aderiv)
aderiv
aD <- D(expression(log(x)), "x")
class(aD)
aD
cat("but \n")
try(D( "~ log(x)", "x")) # fails -- gives NA rather than expected answer due to quotes
try(D( ~ log(x), "x"))
interm <- ~ log(x)
interm
class(interm)
interme <- as.expression(interm)
class(interme)
try(D(interme, "x"))
try(deriv(interme, "x"))
try(deriv(interm, "x"))


nlsDeriv(~ log(x, base=3), "x" ) # OK
try(D(expression(log(x, base=3)), "x" )) # fails - only single-argument calls supported
try(deriv(~ log(x, base=3), "x" )) # fails - only single-argument calls supported
try(deriv(expression(log(x, base=3)), "x" )) # fails - only single-argument calls supported
try(deriv3(expression(log(x, base=3)), "x" )) # fails - only single-argument calls supported
fnDeriv(quote(log(x, base=3)), "x" )

nlsDeriv(~ exp(x), "x")
D(expression(exp(x)), "x") # OK
deriv(~exp(x), "x") # OK, but much more complicated
fnDeriv(quote(exp(x)), "x")

nlsDeriv(~ sin(x), "x")
D(expression(sin(x)), "x")
deriv(~sin(x), "x")
fnDeriv(quote(sin(x)), "x")

nlsDeriv(~ cos(x), "x")
D(expression(cos(x)), "x")
deriv(~ cos(x), "x")
fnDeriv(quote(cos(x)), "x")

nlsDeriv(~ tan(x), "x")
D(expression(tan(x)), "x")
deriv(~ tan(x), "x")
fnDeriv(quote(tan(x)), "x")

nlsDeriv(~ sinh(x), "x")
D(expression(sinh(x)), "x")
deriv(~sinh(x), "x")
fnDeriv(quote(sinh(x)), "x")

nlsDeriv(~ cosh(x), "x")
D(expression(cosh(x)), "x")
deriv(~cosh(x), "x")
fnDeriv(quote(cosh(x)), "x")

nlsDeriv(~ sqrt(x), "x")
D(expression(sqrt(x)), "x")
deriv(~sqrt(x), "x")
fnDeriv(quote(sqrt(x)), "x")

nlsDeriv(~ pnorm(q), "q")
D(expression(pnorm(q)), "q")
deriv(~pnorm(q), "q")
fnDeriv(quote(pnorm(q)), "q")

nlsDeriv(~ dnorm(x, mean), "mean")
D(expression(dnorm(x, mean)), "mean")
deriv(~dnorm(x, mean), "mean")
fnDeriv(quote(dnorm(x, mean)), "mean")

nlsDeriv(~ asin(x), "x")
D(expression(asin(x)), "x")
deriv(~asin(x), "x")
fnDeriv(quote(asin(x)), "x")

nlsDeriv(~ acos(x), "x")
D(expression(acos(x)), "x")
deriv(~acos(x), "x")
fnDeriv(quote(acos(x)), "x")

nlsDeriv(~ atan(x), "x")
D(expression(atan(x)), "x")
deriv(~atan(x), "x")
fnDeriv(quote(atan(x)), "x")

nlsDeriv(~ gamma(x), "x")
D(expression(gamma(x)), "x")
deriv(~gamma(x), "x")
fnDeriv(quote(gamma(x)), "x")

nlsDeriv(~ lgamma(x), "x")
D(expression(lgamma(x)), "x")
deriv(~lgamma(x), "x")
fnDeriv(quote(lgamma(x)), "x")

nlsDeriv(~ digamma(x), "x")
D(expression(digamma(x)), "x")
deriv(~digamma(x), "x")
fnDeriv(quote(digamma(x)), "x")

nlsDeriv(~ trigamma(x), "x")
D(expression(trigamma(x)), "x")
deriv(~trigamma(x), "x")
fnDeriv(quote(trigamma(x)), "x")

nlsDeriv(~ psigamma(x, deriv = 5), "x")
D(expression(psigamma(x, deriv = 5)), "x")
deriv(~psigamma(x, deriv = 5), "x")
fnDeriv(quote(psigamma(x, deriv = 5)), "x")

nlsDeriv(~ x*y, "x")
D(expression(x*y), "x")
deriv(~x*y, "x")
fnDeriv(quote(x*y), "x")

nlsDeriv(~ x/y, "x")
D(expression(x/y), "x")
deriv(~x/y, "x")
fnDeriv(quote(x/y), "x")

nlsDeriv(~ x^y, "x")
D(expression(x^y), "x")
deriv(~x^y, "x")
fnDeriv(quote(x^y), "x")

nlsDeriv(~ (x), "x")
D(expression((x)), "x")
deriv(~(x), "x")
fnDeriv(quote((x)), "x")

nlsDeriv(~ +x, "x")
D(expression(+x), "x")
deriv(~ +x, "x")
fnDeriv(quote(+x), "x")

nlsDeriv(~ -x, "x")
D(expression(- x), "x")
deriv(~ -x, "x")
fnDeriv(quote(-x), "x")

nlsDeriv(~ abs(x), "x")
try(D(expression(abs(x)), "x")) # 'abs' not in derivatives table
try(deriv(~ abs(x), "x"))
fnDeriv(quote(abs(x)), "x")

nlsDeriv(~ sign(x), "x")
try(D(expression(sign(x)), "x")) # 'sign' not in derivatives table
try(deriv(~ sign(x), "x"))
fnDeriv(quote(sign(x)), "x")

## ----chunk10------------------------------------------------------------------
ls(nlsr::sysSimplifications)

## ----chunk11------------------------------------------------------------------
#- Remove ##? to see reproducible error
#- ?? For some reason, if we leave packages attached, we get errors.
#- Here we detach all the non-base packages and then reload nlsr
##? sessionInfo()
##? ##? nlsSimplify(quote(+(a+b)))
##? nlsSimplify(quote(-5))

## ----chunk12------------------------------------------------------------------
#- ?? For some reason, if we leave packages attached, we get errors.
#- Here we detach all the non-base packages and then reload nlsr
sessionInfo()
if ("Deriv" %in% loadedNamespaces()){detach("package:Deriv", unload=TRUE)} 
  #- ?? Do we need to unload too.
if ("nlsr" %in% loadedNamespaces() ){detach("package:nlsr", unload=TRUE)}
if ("Ryacas" %in% loadedNamespaces() ){detach("package:Ryacas", unload=TRUE)}
#- require(Deriv)
#- require(stats)
#- Various simplifications
#- ?? Do we need quote() to stop attempt to evaluate before applying simplification
require(nlsr)
nlsSimplify(quote(+(a+b)))
nlsSimplify(quote(-5))
nlsSimplify(quote(--(a+b)))

nlsSimplify(quote(exp(log(a+b))))
nlsSimplify(quote(exp(1)))

nlsSimplify(quote(log(exp(a+b))))
nlsSimplify(quote(log(1)))

nlsSimplify(quote(!TRUE))
nlsSimplify(quote(!FALSE))

nlsSimplify(quote((a+b)))

nlsSimplify(quote(a + b + 0))
nlsSimplify(quote(0 + a + b))
nlsSimplify(quote((a+b) + (a+b)))
nlsSimplify(quote(1 + 4))

nlsSimplify(quote(a + b - 0))
nlsSimplify(quote(0 - a - b))
nlsSimplify(quote((a+b) - (a+b)))
nlsSimplify(quote(5 - 3))

nlsSimplify(quote(0*(a+b)))
nlsSimplify(quote((a+b)*0))
nlsSimplify(quote(1L * (a+b)))
nlsSimplify(quote((a+b) * 1))
nlsSimplify(quote((-1)*(a+b)))
nlsSimplify(quote((a+b)*(-1)))
nlsSimplify(quote(2*5))

nlsSimplify(quote((a+b) / 1))
nlsSimplify(quote((a+b) / (-1)))
nlsSimplify(quote(0/(a+b)))
nlsSimplify(quote(1/3))

nlsSimplify(quote((a+b) ^ 1))
nlsSimplify(quote(2^10))

nlsSimplify(quote(log(exp(a), 3)))

nlsSimplify(quote(FALSE && b))
nlsSimplify(quote(a && TRUE))
nlsSimplify(quote(TRUE && b))

nlsSimplify(quote(a || TRUE))
nlsSimplify(quote(FALSE || b))
nlsSimplify(quote(a || FALSE))

nlsSimplify(quote(if (TRUE) a+b))
nlsSimplify(quote(if (FALSE) a+b))

nlsSimplify(quote(if (TRUE) a+b else a*b))
nlsSimplify(quote(if (FALSE) a+b else a*b))
nlsSimplify(quote(if (cond) a+b else a+b))

nlsSimplify(quote(--(a+b)))
nlsSimplify(quote(-(-(a+b))))

## ----chunk13------------------------------------------------------------------
#- ?? For some reason, if we leave packages attached, we get errors.
#- Here we detach all the non-base packages and then reload nlsr
sessionInfo()
if ("Deriv" %in% loadedNamespaces()){detach("package:Deriv", unload=TRUE)} 
  #- ?? Do we need to unload too.
if ("Deriv" %in% loadedNamespaces() ){detach("package:nlsr", unload=TRUE)}
if ("Deriv" %in% loadedNamespaces() ){detach("package:Ryacas", unload=TRUE)}
require(Deriv)
#- Various simplifications
#- ?? Do we need quote() to stop attempt to evaluate before applying simplification

Simplify(quote(+(a+b)))
Simplify(quote(-5))
Simplify(quote(--(a+b)))

Simplify(quote(exp(log(a+b))))
Simplify(quote(exp(1)))

Simplify(quote(log(exp(a+b))))
Simplify(quote(log(1)))

Simplify(quote(!TRUE))
Simplify(quote(!FALSE))

Simplify(quote((a+b)))

Simplify(quote(a + b + 0))
Simplify(quote(0 + a + b))
Simplify(quote((a+b) + (a+b)))
Simplify(quote(1 + 4))

Simplify(quote(a + b - 0))
Simplify(quote(0 - a - b))
Simplify(quote((a+b) - (a+b)))
Simplify(quote(5 - 3))

Simplify(quote(0*(a+b)))
Simplify(quote((a+b)*0))
Simplify(quote(1L * (a+b)))
Simplify(quote((a+b) * 1))
Simplify(quote((-1)*(a+b)))
Simplify(quote((a+b)*(-1)))
Simplify(quote(2*5))

Simplify(quote((a+b) / 1))
Simplify(quote((a+b) / (-1)))
Simplify(quote(0/(a+b)))
Simplify(quote(1/3))

Simplify(quote((a+b) ^ 1))
Simplify(quote(2^10))

Simplify(quote(log(exp(a), 3)))

Simplify(quote(FALSE && b))
Simplify(quote(a && TRUE))
Simplify(quote(TRUE && b))

Simplify(quote(a || TRUE))
Simplify(quote(FALSE || b))
Simplify(quote(a || FALSE))

Simplify(quote(if (TRUE) a+b))
Simplify(quote(if (FALSE) a+b))

Simplify(quote(if (TRUE) a+b else a*b))
Simplify(quote(if (FALSE) a+b else a*b))
Simplify(quote(if (cond) a+b else a+b))

#- This one is wrong... the double minus is an error, yet it works ??.
Simplify(quote(--(a+b)))
#- By comparison
Simplify(quote(-(-(a+b))))

## ----chunk14------------------------------------------------------------------
dlogx <- nlsr::nlsDeriv(~ log(x), "x")
str(dlogx)
print(dlogx)

## ----chunk15------------------------------------------------------------------
dlogxs <- nlsr::nlsDeriv(expression(log(x)), "x", do_substitute=FALSE)
str(dlogxs)
print(dlogxs)
cat(as.character(dlogxs), "\n")
fne <- expression(log(x))
dlogxe <- nlsr::nlsDeriv(fne, "x", do_substitute=FALSE)
str(dlogxe)
print(dlogxe)


# base R
dblogx <- D(expression(log(x)), "x")
str(dblogx)
print(dblogx)

require(Deriv)
ddlogx <- Deriv::Deriv(expression(log(x)), "x")
str(ddlogx)
print(ddlogx)
cat(as.character(ddlogx), "\n")
ddlogxf <- ~ ddlogx
str(ddlogxf)

## ----chunk16------------------------------------------------------------------
##
# require(stats)
# require(Deriv)
# require(Ryacas)

# Various derivatives 

new <- codeDeriv(quote(1 + x + y), c("x", "y"))
old <- deriv(quote(1 + x + y), c("x", "y"))
print(new)
# Following generates a very long line on output of knitr (for markdown)
class(new)
str(new)
as.expression(new)
newf <- function(x, y){
   eval(new)
}
newf(3,5)

print(old)
class(old)
str(old)
oldf <- function(x,y){
    eval(old)
}
oldf(3,5)



## ----chunk17------------------------------------------------------------------
fnfromnew <- function(x,y){
    .value <- 1 + x + y
    .grad <- array(0, c(length(.value), 2L), list(NULL, c("x", 
    "y")))
    .grad[, "x"] <- 1
    .grad[, "y"] <- 1
    attr(.value, "gradient") <- .grad
    .value
}

print(fnfromnew(3,5))


## ----chunk18------------------------------------------------------------------
x <- NA
y <- Inf
print(eval(new))
print(eval(old))

## ----chunk19------------------------------------------------------------------
safeD <- function(obj, var) {
   # safeguarded D() function for symbolic derivs
   if (! is.character(var) ) stop("The variable var MUST be character type")
   if (is.character(obj) ) {
       eobj <- parse(text=obj)
       result <- D(eobj, var)
   } else {
       result <- D(obj, var)
   }
}

lxy2 <- expression(log(x+y^2))
clxy2 <- "log(x+y^2)"
try(print(D(clxy2, "y")))
print(try(D(lxy2, "y")))
print(safeD(clxy2, "y"))
print(safeD(lxy2, "y"))

## ----chunk20------------------------------------------------------------------
zzz <- expression(y[3]*r1 + r2)
try(deriv(zzz,c("r1","r2")))
##
try(nlsr::nlsDeriv(zzz, c("r1","r2")))
try(fnDeriv(zzz, c("r1","r2")))
newDeriv(`[`(x,y), stop("no derivative when indexing"))
try(nlsr::nlsDeriv(zzz, c("r1","r2")))
try(nlsr::fnDeriv(zzz, c("r1","r2")))

## ----chunk21------------------------------------------------------------------
try(nlsr::nlsDeriv(zzz, "y[3]"))
try(nlsr::nlsDeriv(y3*r1+r2,"y3"))
try(nlsr::nlsDeriv(y[3]*r1+r2,"y[3]"))

## ----derivexp, eval=TRUE------------------------------------------------------
partialderiv <- D(expression(a * (x ^ b)),"b")
print(partialderiv)

## ----timingshobbsmodel--------------------------------------------------------
require(microbenchmark)
## nls on Hobbs scaled model
weed <- c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443,
          38.558, 50.156, 62.948, 75.995, 91.972)
tt <- 1:12
weeddf <- data.frame(y=weed, tt=tt)
wmods <- y ~ 100*b1/(1+10*b2*exp(-0.1*b3*tt))
stx<-c(b1=2, b2=5, b3=3)
tnls<-microbenchmark((anls<-nls(wmods, start=stx, data=weeddf)), unit="us")
tnls
## nlsr::nlfb() on Hobbs scaled model
tnlxb<-microbenchmark((anlxb<-nlsr::nlxb(wmods, start=stx, data=weeddf)), unit="us")
tnlxb
## minpack.lm::nlsLM() on Hobbs scaled model
tnlsLM<-microbenchmark((anlsLM<-minpack.lm::nlsLM(start=stx, formula=wmods, data=weeddf)), 
                       unit="us")
tnlsLM

## ----timingnlsrminpackfn------------------------------------------------------
require(microbenchmark)
## nlsr::nlfb() on Hobbs scaled
# Scaled Hobbs problem
shobbs.res  <-  function(x){ # scaled Hobbs weeds problem -- residual
  # This variant uses looping
  if(length(x) != 3) stop("shobbs.res -- parameter vector n!=3")
  y  <-  c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443, 
           38.558, 50.156, 62.948, 75.995, 91.972)
  tt  <-  1:12
  res  <-  100.0*x[1]/(1+x[2]*10.*exp(-0.1*x[3]*tt)) - y
}

shobbs.jac  <-  function(x) { # scaled Hobbs weeds problem -- Jacobian
  jj  <-  matrix(0.0, 12, 3)
  tt  <-  1:12
  yy  <-  exp(-0.1*x[3]*tt)
  zz  <-  100.0/(1+10.*x[2]*yy)
  jj[tt,1]   <-   zz
  jj[tt,2]   <-   -0.1*x[1]*zz*zz*yy
  jj[tt,3]   <-   0.01*x[1]*zz*zz*yy*x[2]*tt
  attr(jj, "gradient") <- jj
  jj
}

st1 <- c(b1=1, b2=1, b3=1)
tnlfb<-microbenchmark((anlfb<-nlsr::nlfb(start=st1, resfn=shobbs.res, jacfn=shobbs.jac)), unit="us")
tnlfb
## minpack.lm::nls.lm() on Hobbs scaled
tnls.lm<-microbenchmark((anls.lm<-minpack.lm::nls.lm(par=st1, fn=shobbs.res, jac=shobbs.jac)))
tnls.lm

## ----hiddenexprob1, echo=FALSE------------------------------------------------
# Here we set up an example problem with data
# Define independent variable
t0 <- 0:19
t0a<-t0
t0a[1]<-1e-20 # very small value
# Drop first value in vectors
t0t<-t0[-1]
y1 <- 4 * (t0^0.25)
y1t<-y1[-1]
n <- length(t0)
fuzz <- rnorm(n)
range <- max(y1)-min(y1)
## add some "error" to the dependent variable
y1q <- y1 + 0.2*range*fuzz
edta <- data.frame(t0=t0, t0a=t0a, y1=y1, y1q=y1q)
edtat <- data.frame(t0t=t0t, y1t=y1t)

start1 <- c(a=1, b=1)
try(nlsy0t0ax <- nls(formula=y1~a*(t0a^b), start=start1, data=edta, control=nls.control(maxiter=10000)))
nlsry1t0a <- nlxb(formula=y1~a*(t0a^b), start=start1, data=edta)
library(minpack.lm)
nlsLMy1t0a <- nlsLM(formula=y1~a*(t0a^b), start=start1, data=edta)

## ----nlsstr-------------------------------------------------------------------
str(nlsy0t0ax)

## ----nlsrstr------------------------------------------------------------------
str(nlsry1t0a)

## ----pred1--------------------------------------------------------------------
nudta <- colMeans(edta)
predict(nlsy0t0ax, newdata=nudta)
predict(nlsLMy1t0a, newdata=nudta)
predict(nlsry1t0a, newdata=nudta)

## ----exprob1------------------------------------------------------------------
# Here we set up an example problem with data
# Define independent variable
t0 <- 0:19
t0a<-t0
t0a[1]<-1e-20 # very small value
# Drop first value in vectors
t0t<-t0[-1]
y1 <- 4 * (t0^0.25)
y1t<-y1[-1]
n <- length(t0)
fuzz <- rnorm(n)
range <- max(y1)-min(y1)
## add some "error" to the dependent variable
y1q <- y1 + 0.2*range*fuzz
edta <- data.frame(t0=t0, t0a=t0a, y1=y1, y1q=y1q)
edtat <- data.frame(t0t=t0t, y1t=y1t)

## ----extrynls-----------------------------------------------------------------
cprint <- function(obj){
   # print object if it exists
  sobj<-deparse(substitute(obj))
  if (exists(sobj)) {
      print(obj)
  } else {
      cat(sobj," does not exist\n")
  }
#  return(NULL)
}
start1 <- c(a=1, b=1)
try(nlsy0t0 <- nls(formula=y1~a*(t0^b), start=start1, data=edta))
cprint(nlsy0t0)
# Since this fails to converge, let us increase the maximum iterations
try(nlsy0t0x <- nls(formula=y1~a*(t0^b), start=start1, data=edta,
                    control=nls.control(maxiter=10000)))
cprint(nlsy0t0x)
try(nlsy0t0ax <- nls(formula=y1~a*(t0a^b), start=start1, data=edta, 
                     control=nls.control(maxiter=10000)))
cprint(nlsy0t0ax)
try(nlsy0t0t <- nls(formula=y1t~a*(t0t^b), start=start1, data=edtat))
cprint(nlsy0t0t)

## ----extry1nlsr---------------------------------------------------------------
nlsry1t0 <- try(nlxb(formula=y1~a*(t0^b), start=start1, data=edta))
cprint(nlsry1t0)
nlsry1t0a <- nlxb(formula=y1~a*(t0a^b), start=start1, data=edta)
cprint(nlsry1t0a)
nlsry1t0t <- nlxb(formula=y1t~a*(t0t^b), start=start1, data=edtat)
cprint(nlsry1t0t)

## ----extry1minlm--------------------------------------------------------------
library(minpack.lm)
nlsLMy1t0 <- nlsLM(formula=y1~a*(t0^b), start=start1, data=edta)
nlsLMy1t0
nlsLMy1t0a <- nlsLM(formula=y1~a*(t0a^b), start=start1, data=edta)
nlsLMy1t0a
nlsLMy1t0t <- nlsLM(formula=y1t~a*(t0t^b), start=start1, data=edtat)
nlsLMy1t0t

## ----brownden-----------------------------------------------------------------
m <- 20
t <- seq(1, m) / 5
y <- rep(0,m)
library(nlsr)
library(minpack.lm)

bddata <- data.frame(t=t, y=y)
bdform <- y ~ ((x1 + t * x2 - exp(t))^2 + (x3 + x4 * sin(t) - cos(t))^2)
prm0 <- c(x1=25, x2=5, x3=-5, x4=-1)
fbd <-model2ssgrfun(bdform, prm0, bddata)
cat("initial sumsquares=",as.numeric(crossprod(fbd(prm0))),"\n")
nlsrbd <- nlxb(bdform, start=prm0, data=bddata, trace=FALSE)
nlsrbd

nlsbd10k <- nls(bdform, start=prm0, data=bddata, trace=FALSE, 
                control=nls.control(maxiter=10000))
nlsbd10k

nlsLMbd10k <- nlsLM(bdform, start=prm0, data=bddata, trace=FALSE, 
                    control=nls.lm.control(maxiter=10000, maxfev=10000))
nlsLMbd10k

## ----browndenpred-------------------------------------------------------------
ndata <- data.frame(t=c(5,6), y=c(0,0))
predict(nlsLMbd10k, newdata=ndata)

# now nls
predict(nlsbd10k, newdata=ndata)

# now nlsr
predict(nlsrbd, newdata=ndata)



## ----brownden2----------------------------------------------------------------
bdf2 <-  (x1 + t * x2 - exp(t))^2 ~ - (x3 + x4 * sin(t) - cos(t))^2

## ----bdcheck, eval=FALSE------------------------------------------------------
# #' Brown and Dennis Function
# #'
# #' Test function 16 from the More', Garbow and Hillstrom paper.
# #'
# #' The objective function is the sum of \code{m} functions, each of \code{n}
# #' parameters.
# #'
# #' \itemize{
# #'   \item Dimensions: Number of parameters \code{n = 4}, number of summand
# #'   functions \code{m >= n}.
# #'   \item Minima: \code{f = 85822.2} if \code{m = 20}.
# #' }
# #'
# #' @param m Number of summand functions in the objective function. Should be
# #'   equal to or greater than 4.
# #' @return A list containing:
# #' \itemize{
# #'   \item \code{fn} Objective function which calculates the value given input
# #'   parameter vector.
# #'   \item \code{gr} Gradient function which calculates the gradient vector
# #'   given input parameter vector.
# #'   \item \code{fg} A function which, given the parameter vector, calculates
# #'   both the objective value and gradient, returning a list with members
# #'   \code{fn} and \code{gr}, respectively.
# #'   \item \code{x0} Standard starting point.
# #' }
# #' @references
# #' More', J. J., Garbow, B. S., & Hillstrom, K. E. (1981).
# #' Testing unconstrained optimization software.
# #' \emph{ACM Transactions on Mathematical Software (TOMS)}, \emph{7}(1), 17-41.
# #' \url{https://doi.org/10.1145/355934.355936}
# #'
# #' Brown, K. M., & Dennis, J. E. (1971).
# #' \emph{New computational algorithms for minimizing a sum of squares of
# #' nonlinear functions} (Report No. 71-6).
# #' New Haven, CT: Department of Computer Science, Yale University.
# #'
# #' @examples
# #' # Use 10 summand functions
# #' fun <- brown_den(m = 10)
# #' # Optimize using the standard starting point
# #' x0 <- fun$x0
# #' res_x0 <- stats::optim(par = x0, fn = fun$fn, gr = fun$gr, method =
# #' "L-BFGS-B")
# #' # Use your own starting point
# #' res <- stats::optim(c(0.1, 0.2, 0.3, 0.4), fun$fn, fun$gr, method =
# #' "L-BFGS-B")
# #'
# #' # Use 20 summand functions
# #' fun20 <- brown_den(m = 20)
# #' res <- stats::optim(fun20$x0, fun20$fn, fun20$gr, method = "L-BFGS-B")
# #' @export
# #`
# brown_den <- function(m = 20) {
#   list(
#     fn = function(par) {
#       x1 <- par[1]
#       x2 <- par[2]
#       x3 <- par[3]
#       x4 <- par[4]
# 
#       ti <- (1:m) * 0.2
#       l <- x1 + ti * x2 - exp(ti)
#       r <- x3 + x4 * sin(ti) - cos(ti)
#       f <- l * l + r * r
#       sum(f * f)
#     },
#     gr = function(par) {
#       x1 <- par[1]
#       x2 <- par[2]
#       x3 <- par[3]
#       x4 <- par[4]
# 
#       ti <- (1:m) * 0.2
#       sinti <- sin(ti)
#       l <- x1 + ti * x2 - exp(ti)
#       r <- x3 + x4 * sinti - cos(ti)
#       f <- l * l + r * r
#       lf4 <- 4 * l * f
#       rf4 <- 4 * r * f
#       c(
#         sum(lf4),
#         sum(lf4 * ti),
#         sum(rf4),
#         sum(rf4 * sinti)
#       )
#     },
#     fg = function(par) {
#       x1 <- par[1]
#       x2 <- par[2]
#       x3 <- par[3]
#       x4 <- par[4]
# 
#       ti <- (1:m) * 0.2
#       sinti <- sin(ti)
#       l <- x1 + ti * x2 - exp(ti)
#       r <- x3 + x4 * sinti - cos(ti)
#       f <- l * l + r * r
#       lf4 <- 4 * l * f
#       rf4 <- 4 * r * f
# 
#       fsum <- sum(f * f)
#       grad <- c(
#         sum(lf4),
#         sum(lf4 * ti),
#         sum(rf4),
#         sum(rf4 * sinti)
#       )
# 
#       list(
#         fn = fsum,
#         gr = grad
#       )
#     },
#     x0 = c(25, 5, -5, 1)
#   )
# }
# mbd <- brown_den(m=20)
# mbd
# mbd$fg(mbd$x0)
# bdsolnm <- optim(mbd$x0, mbd$fn, control=list(trace=0))
# bdsolnm
# bdsolbfgs <- optim(mbd$x0, mbd$fn, method="BFGS", control=list(trace=0))
# bdsolbfgs
# 
# library(optimx)
# methlist <- c("Nelder-Mead","BFGS","Rvmmin","L-BFGS-B","Rcgmin","ucminf")
# 
# solo <- opm(mbd$x0, mbd$fn, mbd$gr, method=methlist, control=list(trace=0))
# summary(solo, order=value)
# 
# ## A failure above is generally because a package in the 'methlist' is not installed.
# 

## ----opx----------------------------------------------------------------------
require(optimx)
sq<-function(x){
   nn<-length(x)
   yy<-1:nn
   f<-sum((yy-x)^2)
   f
}
sq.g <- function(x){
   nn<-length(x)
   yy<-1:nn
   gg<- 2*(x - yy)
}
xx <- c(.3, 4)
uncans <- Rvmmin(xx, sq, sq.g)
proptimr(uncans)
mybm <- c(0,1) # fix parameter 1
cans <- Rvmmin(xx, sq, sq.g, bdmsk=mybm)
proptimr(cans)
require(nlsr)
weed <- c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443,
   38.558, 50.156, 62.948, 75.995, 91.972)
ii <- 1:12
wdf <- data.frame(weed, ii)
weedux <- nlxb(weed~b1/(1+b2*exp(-b3*ii)), start=c(b1=200, b2=50, b3=0.3)) 
weedux
weedcx <- nlxb(weed~b1/(1+b2*exp(-b3*ii)), start=c(b1=200, b2=50, b3=0.3), masked=c("b1")) 
weedcx
rfn <- function(bvec, weed=weed, ii=ii){
  res <- rep(NA, length(ii))
  for (i in ii){
    res[i]<- bvec[1]/(1+bvec[2]*exp(-bvec[3]*i))-weed[i]
  }
  res
}
weeduf <- nlfb(start=c(200, 50, 0.3),resfn=rfn,weed=weed, ii=ii,
               control=list(japprox="jacentral"))
weeduf
weedcf <- nlfb(start=c(200, 50, 0.3),resfn=rfn,weed=weed, ii=ii, lower=c(200, 0, 0),
               upper=c(200, 100,100), control=list(japprox="jacentral"))
weedcf

## ----bellx, code=readLines("./BellX.R"), echo=TRUE----------------------------
# BellX.R
library(nlraa)
require(ggplot2)
set.seed(1234)
x <- 1:20
y <- bell(x, 8, -0.0314, 0.000317, 13) + rnorm(length(x), 0, 0.5)
dat <- data.frame(x = x, y = y)
fit <- nls(y ~ SSbell(x, ymax, a, b, xc), data = dat)
fit
gfrm<- y ~ ymax * exp(a *(x - xc)^2)
gfrmx<- y ~ ymax * exp(a *(x - xc)^2 + b*(x - xc)^3)
stgauss<-c(ymax=max(y), a=-0.5/(sd(x)^2), xc=mean(x))
cat("stgauss:"); print(stgauss)
st2<-c(ymax=8, a= 0.03, xc= 13)
st3<-c(ymax=8, a= 0.03, b=0, xc= 13)
fit2 <- nls(gfrm, start=st2, data=dat, trace=TRUE)
summary(fit2)
# Use the selfStart model
xfrm <- y ~ SSbell(x, ymax, a, b, xc)
library(nlsr)
fx2mx <- nlxb(xfrm, start=st3, data=dat, lower=c(0, -1e5, 0, 0),
             upper=c(1e5, 1e5, 0, 1e5), trace=TRUE, control=list(japprox="jacentral"))
fx2mx
# Or the formula for the same model with Jacobian approximation
fx2mgc <- nlxb(gfrmx, start=st3, data=dat, lower=c(0, -1e5, 0, 0),
              upper=c(1e5, 1e5, 0, 1e5), trace=TRUE, control=list(japprox="jacentral"))
fx2mgc
# Or the formula and analytic derivatives
fx2mg <- nlxb(gfrmx, start=st3, data=dat, lower=c(0, -1e5, 0, 0),
               upper=c(1e5, 1e5, 0, 1e5), trace=TRUE)
fx2mg
# Display results together
pshort(fx2mx)
pshort(fx2mgc)
pshort(fx2mg)

## ----ex01, echo=TRUE----------------------------------------------------------
weed <- c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443,
          38.558, 50.156, 62.948, 75.995, 91.972)
tt <- 1:12
weeddf <- data.frame(tt, weed)
plot(weeddf, main="Hobbs weed infestation data")

## ----ex02set, echo=TRUE-------------------------------------------------------
# model formulas
frmu <- weed ~ b1/(1+b2*exp(-b3*tt))
frms <- weed ~ 100*c1/(1+10*c2*exp(-0.1*c3*tt))
frmt <- weed ~ Asym /(1 + exp((xmid-tt)/scal))
#
# Starting parameter sets
stu1<-c(b1=1, b2=1, b3=1)
sts1<-c(c1=1, c2=1, c3=1)
stt1<-c(Asym=1, xmid=1, scal=1)

## ----ex02nls------------------------------------------------------------------
unls1<-try(nls(formula=frmu, start=stu1, data=weeddf))
summary(unls1)
snls1<-try(nls(formula=frms, start=sts1, data=weeddf))
summary(snls1)
tnls1<-try(nls(formula=frmt, start=stt1, data=weeddf))
summary(tnls1)
cat("\n")

## ----ex02nlsr-----------------------------------------------------------------
library(nlsr)
unlx1<-try(nlxb(formula=frmu, start=stu1, data=weeddf))
print(unlx1) 
pshort(unlx1) # A short form output
snlx1<-try(nlxb(formula=frms, start=sts1, data=weeddf))
# pshort(snlx1)
print(snlx1)
tnlx1<-try(nlxb(formula=frmt, start=stt1, data=weeddf))
# pshort(tnlx1)
print(tnlx1)
ct<-as.list(tnlx1$coefficients) # Need a list to use ct$... in next line
cat("exp(xmid/scal)=",exp(ct$xmid/ct$scal),"\n")
cat("\n")
# explicit residuals (weighted if there are weights)
rtnlx1<-residuals(tnlx1)
print(rtnlx1)
cat("explicit sumsquares =", sum(rtnlx1^2),"\n")

## ----ex02minpack--------------------------------------------------------------
library(minpack.lm)
unlm1<-try(nlsLM(formula=frmu, start=stu1, data=weeddf))
summary(unlm1) # Use summary() to get display
unlm1 # Note the difference. Use this form to get sum of squares
# pnls(unlm1)  # Short form of output
snlm1<-try(nlsLM(formula=frms, start=sts1, data=weeddf))
# pnls(snlm1)
summary(snlm1)
tnlm1<-try(nlsLM(formula=frmt, start=stt1, data=weeddf))
if (inherits(tnlm1, "try-error")) {
   cat("Failure to compute solution -- likely singular Jacobian\n")
} else {  
   pnls(tnlm1) # short form to give sum of squares
   summary(tnlm1)
}   

## ----ex02wrapnlsr-------------------------------------------------------------
## Try the wrapper. Calling wrapnlsr() instead of nlsr() is equivalent
##
unlw1<-try(nlsr(formula=frmu, start=stu1, data=weeddf))
print(unlw1) # using 'unlx1' gives name of object as 'x' only
snlw1<-try(nlsr(formula=frms, start=sts1, data=weeddf))
# pshort(snlx1)
print(snlw1)
tnlw1<-try(nlsr(formula=frmt, start=stt1, data=weeddf))
print(tnlw1)

## ----ex03, echo=TRUE----------------------------------------------------------
stspecial<- c(Asym = 35.532,  xmid = 43376,  scal = -2935.4)
# force to exit at once by setting femax to 1 (maximum number of sum of squares evaluations)
getsvs<-nlxb(formula=frmt, start=stspecial, data=weeddf, control=list(femax=1))
print(getsvs)

## ----ex04, echo=TRUE, eval=FALSE----------------------------------------------
# if (inherits(tnlm1, "try-error")) {
#    cat("Cannot compute solution -- likely singular Jacobian\n")
# } else {
#    Jtm <- tnlm1$m$gradient()
#    svd(Jtm)$d # Singular values
# }

## ----ex05, echo=TRUE----------------------------------------------------------
stspecial<- c(Asym = 35.532,  xmid = 43376,  scal = -2935.4)
# force to exit at once by setting femax to 1 (maximum number of sum of squares evaluations)
badstart<-nlxb(formula=frmt, start=stspecial, data=weeddf)
print(badstart)

## ----ex06, echo=TRUE----------------------------------------------------------
frmtss <- weed ~ SSlogis(tt, Asym, xmid, scal)
ssts1<-nls(formula=frmtss, data=weeddf)
summary(ssts1)
require(minpack.lm)
sstm1<-nlsLM(formula=frmtss, data=weeddf)
summary(sstm1)

## ----ex07, echo=TRUE----------------------------------------------------------
Asymt <- 2*max(weed)
Asymt
pw <- Asymt/weed
pw1<-pw-1
lpw1<-log(pw1)
clin <- coef(lm(lpw1~tt))
clin<-as.list(clin)
scalt <- -1/clin$tt
scalt
xmidt <- scalt*as.numeric(clin[1])
xmidt
library(nlsr)
try1<-nlxb(frmt, data=weeddf, start=c(Asym=1, xmid=1, scal=1))
pshort(try1)
try2<-nlxb(frmt, data=weeddf, start=c(Asym=Asymt, xmid=xmidt, scal=scalt))
pshort(try2)

## ----ex08, eval=TRUE----------------------------------------------------------
frmtssJ <- weed ~ SSlogisJN(tt, Asym, xmid, scal) # The model formula
lstrt <- getInitial(frmtssJ, weeddf) # Here we get the starting parameters
cat("starting parameters:\n")
print(lstrt)
# Next line uses the start. We indicate that the "approximate" Jacobian code is in
#  the selfStart code, though in fact it is not an approximation in this case.
sstx1<-nlxb(formula=frmtssJ, start=lstrt, data=weeddf, control=list(japprox="SSJac"))
print(sstx1)
# If no Jacobian code is included in selfStart module, we can actually use an
#  approximate Jacobian. See "gradient" elements for differences in result.
sstx1a<-nlxb(formula=frmtssJ, start=lstrt, data=weeddf, control=list(japprox="jacentral"))
print(sstx1a)
## We can automate the above in function nlsrSS()
sstSS<-nlsrSS(formula=frmtssJ, data=weeddf)
print(sstSS)

## ----ex09, echo=TRUE----------------------------------------------------------
b1t <- 2*max(weed)
b1t
lpw1<-log(b1t/weed-1)
clin <- as.list(coef(lm(lpw1~tt)))
b3t <- -clin$tt
b3t
b2t <- exp(as.numeric(clin[1]))
b2t
library(nlsr)
try1<-nlxb(frmu, data=weeddf, start=c(b1=1, b2=1, b3=1))
pshort(try1)
try2<-nlxb(frmu, data=weeddf, start=c(b1=b1t, b2=b2t, b3=b3t))
pshort(try2)

## ----ex10, echo=TRUE----------------------------------------------------------
# Start MUST be feasible i.e. on or within bounds
anlshob1b <- nls(frms, start=sts1, data=weeddf, lower=c(0,0,0),
             upper=c(2,6,3), algorithm='port')
pnls(anlshob1b) #  check the answer (short form)
# nlsLM seems NOT to work with bounds in this example
anlsLM1b <- nlsLM(frms, start=sts1, data=weeddf, lower=c(0,0,0), upper=c(2,6,3))
pnls(anlsLM1b)
# also no warning if starting out of bounds, but gets good answer!!
st4<-c(c1=4, c2=4, c3=4)
anlsLMob <- nlsLM(frms, start=st4, data=weeddf, lower=c(0,0,0), upper=c(2,6,3))
pnls(anlsLMob)
# Try nlsr::nlxb()
anlx1b <- nlxb(frms, start=sts1, data=weeddf, lower=c(0,0,0), upper=c(2,6,3))
pshort(anlx1b)

## ----exrosen, eval=TRUE-------------------------------------------------------
frres<-function(x) { ## Rosenbrock as residuals
    x1 <- x[1]
    x2 <- x[2]
    res<-c( 10 * (x2 - x1 * x1), (1 - x1) )
}

frjac<-function(x) { ## Rosenbrock residual derivatives matrix
    x1 <- x[1]
    x2 <- x[2]
   J<-matrix(NA, nrow=2, ncol=2)
   J[1,1]<- -20*x1
   J[1,2]<- 10
   J[2,1]<- -1
   J[2,2]<- 0
   attr(J,"gradient")<-J # NEEDED for nlfb()
   J
}
require(minpack.lm)
require(nlsr)
strt<-c(-1.2,1)
rosnlf<-nlfb(strt, resfn=frres, jacfn=frjac)
print(rosnlf)
rosnlm<-nls.lm(par=strt, fn=frres, jac=frjac)
summary(rosnlm)  

## ----staticwts----------------------------------------------------------------
wts <- 1/weeddf$tt # wts are reciprocal of time value
tnlx1w<-try(nlxb(formula=frmt, start=stt1, data=weeddf, weights=wts))
# pshort(tnlx1)
print(tnlx1w)
ct<-as.list(tnlx1w$coefficients) # Need a list to use ct$... in next line
cat("exp(xmid/scal)=",exp(ct$xmid/ct$scal),"\n")
cat("\n")

rtnlx1w<-residuals(tnlx1w)
print(rtnlx1w)
cat("explicit sumsquares =", sum(rtnlx1w^2),"\n")

## ----puromycinwfct, echo=TRUE, eval=FALSE-------------------------------------
# library(minpack.lm)
# library(nlsr) # for pnls
# Treated <- Puromycin[Puromycin$state == "treated", ]
# # First get raw estimates
# wtt3nlm0<-nlsLM(rate ~ Vm * conc/(K + conc), data = Treated, trace=TRUE,
#                start = c(Vm = 200, K = 0.05))
# fit0<-fitted(wtt3nlm0)
# # Static wts using fit0
# wtt3nlm0s<-nlsLM(rate ~ Vm * conc/(K + conc), data = Treated, trace=TRUE,
#                start = c(Vm = 200, K = 0.05), weights = wfct(1/fit0^2))
# pnls(wtt3nlm0s)
# # and run directly, noting the 2 phase operation
# wtt3nlm<-nlsLM(rate ~ Vm * conc/(K + conc), data = Treated, trace=TRUE,
#                start = c(Vm = 200, K = 0.05), weights = wfct(1/fitted^2))
# pnls(wtt3nlm)
# cat("weights from wtt3nlm\n")
# as.numeric(wtt3nlm$weights)

## ----formulawts---------------------------------------------------------------
library(nlsr)
Treated <- Puromycin[Puromycin$state == "treated", ]
# "regression" formula
w1frm <- rate ~ Vm * conc/(K + conc)
# Explicit dynamic weighted nlxb
w3frm <- rate/(Vm * conc/(K + conc)) ~ 1
dynweighted <- nlxb(w3frm, data = Treated, start = c(Vm = 201.003, K = 0.04696),
                    trace=TRUE, control=list(prtlvl=0))
pshort(dynweighted)
# Compute the model from the ORIGINAL model (w1frm) using parameters from dynweighted
dyn0 <- nlxb(w1frm, start=coefficients(dynweighted), data=Treated, control=list(jemax=0))
wfdyn0 <- 1/fitted(dyn0)^2 # weights
print(as.numeric(wfdyn0))
pshort(dyn0) # Shows sumsquares without weights, but computes weights we used
formulaweighted <- nlxb(w1frm, data = Treated, start = c(Vm = 201.003, K = 0.04696),
                        weights = ~ 1/fitted^2, trace=TRUE, control=list(prtlvl=0))
pshort(formulaweighted)
wtsfromfits<-1/fitted(formulaweighted)^2
print(as.numeric(wtsfromfits))
print(as.numeric(formulaweighted$weights))

# require(nlsr)
require(nlsr)
traceval  <-  TRUE  # traceval set TRUE to debug or give full history

# Data for Hobbs problem
ydat  <-  c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443, 
          38.558, 50.156, 62.948, 75.995, 91.972) # for testing
tdat  <-  seq_along(ydat) # for testing

# A simple starting vector -- must have named parameters for nlxb, nls, wrapnlsr.
start1  <-  c(b1=1, b2=1, b3=1)
startf1  <-  c(b1=1, b2=1, b3=.1)

eunsc  <-   y ~ b1/(1+b2*exp(-b3*tt))

cat("LOCAL DATA IN DATA FRAMES\n")
weeddata1  <-  data.frame(y=ydat, tt=tdat)
weeddata2  <-  data.frame(y=1.5*ydat, tt=tdat)

anlxb1  <-  try(nlxb(eunsc, start=start1, trace=traceval, data= weeddata1, control=list(watch=TRUE)))
print(anlxb1)
summary(anlxb1)
anlxb2  <-  try(nlxb(eunsc, start=start1, trace=traceval, data=weeddata2))
print(anlxb2)


suneasy  <-  c(b1=200, b2=50, b3=0.3)
ssceasy  <-  c(b1=2, b2=5, b3=3)
st1scal  <-  c(b1=100, b2=10, b3=0.1)


cat("EASY start -- unscaled")
anls01  <-  try(nls(eunsc, start=suneasy, trace=traceval, data=weeddata1))
print(anls01)
anl01  <-  try(nlxb(eunsc, start=suneasy, trace=traceval, data=weeddata1))
print(anl01)

cat("All 1s start -- unscaled")
anls02  <-  try(nls(eunsc, start=start1, trace=traceval, data=weeddata1))
if (class(anls02) == "try-error") {
   cat("FAILED:")
   print(anls02)
} else {
   print(anls02)
}
anl02  <-  nlxb(eunsc, start=start1, trace=traceval, data=weeddata1)
print(anl02)

escal  <-   y ~ 100*b1/(1+10*b2*exp(-0.1*b3*tt))

cat("ones start -- scaled")
anls03  <-  try(nls(escal, start=start1, trace=traceval, data=weeddata1))
print(anls03)
anl03  <-  nlxb(escal, start=start1, trace=traceval, data=weeddata1)
print(anl03)

# cat("HARD start -- scaled")
# anls04  <-  try(nls(escal, start=st1scal, trace=traceval, data=weeddata1))
# print(anls04)
# anl04  <-  nlxb(escal, start=st1scal, trace=traceval, data=weeddata1)
# print(anl04)

cat("EASY start -- scaled")
anls05  <-  try(nls(escal, start=ssceasy, trace=traceval, data=weeddata1))
print(anls05)
anl05  <-  nlxb(escal, start=ssceasy, trace=traceval, data=weeddata1)
print(anl05)

traceval  <-  TRUE  # traceval set TRUE to debug or give full history

# Data for Hobbs problem
ydat  <-  c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443, 
          38.558, 50.156, 62.948, 75.995, 91.972) # for testing
tdat  <-  seq_along(ydat) # for testing

# A simple starting vector -- must have named parameters for nlxb, nls, wrapnlsr.
start1  <-  c(b1=1, b2=1, b3=1)
startf1  <-  c(b1=1, b2=1, b3=.1)

eunsc  <-   y ~ b1/(1+b2*exp(-b3*tt))

cat("LOCAL DATA IN DATA FRAMES\n")
weeddata1  <-  data.frame(y=ydat, tt=tdat)
weeddata2  <-  data.frame(y=1.5*ydat, tt=tdat)

cat("TEST MASKS\n")

anlsmnqm  <-  try(nlxb(eunsc, start=start1, lower=c(b1=0, b2=0, b3=0), 
   upper=c(b1=500, b2=100, b3=5), masked=c("b2"), trace=traceval, data=weeddata1))
print(anlsmnqm)

cat("UNCONSTRAINED\n")
an1q  <-  try(nlxb(eunsc, start=start1, trace=traceval, data=weeddata1))
print(an1q)


cat("MASKED\n")

an1qm3  <-  try(nlxb(eunsc, start=start1, trace=traceval, data=weeddata1, 
                masked=c("b3")))
print(an1qm3)

# Note that the parameters are put in out of order to test code.
an1qm123  <-  try(nlxb(eunsc, start=start1, trace=traceval, data=weeddata1, 
                  masked=c("b2","b1","b3")))
print(an1qm123)

jand <- function(pars, resfn=NULL, bdmsk=NULL, resbest=NULL, ndstep=1e-7, ...){
   # ndstep not currently used
   if (is.null(resfn)) stop("You must supply a residual function")
   # numDeriv jacobian approximation
   jacmat<-numDeriv::jacobian(resfn, pars,...) # don't specify other numDeriv parameters here yet
   attr(jacmat,"gradient") <- jacmat # to satisfy needs of nlxb(), nlfb() working
} # end jand

jafwd <- function(pars, resfn=NULL, bdmsk=NULL, resbest=NULL, ndstep=1e-7, ...){
  # Forward difference jacobian approximation
   if (is.null(resfn)) stop("You must supply a residual function")
   npar<-length(pars)
   nres<-length(resbest)
   jacmat<-matrix(0, ncol=npar, nrow=nres)
   tpars<-pars # need to set because values used for each dimension
   for (j in 1:npar){
      if (is.null(bdmsk) || (bdmsk[j]!=0)) {
         step<-ndstep*(abs(pars[j])+ndstep)
         tpars[j]<-pars[j]+step
         jacmat[,j]<-(resfn(tpars,...)-resbest)/step
         tpars[j]<-pars[j]
      }
   }
   attr(jacmat,"gradient") <- jacmat # to satisfy needs of nlxb(), nlfb() working
   jacmat
} # end jafwd

jaback <- function(pars, resfn=NULL, bdmsk=NULL, resbest=NULL, ndstep=1e-7, ...){
   # Backward difference jacobian approximation
   if (is.null(resfn)) stop("You must supply a residual function")
   npar<-length(pars)
   nres<-length(resbest)
   jacmat<-matrix(0, ncol=npar, nrow=nres)
   tpars<-pars
   for (j in 1:npar){
      if (is.null(bdmsk) || (bdmsk[j]!=0)) {
         step<-ndstep*(abs(pars[j])+ndstep)
         tpars[j]<-tpars[j]-step
         jacmat[,j]<-(resbest-resfn(tpars,...))/step
         tpars[j]<-pars[j]
      }
   }
   attr(jacmat,"gradient") <- jacmat # to satisfy needs of nlxb(), nlfb() working
   jacmat
} # end jaback

jacentral <- function(pars, resfn=NULL, bdmsk=NULL, resbest=NULL, ndstep=1e-7, ...){
   # Central difference jacobian approximation
   if (is.null(resfn)) stop("You must supply a residual function")
   npar<-length(pars)
   nres<-length(resbest)
   jacmat<-matrix(0, ncol=npar, nrow=length(resbest))
   tparf<-pars
   tparm<-pars
   for (j in 1:npar){
      if (is.null(bdmsk) || (bdmsk[j]!=0)) {
         step<-ndstep*(abs(pars[j])+ndstep)
         tparf[j]<-tparf[j]+step
         tparm[j]<-tparm[j]-step
         jacmat[,j]<-0.5*(resfn(tparf,...)-resfn(tparm,...))/step
         tparf[j]<-pars[j]
         tparm[j]<-pars[j]
      }
   }
   attr(jacmat,"gradient") <- jacmat # to satisfy needs of nlxb(), nlfb() working
   jacmat
} # end jacentral


resgr <- function(prm, resfn, jacfn, ...) {
    # computes the gradient 2 * J' %*% res for residuals (res)
    # and jacobian (Jac) defined by resfn and jacfn at
    # parameters prm and extra variables defined in the
    # dot-arguments J C Nash 2012-4-26
    res <- resfn(prm, ...)  # Should we use try()?
    if (missing(jacfn) || is.null(jacfn)){
       stop("MUST PROVIDE jacobian function or specify approximation")
    } else {  # Need to see if jacfn has quotes
      if (is.character(jacfn)) { # have quoted jacobian function -- an approximation
        myjac <- match.fun(jacfn)
        resval <- resfn(prm)
        cat("myjac:"); print(myjac)
        Jac <- myjac(prm, resfn, resbest=resval, ...)
      } else { # non-null, non-quoted jacfn
        Jac<-jacfn(prm, ...)
        cat("jacfn:"); print(jacfn)
      }
    }
    grj <- 2 * as.numeric(crossprod(Jac, res))
    attr(res, "Jacobian") <- Jac 
    attr(res, "gradient") <- grj
    res
}


resss <- function(prm, resfn, ...) {
    # computes sumsquares function from residuals defined by
    # resfn at parameters prm and extra variables defined in
    # the dot-arguments J C Nash 2012-4-26
    resids <- resfn(prm, ...)  #  try() ?
    ss <- as.numeric(crossprod(resids))
}

numericDerivR <- function(expr, theta, rho = parent.frame(), dir = 1,
                 eps = .Machine$double.eps ^ (1/if(central) 3 else 2), central = FALSE)
## Note: Must this expr must be set up as a call to work properly?
## We set eps conditional on central. But central set AFTER eps. Is this OK?
{   ## cat("numericDeriv-Alt\n")
    dir <- rep_len(dir, length(theta))
    stopifnot(is.finite(eps), eps > 0)
##    rho1 <- new.env(FALSE, rho, 0)
    if (!is.character(theta) ) {stop("'theta' should be of type character")}
    if (is.null(rho)) {
            stop("use of NULL environment is defunct")
            #        rho <- R_BaseEnv;
    } else {
          if(! is.environment(rho)) {stop("'rho' should be an environment")}
          #    int nprot = 3;
    }
    if( ! ((length(dir) == length(theta) ) & (is.numeric(dir) ) ) )
              {stop("'dir' is not a numeric vector of the correct length") }
    if(is.na(central)) { stop("'central' is NA, but must be TRUE or FALSE") }
    res0 <- eval(expr, rho) # the base residuals. C has a check for REAL ANS=res0
    nt <- length(theta) # number of parameters
    mr <- length(res0) # number of residuals
    JJ <- matrix(NA, nrow=mr, ncol=nt) # Initialize the Jacobian
    for (j in 1:nt){
       origPar<-get(theta[j],rho) # This is parameter by NAME, not index
       xx <- abs(origPar)
       delta <- if (xx == 0.0) {eps} else { xx*eps }
       ## JN: I prefer eps*(xx + eps)  which is simpler?
       prmx<-origPar+delta*dir[j]
       assign(theta[j],prmx,rho)
       res1 <- eval(expr, rho) # new residuals (forward step)
#       cat("res1:"); print(res1)
       if (central) { # compute backward step resids for central diff
          prmb <- origPar - dir[j]*delta
          assign(theta[j], prmb, envir=rho) # may be able to make more efficient later?
          resb <- eval(expr, rho)
          JJ[, j] <- dir[j]*(res1-resb)/(2*delta) # vectorized
       } else { ## forward diff
          JJ[, j] <- dir[j]*(res1-res0)/delta
       }  # end forward diff
       assign(theta[j],origPar, rho) # reset value 
    } # end loop over the parameters
    attr(res0, "gradient") <- JJ
    return(res0)
}

model2rjfun <- function(modelformula, pvec, data = NULL, jacobian = TRUE, 
                          testresult = TRUE, ...) {
    # add dots 161127, but they have NOT been tested

    stopifnot(inherits(modelformula, "formula"))

    if (length(modelformula) == 2) { ## Explain this in vignette!?
        residexpr <- modelformula[[2]]
    } else if (length(modelformula) == 3) {
        residexpr <- call("-", modelformula[[3]], modelformula[[2]])
    } else stop("Unrecognized formula")
    
    if (is.null(names(pvec))){
          cat("apparently no names in pvec\n")
	  names(pvec) <- paste0("p_", seq_along(pvec))
    }
    
    if (jacobian)
        residexpr <- deriv(residexpr, names(pvec))
## SHOULD WE TRY:?
##	residexpr <- fnDeriv(residexpr, names(pvec))
##  data <- list(...)	
    if (is.null(data)) {
	  data <- environment(modelformula) # this will handle variables in the parent frame
    }  ## Don't yet handle variable in dot args. But no dot args here. Why?
    else if (is.list(data))
	data <- list2env(data, parent = environment(modelformula))
    else if (!is.environment(data))
        	stop("'data' must be a dataframe, list, or environment")
    rjfun <- function(prm) {
        if (is.null(names(prm))) 
	    names(prm) <- names(pvec)
	  localdata <- list2env(as.list(prm), parent = data)
	  eval(residexpr, envir = localdata)  
      # Saves Jacobian matrix as "gradient" attribute (consistent with deriv())
    }
    
    if (testresult) {
	resids <- rjfun(pvec)
	if (any(bad <- !is.finite(resids))) 
	    stop("residuals contain ", unique(resids[bad]))
	if (jacobian && any(bad <- !is.finite(attr(resids, "gradient"))))
	    stop("Jacobian contains ", unique(attr(resids, "gradient")[bad]))
	rm(resids, bad)  # Don't want to capture these in the environment of rjfun
    }
    rjfun
}

model2ssgrfun <- function(modelformula, pvec, data = NULL, gradient = TRUE, 
                          testresult = TRUE, ...) {
    rjfun <- model2rjfun(modelformula, pvec, data = data, jacobian = gradient, 
                         testresult = testresult, ...)
			
    function(prm) {
    	resids <- rjfun(prm)
	    ss <- as.numeric(crossprod(resids))
	    if (gradient) {
	        jacval <- attr(resids, "gradient")
	        grval <- 2*as.numeric(crossprod(jacval, resids))
	        attr(ss, "gradient") <- grval
	    }
	    attr(ss, "resids") <- resids
	    ss
    }
}

modelexpr <- function(fun) {
    env <- environment(fun)
    if (exists("rjfun", env))
	env <- environment(env$rjfun)
    env$residexpr
}


SSmod2rjfun <- function(modelformula, pvec, data = NULL, jacobian = TRUE, 
                        testresult = TRUE, ...) {
  stopifnot(inherits(modelformula, "formula"))
  
  if (length(modelformula) == 2) { ## Explain this in vignette!?
    residexpr <- modelformula[[2]]
    rhs <- residexpr # for SS case
  } else if (length(modelformula) == 3) {
    residexpr <- call("-", modelformula[[3]], modelformula[[2]])
    rhs <- modelformula[[3]] # for SS case
  } else stop("Unrecognized formula")

  if (is.null(names(pvec))){
    cat("apparently no names in pvec\n")
    names(pvec) <- paste0("p_", seq_along(pvec))
  }
  
  if (jacobian) {
    if (is.null(data)) {
      data <- environment(modelformula) # this will handle variables in the parent frame
    }  ## Don't yet handle variable in dot args. But no dot args here.
    else if (is.list(data))
      data <- list2env(data, parent = environment(modelformula))
    else if (!is.environment(data))
      stop("'data' must be a dataframe, list, or environment")
    rjfun <- function(prm) {
      if (is.null(names(prm))) 
        names(prm) <- names(pvec)
      localdata <- list2env(as.list(prm), parent = data)
      eval(residexpr, envir = localdata)  
      # Saves Jacobian matrix as "gradient" attribute (consistent with deriv())
    }
  }  

  if (testresult) {
    resids <- rjfun(pvec)
    if (any(bad <- !is.finite(resids))) 
      stop("residuals contain ", unique(resids[bad]))
    if (jacobian && any(bad <- !is.finite(attr(resids, "gradient"))))
      stop("Jacobian contains ", unique(attr(resids, "gradient")[bad]))
    rm(resids, bad)  # Don't want to capture these in the environment of rjfun
  }
  rjfun
}

traceval  <-  TRUE  # traceval set TRUE to debug or give full history

# Data for Hobbs problem
ydat  <-  c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443, 
          38.558, 50.156, 62.948, 75.995, 91.972) # for testing
tdat  <-  seq_along(ydat) # for testing

eunsc  <-   y ~ b1/(1+b2*exp(-b3*tt))

cat("LOCAL DATA IN DATA FRAMES\n")
weeddata1  <-  data.frame(y=ydat, tt=tdat)
weeddata2  <-  data.frame(y=1.5*ydat, tt=tdat)


cat("BOUNDS")
start2  <-  c(b1=100, b2=10, b3=0.1)
an1qb1  <-  try(nlxb(eunsc, start=start2, trace=traceval, data=weeddata1, 
                     lower=c(0,0,0), upper=c(200, 60, .3)))
print(an1qb1)

cat("BOUNDS and MASK")

an1qbm2  <-  try(nlxb(eunsc, start=start2, trace=traceval, data=weeddata1, 
                      lower=c(0,0,0), upper=c(200, 60, .3), masked=c("b2")))
print(an1qbm2)

##*## SSlogisJN - logistic model for nonlinear regression
## Revision by John Nash 2022-8-30 to avoid plinear call.

SSlogisJN <- # selfStart(~ Asym/(1 + exp((xmid - input)/scal)),
    selfStart(
        function(input, Asym, xmid, scal)
        {
              .expr1 <- xmid - input
              .expr3 <- exp(.e2 <- .expr1/scal)
              .expr4 <- 1 + .expr3
              .value <- Asym/.expr4
              .actualArgs <- as.list(match.call()[c("Asym", "xmid", "scal")])
              if(all(vapply(.actualArgs, is.name, NA)))
              {
		  .expr10 <- .expr4^2
                  .grad <- array(0, c(length(.value), 3L), list(NULL, c("Asym", "xmid", "scal")))
                  .grad[, "Asym"] <- 1/.expr4
		  .grad[, "xmid"] <- - (xm <- Asym * .expr3/scal/.expr10)
		  .grad[, "scal"] <- xm * .e2
                  dimnames(.grad) <- list(NULL, .actualArgs)
                  attr(.value, "gradient") <- .grad
              }
              .value
        },
		    initial = function(mCall, data, LHS, ...) {
		          xy <- sortedXyData(mCall[["input"]], LHS, data)
##		          cat("xy:\n"); print(xy)
		          if(nrow(xy) < 4) {
		            stop("too few distinct input values to fit a logistic model")
		          }
		          z <- xy[["y"]]
		          z[which(z<=0)] <- 1e-9 # Just in case
		          Asym<-2*max(z)
		          xy[["z"]] <- log(Asym/z - 1)
		          aux <- coef(lm(z ~ x, xy))
		          scal <- -1/aux[2]
		          xmid <- aux[1]*scal
		          pars<-c(Asym=Asym, xmid=xmid, scal=scal)
		          names(pars)<-c("Asym", "xmid", "scal")
		          pars
		          },
              parameters = c("Asym", "xmid", "scal"))

# Test of nlsr hessian and gradient

fexp <- ~ ((x-1)^2)*((y-3)^2)
fexp
library(nlsr)
myder <- fnDeriv(fexp, c("x","y"), hessian=TRUE)
myder
old <- c(x=0, y=-0)
myd <- function(prm=old){
  x <- prm[1]
  y <- prm[2]
  val <- myder(x, y)
}
aold <- myd(old)
aold
hm1a <- attr(aold, "hessian")
hm <- as.matrix(hm1a[1,,])
hm
g1 <- attr(aold, "gradient")
g1 <- as.vector(g1)
g1

old <- c(1e20, 1e20)
new <- c(x=0, y=-0)

count <- 0
while (max(abs(new-old)) > 1e-6 ) {
  old <- new
  count <- count + 1
  cat("iteration ", count,"  ")
  newf<-myd(new)
  newf
  hm1a <- attr(newf, "hessian")
  hm <- as.matrix(hm1a[1,,])
  hm
  g1 <- attr(newf, "gradient")
  g1 <- as.vector(g1)
  g1
  delta <- solve(hm, g1)
#  cat("delta:")
#  print(delta)
  new <- old-delta
  cat("new:")
  print(new)
  newf<-myd(new)
  cat("Function value = ",newf,"\n")
  hm1a <- attr(newf, "hessian")
  hm <- as.matrix(hm1a[1,,])
  hm
  g1 <- attr(newf, "gradient")
  g1 <- as.vector(g1)
  g1
#  print(new)
#  print(old)
  tmp <- readline("continue?")
}

nlsrSS <- function(formula, data){
    # wrapper to allow nlxb to self-start
    strt<-getInitial(formula, data)
    cat("suggested start=\n")
    print(strt)
    fit <- nlxb(formula, data, strt, control=list(japprox="SSJac"))
    fit
}

wrapnlsr <- function(formula=NULL, data=NULL, start=NULL, control = NULL, 
     trace = FALSE, subset=NULL, lower = -Inf, upper = Inf, weights = NULL, ...) {
    # A wrapper to call nlxb() and then call nls() with the
    # solution.  The calling sequence matches that of nlsmnq()
    if (is.null(formula)) stop("No formula")
  
  ## NOTE: The next few lines are suggestions by F. Miguez (2022-09-19) mod by JN 2022-9-24
    if (is.null(start)) {
    if (trace) cat("start is NULL -- try getInitial\n")
    issvalues <- try(getInitial(object = formula, data = data), silent = TRUE)
    if(!inherits(issvalues, "try-error")){
      if(is.null(control)){
        control <- list(japprox = "SSJac")
      }else{
        control[["jjaprox"]] <- "SSJac"
      }  
      start <- issvalues
    } 
    }
    if (is.null(start)) stop("No start")
    if (trace) {
       cat("control list for wrapnlsr:\n")
       print(control)
    }
    if (trace && ! is.null(weights)) { 
       cat("weights:"); print(str(weights)) 
    }
    nobs <- dim(data)[1]
    swts <- weights
    if (is.null(weights)) swts <- rep(1,nobs)
    sform<-formula
    if (trace) print(str(sform))
# Ensure we have controls.
    jfn<-NULL # default for nlxb
    ctrl <- nlsr.control()
    if(!missing(control)) {
	control <- as.list(control)
        if (! is.null(control$japprox)) jfn <- control$japprox
	ctrl[names(control)] <- control
    }
    ctrl$japprox<-jfn
    if (is.null(data)) stop("wrapnls() must have 'data' supplied")
    if(! is.null(subset)) {
##        stop("subset NOT used in wrapnlsr(). Please explicitly subset your data.")
       swts[-subset] <- 0 # handle subsetting through weights
       if (trace) { cat("swts:"); print(swts) }
       warning("subset effected by zero weights")
    }
    npar <- length(start)
    if (length(lower) < npar) {
        if (length(lower) == 1) 
            lower <- rep(lower, npar) else stop("lower bounds wrong length")
    }
    if (length(upper) < npar) {
        if (length(upper) == 1) 
            upper <- rep(upper, npar) else stop("upper bounds wrong length")
    }
    if (trace) {
        cat("wrapnlsr call with lower="); print(lower)
        cat("and upper="); print(upper)
    }
    first <- nlxb(formula=sform, start=start, trace = trace, data = data,
        lower = lower, upper = upper, weights=swts, control = ctrl, ...)
    # Should check this has worked, but ...
    if (trace) pshort(first)
    newstart <- first$coefficients
    if (trace) { nvec(newstart) }
    if (all(is.infinite(lower)) && all(is.infinite(upper))) {
        if (trace) cat("nls call with no bounds\n")
        second <- try(do.call("nls", list(formula=sform, data=data, start=newstart, 
		control=ctrl, algorithm=NULL, trace=trace, weights = swts, ...)))
     } else {
        if (trace) cat("Now try nls - bounded\n")
        second <- try(do.call("nls", list(formula=sform, data=data, start=newstart, 
		control=list(), algorithm="port", trace=trace, lower = lower, 
		upper = upper, weights = swts, ...)))

    }
    second
}

   nlsr <- wrapnlsr

## End of file ##

dex <- function(x, do_substitute = NA, verbose = FALSE) {
  expr <- substitute(x)
  if (is.na(do_substitute) || !do_substitute)
    value <- tryCatch(x, error = function(e) e)
  if (is.na(do_substitute)) {
    if (verbose)
      message("Determining whether to use expression or value...")
    if (inherits(value, "error"))
      do_substitute <- TRUE
    else if (is.character(value))
      do_substitute <- FALSE
    else if (is.name(expr))
      do_substitute <- FALSE
    else
      do_substitute <- TRUE
  } else if (!do_substitute && inherits(value, "error"))
    stop(conditionMessage(value), call. = FALSE)
  if (verbose) {
    if (do_substitute) 
      message("Using expression.")
    else message("Using value.")
  }  
  if (!do_substitute) {
    if (is.character(value)) {
      if (verbose) 
        message("Parsing value.")
      expr <- parse(text=value)
    } else
      expr <- value
  }
  if (is.expression(expr) && length(expr) == 1)
    expr <- expr[[1]]
  else if (is.call(expr) && as.character(expr[[1]]) == "~") {
    if (length(expr) == 3) {
      warning("Left hand side of formula will be ignored.")
      expr <- expr[[3]]
    } else
      expr <- expr[[2]]
  }
  expr
}

sysDerivs <- new.env(parent = emptyenv())

sysSimplifications <- new.env(parent = emptyenv())

newDeriv <- function(expr, deriv, derivEnv = sysDerivs) {
    if (missing(expr))
    	return(ls(derivEnv))
    expr <- substitute(expr) # Get parse tree of expr 
    if (!is.call(expr))
    	stop("expr must be a call to a function")
    fn <- as.character(expr[[1]])
    if (missing(deriv)) 
    	return(derivEnv[[fn]])
    deriv <- substitute(deriv)
    args <- expr[-1]
    argnames <- names(args)
    if (is.null(argnames))
    	argnames <- rep("", length(args))
    required <- which(argnames == "")
    for (i in required)
    	argnames[i] <- as.character(args[[i]])
    value <- list(expr = expr, argnames = argnames, 
                  required = required, deriv = deriv)
    if (!is.null(oldval <- derivEnv[[fn]]) && !identical(value, oldval))
      warning(gettextf("changed derivative for %s", dQuote(fn)))
    assign(fn, value, envir = derivEnv)
    invisible(value) # returns but does not print
} 

newSimplification <- function(expr, test, simplification, 
                              do_eval = FALSE, simpEnv = sysSimplifications) {
    if (missing(expr))
    	return(ls(simpEnv))
    expr <- substitute(expr)
    if (!is.call(expr))
    	stop("expr must be a call to a function")
    fn <- as.character(expr[[1]])
    nargs <- length(expr) - 1L
    simps <- simpEnv[[fn]]
    if (missing(test)) {
        if (nargs <= length(simps))	
    	    return(simps[[nargs]])
    	else
    	    return(NULL)
    }
    test <- substitute(test)
    simplification <- substitute(simplification)
    
    args <- expr[-1]
    if (!is.null(names(args)))
    	stop("expr should not have named arguments")
    if (!all(sapply(args, is.name)))
    	stop("expr should have simple names as arguments")
    argnames <- sapply(args, as.character)
    if (any(duplicated(argnames)))
    	stop("expr names should be unique")
    	
    if (is.null(simps)) simps <- list()
    if (nargs <= length(simps)) 
    	simpn <- simps[[nargs]]
    else
    	simpn <- list()
    simpn <- c(simpn, list(list(expr = expr, argnames = argnames, test = test, 
                                simplification = simplification, do_eval = do_eval)))
    simps[[nargs]] <- simpn
    assign(fn, simps, envir = simpEnv)
}
    	
nlsDeriv <- function(expr, name, derivEnv = sysDerivs, do_substitute = FALSE, verbose = FALSE, ...) {
    Recurse <- function(expr) {
    	if (is.call(expr)) {
    	    if (as.character(expr[[1]]) == "D")
    	    	expr <- nlsDeriv(expr[[2]], name, derivEnv, do_substitute = FALSE, verbose = verbose, ...)
    	    else
    	    	for (i in seq_along(expr)[-1])
    	    	    expr[[i]] <- Recurse(expr[[i]])
    	}
    	expr
    }
    expr <- dex(expr, do_substitute = do_substitute, verbose = verbose)
    if (is.expression(expr))
    	return(as.expression(lapply(expr, nlsDeriv, name = name, derivEnv = derivEnv, do_substitute = FALSE, verbose = verbose, ...)))
    else if (is.numeric(expr) || is.logical(expr))
    	return(0)
    else if (is.call(expr)) {
    	fn <- as.character(expr[[1]])
	if (fn == "expression")
	    return(as.expression(lapply(as.list(expr)[-1], nlsDeriv, name = name, derivEnv = derivEnv, do_substitute = FALSE, verbose = verbose, ...)))
    	model <- derivEnv[[fn]]
    	if (is.null(model))
    	    stop("no derivative known for '", fn, "'")
 	if (missing(name) || verbose) {
 	    message(paste("Expr:", deparse(expr)))
 	    message(if (missing(name)) "Pattern for" else "Using pattern")
 	    message(paste("  ", deparse(model$expr), collapse = "\n"))
 	    message("is")
 	    message(paste("  ", deparse(model$deriv), collapse = "\n"))
 	    if (missing(name))
 	    	return(invisible(NULL))
 	}
        args <- expr[-1]
        argnames <- names(args)
        if (is.null(argnames)) 
            argnames <- rep("", length(args))
        modelnames <- model$argnames
        argnum <- pmatch(argnames, modelnames)
        if (any(bad <- is.na(argnum) & argnames != ""))
            stop("Argument names not matched: ", paste(argnames[bad], collapse = ", "))
        unused <- setdiff(seq_along(modelnames), argnum)
        nonamecount <- sum(is.na(argnum))
        length(unused) <- nonamecount
        argnum[which(is.na(argnum))] <- unused
        default <- setdiff(seq_along(modelnames), argnum)
        if (length(bad <- setdiff(model$required, argnum)))
            stop("Missing required arguments: ", paste(modelnames[bad], collapse = ", "))
        
        # Now do the substitutions
        subst <- list()
        subst[argnum] <- as.list(args)
        subst[default] <- as.list(model$expr[-1])[default]
        names(subst) <- modelnames
        result <- do.call(substitute, list(model$deriv, subst))
        result <- Recurse(result)
        nlsSimplify(result, verbose = verbose, ...)
    } else if (is.name(expr))
        return( as.numeric(as.character(expr) == name) )
}

codeDeriv <- function(expr, namevec, 
       hessian = FALSE, derivEnv = sysDerivs, 
       do_substitute = FALSE, verbose = FALSE, ...) {
  
  # Make sure expr is a single expression object
  expr <- dex(expr, do_substitute = do_substitute, verbose = verbose)
  expr <- as.expression(expr)
  if (length(expr) > 1)
    stop("Only single expressions allowed")
  
  # Set exprs[[1]] to exprs, fill in the rest with the derivatives
  # that were requested
  exprs <- as.list(expr)
  n <- length(namevec)
  length(exprs) <- n + 1L
  for (i in seq_len(n))
    exprs[[i + 1]] <- nlsDeriv(expr[[1]], namevec[i], derivEnv = derivEnv, do_substitute = FALSE, verbose = verbose, ...)
  names(exprs) <- c(".value", namevec)
  
  # If a Hessian is requested, extend exprs to hold its entries
  if (hessian) {
    m <- length(exprs)
	  length(exprs) <- m + n*(n+1)/2
	  for (i in seq_len(n))
	    for (j in seq_len(n-i+1) + i-1) {
		    m <- m + 1
		    exprs[[m]] <- nlsDeriv(exprs[[i + 1]], namevec[j], derivEnv = derivEnv, 
		                    do_substitute = FALSE, verbose = verbose, ...)
	    }
  }
  
  # Find common subexpressions in the result 
  exprs <- as.expression(exprs)
  subexprs <- findSubexprs(exprs)
  m <- length(subexprs)
  final <- subexprs[[m]]
  subexprs[[m]] <- substitute(.value <- expr, list(expr = final[[".value"]]))
  subexprs[[m+1]] <- substitute(.grad <- array(0, c(length(.value), namelen), list(NULL, namevec)),
				  list(namevec = namevec, namelen = length(namevec)))
  if (hessian)
    subexprs[[m+2]] <- substitute(.hessian <- array(0, c(length(.value), namelen, namelen), 
					list(NULL, namevec, namevec)), 
					list(namelen = length(namevec), namevec = namevec))
  m <- length(subexprs)
  for (i in seq_len(n))
    subexprs[[m+i]] <- substitute(.grad[, name] <- expr,
			          list(name = namevec[i], expr = final[[namevec[i]]]))
  m <- length(subexprs)
  h <- 0
  if (hessian) {
    for (i in seq_len(n))
      for (j in seq_len(n-i+1) + i-1) {
		    h <- h + 1
		    if (i == j)
		      subexprs[[m + h]] <- substitute(.hessian[, i, i] <- expr,
				    list(i = namevec[i], expr = final[[1 + n + h]]))
		    else
		      subexprs[[m + h]] <- substitute(.hessian[, i, j] <- .hessian[, j, i] <- expr,
				    list(i = namevec[i], j = namevec[j], expr = final[[1 + n + h]]))
	    }	
	  h <- h + 1
	  subexprs[[m + h]] <- quote(attr(.value, "hessian") <- .hessian)
  }
  m <- length(subexprs)
  subexprs[[m+1]] <- quote(attr(.value, "gradient") <- .grad)
  subexprs[[m+2]] <- quote(.value)
  
  # Return the version that reuses common subexpressions
  subexprs
}

fnDeriv <- function(expr, namevec, args = all.vars(expr), env = environment(expr),
                    do_substitute = FALSE, verbose = FALSE, ...) {
  # Create a dummy function
  fn <- function() NULL
  
  # Set the body to the sequence of computations returned by codeDeriv
  expr <- dex(expr, do_substitute = do_substitute, verbose = verbose)
  body(fn) <- codeDeriv(expr, namevec, do_substitute = FALSE, verbose = verbose, ...)
  
  # Set the args 
  if (is.character(args)) {
    formals <- rep(list(bquote()), length = length(args))
    names(formals) <- args
    args <- formals
  }
  formals(fn) <- args
  
  # Set the environment and return it
  if (is.null(env))
    environment(fn) <- parent.frame()
  else
    environment(fn) <- as.environment(env)
  fn
}

isZERO <- function(x) is.numeric(x) && length(x) == 1 && x == 0

isONE  <- function(x) is.numeric(x) && length(x) == 1 && x == 1

isMINUSONE <- function(x) is.numeric(x) && length(x) == 1 && x == -1

isCALL <- function(x, name) is.call(x) && as.character(x[[1]]) == name


nlsSimplify <- function(expr, simpEnv = sysSimplifications, verbose = FALSE) {
    
    if (is.expression(expr))
    	return(as.expression(lapply(expr, nlsSimplify, simpEnv, verbose = verbose)))    

    if (is.call(expr)) {    	
	for (i in seq_along(expr)[-1])
	    expr[[i]] <- nlsSimplify(expr[[i]], simpEnv, verbose = verbose)
	fn <- as.character(expr[[1]])
	nargs <- length(expr) - 1
	while (!identical(simpEnv, emptyenv())) {
	    simps <- simpEnv[[fn]]
	    if (nargs > length(simps))
		return(expr)
	    simpn <- simps[[nargs]]
	    for (i in seq_along(simpn)) {  	
		argnames <- simpn[[i]]$argnames
		substitutions <- lapply(seq_along(argnames)+1L, function(i) expr[[i]])
		names(substitutions) <- argnames
		test <- simpn[[i]]$test
		if (with(substitutions, eval(test))) {
		    if (verbose) {
	    		message(paste("Simplifying", deparse(expr)))		    	
	    		message("Applying simplification:")
		    	print(simpn[[i]])
		    }
		    simplification <- do.call(substitute, list(simpn[[i]]$simplification, substitutions))
	   	    if (simpn[[i]]$do_eval)
	    		simplification <- eval(simplification)
	    	    return(simplification)
	        }
	     }
	     simpEnv <- parent.env(simpEnv)
	}
    }
    expr
}

findSubexprs <- function(expr, simplify = FALSE, tag = ".expr", verbose = FALSE, ...) {
    digests <- new.env(parent = emptyenv())
    subexprs <- list()
    subcount <- 0
    
    record <- function(index) {
        if (simplify)
	         expr[[index]] <<- subexpr <- nlsSimplify(expr[[index]], 
	                     verbose = verbose, ...)
      	else
     	    subexpr <- expr[[index]]
        	if (is.call(subexpr)) {
	           digest <- digest(subexpr)
	           for (i in seq_along(subexpr))
		         record(c(index,i))
	           prev <- digests[[digest]]
	           if (is.null(prev)) assign(digest, index, envir = digests)
        	   else if (is.numeric(prev))  { # the index where we last saw this
      	        subcount <<- subcount + 1
		            name <- as.name(paste0(tag, subcount))
		            assign(digest, name, envir = digests)
        	   }
	        }
     }
    
     edit <- function(index) {
	     subexpr <- expr[[index]]
	     if (is.call(subexpr)) {
	       digest <- digest(subexpr)	    
	       for (i in seq_along(subexpr)) edit(c(index,i))
    	   prev <- digests[[digest]]
	       if (is.name(prev)) {
		       num <- as.integer(substring(as.character(prev), nchar(tag)+1L))
		       subexprs[[num]] <<- call("<-", prev, expr[[index]])
		       expr[[index]] <<- prev
	       } 
	     }
     }
     for (i in seq_along(expr)) record(i)
     for (i in seq_along(expr)) edit(i)
     result <- quote({})
     result[seq_along(subexprs)+1] <- subexprs
     result[[length(result)+1]] <- expr
     result
}
    
# These are the derivatives supported by deriv()

newDeriv(log(x, base = exp(1)), 
         D(x)/(x*log(base)) - (1/base)*log(x, base)/log(base)*D(base))
newDeriv(exp(x), exp(x)*D(x))
newDeriv(sin(x), cos(x)*D(x))
newDeriv(cos(x), -sin(x)*D(x))
newDeriv(tan(x), 1/cos(x)^2*D(x))
newDeriv(sinh(x), cosh(x)*D(x))
newDeriv(cosh(x), sinh(x)*D(x))
newDeriv(sqrt(x), D(x)/2/sqrt(x))
newDeriv(pnorm(q, mean = 0, sd = 1, lower.tail = TRUE, log.p = FALSE),
  (if (lower.tail && !log.p) 
  	dnorm((q-mean)/sd)*(D(q)/sd - D(mean)/sd - D(sd)*(q-mean)/sd^2) 
  else if (!lower.tail && !log.p) 
  	-dnorm((q-mean)/sd)*(D(q)/sd - D(mean)/sd - D(sd)*(q-mean)/sd^2) 
  else if (lower.tail && log.p) 
  	(D(q)/sd - D(mean)/sd - D(sd)*(q-mean)/sd^2)*exp(dnorm((q-mean)/sd, log = TRUE) 
  		- pnorm((q-mean)/sd, log = TRUE))
  else if (!lower.tail && log.p)	
  	-(D(q)/sd - D(mean)/sd - D(sd)*(q-mean)/sd^2)*exp(dnorm((q-mean)/sd, log = TRUE) 
  		- pnorm((q-mean)/sd, lower.tail = FALSE, log = TRUE)))
  + stop("cannot take derivative wrt 'lower.tail' or 'log.p'")*(D(lower.tail) + D(log.p)))
newDeriv(dnorm(x, mean = 0, sd = 1, log = FALSE),
  (if (!log) 
  	dnorm((x-mean)/sd)/sd^2*((D(mean)-D(x))*(x-mean)/sd + D(sd)*((x-mean)^2/sd^2 - 1)) 
  else if (log)
        ((D(mean)-D(x))*(x-mean)/sd + D(sd)*((x-mean)^2/sd^2 - 1))/sd) 
  + stop("cannot take derivative wrt 'log'")*D(log))
newDeriv(asin(x), D(x)/sqrt(1+x^2))
newDeriv(acos(x), -D(x)/sqrt(1+x^2))
newDeriv(atan(x), D(x)/(1+x^2))
newDeriv(gamma(x), gamma(x)*digamma(x)*D(x))
newDeriv(lgamma(x), digamma(x)*D(x))
newDeriv(digamma(x), trigamma(x)*D(x))
newDeriv(trigamma(x), psigamma(x, 2L)*D(x))
newDeriv(psigamma(x, deriv = 0L), 
          psigamma(x, deriv + 1L)*D(x) 
        + stop("cannot take derivative wrt 'deriv'")*D(deriv))

newDeriv(x*y, x*D(y) + D(x)*y)
newDeriv(x/y, D(x)/y - x*D(y)/y^2)
newDeriv(x^y, y*x^(y-1)*D(x) + x^y*log(x)*D(y))
newDeriv((x), D(x))
# Need to be careful with unary + or -
newDeriv(`+`(x, y = .MissingVal), if (missing(y)) D(x) else D(x) + D(y))
newDeriv(`-`(x, y = .MissingVal), if (missing(y)) -D(x) else D(x) - D(y))

# These are new

newDeriv(abs(x), sign(x)*D(x))
newDeriv(sign(x), 0)

newDeriv(`~`(x, y = .MissingVal), if (missing(y)) D(x) else D(y))

# Now, the simplifications

newSimplification(+a, TRUE, a)
newSimplification(-a, is.numeric(a), -a, do_eval = TRUE)
newSimplification(-a, isCALL(a, "-") && length(a) == 2, quote(a)[[2]], do_eval = TRUE)

newSimplification(exp(a), isCALL(a, "log") && length(a) == 2, quote(a)[[2]], do_eval = TRUE)
newSimplification(exp(a), is.numeric(a), exp(a), do_eval = TRUE)

newSimplification(log(a), isCALL(a, "exp") && length(a) == 2, quote(a)[[2]], do_eval = TRUE)
newSimplification(log(a), is.numeric(a), log(a), do_eval = TRUE)

newSimplification(!a, isTRUE(a), FALSE)
newSimplification(!a, isFALSE(a), TRUE)

newSimplification((a), TRUE, a)

newSimplification(a + b, isZERO(b), a)
newSimplification(a + b, isZERO(a), b)
newSimplification(a + b, identical(a, b), nlsSimplify(quote(2*a)), do_eval = TRUE)
newSimplification(a + b, is.numeric(a) && is.numeric(b), a+b, do_eval = TRUE)
# Add these to support our error scheme, don't test for stop() everywhere.
newSimplification(a + b, isCALL(a, "stop"), a)
newSimplification(a + b, isCALL(b, "stop"), b)

newSimplification(a - b, isZERO(b), a)
newSimplification(a - b, isZERO(a), nlsSimplify(quote(-b)), do_eval = TRUE)
newSimplification(a - b, identical(a, b), 0)
newSimplification(a - b, is.numeric(a) && is.numeric(b), a - b, do_eval = TRUE)

newSimplification(a * b, isZERO(a), 0)
newSimplification(a * b, isZERO(b), 0)
newSimplification(a * b, isONE(a), b)
newSimplification(a * b, isONE(b), a)
newSimplification(a * b, isMINUSONE(a), nlsSimplify(quote(-b)), do_eval = TRUE)
newSimplification(a * b, isMINUSONE(b), nlsSimplify(quote(-a)), do_eval = TRUE)
newSimplification(a * b, is.numeric(a) && is.numeric(b), a * b, do_eval = TRUE)

newSimplification(a / b, isONE(b), a)
newSimplification(a / b, isMINUSONE(b), nlsSimplify(quote(-a)), do_eval = TRUE)
newSimplification(a / b, isZERO(a), 0)
newSimplification(a / b, is.numeric(a) && is.numeric(b), a / b, do_eval = TRUE)

newSimplification(a ^ b, isONE(b), a)
newSimplification(a ^ b, is.numeric(a) && is.numeric(b), a ^ b, do_eval = TRUE)

newSimplification(log(a, base), isCALL(a, "exp"), nlsSimplify(call("/", quote(a)[[2]], quote(log(base)))), do_eval = TRUE)

newSimplification(a && b, isFALSE(a) || isFALSE(b), FALSE)
newSimplification(a && b, isTRUE(a), b)
newSimplification(a && b, isTRUE(b), a)

newSimplification(a || b, isTRUE(a) || isTRUE(b), TRUE)
newSimplification(a || b, isFALSE(a), b)
newSimplification(a || b, isFALSE(b), a)

newSimplification(if (cond) a, isTRUE(cond), a)
newSimplification(if (cond) a, isFALSE(cond), NULL)

newSimplification(if (cond) a else b, isTRUE(cond), a)
newSimplification(if (cond) a else b, isFALSE(cond), b)
newSimplification(if (cond) a else b, identical(a, b), a)

# This one is used to fix up the unary -
newSimplification(missing(a), identical(a, quote(.MissingVal)), TRUE)
newSimplification(missing(a), !identical(a, quote(.MissingVal)), FALSE)

rhs <- function(formula) {
  if (length(formula) == 2)
    formula[[2]]
  else
    formula[[3]]
}

nlxb <- function(formula, data=parent.frame(), start, trace = FALSE,  lower = NULL,
                 upper = NULL, weights=NULL, control=list(), ...) {
  
    weights0 <- weights
    
    if("subset" %in% names(list(...)))
        stop("subset NOT used in nlxb(). Please explicitly subset your data.")
    ctrl <- nlsr.control(control)
    if (missing(start)) stop("A start vector MUST be provided")
    pnames <- names(start)
    start <- as.numeric(start) # ensure we convert (e.g., if matrix)
    names(start) <- pnames ## as.numeric strips names, so this is needed
    # bounds
    npar <- length(start)  # number of parameters
    if (is.null(lower)) lower<- -Inf
    if (is.null(upper)) upper<- Inf
    if (length(lower) == 1) 
        lower <- rep(lower, npar) # expand to full dimension
    if (length(upper) == 1) 
        upper <- rep(upper, npar)
# more tests on bounds
    if (length(lower) != npar) 
        stop("Wrong length: lower")
    if (length(upper) != npar) 
        stop("Wrong length: upper")
    if (any(start < lower) || any(start > upper)) {
        if(trace) {  cat("start:"); print(start)
                     cat("\n");cat("lower:"); print(lower)
                     cat("\n"); cat("upper:"); print(upper) 
                  }
        stop("Infeasible start")
    }
    if (trace && ctrl$prtlvl>1) {
        cat("formula: ")
        print(formula)
        cat("lower:"); print(lower); cat("upper:"); print(upper) 
    }
    if (trace && ctrl$prtlvl>2) print(str(ctrl))
# Note spelling of lamda -- a throwback to Ag Can 1974 and way to see if folk are copying code.
# First get all the variable names:
    vn <- all.vars(formula)
    # Then see which ones are parameters (get their positions
    # in the set xx
    pnum <- start
    pnames <- names(pnum)
    bdmsk <- rep(1, npar)  # set all params free for now
    maskidx <- which(lower==upper)
    if (length(maskidx) > 0 && trace) {
        cat("The following parameters are masked:")
        print(pnames[maskidx])
    }
    bdmsk[maskidx] <- 0  # fixed parameters
    if (trace && ctrl$prtlvl>1) { # diagnostic printout
        cat("Finished masks check\n")
        parpos <- match(pnames, vn) # Can we explain the 'why'?
        datvar <- vn[-parpos]  # NOT the parameters
        cat("datvar:")
        print(datvar)
        for (i in 1:length(datvar)) {
            dvn <- datvar[[i]]
            cat("Data variable ", dvn, ":")
            if (is.null(data)) { 
                print(eval(parse(text = dvn)))
            } else {
                print(with(data, eval(parse(text = dvn)))) 
            }
        }
    }
    if (is.null(ctrl$japprox)) {
       trjfn<-model2rjfun(formula, pnum, data=data) 
    } else {
      if(ctrl$japprox == "SSJac") {
         trjfn<-SSmod2rjfun(formula, pnum, data=data)
      } else {
         trjfn<-model2rjfun(formula, pnum, data=data, jacobian=FALSE)
      }
    } 
    if (trace && ctrl$prtlvl>2) { cat("str(trjfn):\n"); print(str(trjfn))}
    if (trace && ctrl$prtlvl>2){ cat("ctrl:\n"); print(ctrl) }
    if (inherits(weights, "formula")) {
      # This is a local function so that it has access to 
      # local variables like data, formula
      weights2fun <- function(weights) {
        if (length(weights) != 2)
          warning("Left hand side of weights formula was ignored.")
        weightsexpr <- rhs(weights)
        # Getting fitted values may be slow, so only get
        # them if necessary
        needfitted <- "fitted" %in% all.vars(weightsexpr)
        if (is.null(data))
          data <- environment(weights)
        else if (is.list(data))
          data <- list2env(data, parent = environment(weights))
        else if (!is.environment(data))
          stop("'data' must be a dataframe, list, or environment")
        function(prm, resids) {
          if (is.null(names(prm))) 
            names(prm) <- names(pnum)
          localdata <- list2env(as.list(prm), parent = data)
          localdata$resid <- resids
          if (needfitted)
            localdata$fitted <- eval(rhs(formula), localdata)
          eval(weightsexpr, localdata)
        }
      } 
      weights <- weights2fun(weights)
    }
    if (is.null(ctrl$japprox)) { # call regularly when no jacobian approx
       resfb <- nlfb(start=pnum, resfn=trjfn, jacfn=trjfn, trace=trace, 
            data=data, lower=lower, upper=upper, 
            weights=weights, ctrlcopy=TRUE, control=ctrl)
    } else { # need approx
      if(!is.character(ctrl$japprox)) stop("Non-character ctrl$japprox")
      if(ctrl$japprox == "SSJac") { # Jacobian from selfStart model
        if (trace) { cat('Using Jacobian code from selfStart model \n') }
       resfb <- nlfb(start=pnum, resfn=trjfn, jacfn=trjfn, trace=trace, 
            data=data, lower=lower, upper=upper, 
            weights=weights, ctrlcopy=TRUE, control=ctrl)
      } else {
      if (trace) { cat('Using approximation ',as.character(ctrl$japprox),' 
		 is.character(ctrl$japprox)=', is.character(ctrl$japprox),'\n') }
       resfb <- nlfb(start=pnum, resfn=trjfn, jacfn=ctrl$japprox, trace=trace, 
            data=data, lower=lower, upper=upper, 
            weights=weights, ctrlcopy=TRUE, control=ctrl)
      }
    }
    resfb$formula <- formula # 190805 to add formula to solution (for predict?)
    resfb$resfn <- trjfn # May be lacking Jacobian
    resfb$data <- substitute(data) # Should there be any ... arguments?
    resfb$weights0 <- weights0
    pnum <- as.vector(resfb$coefficients)
    names(pnum) <- pnames # Make sure names re-attached. Is this needed?
    names(resfb$coefficients) <- pnames
    result <- resfb
    class(result) <- "nlsr" ## needed for print method
    result
}

## ----chunk01------------------------------------------------------------------
dx2x <- deriv(~ x^2, "x") 
dx2x
mode(dx2x)
str(dx2x)
x <- -1:2
eval(dx2x) # This is evaluated at -1, 0, 1, 2, with the result in 
# the "gradient" attribute.
# Note that we cannot (easily) differentiate this again.
firstd <- attr(dx2x,"gradient")
str
#  ... and the following gives an error
d2x2x <- try(deriv(firstd, "x"))
str(d2x2x)
#- Build a function from the expression
fdx2x<-function(x){eval(dx2x)}
fdx2x(1)
fdx2x(3.21)
fdx2x(1:5)
#- # Now try D()
Dx2x <- D(expression(x^2), "x")
Dx2x
x <- -1:2
eval(Dx2x)
# We can differentiate aggain
D2x2x <- D(Dx2x,"x")
D2x2x
eval(D2x2x) #- But we don't get a vector -- could be an issue in gradients/Jacobians
#- Note how we handle an expression stored in a string via parse(text=  ))
sx2 <- "x^2"
sDx2x <- D(parse(text=sx2), "x")
sDx2x
#- But watch out! The following "seems" to work, but the answer is not as intended.  
#- The problem is that the first argument is evaluated before being used.  Since 
#- x exists, it fails
x
Dx2xx <- D(x^2, "x")
Dx2xx
eval(Dx2xx)
#-  Something 'tougher':
trig.exp <- expression(sin(cos(x + y^2)))
( D.sc <- D(trig.exp, "x") )
all.equal(D(trig.exp[[1]], "x"), D.sc)
( dxy <- deriv(trig.exp, c("x", "y")) )
y <- 1
eval(dxy)
eval(D.sc)
#-  function returned:
deriv((y ~ sin(cos(x) * y)), c("x","y"), func = TRUE)
#- ??#-  Surely there is an error, since documentation says no lhs! i.e.,
#- "expr: a 'expression' or 'call' or (except 'D') a formula with no lhs."
#-  function with defaulted arguments:
(fx <- deriv(y ~ b0 + b1 * 2^(-x/th), c("b0", "b1", "th"),
             function(b0, b1, th, x = 1:7){} ) )
fx(2, 3, 4)
#-  First derivative
D(expression(x^2), "x")
#-  stopifnot(D(as.name("x"), "x") == 1) #- A way of testing. 
#- This works by coercing "x" to name/symbol, and derivative should be 1.
#- Would fail only if "x" cannot be so coerced. How could this happen??
#-  Higher derivatives showing deriv3
myd3 <- deriv3(y ~ b0 + b1 * 2^(-x/th), c("b0", "b1", "th"),
     c("b0", "b1", "th", "x") )
myd3(2,3,4, x=1:7)
#- check against deriv()
myd3a <- deriv(y ~ b0 + b1 * 2^(-x/th), c("b0", "b1", "th"),
     c("b0", "b1", "th", "x"), hessian=TRUE )
myd3a(2,3,4, x=1:7)
identical(myd3a, myd3) #- Remember to check things!

#-  Higher derivatives:
DD <- function(expr, name, order = 1) {
   if(order < 1) stop("'order' must be >= 1")
   if(order == 1) D(expr, name)
   else DD(D(expr, name), name, order - 1)
}
DD(expression(sin(x^2)), "x", 3)
#-  showing the limits of the internal "simplify()" :
#-  -sin(x^2) * (2 * x) * 2 + ((cos(x^2) * (2 * x) * (2 * x) + sin(x^2) *
#-     2) * (2 * x) + sin(x^2) * (2 * x) * 2)

## ----chunk02------------------------------------------------------------------
require(nlsr)
dx2xn <- nlsDeriv(~ x^2, "x")
dx2xn
mode(dx2xn)
str(dx2xn)
x <- -1:2
eval(dx2xn) # This is evaluated at -1, 0, 1, 2, BUT result is returned directly,
#-  NOT in "gradient" attribute
firstdn <- dx2xn
str(firstdn)
d2x2xn <- nlsDeriv(firstdn, "x")
d2x2xn
d2x2xnF <- nlsDeriv(firstdn, "x", do_substitute=FALSE)
d2x2xnF # in this case we get the same result
d2x2xnT <- nlsDeriv(firstdn, "x", do_substitute=TRUE)
d2x2xnT # 0 ## WATCH OUT

#- ?? We can iterate the derivatives
nlsDeriv(d2x2xn, "x")

nlsDeriv(x^2, "x")# 0
nlsDeriv(x^2, "x", do_substitute=FALSE)# 0
nlsDeriv(x^2, "x", do_substitute=TRUE) # 2 * x
nlsDeriv(~ x^2, "x") # 2 * x
nlsDeriv(~ x^2, "x", do_substitute=FALSE) # 2 * x
nlsDeriv(~ x^2, "x", do_substitute=TRUE) # 2 * x

#- Some things to check and explain??
#?? firstde <- quote(firstd)
#?? firstde
#?? firstde <- bquote(firstd)
#?? firstde
#?? nlsDeriv(firstde, "x")
d2 <- nlsDeriv(2 * x, "x")
str(d2)
d2
#?? firstc <- as.call(firstd)
#?? nlsDeriv(firstc, "x")
#- Build a function from the expression
#?? fdx2xn<-function(x){eval(dx2xn)}
#?? fdx2xn(1)
#?? fdx2xn(3.21)
#?? fdx2xn(1:5)

## ----chunk03------------------------------------------------------------------
codeDeriv(parse(text="b0 + b1 * 2^(-x/th)"), c("b0", "b1", "th")) 
#- Include parameters as arguments
fj.1 <- fnDeriv(parse(text="b0 + b1 * 2^(-x/th)"), c("b0", "b1", "th")) 
head(fj.1)
fj.1(1,2,3,4)
#- Get all parameters from the calling environment
fj.2 <- fnDeriv(parse(text="b0 + b1 * 2^(-x/th)"), c("b0", "b1", "th"),
        args = character())
head(fj.2)
b0 <- 1
b1 <- 2
x <- 3
th <- 4 
fj.2()

#- Just use an expression
fje <- codeDeriv(parse(text="b0 + b1 * 2^(-x/th)"), c("b0", "b1", "th"))
eval(fje)

dx2xnf <- fnDeriv(~ x^2, "x") #- Use tilde
dx2xnf <- fnDeriv(expression(x^2), "x") #- or use expression()
dx2xnf
mode(dx2xnf)
str(dx2xnf)
x <- -1:2
#?? eval(dx2xnf) # This is evaluated at -1, 0, 1, 2, BUT result is returned directly,
#-  NOT in "gradient" attribute
# Note that we cannot (easily) differentiate this again.
# firstd <- dx2xnf
# str(firstd)
# d2x2xnf <- try(nlsDeriv(firstd, "x")) #- this APPEARS to work, but WRONG answer 
# str(d2x2xnf)
# d2x2xnf
# eval(d2x2xnf)

# dx2xnfh <- fnDeriv(expression(x^2), "x", hessian=TRUE) #- Try for second derivatives
# dx2xnfh
# mode(dx2xnfh)
# str(dx2xnfh)
# x <- -1
# eval(dx2xnfh) # This is evaluated at -1, 0, 1, 2, BUT result is returned directly,

## ----chunk04------------------------------------------------------------------
require(Deriv)

f <- function(x) x^2
Deriv(f)
#- Should see
#- function (x)
#- 2 * x
#- Now save the derivative
f1 <- Deriv(f)
f1 #- print it
f2 <- Deriv(f1) #- and take second derivative
f2 #- print it

f <- function(x, y) sin(x) * cos(y)
f_ <- Deriv(f)
f_ #- print it
#- Should see
#- function (x, y)
#- c(x = cos(x) * cos(y), y = -(sin(x) * sin(y)))
f_(3, 4)
#- Should see
#-              x         y
#- [1,] 0.6471023 0.1068000

f2 <- Deriv(~ f(x, y^2), "y") 
#- This has a tilde to render the 1st argument as a formula object
#- Also we are substituting in y^2 for y
f2 #- print it
#- -(2 * (y * sin(x) * sin(y^2)))
mode(f2) #- check what type of object it is
arg1 <- ~ f(x,y^2)
mode(arg1) #- check the type
f2a <- Deriv(arg1, "y")
f2a #- and print to see if same as before
#- try evaluation of f using current x and y
x
y
f(x,y^2)
eval(f2a) #- We need x and y defined to do this.

f3 <- Deriv(quote(f(x, y^2)), c("x", "y"), cache.exp=FALSE) #- check cache.exp operation
#- Note that we need to quote or will get evaluation at current x, y values (if they exist)
f3 #- print it
#- c(x = cos(x) * cos(y^2), y = -(2 * (y * sin(x) * sin(y^2))))
f3c <- Deriv(quote(f(x, y^2)), c("x", "y"), cache.exp=TRUE) #- check cache.exp operation
f3c #- print it
#- Now want to evaluate the results
#- First must provide some data
x <- 3
y <- 4
eval(f3c)
#- Should see
#- x         y 
#- 0.9480757 0.3250313 
eval(f3) #- check this also
#- or we can create functions
f3cf <- function(x, y){eval(f3c)}
f3cf(x=1, y=2)
#-          x          y 
#- -0.3531652  2.5473094 
f3f <- function(x,y){eval(f3)}
f3f(x=3, y=4)
#-         x         y 
#- 0.9480757 0.3250313 

#- try an expression
Deriv(expression(sin(x^2) * y), "x")
#- should see
#- expression(2 * (x * y * cos(x^2)))

#- quoted string
Deriv("sin(x^2) * y", "x") # differentiate only by x
#- Should see
#- "2 * (x * y * cos(x^2))"

Deriv("sin(x^2) * y", cache.exp=FALSE) #- differentiate by all variables (here by x and y)
#- Note that default is to differentiate by all variables.
#- Should see
#- "c(x = 2 * (x * y * cos(x^2)), y = sin(x^2))"

#- Compound function example (here abs(x) smoothed near 0)
#- Note that this introduces the possibilty of `if` statements in the code
#- BUT (JN) seems to give back quoted string, so we must parse.
fc <- function(x, h=0.1) if (abs(x) < h) 0.5*h*(x/h)**2 else abs(x)-0.5*h
efc1 <- Deriv("fc(x)", "x", cache.exp=FALSE) 
#- "if (abs(x) < h) x/h else sign(x)"
#- A few checks on the results
efc1

fc1 <- function(x,h=0.1){ eval(parse(text=efc1)) }
fc1
## h=0.1
fc1(1)
fc1(0.001)
fc1(-0.001)
fc1(-10)
fc1(0.001, 1)

#- Example of a first argument that cannot be evaluated in the current environment:
  try(suppressWarnings(rm("xx", "yy"))) #- Make sure there are no objects xx or yy
  Deriv(~ xx^2+yy^2)
#- Should show
#- c(xx = 2 * xx, yy = 2 * yy)
#- ?? What is the meaning / purpose of this construct?
  
#- ?? Is following really AD?  
#- Automatic differentiation (AD), note intermediate variable 'd' assignment
Deriv(~{d <- ((x-m)/s)^2; exp(-0.5*d)}, "x")
# Note that the result we see does NOT match what follows in the example(Deriv) (JN ??)
#{
#   d <- ((x - m)/s)^2
#   .d_x <- 2 * ((x - m)/s^2)
#   -(0.5 * (.d_x * exp(-(0.5 * d))))
#}
#- For some reason the intermediate variable d is NOT included.??


#- Custom derivative rule. Note that this needs explanations??
  myfun <- function(x, y=TRUE) NULL #- do something useful
  dmyfun <- function(x, y=TRUE) NULL #- myfun derivative by x.
  drule[["myfun"]] <- alist(x=dmyfun(x, y), y=NULL) #- y is just a logical
  Deriv(myfun(z^2, FALSE), "z")
  # 2 * (z * dmyfun(z^2, FALSE))

#- Differentiation by list components
  theta <- list(m=0.1, sd=2.) #- Why do we set values??
  x <- names(theta) #- and why these particular names??
  names(x)=rep("theta", length(theta))
  Deriv(~exp(-(x-theta$m)**2/(2*theta$sd)), x, cache.exp=FALSE)
#- Should show the following (but why??)
#- c(theta_m = exp(-((x - theta$m)^2/(2 * theta$sd))) *
#-  (x - theta$m)/theta$sd, theta_sd = 2 * (exp(-((x - theta$m)^2/
#-  (2 * theta$sd))) * (x - theta$m)^2/(2 * theta$sd)^2))
lderiv <-  Deriv(~exp(-(x-theta$m)**2/(2*theta$sd)), x, cache.exp=FALSE)
fld <- function(x){ eval(lderiv)} #- put this in a function
fld(2) #- and evaluate at a value

## ----chunk05------------------------------------------------------------------
library(Deriv)
rm(x) # ensures x is undefined
Deriv(~ x, "x")  # returns [1] 1 -- clearly a bug!
Deriv(~ x^2, "x")   # returns 2 * x
x <- quote(x^2)
Deriv(x, "x") # returns 2 * x

## ----chunk06------------------------------------------------------------------
rm(x) # in case it is defined
library(nlsr)
try(nlsDeriv(x, "x")  ) # fails, not a formula
try(nlsDeriv(as.expression("x"), "x")  ) # expression(NULL)
try(nlsDeriv(~x, "x")  ) # 1
try(nlsDeriv(x^2, "x"))   # fails
try(nlsDeriv(~x^2, "x")) # 2 * x
x <- quote(x^2)
try(nlsDeriv(x, "x")) # returns 2 * x

## ----chunk07, eval = require(Ryacas)------------------------------------------
require(nlsr)
dnlsr <- nlsr::nlsDeriv(~ sin(x+y), "x")
print(dnlsr)
class(dnlsr)
detach("package:nlsr", unload=TRUE)
detach("package:Deriv", unload=TRUE)

## New Ryacas mechanism as of 2019-8-29 from mikl@math.aau.dk (Mikkel Meyer Andersen)

yac_str("D(x) Sin(x+y)") 

# or if an expression is needed:
ex <- yac_expr("D(x) Sin(x+y)")
ex
expression(cos(x + y))
eval(ex, list(x = pi, y = pi/2))

## Previous syntax for Ryacas was
## x <- Sym("x")
## y <- Sym("y")
## dryacas <- deriv(sin(x+y), x)
## print(dryacas)
## class(dryacas)

detach("package:Ryacas", unload=TRUE)

## ----chunk08------------------------------------------------------------------
ls(nlsr::sysDerivs)

## ----chunk09------------------------------------------------------------------
require(nlsr)
## Try different ways to supply the log function
aDeriv <- nlsDeriv(~ log(x), "x")
class(aDeriv)
aDeriv
aderiv <- try(deriv( ~ log(x), "x"))
class(aderiv)
aderiv
aD <- D(expression(log(x)), "x")
class(aD)
aD
cat("but \n")
try(D( "~ log(x)", "x")) # fails -- gives NA rather than expected answer due to quotes
try(D( ~ log(x), "x"))
interm <- ~ log(x)
interm
class(interm)
interme <- as.expression(interm)
class(interme)
try(D(interme, "x"))
try(deriv(interme, "x"))
try(deriv(interm, "x"))


nlsDeriv(~ log(x, base=3), "x" ) # OK
try(D(expression(log(x, base=3)), "x" )) # fails - only single-argument calls supported
try(deriv(~ log(x, base=3), "x" )) # fails - only single-argument calls supported
try(deriv(expression(log(x, base=3)), "x" )) # fails - only single-argument calls supported
try(deriv3(expression(log(x, base=3)), "x" )) # fails - only single-argument calls supported
fnDeriv(quote(log(x, base=3)), "x" )

nlsDeriv(~ exp(x), "x")
D(expression(exp(x)), "x") # OK
deriv(~exp(x), "x") # OK, but much more complicated
fnDeriv(quote(exp(x)), "x")

nlsDeriv(~ sin(x), "x")
D(expression(sin(x)), "x")
deriv(~sin(x), "x")
fnDeriv(quote(sin(x)), "x")

nlsDeriv(~ cos(x), "x")
D(expression(cos(x)), "x")
deriv(~ cos(x), "x")
fnDeriv(quote(cos(x)), "x")

nlsDeriv(~ tan(x), "x")
D(expression(tan(x)), "x")
deriv(~ tan(x), "x")
fnDeriv(quote(tan(x)), "x")

nlsDeriv(~ sinh(x), "x")
D(expression(sinh(x)), "x")
deriv(~sinh(x), "x")
fnDeriv(quote(sinh(x)), "x")

nlsDeriv(~ cosh(x), "x")
D(expression(cosh(x)), "x")
deriv(~cosh(x), "x")
fnDeriv(quote(cosh(x)), "x")

nlsDeriv(~ sqrt(x), "x")
D(expression(sqrt(x)), "x")
deriv(~sqrt(x), "x")
fnDeriv(quote(sqrt(x)), "x")

nlsDeriv(~ pnorm(q), "q")
D(expression(pnorm(q)), "q")
deriv(~pnorm(q), "q")
fnDeriv(quote(pnorm(q)), "q")

nlsDeriv(~ dnorm(x, mean), "mean")
D(expression(dnorm(x, mean)), "mean")
deriv(~dnorm(x, mean), "mean")
fnDeriv(quote(dnorm(x, mean)), "mean")

nlsDeriv(~ asin(x), "x")
D(expression(asin(x)), "x")
deriv(~asin(x), "x")
fnDeriv(quote(asin(x)), "x")

nlsDeriv(~ acos(x), "x")
D(expression(acos(x)), "x")
deriv(~acos(x), "x")
fnDeriv(quote(acos(x)), "x")

nlsDeriv(~ atan(x), "x")
D(expression(atan(x)), "x")
deriv(~atan(x), "x")
fnDeriv(quote(atan(x)), "x")

nlsDeriv(~ gamma(x), "x")
D(expression(gamma(x)), "x")
deriv(~gamma(x), "x")
fnDeriv(quote(gamma(x)), "x")

nlsDeriv(~ lgamma(x), "x")
D(expression(lgamma(x)), "x")
deriv(~lgamma(x), "x")
fnDeriv(quote(lgamma(x)), "x")

nlsDeriv(~ digamma(x), "x")
D(expression(digamma(x)), "x")
deriv(~digamma(x), "x")
fnDeriv(quote(digamma(x)), "x")

nlsDeriv(~ trigamma(x), "x")
D(expression(trigamma(x)), "x")
deriv(~trigamma(x), "x")
fnDeriv(quote(trigamma(x)), "x")

nlsDeriv(~ psigamma(x, deriv = 5), "x")
D(expression(psigamma(x, deriv = 5)), "x")
deriv(~psigamma(x, deriv = 5), "x")
fnDeriv(quote(psigamma(x, deriv = 5)), "x")

nlsDeriv(~ x*y, "x")
D(expression(x*y), "x")
deriv(~x*y, "x")
fnDeriv(quote(x*y), "x")

nlsDeriv(~ x/y, "x")
D(expression(x/y), "x")
deriv(~x/y, "x")
fnDeriv(quote(x/y), "x")

nlsDeriv(~ x^y, "x")
D(expression(x^y), "x")
deriv(~x^y, "x")
fnDeriv(quote(x^y), "x")

nlsDeriv(~ (x), "x")
D(expression((x)), "x")
deriv(~(x), "x")
fnDeriv(quote((x)), "x")

nlsDeriv(~ +x, "x")
D(expression(+x), "x")
deriv(~ +x, "x")
fnDeriv(quote(+x), "x")

nlsDeriv(~ -x, "x")
D(expression(- x), "x")
deriv(~ -x, "x")
fnDeriv(quote(-x), "x")

nlsDeriv(~ abs(x), "x")
try(D(expression(abs(x)), "x")) # 'abs' not in derivatives table
try(deriv(~ abs(x), "x"))
fnDeriv(quote(abs(x)), "x")

nlsDeriv(~ sign(x), "x")
try(D(expression(sign(x)), "x")) # 'sign' not in derivatives table
try(deriv(~ sign(x), "x"))
fnDeriv(quote(sign(x)), "x")

## ----chunk10------------------------------------------------------------------
ls(nlsr::sysSimplifications)

## ----chunk11------------------------------------------------------------------
#- Remove ##? to see reproducible error
#- ?? For some reason, if we leave packages attached, we get errors.
#- Here we detach all the non-base packages and then reload nlsr
##? require(nlsr)
##? sessionInfo()
##? ##? nlsSimplify(quote(+(a+b)))
##? nlsSimplify(quote(-5))

## ----chunk12------------------------------------------------------------------
sessionInfo()
if ("Deriv" %in% loadedNamespaces()){detach("package:Deriv", unload=TRUE)} 
if ("nlsr" %in% loadedNamespaces() ){detach("package:nlsr", unload=TRUE)}
if ("Ryacas" %in% loadedNamespaces() ){detach("package:Ryacas", unload=TRUE)}
require(nlsr)
#- require(Deriv)
#- require(stats)
#- Various simplifications
#- ?? Do we need quote() to stop attempt to evaluate before applying simplification

nlsSimplify(quote(+(a+b)))
nlsSimplify(quote(-5))
nlsSimplify(quote(--(a+b)))

nlsSimplify(quote(exp(log(a+b))))
nlsSimplify(quote(exp(1)))

nlsSimplify(quote(log(exp(a+b))))
nlsSimplify(quote(log(1)))

nlsSimplify(quote(!TRUE))
nlsSimplify(quote(!FALSE))

nlsSimplify(quote((a+b)))

nlsSimplify(quote(a + b + 0))
nlsSimplify(quote(0 + a + b))
nlsSimplify(quote((a+b) + (a+b)))
nlsSimplify(quote(1 + 4))

nlsSimplify(quote(a + b - 0))
nlsSimplify(quote(0 - a - b))
nlsSimplify(quote((a+b) - (a+b)))
nlsSimplify(quote(5 - 3))

nlsSimplify(quote(0*(a+b)))
nlsSimplify(quote((a+b)*0))
nlsSimplify(quote(1L * (a+b)))
nlsSimplify(quote((a+b) * 1))
nlsSimplify(quote((-1)*(a+b)))
nlsSimplify(quote((a+b)*(-1)))
nlsSimplify(quote(2*5))

nlsSimplify(quote((a+b) / 1))
nlsSimplify(quote((a+b) / (-1)))
nlsSimplify(quote(0/(a+b)))
nlsSimplify(quote(1/3))

nlsSimplify(quote((a+b) ^ 1))
nlsSimplify(quote(2^10))

nlsSimplify(quote(log(exp(a), 3)))

nlsSimplify(quote(FALSE && b))
nlsSimplify(quote(a && TRUE))
nlsSimplify(quote(TRUE && b))

nlsSimplify(quote(a || TRUE))
nlsSimplify(quote(FALSE || b))
nlsSimplify(quote(a || FALSE))

nlsSimplify(quote(if (TRUE) a+b))
nlsSimplify(quote(if (FALSE) a+b))

nlsSimplify(quote(if (TRUE) a+b else a*b))
nlsSimplify(quote(if (FALSE) a+b else a*b))
nlsSimplify(quote(if (cond) a+b else a+b))

nlsSimplify(quote(--(a+b)))
nlsSimplify(quote(-(-(a+b))))

## ----chunk13------------------------------------------------------------------
#- For some reason, if we leave packages attached, we get errors.
#- Here we detach all the non-base packages and then reload nlsr
sessionInfo()
if ("Deriv" %in% loadedNamespaces()){detach("package:Deriv", unload=TRUE)} 
if ("Deriv" %in% loadedNamespaces() ){detach("package:nlsr", unload=TRUE)}
if ("Deriv" %in% loadedNamespaces() ){detach("package:Ryacas", unload=TRUE)}
require(Deriv)
#- Various simplifications
#- ?? Do we need quote() to stop attempt to evaluate before applying simplification?

Simplify(quote(+(a+b)))
Simplify(quote(-5))
Simplify(quote(--(a+b)))

Simplify(quote(exp(log(a+b))))
Simplify(quote(exp(1)))

Simplify(quote(log(exp(a+b))))
Simplify(quote(log(1)))

Simplify(quote(!TRUE))
Simplify(quote(!FALSE))

Simplify(quote((a+b)))

Simplify(quote(a + b + 0))
Simplify(quote(0 + a + b))
Simplify(quote((a+b) + (a+b)))
Simplify(quote(1 + 4))

Simplify(quote(a + b - 0))
Simplify(quote(0 - a - b))
Simplify(quote((a+b) - (a+b)))
Simplify(quote(5 - 3))

Simplify(quote(0*(a+b)))
Simplify(quote((a+b)*0))
Simplify(quote(1L * (a+b)))
Simplify(quote((a+b) * 1))
Simplify(quote((-1)*(a+b)))
Simplify(quote((a+b)*(-1)))
Simplify(quote(2*5))

Simplify(quote((a+b) / 1))
Simplify(quote((a+b) / (-1)))
Simplify(quote(0/(a+b)))
Simplify(quote(1/3))

Simplify(quote((a+b) ^ 1))
Simplify(quote(2^10))

Simplify(quote(log(exp(a), 3)))

Simplify(quote(FALSE && b))
Simplify(quote(a && TRUE))
Simplify(quote(TRUE && b))

Simplify(quote(a || TRUE))
Simplify(quote(FALSE || b))
Simplify(quote(a || FALSE))

Simplify(quote(if (TRUE) a+b))
Simplify(quote(if (FALSE) a+b))

Simplify(quote(if (TRUE) a+b else a*b))
Simplify(quote(if (FALSE) a+b else a*b))
Simplify(quote(if (cond) a+b else a+b))

#- This one is wrong... the double minus is an error, yet it works ??.
Simplify(quote(--(a+b)))
#- By comparison
Simplify(quote(-(-(a+b))))

## ----chunk14------------------------------------------------------------------
require(nlsr)
dlogx <- nlsr::nlsDeriv(~ log(x), "x")
str(dlogx)
print(dlogx)

## ----chunk15------------------------------------------------------------------
require(nlsr)
dlogxs <- nlsr::nlsDeriv(expression(log(x)), "x", do_substitute=FALSE)
str(dlogxs)
print(dlogxs)
cat(as.character(dlogxs), "\n")
fne <- expression(log(x))
dlogxe <- nlsr::nlsDeriv(fne, "x", do_substitute=FALSE)
str(dlogxe)
print(dlogxe)


# base R
dblogx <- D(expression(log(x)), "x")
str(dblogx)
print(dblogx)

require(Deriv)
ddlogx <- Deriv::Deriv(expression(log(x)), "x")
str(ddlogx)
print(ddlogx)
cat(as.character(ddlogx), "\n")
ddlogxf <- ~ ddlogx
str(ddlogxf)

## ----chunk16------------------------------------------------------------------
require(nlsr)
# require(stats)
# require(Deriv)
# require(Ryacas)

# Various derivatives 

new <- codeDeriv(quote(1 + x + y), c("x", "y"))
old <- deriv(quote(1 + x + y), c("x", "y"))
print(new)
# Following generates a very long line on output of knitr (for markdown)
class(new)
str(new)
as.expression(new)
newf <- function(x, y){
   eval(new)
}
newf(3,5)

print(old)
class(old)
str(old)
oldf <- function(x,y){
    eval(old)
}
oldf(3,5)



## ----chunk17------------------------------------------------------------------
fnfromnew <- function(x,y){
    .value <- 1 + x + y
    .grad <- array(0, c(length(.value), 2L), list(NULL, c("x", 
    "y")))
    .grad[, "x"] <- 1
    .grad[, "y"] <- 1
    attr(.value, "gradient") <- .grad
    .value
}

print(fnfromnew(3,5))


## ----chunk18------------------------------------------------------------------
x <- NA
y <- Inf
print(eval(new))
print(eval(old))

## ----chunk19------------------------------------------------------------------
safeD <- function(obj, var) {
   # safeguarded D() function for symbolic derivs
   if (! is.character(var) ) stop("The variable var MUST be character type")
   if (is.character(obj) ) {
       eobj <- parse(text=obj)
       result <- D(eobj, var)
   } else {
       result <- D(obj, var)
   }
}

lxy2 <- expression(log(x+y^2))
clxy2 <- "log(x+y^2)"
try(print(D(clxy2, "y")))
print(try(D(lxy2, "y")))
print(safeD(clxy2, "y"))
print(safeD(lxy2, "y"))

## ----chunk20------------------------------------------------------------------
zzz <- expression(y[3]*r1 + r2)
try(deriv(zzz,c("r1","r2")))
require(nlsr)
try(nlsr::nlsDeriv(zzz, c("r1","r2")))
try(fnDeriv(zzz, c("r1","r2")))
newDeriv(`[`(x,y), stop("no derivative when indexing"))
try(nlsr::nlsDeriv(zzz, c("r1","r2")))
try(nlsr::fnDeriv(zzz, c("r1","r2")))

## ----chunk21------------------------------------------------------------------
try(nlsr::nlsDeriv(zzz, "y[3]"))
try(nlsr::nlsDeriv(y3*r1+r2,"y3"))
try(nlsr::nlsDeriv(y[3]*r1+r2,"y[3]"))

## Problem in 1 parameter to ensure methods work in trivial case
require(nlsr)
cat("1 parameter problem\n")
nobs<-8
tt <- seq(1,nobs)
dd <- 1.23*tt + 4*runif(nobs)

df <- data.frame(tt, dd)
print(df)
a1par<-nlxb(dd ~ a*tt, start=c(a=1), data=df)
a1par

# BellX.R
library(nlraa)
require(ggplot2)
set.seed(1234)
x <- 1:20
y <- bell(x, 8, -0.0314, 0.000317, 13) + rnorm(length(x), 0, 0.5)
dat <- data.frame(x = x, y = y)
fit <- nls(y ~ SSbell(x, ymax, a, b, xc), data = dat)
fit
gfrm<- y ~ ymax * exp(a *(x - xc)^2)
gfrmx<- y ~ ymax * exp(a *(x - xc)^2 + b*(x - xc)^3)
stgauss<-c(ymax=max(y), a=-0.5/(sd(x)^2), xc=mean(x))
cat("stgauss:"); print(stgauss)
st2<-c(ymax=8, a= 0.03, xc= 13)
st3<-c(ymax=8, a= 0.03, b=0, xc= 13)
fit2 <- nls(gfrm, start=st2, data=dat, trace=TRUE)
summary(fit2)
# Use the selfStart model
xfrm <- y ~ SSbell(x, ymax, a, b, xc)
library(nlsr)
fx2mx <- nlxb(xfrm, start=st3, data=dat, lower=c(0, -1e5, 0, 0),
             upper=c(1e5, 1e5, 0, 1e5), trace=TRUE, control=list(japprox="jacentral"))
fx2mx
# Or the formula for the same model with Jacobian approximation
fx2mgc <- nlxb(gfrmx, start=st3, data=dat, lower=c(0, -1e5, 0, 0),
              upper=c(1e5, 1e5, 0, 1e5), trace=TRUE, control=list(japprox="jacentral"))
fx2mgc
# Or the formula and analytic derivatives
fx2mg <- nlxb(gfrmx, start=st3, data=dat, lower=c(0, -1e5, 0, 0),
               upper=c(1e5, 1e5, 0, 1e5), trace=TRUE)
fx2mg
# Display results together
pshort(fx2mx)
pshort(fx2mgc)
pshort(fx2mg)

library(nlsr)

# Various derivatives 

fnDeriv(quote(1 + x + y), c("x", "y"))

nlsDeriv( ~ log(x), "x")
nlsDeriv( ~ log(x, base=3), "x" )

nlsDeriv( ~ exp(x), "x")
nlsDeriv( ~ sin(x), "x")
nlsDeriv( ~ cos(x), "x")
nlsDeriv( ~ tan(x), "x")
nlsDeriv( ~ sinh(x), "x")
nlsDeriv( ~ cosh(x), "x")
nlsDeriv( ~ sqrt(x), "x")
nlsDeriv( ~ pnorm(q), "q")
nlsDeriv( ~ dnorm(x, mean), "mean")
nlsDeriv( ~ asin(x), "x")
nlsDeriv( ~ acos(x), "x")
nlsDeriv( ~ atan(x), "x")
nlsDeriv( ~ gamma(x), "x")
nlsDeriv( ~ lgamma(x), "x")
nlsDeriv( ~ digamma(x), "x")
nlsDeriv( ~ trigamma(x), "x")
nlsDeriv( ~ psigamma(x, deriv = 5), "x")
nlsDeriv( ~ x*y, "x")
nlsDeriv( ~ x/y, "x")
nlsDeriv( ~ x^y, "x")
nlsDeriv( ~ (x), "x")
nlsDeriv( ~ +x, "x")
nlsDeriv( ~ -x, "x")
nlsDeriv( ~ abs(x), "x")
nlsDeriv( ~ sign(x), "x")

# Various simplifications

nlsSimplify(quote(+(a+b)))
nlsSimplify(quote(-5))
nlsSimplify(quote(--(a+b)))

nlsSimplify(quote(exp(log(a+b))))
nlsSimplify(quote(exp(1)))

nlsSimplify(quote(log(exp(a+b))))
nlsSimplify(quote(log(1)))

nlsSimplify(quote(!TRUE))
nlsSimplify(quote(!FALSE))

nlsSimplify(quote((a+b)))

nlsSimplify(quote(a + b + 0))
nlsSimplify(quote(0 + a + b))
nlsSimplify(quote((a+b) + (a+b)))
nlsSimplify(quote(1 + 4))

nlsSimplify(quote(a + b - 0))
nlsSimplify(quote(0 - a - b))
nlsSimplify(quote((a+b) - (a+b)))
nlsSimplify(quote(5 - 3))

nlsSimplify(quote(0*(a+b)))
nlsSimplify(quote((a+b)*0))
nlsSimplify(quote(1L * (a+b)))
nlsSimplify(quote((a+b) * 1))
nlsSimplify(quote((-1)*(a+b)))
nlsSimplify(quote((a+b)*(-1)))
nlsSimplify(quote(2*5))

nlsSimplify(quote((a+b) / 1))
nlsSimplify(quote((a+b) / (-1)))
nlsSimplify(quote(0/(a+b)))
nlsSimplify(quote(1/3))

nlsSimplify(quote((a+b) ^ 1))
nlsSimplify(quote(2^10))

nlsSimplify(quote(log(exp(a), 3)))

nlsSimplify(quote(FALSE && b))
nlsSimplify(quote(a && TRUE))
nlsSimplify(quote(TRUE && b))

nlsSimplify(quote(a || TRUE))
nlsSimplify(quote(FALSE || b))
nlsSimplify(quote(a || FALSE))

nlsSimplify(quote(if (TRUE) a+b))
nlsSimplify(quote(if (FALSE) a+b))

nlsSimplify(quote(if (TRUE) a+b else a*b))
nlsSimplify(quote(if (FALSE) a+b else a*b))
nlsSimplify(quote(if (cond) a+b else a+b))

# This one was wrong...
nlsSimplify(quote(--(a+b)))


# From the Weeds problem

  modelformula <- y ~ ms * b1/(1 + b2 * exp(-b3 * tt))
  pvec <- c(b1=1, b2=1, b3=1)
  cat("model2rjfunx: modelformula = ")
  print(modelformula)
  print(class(modelformula))

  if (length(modelformula) == 2) {
       residexpr <- modelformula[[2]]
  } else if (length(modelformula) == 3) {
       residexpr <- call("-", modelformula[[3]], modelformula[[2]])
  } else stop("Unrecognized formula")
    
  if (is.null(names(pvec)))
    names(pvec) <- paste0("p", seq_along(pvec))
    
  residexpr1 <- nlsDeriv( ~ residexpr, names(pvec))
  cat("residexpr1:\n")
  print(residexpr1) 
## SHOULD TRY:
  residexpr2 <- fnDeriv(residexpr, names(pvec))
  cat("residexpr2:\n")
  print(residexpr2)

# rm(list=ls())
# require(nlsr)

traceval  <-  TRUE  # traceval set TRUE to debug or give full history

# Data for Hobbs problem
ydat  <-  c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443, 
          38.558, 50.156, 62.948, 75.995, 91.972) # for testing
tdat  <-  seq_along(ydat) # for testing

# A simple starting vector -- must have named parameters for nlxb, nls, wrapnlsr.

eunsc  <-   y ~ b1/(1+b2*exp(-b3*tt))

cat("LOCAL DATA IN DATA FRAMES\n")
weeddata1  <-  data.frame(y=ydat, tt=tdat)
startf1  <-  c(b1=1, b2=1, b3=.1)
start1  <-  c(b1=1, b2=1, b3=1)

## With BOUNDS


anlxb1  <-  try(nlxb(eunsc, start=startf1, lower=c(b1=0, b2=0, b3=0), 
      upper=c(b1=500, b2=100, b3=5), trace=traceval, data=weeddata1))

print(anlxb1)

## anlxb1  <-  try(nlxb(eunsc, start=startf1, lower=c(b1=0, b2=0, b3=0), upper=c(b1=500, b2=40, b3=5), trace=traceval, data=weeddata1))

# Check nls too

anlsb1  <-  try(nls(eunsc, start=start1, lower=c(b1=0, b2=0, b3=0), 
     upper=c(b1=500, b2=100, b3=5), trace=traceval, data=weeddata1, 
             algorithm='port'))
print(anlsb1)


anlxb2  <-  try(nlxb(eunsc, start=start1, lower=c(b1=0, b2=0, b3=0), 
        upper=c(b1=500, b2=100, b3=.25), trace=traceval, data=weeddata1))
print(anlxb2)


anlsb2  <-  try(nls(eunsc, start=start1, lower=c(b1=0, b2=0, b3=0), 
                upper=c(b1=500, b2=100, b3=.25), trace=traceval, 
                data=weeddata1, algorithm='port'))
print(anlsb2)

nlfb <-function(start, resfn, jacfn = NULL, trace = FALSE, 
		lower = -Inf, upper = Inf, weights=NULL,
		data=NULL, ctrlcopy=FALSE, control=list(), ...){
  
getweights <- function() { # local function
    if (is.null(weights0))
      rep(1, length(resraw))
    else if (is.function(weights0))
      weights0(pnum, resraw)
    else
      weights0
} 
  weights0 <- weights # save the original, possibly a function
  
  if (ctrlcopy) ctrl<-control else ctrl <- nlsr.control(control) 
  if (trace && ctrl$prtlvl>0) cat("psi=",ctrl$psi,"  phi=",ctrl$phi,"\n")
  # Function to display SS and point
  roff <- NA # initially undefined
  showprms<-function(SS, roff, pnum){
    pnames<-names(pnum)
#    npar<-length(pnum) # Will already be set before showprms called
    cat("lamda:",ctrl$lamda," SS=",SS,"(",roff,") at")
    for (i in 1:npar){
       cat(" ",pnames[i],"=",pnum[i])
    }
    cat(" f/j ",feval,"/",jeval)
    cat("\n")
  } # end showprms

  if (trace && ctrl$prtlvl>1) {
    if (is.null(weights)) {
       cat("no weights\n") 
    } else {
      cat("weights:")
      print(weights)
    }
  }
  # ensure params in vector
  if (missing(start)) stop("MUST provide starting parameters")
  pnames<-names(start)
  npar<-length(start)
  if (is.null(pnames)) {
    for (j in 1:npar) {  pnames[[j]]<- paste("p",j,sep='')}
  } 
  if (trace && ctrl$prtlvl>2) {cat("pnames:"); print(pnames)}
  pnum<-start # may simplify later
  names(pnum)<-pnames
  start<-as.numeric(start)
  names(start)<-pnames # needed?
  # bounds
  npar<-length(start) # number of parameters
  resraw <- resfn(pnum, ...)
  feval<-1 # number of function evaluations
  jeval<-0 # number of Jacobian evaluations
  if (length(lower)==1) lower<-rep(lower,npar)
  if (length(upper)==1) upper<-rep(upper,npar) 
  if (length(lower)!=npar) stop("Wrong length: lower")
  if (length(upper)!=npar) stop("Wrong length: upper")
  if (any(start < lower) || any(start > upper)) {
     if(trace) {  cat("start:"); print(start)
                     cat("\n");cat("lower:"); print(lower)
                     cat("\n"); cat("upper:"); print(upper) 
                  }
     stop("Infeasible start")
  }
  if (any(start<lower) || any(start>upper)) stop("Infeasible start")
  maskidx <- which(lower == upper) # MAY want to add tolerance?
  if (trace && ctrl$prtlvl>2) {cat("lower:"); print(lower); cat("upper:"); print(upper)}
  epstol<-(.Machine$double.eps)*ctrl$offset
  #   cat("ctrl$phi=",ctrl$phi,"\n")
  if (ctrl$phi == 0.0) phiroot<-0.0 else phiroot<-sqrt(ctrl$phi)
  if (ctrl$psi == 0.0) psiroot<-0.0 else psiroot<-sqrt(ctrl$psi)
  ctrl$stepredn <- ctrl$stepredn
  nbtlim <- ctrl$nbtlim 
  if (ctrl$stepredn <= 0) {
     nbtlim <- 1 # only 1 step if not backtracking
     if (trace) cat("No backtrack\n")
  } else {
     if (trace) cat("Bactrack with step reduction=",ctrl$stepredn,"\n")
  }
  bdmsk<-rep(1,npar) # set all params free for now
  if (length(maskidx)>0 && trace) {
       cat("The following parameters are masked:")
       print(pnames[maskidx]) # Change 140716
  }
  bdmsk[maskidx]<-0 # fixed parameters
  # Jacobian computations and approximations
  if (trace && ctrl$prtlvl>2) {cat("jacfn=",as.character(substitute(jacfn)),"\n") }
  appjac<-NULL # start NULL and test at end
  if (missing(jacfn) || is.null(jacfn)){
    if(is.null(ctrl$japprox)) stop("MUST PROVIDE jacobian function or specify approximation")
    if (trace && ctrl$prtlvl>0) cat("Using default jacobian approximation ",ctrl$japprox,"\n")
    if (trace) cat("ctrl$japprox=",ctrl$japprox,"\n")
    myjac<-match.fun(ctrl$japprox)
    appjac<-TRUE
  } else { 
## Need to see if jacfn has quotes and act appropriately
    if (is.character(jacfn)) { # have quoted jacobian function -- an approximation
      appjac <- TRUE # to inform us that we are using approximation
      myjac <- match.fun(jacfn)
      if (trace && ctrl$prtlvl>2) {cat("myjac="); print(myjac)}
    } else { # non-null, non-quoted jacfn
      if (trace && ctrl$prtlvl>2) {cat("We are using specified jacfn =",as.character(substitute(jacfn)),"\n")} 
      appjac<-FALSE # probably NOT needed
      myjac<-jacfn
    }
  }
  if (is.null(appjac)) stop("Failed to define jacfn -- appjac is null")
  if (trace && ctrl$prtlvl>1) {cat("Starting pnum="); print(pnum)}
  weights <- getweights()
  swts <- sqrt(weights)
  resbest<-resfn(pnum, ...)*swts
  ssbest<-as.numeric(crossprod(resbest))
  if (trace) { cat("00"); showprms(ssbest, roff, pnum) }
  ssminval <- ssbest*epstol^4
#  if (ctrl$watch) cat("ssminval =",ssminval,"\n")
  pbest<-pnum
  if (trace && ctrl$prtlvl>1) { 
    cat("Start:"); showprms(ssbest,roff,pnum);
    # if (ctrl$watch) tmp<-readline("Continue")
  }
  if (length(maskidx) == npar) {
    warning("All parameters are masked")
    result <- list(resid = resbest, jacobian = NA, feval = feval, 
            jeval = jeval, coefficients = pnum, ssquares = ssbest, 
            lower=lower, upper=upper, maskidx=maskidx, weights=weights0, 
            formula=NULL) 
    class(result)<-"nlsr"
    return(result)
  }
  ssquares<-.Machine$double.xmax # make it big
  newjac<-TRUE # set control for computing Jacobian
  eqcount<-0
  roffstop <- FALSE
  smallstop <- FALSE
  while ((! roffstop) && (eqcount < npar) && (feval <= ctrl$femax) 
           && (jeval <= ctrl$jemax) && (! smallstop) ) {
    ## if (trace) cat("Inside top main loop -- newjac=",newjac,"\n")
    if (newjac) {
      bdmsk<-rep(1,npar)
      bdmsk[which(pnum-lower<epstol*(abs(lower)+epstol))]<- -3 
      bdmsk[which(upper-pnum<epstol*(abs(upper)+epstol))]<- -1
      bdmsk[maskidx]<-0 # MUST be last -- or don't get masking
      if (ctrl$watch) { cat("bdmsk:"); print(bdmsk) }
      if (appjac) { # use approximation
       Jac<-myjac(pbest, resfn=resfn, bdmsk=bdmsk, 
                resbest=resbest, ndstep=ctrl$ndstep, ...)
      }
      else {Jac <- attr(myjac(pbest, ...),"gradient") }
      if (is.null(Jac)) stop("nlfb: Jac is not defined! (Is attr 'gradient' set?)")
      weights<-getweights()
      swts <- sqrt(weights)
      Jac <- Jac * swts
      resw <- resbest # already wtd
      ## NOTE: by insisting on using the "gradient" attribute, we can use same
      ## fn for gradient and residual
      jeval<-jeval+1 # count Jacobians
      if (any(is.na(Jac))) stop("NaN in Jacobian")
      gjty<-t(Jac)%*%resw # raw gradient
      if (trace && ctrl$prtlvl>2) { cat("gjty:"); print(as.vector(gjty)) }
      for (i in 1:npar){
        bmi<-bdmsk[i]
        if (bmi==0) {
          gjty[i]<-0 # masked
          Jac[,i]<-0
        }
        if (bmi<0) {
          if((2+bmi)*gjty[i] > 0) { # free parameter
            bdmsk[i]<-1
            if (ctrl$watch) cat("freeing parameter ",i," now at ",pnum[i],"\n")
          } 
          else {
            gjty[i]<-0 # active bound
            Jac[,i]<-0
            if (ctrl$watch) cat("active bound ",i," at ",pnum[i],"\n") 
          }
        } # bmi
      } # end for loop
      if (trace && ctrl$prtlvl>2) { cat("gjty-adj:"); print(as.vector(gjty)) }
      if (psiroot > 0.0) {
        if (npar == 1) dee <- diag(as.matrix(sqrt(diag(crossprod(Jac)))))
        else dee <- diag(sqrt(diag(crossprod(Jac))))  # to append to Jacobian
      }
    } # end newjac
    if (ctrl$jemax > 0) { # DO NOT COMPUTE CHANGE IF EVAL ONLY OPTION    
        lamroot<-sqrt(ctrl$lamda)
        JJ <- Jac
        rplus <- resw
        if (psiroot > 0.0) {
          JJ<-rbind(JJ,lamroot*psiroot*dee) # start building the matrix
          rplus <- c(rplus, rep(0, npar)) 
        }
        if (phiroot > 0.0) {
          JJ<-rbind(JJ,lamroot*phiroot*diag(npar)) # start building the matrix
          rplus <- c(rplus, rep(0, npar)) 
        }
        if (trace && ctrl$prtlvl>1) {
             cat("JJ\n"); print(JJ)
        }
        JQR<-qr(JJ)# Should we use try()?
        roff <- max(abs(as.numeric(crossprod(qr.Q(JQR), rplus))))/sqrt(ssbest+ctrl$scaleOffset)
        if (ctrl$watch) cat("roff =", roff,"  converged = ",(roff <= sqrt(epstol)),"\n")
        if (ctrl$rofftest && (roff <= sqrt(epstol))) roffstop <- TRUE
#       if (trace) cat("roffstop now ",roffstop,"\n")
        if (roffstop) break # added 220619
        #        tmp <- readline('cont')
        delta<-try(qr.coef(JQR,-rplus)) # Note appended rows of y)
        if (trace && ctrl$prtlvl>2) { cat("delta:"); print(delta) }
        if (inherits(delta, "try-error")) {
          if (ctrl$lamda<1000*.Machine$double.eps) ctrl$lamda<-1000*.Machine$double.eps
          ctrl$lamda<-ctrl$laminc*ctrl$lamda
          newjac<-FALSE # increasing lamda -- don't re-evaluate
          if (trace) cat(" Equation solve failure\n")
        } else { # solution OK
          gproj<-crossprod(delta,gjty)
          gangle <- gproj/sqrt(crossprod(gjty) * crossprod(delta))
          gangle <- 180 * acos(sign(gangle)*min(1, abs(gangle)))/pi
          if (ctrl$watch) cat("gradient projection = ",gproj," g-delta-angle=",gangle,"\n")
          if (is.na(gproj) || (gproj >= 0) ) { # uphill direction -- should NOT be possible
            if (ctrl$lamda<1000*.Machine$double.eps) ctrl$lamda <- 1000*.Machine$double.eps
            ctrl$lamda <- ctrl$laminc*ctrl$lamda
            newjac<-FALSE # increasing lamda -- don't re-evaluate
            if (trace) cat(" Uphill search direction\n")
            # May have trouble in bounded case!?
          } else { # downhill  
            delta[maskidx]<-0
            delta<-as.numeric(delta)
            if (ctrl$watch) {cat("delta:"); print(delta)}
            step<-rep(1,npar)
            for (i in 1:npar){
              bd<-bdmsk[i]
              da<-delta[i]
              #  if (ctrl$watch) cat(i," bdmsk=",bd,"  delta=",da,"\n")
              if (bd==0 || ((bd==-3) && (da<0)) ||((bd==-1)&& (da>0))) {
                delta[i]<-0
              } else {
                if (delta[i]>0) step[i]<-(upper[i]-pbest[i])/delta[i]
                if (delta[i]<0) step[i]<-(lower[i]-pbest[i])/delta[i] # positive
              }
            } # end loop over parameters/bounds
            stepsize<-min(1,step[which(delta != 0)]) # reduce stepsize if needed
            if (stepsize < .Machine$double.eps) {
               if (ctrl$lamda<1000*.Machine$double.eps) lamda<-1000*.Machine$double.eps
               ctrl$lamda<-ctrl$laminc*ctrl$lamda
               newjac<-FALSE # increasing lamda -- don't re-evaluate
               if (trace) cat(" Step too small or not possible; lamda now ",ctrl$lamda,"\n")
            }
            else { # stepsize not too small
              nbt=0 # number of backtracks (always do 1 evaluation)
              ssquares<-2*ssbest+1.0 # guarantee bigger
              while ((nbt < nbtlim) && (ssquares >= ssbest) ) { # backtrack loop 
                ## don't loop if ctrl$stepredn==0, and only until ssquares < ssbest
                pnum<-pbest+stepsize*delta # adjust step
                eqcount<-length(which((ctrl$offset+pbest)==(ctrl$offset+pnum))) 
                if (eqcount < npar) { # NOT all equal, so some change, computer resfn
                  feval<-feval+1 # count evaluations
                  nbt <- nbt+1
                  resid <- resfn(pnum, ...) * swts
                  ssquares<-sum(resid^2L) # get sum of squares
                  if (is.na(ssquares)) ssquares<-.Machine$double.xmax # Caution!
                  if (trace && ctrl$prtlvl>0)  cat(nbt," stepsize=",stepsize," lamda=",
                                   ctrl$lamda," ssquares=",ssquares,"\n")
                  if (ssquares >= ssbest) { # failed step, decrease stepsize
                     if ( (ctrl$stepredn <= 0.0) || (nbt >= ctrl$nbtlim) ) break # out of backtrack loop     
                     stepsize <- stepsize * ctrl$stepredn 
                  }
                } # end equality check
                else { # no change -- Do we need to fix in bounded case?
                  if (trace && ctrl$prtlvl>0) cat("No change in parameters(1)\n")
                  break # no change (eqcount == npar)
                }
              } # end while ssquares not reduced 
              # at this point we have either smaller ssquares (good), or failed step
              # failed step is either no change (converged) or we increase lambda
              if (ssquares < ssbest) { # smaller sumsquares, SUCCESS. Clean up and new iteration
                ctrl$lamda<-ctrl$lamdec*ctrl$lamda/ctrl$laminc  
                   # reduction via lamdec/laminc e.g., 4/10
                if (trace) { cat("<<"); showprms(ssquares, roff, pnum) }
                ssbest<-ssquares # save "best" so far
                if (ctrl$smallsstest) { smallstop<-(ssbest <= ssminval) } # capture small ss
                resbest<-resid
                pbest<-pnum
                newjac<-TRUE # indicate to compute new gradient
              } # end reduced sumsquares
              else { # Converged or increasing lambda
                if (eqcount < npar) {
                  if (ctrl$lamda<1000*.Machine$double.eps) ctrl$lamda<-1000*.Machine$double.eps
                  ctrl$lamda<-ctrl$laminc*ctrl$lamda
                  newjac<-FALSE # increasing lamda -- don't re-evaluate
                  if(trace && ctrl$prtlvl>2) showprms(ssquares, roff, pnum)
                }
                else {# equcount >= npar (will be ==). No progress.
                   if (trace) cat("No parameter change - stop!\n")
                }
              } # converged or increasing lambda
            } # end else stepsize not too small
          } # downhill
        } # solution OK
        if (ctrl$watch) tmp<-readline("Cycle")
     }
  } # end main while loop 
  pnum<-as.vector(pnum)
  names(pnum) <- pnames
  result <- list(resid = resbest, jacobian = Jac, feval = feval, 
            jeval = jeval, coefficients = pnum, ssquares = ssbest, lower=lower, 
            upper = upper, maskidx = maskidx, weights0 = weights0, 
            weights = weights, formula = NULL) # chg 190805
  class(result) <- "nlsr" ## Needed for print method
  result
} # end nlfb

nlsr.package <- function(){
   cat("nlsr package: a set of tools for solving nonlinear least squares problems\n")
   return(1)
}

nlsr.control <- function(control) {
  # NEED TO PUT IN CHECKS of validity
  # Defaults
  defctrl=list(femax=10000,
              japprox=NULL,
              jemax=5000,
              lamda=0.0001,
	      laminc=10, 
	      lamdec=4, 
          nbtlim=6,
	      ndstep=1e-7,
	      offset=100, 
	      phi=1.0, 
	      psi=0.0,
	      rofftest = TRUE, 
	      smallsstest = TRUE, 
	      stepredn=0.0,
	      watch=FALSE,
              prtlvl=0,
              scaleOffset=1.0)
  if (missing(control) || is.null(control)){control <- defctrl}
  else {
      namctl <- names(control)
      namdef <- names(defctrl)
      if (! all(namctl %in% namdef) ) stop("unrecognized control names")
      for (onename in namctl) {defctrl[onename]<-control[onename]}
      control<-defctrl
  }
  control # return the control list
}

coef.nlsr <- function(object, ...) {
       out <- object$coefficients
       attr(out,"pkgname")<-"nlsr"
       out # JN 170109
}

summary.nlsr <- function(object, ...) {
  smalltol <- .Machine$double.eps * 1000
  options(digits = 5) # 7 is default
  resname <- deparse(substitute(object))
  JJ <- object$jacobian
  res <- object$resid # weighted
  coeff <- object$coefficients
  pnames<-names(coeff)
#  cat("pnames:"); print(pnames)
  npar <- length(coeff)
  w <- object$weights
  nobs <- if (!is.null(w)) sum(w > 0) else length(res)
  lo <- object$lower
  if (is.null(lo)) lo <- rep( -Inf, npar)
  up <- object$upper
  if (is.null(up)) up <- rep( Inf, npar)
  mi <- object$maskidx
  mt <- rep(" ",npar) # start with all "unmasked"
  mt[mi] <- "M" # Put in the masks
  bdmsk <- rep(1, npar) # bounds and masks indicator. Should it be 1L?
  bdmsk[mi] <- 0 # masked
  ct <- rep(" ",npar) # start with all "free"
  for (i in seq_along(coeff)){
    if (lo[[i]] - coeff[[i]] > 0) {
      ct[[i]] <- "-" # lower bound violation
      if (bdmsk[[i]] == 1) bdmsk[[i]] <- -3
    } else { 
      if (coeff[[i]] - lo[[i]] < smalltol*(abs(coeff[[i]])+smalltol) ) {
        ct[[i]] <- "L" # "at" lower bound
        if (bdmsk[[i]] != 0) bdmsk[[i]] <- -3 # leave mask indication intact
      }
    }
    if (coeff[[i]] - up[[i]] > 0) {
      ct[[i]] <- "+" # upper bound violation
      if (bdmsk[[i]] == 1) bdmsk[[i]] <- -1
    } else { 
      if (up[[i]] - coeff[[i]] < smalltol*(abs(coeff[[i]])+smalltol) ) {
        ct[[i]] <- "U" # "at" upper bound
        if (bdmsk[[i]] != 0) bdmsk[[i]] <- -1 # leave mask indication intact
      }
    }
  }
  notmask<-which(bdmsk != 0)
  ss <- object$ssquares
  rdf <- nobs - length(notmask) # New way 220801
  if (rdf <= 0) {
    if (rdf < 0) { stop(paste("Inadmissible degrees of freedom =",rdf,sep='')) }
    else { resvar <- Inf }
  } else {
    resvar <- ss/rdf
  }
  nanc<-rep(NA,npar)
  Sdout <- SEs <- nanc # Set so it is defined and right length. Would Inf be preferred?  
  XtXinv <- matrix(NA, nrow=npar, ncol=npar) # default. 
  gr <- nanc # default to NA
  if (length(notmask) < 1) {
    warning("No unmasked parameters")
    XTXinv <- NULL
    param <- cbind(coeff, nanc, nanc, nanc)
  }
  else {
    Jwork<-JJ[, notmask]
    if (any(is.na(Jwork))){ stop("Bad working Jacobian") }
    else {
      dec <- svd(Jwork)
      U <- dec$u
      V <- dec$v
      Sd <- dec$d
    }
#    if (min(Sd) <= smalltol * max(Sd)) { # singular
#      XtXinv <- matrix(NA, nrow=npar, ncol=npar)
#    } else {  ## Above not needed because we have set default
    if (min(Sd) > smalltol * max(Sd)) { # singular
        Sinv <- 1/Sd
      if (length(notmask) > 1)  { # 220809 use 
        VS <- crossprod(t(V), diag(Sinv))
      } else {
        VS <- V/Sinv
      }
      XtXinv <- crossprod(t(VS))
      SEs[notmask] <- sqrt(diag(XtXinv) * resvar) # SEs of masked params still NA
    }
    Sdout[notmask] <- Sd
    gr <- crossprod(JJ, res)
    tstat<-rep(NA, npar)
    tstat[notmask] <- coeff[notmask]/SEs[notmask]
    tval <- coeff/SEs
    if (rdf > 0) {
       pval <- pt(abs(tval), rdf, lower.tail = FALSE)
    } else { pval <- NaN }
    param <- cbind(coeff, SEs, tval, 2 * pval)
  }
  # Note: We don't return formula because we may be doing nlfb summary 
  #   i.e., resfn and jacfn approach.
#  dimnames(param) <-
#    list(pnames, c("Estimate", "Std. Error", "t value", "Pr(>|t|)"))
  colnames(param) <- c("Estimate", "Std. Error", "t value", "Pr(>|t|)")
  rownames(param) <- pnames
  ans <- list(residuals = res, sigma = sqrt(resvar),  
              df = c(npar, rdf), cov.unscaled = XtXinv,
              param = param, resname=resname, ssquares=ss, nobs=nobs, 
              ct=ct, mt=mt, Sd=Sdout, gr=gr, jeval=object$jeval,feval=object$feval)
  class(ans)<-"nlsr"
  ans
} # end summary()

print.nlsr <- function(x, ...) {
  if (inherits(x,"try-error") || is.null(x$coefficients)) {
    cat("Object has try-error or missing parameters\n")
    return(invisible(x))
  }
  xx<-summary(x) # calls summary to get information
  with(xx, { 
    pname<-rownames(param) # param is augmented coefficients with SEs and tstats
    npar <- dim(param)[1] # previously length(coeff) 
    cat("residual sumsquares = ",ssquares," on ",nobs,"observations\n")
    cat("    after ",jeval,"   Jacobian and ",feval,"function evaluations\n")
    cat("  name     ","      coeff    ","     SE   ","   tstat  ",
        "   pval  ","   gradient  "," JSingval  ","\n")
    SEs <- param[,2]
    tstat <- param[,3]
    pval <- param[,4]
#    for (i in seq_along(param[,1])){
    for (i in 1:dim(param)[1]){
      tmpname<-pname[i]
      if (is.null(tmpname)) {tmpname <- paste("p_",i,sep='')}
      cat(format(tmpname, width=10)," ")
      cat(format(param[[i]], digits=6, width=12))
      cat(ct[[i]],mt[[i]]," ")
      cat(format(SEs[[i]], digits=4, width=9)," ")
      cat(format(tstat[[i]], digits=4, width=9)," ")
      cat(format(pval[[i]], digits=4, width=9)," ")
      cat(format(gr[[i]], digits=4, width=10)," ")
      cat(format(Sd[[i]], digits=4, width=10)," ")
      cat("\n")
    }
  }) # remember to close with()
  # return(NULL) 
  invisible(x)
} # end print.nlsr()

pshort <- function(x) {
  xname<-deparse(substitute(x))
  cat(xname," -- ss=",x$ssquares,":")
  prm<-x$coefficients
  pnam<-names(prm)
  for (i in 1:length(prm)){ cat(" ",pnam[i],"=",prm[i]) }
  cat(";",x$feval,"res/",x$jeval,"jac\n")
  invisible(x)
}


resid.nlsr <- function(object, ...){ # weighted
  resids <- object$resid
  resids # so the function prints
}

residuals.nlsr <- function(object, ...){
  resids <- object$resid
  resids # so the function prints
}


predict.nlsr <- function(object=NULL, newdata=list(), ...) { 
#  This ONLY works if we have used nlxb. Do we want to add class 'nlxb'?
    if( is.null(object) ) stop("predict.nlsr REQUIRES an nlsr solution object")
    form <- object$formula
    if (is.null(form)) stop("nlsr.predict works only if formula is defined")
# Give more output?
#  Note: we assume a formula of style y~something, and use the something
#  In some ways need to check this more carefully
    env4coefs <- list2env(as.list(object$coefficients))
    preds <- eval(form[[3]], as.list(newdata), env4coefs)
    class(preds)<- "predict.nlsr"
    attr(preds,"pkgname") <- "nlsr"
    preds
}

fitted.nlsr <- function(object=NULL, data=parent.frame(), ...) { 
  #  This ONLY works if we have used nlxb. Do we want to add class 'nlxb'?
  # Is parent.frame() right?  What about dotargs?
  if( is.null(object) ) stop("predict.nlsr REQUIRES an nlsr solution object")
  form <- object$formula
  if (is.null(form)) stop("fitted.nlsr works only if formula is defined")
  data <- eval(object$data)
  # Give more output
  #  we assume a formula of style y~something, and use the something
  env4coefs <- list2env(as.list(object$coefficients))
  fits <- eval(form[[3]], as.list(data), env4coefs)
  class(fits)<- "predict.nlsr" # Do we want all this?
#  attr(fits,"pkgname") <- "nlsr" # Do we want all this?
  fits
}

rawres <- function(object=NULL, data=parent.frame(), ...) { 
  #  This ONLY works if we have used nlxb. Do we want to add class 'nlxb'?
  # Is parent.frame() right?  What about dotargs?
  if( is.null(object) ) stop("predict.nlsr REQUIRES an nlsr solution object")
  form <- object$formula
  data <- eval(object$data) # Nov 17, 2022
  if (is.null(form)) stop("fitted.nlsr works only if formula is defined")
  env4coefs <- list2env(as.list(object$coefficients))
  rawres <- eval(form[[3]]-form[[1]], as.list(data), env4coefs)
  class(rawres)<- "nlxb" # Do we want all this?
  rawres
}

pctrl <- function(control) { 
   nprow <- 4 # default to 4 per row. May change later.
   nc <- length(control) # note some items may be multiple
   namctl<-names(control)
   for (ii in 1:nc) {
      item <- control[[ii]]
      if (length(item) > 1) item <- "vec/list"
      cat("  ",namctl[[ii]],"=",item)
      if ((ii %% nprow) == 0) cat("\n")
   }
   if ((ii %% nprow) != 0) cat("\n")
   invisible(1)
}

pnlslm <- function(x){ # short output for nls.lm result
  xname <- deparse(substitute(x))
  ss<-deviance(x)
  prm<-coef(x)
  pnam <- names(prm)   
  cat(xname," -- ss=",ss,":")
  for (i in 1:length(prm)){cat(" ",pnam[i],"=",prm[i])}
  cat(";",x$niter," itns\n") # only diff from pnlsLM
  invisible(1)
}

pnls <- function(x){ # short output for nls() result
  xname <- deparse(substitute(x))
  ss<-deviance(x)
  prm<-coef(x)
  pnam <- names(prm)   
  cat(xname," -- ss=",ss,":")
  for (i in 1:length(prm)){cat(" ",pnam[i],"=",prm[i])}
  cat(";",x$convInfo$finIter," itns\n")
  invisible(1)
}

# pnlsLM <- function(x){ # short output for nlsLM result
#   xname <- deparse(substitute(x))
#   ss<-deviance(x)
#   prm<-coef(x)
#   pnam <- names(prm)   
#   cat(xname," -- ss=",ss,":")
#   for (i in 1:length(prm)){cat(" ",pnam[i],"=",prm[i])}
#   cat(";",x$convInfo$finIter," itns\n")
#   invisible(1)
# }
# 


nvec <- function(vec){ # tidy display for named vector
   vnam<-deparse(substitute(vec))
   cat(vnam,":")
   pnam <- names(vec)
   for (i in 1:length(pnam)){ cat(pnam[i],"=",as.numeric(vec[i])," ")}
   cat("\n")
   invisible(1)
}

prt <- function(x) { # to provide naming for generic print 221123
  xname<-deparse(substitute(x))
  cat("nlsr object:",xname,"\n")
  print(x)
  invisible(x)
}

## Problem in 1 parameter to ensure methods work in trivial case
require(nlsr)
cat("Check nlsr.control\n")
cat("defaults\n")
ctrl <- nlsr.control()
pctrl(ctrl)
ctrl<-NULL
cat("Try control<-list(femax=4, phi=.12)\n")
control<-list(femax=4, phi=.12)
ctrl <- nlsr.control(control)
pctrl(ctrl)
ctrl<-NULL
cat("Try INVALID control<-list(fellmax=4, phi=.12) [bad name]\n")
control<-list(fellmax=4, phi=.12)
ctrl <- try(nlsr.control(control))
pctrl(ctrl)

## globaldata1.R
## tests to see that nlxb fails if user assumes that workspace
## data for y and tt will be used.

# traceval set TRUE to debug or give full history
traceval  <-  TRUE

# Data for Hobbs problem
ydat  <-  c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443, 
          38.558, 50.156, 62.948, 75.995, 91.972) # for testing
y  <-  ydat  # for testing
tdat  <-  seq_along(ydat) # for testing
# WARNING -- using T can get confusion with TRUE
tt  <-  tdat
# A simple starting vector -- must have named parameters for nlxb, nls, wrapnlsr.
start1  <-  c(b1=1, b2=1, b3=1)

eunsc  <-   y ~ b1/(1+b2*exp(-b3*tt))

cat("GLOBAL DATA -- does not work in nlmrt\n")

anls1g  <-  try(nls(eunsc, start=start1, trace=traceval))
print(anls1g)

cat("GLOBAL DATA AND EXPRESSION\n")
anlxb1g  <-  try(nlxb(eunsc, start=start1, trace=traceval))
print(anlxb1g)

---
title: "An introduction to nlsr"
author: "John C. Nash"
date: "`r Sys.Date()`"
output: pdf_document
bibliography: ImproveNLS.bib
vignette: >
  %\VignetteIndexEntry{nlsr Introduction}
  %\usepackage[utf8]{inputenc}
  %\VignetteEngine{knitr::rmarkdown}
---

**R** has several tools for estimating nonlinear models and minimizing sums of 
squares functions. The base R
distribution includes a function `nls()` that is feature-rich but has some 
annoying weaknesses (@JNABRefactoringNLS22). Some of these
are addressed in the package **nlsr** (@nlsr2019manual), for which this vignette 
provides an introduction with examples.

## What does nlsr do?

**nlsr** has functions to find a set of parameters that minimizes the sum of 
squares (or deviance) 
of differences between a model specified either as a formula using function 
`nlxb()` or as both residual
and Jacobian R functions using function `nlfb()`. As described below, the 
Jacobian may be approximated
by particular control settings.

Existing R tools that are similar to `nlxb()` are `nls()` and `minpack.lm::nlsLM()`;  
`minpack.lm::nls.lm()` is similar to `nlfb()`.

There are several important differences in approach and results, some of which 
are detailed in this document.

### Output object 

The output object of `nlxb()` is smaller than that of `nls()` and is focused on 
the solution and its properties
rather than the **modeling** task. That is, package `nlsr` emphasizes the solution 
of the nonlinear least squares problem rather than the estimation of a nonlinear 
model that fits or explains the data. `nls()` and `nlsLM` return an object of 
class `nls`, which allows for
a number of specialized modeling and diagnostic extensions. For compatibility, 
the `nlsr` package has function `wrapnlsr()`, for which `nlsr()` is an alias. 
This uses `nlxb()` to find good parameters, then calls `nls()` to return the class 
`nls` object. Unless particular modeling features are needed, the use of 
`wrapnlsr()` is unnecessary and wasteful of resources.

### Jacobian calculation

Internally, `nlxb()` uses symbolic and automatic differentiation tools to create 
the residual and Jacobian functions needed for `nlfb()` then uses it to solve 
the relevant nonlinear least squares minimization problem. 

As with other nonlinear least squares packages, we provide the Jacobian code as 
the "gradient" attribute of the Jacobian function. This lets us embed the code 
for the Jacobian as this attribute of the **residual** function so that the call 
to `nlfb()` can be made with the same name used for 
both residual and Jacobian function arguments. This programming trick saves a 
lot of trouble for the package developer, but it can be a nuisance for users 
trying to understand the code. 

Generally `nls()` and `nlsLM()` use a fairly simple forward difference 
approximation of derivatives for the Jacobian, though a central approximation 
can be specified in control parameters. Package `nlsr` provides four approximation 
options, with a further control `ndstep` for the size of the
step used in the approximation.

### Stabilization of Gauss-Newton computations

The iteration in all the major tools mentioned is the solution of the Gauss-Newton 
equations. `nls()` uses a variant of an approach suggested by @Hartley61, while 
`nlsr` and `minpack.lm` use variants of the method published by @Marquardt1963, 
though it was suggested earlier by @Levenberg1944. The details of the method in 
`nlsr` are discussed in  @JNABRefactoringNLS22.
Though it is unlikely to be useful in general, control settings for `nlxb()` or `nlfb()`
allow for those routines to perform a variety of Hartley and Marquardt algorithms. This
has been useful for learning about the algorithms, but tangential to simply finding
parameters of nonlinear models.

In general, the Levenberg-Marquardt stabilization is important in obtaining solutions.
The default method of `nls()` terminates with `singular gradient` unnecessarily in 
too many instances.

### Programming language

An important choice made in developing `nlsr` was to code entirely within the R 
programming language. `nls()` uses a mix of R, C and Fortran, as does `minpack.lm`. 
Generally, I believe that keeping to a single programming language allows for 
easier maintenance and upgrades. On the other hand, R is usually somewhat slower 
in performing computations because it keeps track of names and structures and 
because it is usually interpreted rather than compiled. In recent years the 
performance penalty for using code entirely in R has been much reduced
with the just-in-time compiler and other improvements, so that using an all-R 
package offers acceptable performance. Indeed, in `nlsr` the use of R may be 
less of a performance cost than the attempt to ensure the solution has been found, 
which forces more iterations to be used.

## An illustrative example

The Hobbs weed infestation problem (@cnm79 [page 120]) is a growth curve modeling 
task which is seemingly straightforward but turns out to be quite nasty. 

The data and a graph of it is given below.

```{r ex01, echo=TRUE}
weed <- c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443,
          38.558, 50.156, 62.948, 75.995, 91.972)
tt <- 1:12
weeddf <- data.frame(tt, weed)
plot(weeddf, main="Hobbs weed infestation data")
```

Most nonlinear least squares problems encountered by R users have data, but it is not
strictly required, as we shall see below in the section on functional specification 
of nonlinear least squares problems. What is required is a definition of 
the (residual) functions that are to be squared and then their squares 
summed to provide the  **loss function** or **objective function** 
to be minimized by adjusting the **parameters** that define the 
set of functions. In the case of the Hobbs problem, I was told the scientists
who collected the data believe that the growth of weeds followed a 3-parameter
logistic sigmoid growth curve where $y$ (the `weed` data) changes with time ($t$) 
(the `tt` index) according to the function

**Logistic3U**:
$$  y \approx  b_1 / (1 + b_2 * exp(- b_3 * t)) $$

The problem (using the 3 parameter logistic form above) has very bad scaling and regions
in the parameter space where the sum of squares objective is nearly "flat", so that its
Hessian, or matrix of second derivatives, is almost singular. When this occurs, 
methods cannot easily make progress towards the optimal solution.

Solution methods also need an initial guess for the parameters to be adjusted.
There are often some ways to provide such initial parameters, though in my 
experience they take quite a lot of work to set up reliably, but R does have
some worthwhile possibilities in some, though not all, of the **selfStart**
modelling functions, especially those in the package `nlraa` (@MiguezNLRAA2021).

There are, however, alternatives to the model above. For example, a simple scaling 
can make the solution to the problem easier to find, such as

**Logistic3S**:
$$ y \approx  100 * c_1 / (1 + 10 * c_2 * exp(- 0.1 * c_3 * t)) $$
Another form is

**Logistic3T**:
$$ y \approx Asym / (1 + exp((xmid - t)/scal)) $$

The functions above are equivalent. Their parameters are related as follows:

$$   Asym =  b_1 = 100 * c_1 $$
$$ exp(xmid/scal)  =  b_2 = 10 * c_2 $$
$$ 1/scal  =  b_3 $$

`nlsr` tries to find solutions from very poor initial parameter values. While
this is not always possible, `nslr` has generally been able to succeed in finding
solutions, even when all parameters are started at 1. Let us compare its results
with those of `nls()` and `nlsLM()`.

### Problem setup
  
```{r ex02set, echo=TRUE}
# model formulas
frmu <- weed ~ b1/(1+b2*exp(-b3*tt))
frms <- weed ~ 100*c1/(1+10*c2*exp(-0.1*c3*tt))
frmt <- weed ~ Asym /(1 + exp((xmid-tt)/scal))
#
# Starting parameter sets
stu1<-c(b1=1, b2=1, b3=1)
sts1<-c(c1=1, c2=1, c3=1)
stt1<-c(Asym=1, xmid=1, scal=1)
```

### Solution attempts with nls()

```{r ex02nls}
unls1<-try(nls(formula=frmu, start=stu1, data=weeddf))
summary(unls1)
snls1<-try(nls(formula=frms, start=sts1, data=weeddf))
summary(snls1)
tnls1<-try(nls(formula=frmt, start=stt1, data=weeddf))
summary(tnls1)
cat("\n")
```

### Solution attempts with nlsr tools

<!-- # source("/home/john/current/GSoC2021/improvenls/sp.R") -->


```{r ex02nlsr}
library(nlsr)
unlx1<-try(nlxb(formula=frmu, start=stu1, data=weeddf))
print(unlx1) 
pshort(unlx1) # A short form output
snlx1<-try(nlxb(formula=frms, start=sts1, data=weeddf))
# pshort(snlx1)
print(snlx1)
tnlx1<-try(nlxb(formula=frmt, start=stt1, data=weeddf))
# pshort(tnlx1)
print(tnlx1)
ct<-as.list(tnlx1$coefficients) # Need a list to use ct$... in next line
cat("exp(xmid/scal)=",exp(ct$xmid/ct$scal),"\n")
cat("\n")
# explicit residuals (weighted if there are weights)
rtnlx1<-residuals(tnlx1)
print(rtnlx1)
cat("explicit sumsquares =", sum(rtnlx1^2),"\n")
```

### Solution attempts with minpack.lm

NOTE: The original version of this vignette passed all package checks for Linux,
Windows and (Intel-based) Macintosh computers, but failed on Macintosh machines
using the M1 chip. 

```{r ex02minpack}
library(minpack.lm)
unlm1<-try(nlsLM(formula=frmu, start=stu1, data=weeddf))
summary(unlm1) # Use summary() to get display
unlm1 # Note the difference. Use this form to get sum of squares
# pnls(unlm1)  # Short form of output
snlm1<-try(nlsLM(formula=frms, start=sts1, data=weeddf))
# pnls(snlm1)
summary(snlm1)
tnlm1<-try(nlsLM(formula=frmt, start=stt1, data=weeddf))
if (inherits(tnlm1, "try-error")) {
   cat("Failure to compute solution -- likely singular Jacobian\n")
} else {  
   pnls(tnlm1) # short form to give sum of squares
   summary(tnlm1)
}   
```

### Solution attempts with wrapnlsr() wrapper

```{r ex02wrapnlsr}
## Try the wrapper. Calling wrapnlsr() instead of nlsr() is equivalent
##
unlw1<-try(nlsr(formula=frmu, start=stu1, data=weeddf))
print(unlw1) # using 'unlx1' gives name of object as 'x' only
snlw1<-try(nlsr(formula=frms, start=sts1, data=weeddf))
# pshort(snlx1)
print(snlw1)
tnlw1<-try(nlsr(formula=frmt, start=stt1, data=weeddf))
print(tnlw1)
```

### Commentary

There are several details that may be important for users.

- `nlsr` is set up to use `print()` to output standard errors and singular values
  of the Jacobian (for diagnostic purposes). By contrast, `minpack.lm` and `nls()`
  use `summary()`, which does NOT display the sum of squares, while `print()` 
  gives the sum of squares, but not the standard errors.
- The singular values displayed by `print.nlsr()` (the internal name for the adaptation
  of the generic `print()`) are displayed in a column to the right of the coefficient
  and standard error display, but are NOT specific to the parameters. Their position is
  purely for efficient use of page space. 
- The most common use of the singular values is to gauge how "nearly singular" the 
  Jacobian is at the solution, and the ratio of the largest to smallest of the 
  singular values is a simple but effective measure. In the above example, we note 
  that the scaled logistic has the smallest ratio. 
- The result from `nlsLM` for the transformed model has a very large sum of squares,
  which may suggest that the program has failed. Since neither `nls()` nor `nlsLM()`
  offer the singular values, we can "cheat" and use `nlxb()`, though the Jacobian
  used will be the analytic one used by this last program rather than the forward
  difference approximation that is generally used by the others.
  
```{r ex03, echo=TRUE}  
stspecial<- c(Asym = 35.532,  xmid = 43376,  scal = -2935.4)
# force to exit at once by setting femax to 1 (maximum number of sum of squares evaluations)
getsvs<-nlxb(formula=frmt, start=stspecial, data=weeddf, control=list(femax=1))
print(getsvs)
```

- The trick used above to get singular values is convenient, but it is actually
  quite easy to get the Jacobian from a class `nls` object such as `tnlm1` as follows.
  
```{r ex04, echo=TRUE, eval=FALSE}  
if (inherits(tnlm1, "try-error")) {
   cat("Cannot compute solution -- likely singular Jacobian\n")
} else {  
   Jtm <- tnlm1$m$gradient()
   svd(Jtm)$d # Singular values
}   
```

We see that there are differences in detail, but the more important result is that
two out of three singular values are essentially 0. Our Jacobian is singular, and no 
method of the Gauss-Newton type should be able to continue. Indeed, from this set of
parameters, `nlxb` also stops.

```{r ex05, echo=TRUE}  
stspecial<- c(Asym = 35.532,  xmid = 43376,  scal = -2935.4)
# force to exit at once by setting femax to 1 (maximum number of sum of squares evaluations)
badstart<-nlxb(formula=frmt, start=stspecial, data=weeddf)
print(badstart)
```

## selfStart options

The form of the logistic given by `frmt` is available as a **selfStart** model,
needing no starting parameters with `nls()`. The code is in base R in location 
` src/library/stats/R/zzModels.R`.

```{r ex06, echo=TRUE}  
frmtss <- weed ~ SSlogis(tt, Asym, xmid, scal)
ssts1<-nls(formula=frmtss, data=weeddf)
summary(ssts1)
require(minpack.lm)
sstm1<-nlsLM(formula=frmtss, data=weeddf)
summary(sstm1)
```

The essence of selfStart models is to provide starting parameters for the iterative methods
used to find the final parameters. However, an examination of the code for the `SSlogis()`
function shows that it makes use of a DIFFERENT solver and returns the final results of
that solver. The computational effort in doing this is, I believe, greater than 
that expended by `nlxb()` from an extremely crude start (all 1's). An alternative
selfStart for this version of the logistic is included in package `nlsr` as 
`SSlogisJN.R`.

Users should be aware that the programming of selfStart
models involves quite esoteric aspects of R which I find prone to errors and difficult to 
grasp easily. Developers who have built them for us deserve our thanks. A particularly
important collection of such tools is in the package @MiguezNLRAA2021. 

###  SSlogisJN.R for the 3-parameter logistic

`SSlogisJN.R` uses ideas for selfStart from @Ratkowsky1983 for the Logistic3T form of the 
model. It produces only an approximate set of starting parameters, unlike `SSlogis.R`. 
The core of the idea is as follows, where we use the abbreviated output function to
save page space.
  
```{r ex07, echo=TRUE}  
Asymt <- 2*max(weed)
Asymt
pw <- Asymt/weed
pw1<-pw-1
lpw1<-log(pw1)
clin <- coef(lm(lpw1~tt))
clin<-as.list(clin)
scalt <- -1/clin$tt
scalt
xmidt <- scalt*as.numeric(clin[1])
xmidt
library(nlsr)
try1<-nlxb(frmt, data=weeddf, start=c(Asym=1, xmid=1, scal=1))
pshort(try1)
try2<-nlxb(frmt, data=weeddf, start=c(Asym=Asymt, xmid=xmidt, scal=scalt))
pshort(try2)
```

We see a dramatic reduction in the computational effort as measured by residual and 
Jacobian evaluations.

### Running selfStart models with nlxb()

There are two important steps in using selfStart models with `nlxb()`:

- we must provide the starting parameters explicitly to `nlxb()`, which seems
counter to the goal of selfStart models. However, we use function `getInitial()`
to exploit particular selfStart modeling functions.
- we must point to a mechanism to compute the Jacobian, since the
  selfStart function is unlikely to be in the derivatives table of
  analytic or automatic derivatives. Note that Jacobian ("gradient")
  code is generally part of selfStart functions. If code is included
  for the analytic Jacobian, it is likely worth using. However, because
  it is easy to make errors in coding derivatives, it
  may also be worth checking results of such code, for example,
  using tools from package `numDeriv`. I have made more such errors
  over my career than I like to admit.
  
These steps can be automated, and this has been carried out in function
`nlsrSS()`. The automation is built into the functions `nls()` and 
`minpack.lm::nlsLM()`, but I have preferred to make the process explicit
and give it a separate function.

Let us illustrate how to use the `getInitial()` function (part of
base R) to do this.

```{r ex08, eval=TRUE}
frmtssJ <- weed ~ SSlogisJN(tt, Asym, xmid, scal) # The model formula
lstrt <- getInitial(frmtssJ, weeddf) # Here we get the starting parameters
cat("starting parameters:\n")
print(lstrt)
# Next line uses the start. We indicate that the "approximate" Jacobian code is in
#  the selfStart code, though in fact it is not an approximation in this case.
sstx1<-nlxb(formula=frmtssJ, start=lstrt, data=weeddf, control=list(japprox="SSJac"))
print(sstx1)
# If no Jacobian code is included in selfStart module, we can actually use an
#  approximate Jacobian. See "gradient" elements for differences in result.
sstx1a<-nlxb(formula=frmtssJ, start=lstrt, data=weeddf, control=list(japprox="jacentral"))
print(sstx1a)
## We can automate the above in function nlsrSS()
sstSS<-nlsrSS(formula=frmtssJ, data=weeddf)
print(sstSS)
```

### Starting parameters for the Logistic3U model

A similar approach for the Logistic3U model can be used.

```{r ex09, echo=TRUE} 
b1t <- 2*max(weed)
b1t
lpw1<-log(b1t/weed-1)
clin <- as.list(coef(lm(lpw1~tt)))
b3t <- -clin$tt
b3t
b2t <- exp(as.numeric(clin[1]))
b2t
library(nlsr)
try1<-nlxb(frmu, data=weeddf, start=c(b1=1, b2=1, b3=1))
pshort(try1)
try2<-nlxb(frmu, data=weeddf, start=c(b1=b1t, b2=b2t, b3=b3t))
pshort(try2)
```

Once again, there is a significant saving in the number of iterations.
We have not coded a selfStart function for Logistic3U, preferring to 
use the scaled form Logistic3S for which we do not need good starting
values.

## Jacobian computation

`nlxb()` offers more options for how the Jacobian is computed than either `nls()` or
`minpack.lm::nlsLM()`. The choice is made using the `control` list element
`japprox`. If this is NOT specified, `nlxb()` attempts to build Jacobian code
using analytic or automatic derivative codes. This is not, of course, always possible,
and sometimes we will get an "error" message that required information is not in 
the derivatives table. This will be the case when we use a function like `SSlogisJN()`
unless we indicate that the code is available from a selfStart module by using a value 
of `SSJac` for `japprox`. The example also showed how we can specify an approximation
method, which is also useful when a formula does not have analytic derivatives available.
In such cases `control` element `japprox` can be set to one of `jafwd`, `jaback`, 
`jacentral` or `jand`, giving forward, backward, central and package `numDeriv`
approximations. I recommend `jacentral` as a reasonable compromise between 
accuracy of approximation and amount of computational work. 

## Bounds constraints on parameters

In some cases, we know that parameters cannot be bigger or smaller than some
externally known limits. Cash reserves cannot be negative. Animals have a 
minimum need for water. Airplanes cannot carry more than a known or 
legislated cargo weight. Such limits can be built into models, but there 
are some important details for using the tools in R.

- `nls()` can only impose bounds if the `algorithm="port"` argument is 
used in the call. Unfortunately, the documentation warns us:

  *The algorithm = "port" code appears unfinished, and does not even 
   check that the starting value is within the bounds. Use with 
   caution, especially where bounds are supplied.*

- `nlsLM()` includes bounds in the standard call, but I have observed cases
   where it fails to get the correct answer. From my examination of the code,
   I believe the authors have not taken into account all possibilities, though
   it should be noted that I regard all programs as having some weakness in
   regard to constrained optimization. Software creators have to work with
   assumptions on the extremity of scale that they are willing to countenance,
   and sometimes problems will be outside the scope envisaged.

```{r ex10, echo=TRUE} 
# Start MUST be feasible i.e. on or within bounds
anlshob1b <- nls(frms, start=sts1, data=weeddf, lower=c(0,0,0),
             upper=c(2,6,3), algorithm='port')
pnls(anlshob1b) #  check the answer (short form)
# nlsLM seems NOT to work with bounds in this example
anlsLM1b <- nlsLM(frms, start=sts1, data=weeddf, lower=c(0,0,0), upper=c(2,6,3))
pnls(anlsLM1b)
# also no warning if starting out of bounds, but gets good answer!!
st4<-c(c1=4, c2=4, c3=4)
anlsLMob <- nlsLM(frms, start=st4, data=weeddf, lower=c(0,0,0), upper=c(2,6,3))
pnls(anlsLMob)
# Try nlsr::nlxb()
anlx1b <- nlxb(frms, start=sts1, data=weeddf, lower=c(0,0,0), upper=c(2,6,3))
pshort(anlx1b)
```

## Functional specification of nonlinear least squares problems

We illustrate how to solve nonlinear least squares problems where the set of
functions to be squared is provided by an R function, as is the Jacobian
matrix. Note that `nlsr::nlfb()` gets this information from the object
returned by the `jacfn` argument in the "gradient" attribute of that object.
This is a programming artifice to allow a simplification of the `nlxb()` code.
The example is the well-known Banana shaped valley of @Rosenbrock1960.

```{r exrosen, eval=TRUE}
frres<-function(x) { ## Rosenbrock as residuals
    x1 <- x[1]
    x2 <- x[2]
    res<-c( 10 * (x2 - x1 * x1), (1 - x1) )
}

frjac<-function(x) { ## Rosenbrock residual derivatives matrix
    x1 <- x[1]
    x2 <- x[2]
   J<-matrix(NA, nrow=2, ncol=2)
   J[1,1]<- -20*x1
   J[1,2]<- 10
   J[2,1]<- -1
   J[2,2]<- 0
   attr(J,"gradient")<-J # NEEDED for nlfb()
   J
}
require(minpack.lm)
require(nlsr)
strt<-c(-1.2,1)
rosnlf<-nlfb(strt, resfn=frres, jacfn=frjac)
print(rosnlf)
rosnlm<-nls.lm(par=strt, fn=frres, jac=frjac)
summary(rosnlm)  
```

I find it awkward to solve problems specified this way with `nls()` and do not believe
it is worth pursuing that topic.


## Weighted nonlinear regression

Regression attempts to explain a *dependent* (or *predicted*) variable as a function of
*independent* (or *explanatory* or *predictor*) variables and estimated parameters. The
estimation method commonly used is minimization of the sum of squared
residuals. However, these residuals may be weighted. In the tools available
in **R** for nonlinear least squares, weights can be specified which multiply
the **squared** residuals, and we need a weight for each term in the sum.
Each residual is therefore multiplied by the square root of its respective
weight.

Typically, the weights are given as a vector of numbers. A popular choice is
the reciprocal of a measure of the variance of each data observation.
Replicated data allows for sample variances to be used; otherwise we need some
proxy. The concept is that the weights are proportional to our belief that the
observations are correct.

### Static weights

Let us demonstrate with weights that are simply the reciprocal of the
independent variable. This may not make sense for statistical modeling,
but it lets us see that the `residuals()` function returns weighted
residuals.

```{r staticwts}
wts <- 1/weeddf$tt # wts are reciprocal of time value
tnlx1w<-try(nlxb(formula=frmt, start=stt1, data=weeddf, weights=wts))
# pshort(tnlx1)
print(tnlx1w)
ct<-as.list(tnlx1w$coefficients) # Need a list to use ct$... in next line
cat("exp(xmid/scal)=",exp(ct$xmid/ct$scal),"\n")
cat("\n")

rtnlx1w<-residuals(tnlx1w)
print(rtnlx1w)
cat("explicit sumsquares =", sum(rtnlx1w^2),"\n")
```

### Weights that are functions of the model parameters

Possible choices for weights include the reciprocal of the squared residuals
or squared fitted values. Unfortunately, such quantities depend on the
parameter values. Consider our Logistic example. We want to predict
$y$ with

$$  fitted(b_1, b_2, b_3, t) = b_1 / (1 + b_2 * exp(- b_3 * t)) $$
Thus the raw residuals are

$$  rawres(b_1, b_2, b_3, t) = y -  fitted(b_1, b_2, b_3, t) $$
But if the weights are $(1/ fitted(b_1, b_2, b_3, t)$, then the
residuals for which the sum of squares is to be calculated are

$$ resid(b_1, b_2, b_3, t) = rawres(b_1, b_2, b_3, t) * (1/fitted(b_1, b_2, b_3, t) $$

$$                          =  y/(b_1 / (1 + b_2 * exp(- b_3 * t))) - 1 $$

However, we still need to keep in mind that the user needs to use the formula
for the fitted values to get predictions. While the software needs to minimize
the weighted (hence modified) sum of squares, it also must keep track of the
fitted/predicted values.

`minpack.lm` offers a function `wfct()` that can be used within the argument 
supplied as `weights` in the call to `nlsLM()` or `nls()`, though it usually
fails with `nlsr::nlxb()`. The action of this function was surprising to me, 
as `wfct()` takes an expression as its argument (and may itself be part of
an expression in the argument after the `=` sign). If the argument to 
`wfct` contains `fitted`, `resid` or `error`, then the call to `nlsLM` or
`nls` will trigger a further call to one of these functions, evaluate the
`weights` and then run again to compute the solution. This can be seen by
setting `trace = TRUE`. The process can be verified explicitly for the 
example used in the `wfct` documentation. Unfortunately, `wfct()` appears
to crash `knitr` or `rmarkdown`, so we have evaluated the example outside
of Rstudio and included the output below.

```{r puromycinwfct, echo=TRUE, eval=FALSE}
library(minpack.lm)
library(nlsr) # for pnls
Treated <- Puromycin[Puromycin$state == "treated", ]
# First get raw estimates
wtt3nlm0<-nlsLM(rate ~ Vm * conc/(K + conc), data = Treated, trace=TRUE,
               start = c(Vm = 200, K = 0.05))
fit0<-fitted(wtt3nlm0)
# Static wts using fit0
wtt3nlm0s<-nlsLM(rate ~ Vm * conc/(K + conc), data = Treated, trace=TRUE,
               start = c(Vm = 200, K = 0.05), weights = wfct(1/fit0^2))
pnls(wtt3nlm0s)
# and run directly, noting the 2 phase operation
wtt3nlm<-nlsLM(rate ~ Vm * conc/(K + conc), data = Treated, trace=TRUE,
               start = c(Vm = 200, K = 0.05), weights = wfct(1/fitted^2))
pnls(wtt3nlm)
cat("weights from wtt3nlm\n")
as.numeric(wtt3nlm$weights)
```

```
It.    0, RSS =    1636.59, Par. =        200       0.05
It.    1, RSS =    1205.62, Par. =    211.157  0.0616271
It.    2, RSS =    1195.57, Par. =    212.511  0.0638418
It.    3, RSS =    1195.45, Par. =    212.666  0.0640939
It.    4, RSS =    1195.45, Par. =    212.682  0.0641186
It.    5, RSS =    1195.45, Par. =    212.684   0.064121

It.    0, RSS =    0.22811, Par. =        200       0.05
It.    1, RSS =   0.227222, Par. =    200.913  0.0494747
It.    2, RSS =   0.227221, Par. =    200.847  0.0494303
It.    3, RSS =   0.227221, Par. =    200.842   0.049426
It.    4, RSS =   0.227221, Par. =    200.841  0.0494256
wtt3nlm0s  -- ss= 0.2272213 :  Vm = 200.841  K = 0.04942558; 4  itns
It.    0, RSS =    1636.59, Par. =        200       0.05
It.    1, RSS =    1205.62, Par. =    211.157  0.0616271
It.    2, RSS =    1195.57, Par. =    212.511  0.0638418
It.    3, RSS =    1195.45, Par. =    212.666  0.0640939
It.    4, RSS =    1195.45, Par. =    212.682  0.0641186
It.    5, RSS =    1195.45, Par. =    212.684   0.064121
It.    0, RSS =    0.22811, Par. =        200       0.05
It.    1, RSS =   0.227222, Par. =    200.913  0.0494747
It.    2, RSS =   0.227221, Par. =    200.847  0.0494303
It.    3, RSS =   0.227221, Par. =    200.842   0.049426
It.    4, RSS =   0.227221, Par. =    200.841  0.0494256
wtt3nlm  -- ss= 0.2272213 :  Vm = 200.841  K = 0.04942558; 4  itns
Show in New Window
weights from wtt3nlm
 [1] 3.910941e-04 3.910941e-04 9.460635e-05 9.460635e-05 5.539227e-05 5.539227e-05 
 [7] 3.687173e-05 3.687173e-05 2.745956e-05 2.745956e-05 2.475956e-05 2.475956e-05
```

The documentation of `wfct` suggests that it can also use the name of the
response (dependent) variable or the name of the predictor (independent)
variable. However, some models have more than one independent variable, and
I have not explored what happens in such cases.

It would be helpful if `nlsr` had the capability to include functional weights. 
Duncan Murdoch suggested a patch that allows this. We modify the `nlfb()` routine
to detect and act on functional weights. 


```{r formulawts}
library(nlsr)
Treated <- Puromycin[Puromycin$state == "treated", ]
# "regression" formula
w1frm <- rate ~ Vm * conc/(K + conc)
# Explicit dynamic weighted nlxb
w3frm <- rate/(Vm * conc/(K + conc)) ~ 1
dynweighted <- nlxb(w3frm, data = Treated, start = c(Vm = 201.003, K = 0.04696),
                    trace=TRUE, control=list(prtlvl=0))
pshort(dynweighted)
# Compute the model from the ORIGINAL model (w1frm) using parameters from dynweighted
dyn0 <- nlxb(w1frm, start=coefficients(dynweighted), data=Treated, control=list(jemax=0))
wfdyn0 <- 1/fitted(dyn0)^2 # weights
print(as.numeric(wfdyn0))
pshort(dyn0) # Shows sumsquares without weights, but computes weights we used
formulaweighted <- nlxb(w1frm, data = Treated, start = c(Vm = 201.003, K = 0.04696),
                        weights = ~ 1/fitted^2, trace=TRUE, control=list(prtlvl=0))
pshort(formulaweighted)
wtsfromfits<-1/fitted(formulaweighted)^2
print(as.numeric(wtsfromfits))
print(as.numeric(formulaweighted$weights))
```

There are differences in the (weighted) sums of squares. The **dynweighted** case minimizes
the residuals weighted
by the actual CURRENT fits as expressed by the formula. The **formulaweighted** case
computes the weights at each
iteration from the previous (or original) fit. It is a form of iteratively weighted least
squares. The case
**wtt3nlm** above uses `nlsLM` with weights specified in `wfct()` and computes the weights
once from an unweighted
model, then runs a second, statically weighted calculation. Keeping track of these
differences requires a lot of
care, and I advise documenting what is done carefully.


## Ongoing efforts

The examples above show how to use package `nlsr` as it exists at the end of December
2022.
The story, however, is not finished. It would be very nice to be able to identify and work
with models that are partially linear. Solving such models is possible with the `algorithm="plinear"`
option of `nls()`, but identifying which parameters enter linearly is not easy for most users. 
Moreover, the specification
of the model to the `VARPRO` solver that is called by the `plinear` option is inconsistent
with the general specification used by `nls()` and other nonlinear least squares tools for R.

There are also a number of possible tweaks to details in `nlsr` and efforts to include such
improvements are a continuing interest.
I welcome communication and collaboration to continue the improvement of the package and
R in general. 


## References

---
title: "nlsr and Other Approaches to Derivatives in R"
author: "John C. Nash"
date: "`r Sys.Date()`"
output: rmarkdown::pdf_document
bibliography: ImproveNLS.bib
vignette: >
  %\VignetteIndexEntry{nlsr Derivatives}
  %\VignetteEngine{knitr::rmarkdown}
  %\usepackage[utf8]{inputenc}
---

This long and incomplete vignette is an attempt to catalog and illustrate the various 
capabilities in the **R** statistical computing system to 
perform differentiation. There are many traps and pitfalls for
the unwary in doing this. It is hoped that by collecting examples
in this long treatment will serve to record these and show how to avoid
them, and how to reliably compute the derivatives desired.
Derivative capabilities of **R** are in the base system (essentially
the functions `D()` and `deriv()`) and in
different packages, namely `nlsr` (or more recently `nlsr`), `Deriv`, 
and `Ryacas`. General tools for
approximations to derivatives are found in the package `numDeriv` as
well as `optimx`. Other approximations may be embedded in 
various packages, but not necessarily exported for use in scripts
or packages.

As a way of recording where attention is needed either to this document
or to the functions and methods described, I have put double question marks
in various places. Contributions and collaborations to extend and complete 
the treatment are welcome.

Note: To distinguish output results (which are prefaced '`## `' by **knitr**, 
I have attempted to put comments in the **R** code with the preface '`#- `'.)


## Available analytic differentiation tools

**R** has a number of tools for finding analytic derivatives.

- **stats**: tools `D()` and `deriv()` (@Rcite)

- **nlsr**: tools `nlsDeriv()`, `fnDeriv()`, and the interface `model2rjfun` (@nlsr2019manual)

- **Deriv**: tools `Deriv()` (@Deriv-manual)

- **Ryacas**: tools ??  (@Ryacas-manual)

- In 2018, Changcheng Li conducted a Google Summer of Code project to link R to Julia's
Automatic Differentiation tools, resulting in the experimental package `autodiffr`
(see https://github.com/Non-Contradiction/autodiffr).

<!-- - ?? any other packages that give analytic derivatives? -->

## How the tools are used

This is an overview section to give an idea of the capabilities. It is
not intended to be exhaustive, but to give pointers to how the tools can
be used quickly.

An important issue that may cause a lot of difficulty is the iterating of the tools.
That is, we compute a derivative, then want to apply a tool to the derivative to get
a second derivative. In doing so, we need to be careful that the type (class??) of
the quantity output by the tool is passed back into the tool in a form that will
generate a derivative expression. Some examples are presented. 

We also note that the **Deriv** package will give a result in cases when the input
is undefined. This is clearly a bug. There is an example below on the section for
`Deriv`.

### stats (i.e., the base R installation)

`D()`, `deriv()` and `deriv3()`: As `deriv3()` is stated to be the same as `deriv()` but with
argument `hessian=TRUE`, we will for now only consider the first two.

```{r chunk01}
dx2x <- deriv(~ x^2, "x") 
dx2x
mode(dx2x)
str(dx2x)
x <- -1:2
eval(dx2x) # This is evaluated at -1, 0, 1, 2, with the result in 
# the "gradient" attribute.
# Note that we cannot (easily) differentiate this again.
firstd <- attr(dx2x,"gradient")
str
#  ... and the following gives an error
d2x2x <- try(deriv(firstd, "x"))
str(d2x2x)
#- Build a function from the expression
fdx2x<-function(x){eval(dx2x)}
fdx2x(1)
fdx2x(3.21)
fdx2x(1:5)
#- # Now try D()
Dx2x <- D(expression(x^2), "x")
Dx2x
x <- -1:2
eval(Dx2x)
# We can differentiate aggain
D2x2x <- D(Dx2x,"x")
D2x2x
eval(D2x2x) #- But we don't get a vector -- could be an issue in gradients/Jacobians
#- Note how we handle an expression stored in a string via parse(text=  ))
sx2 <- "x^2"
sDx2x <- D(parse(text=sx2), "x")
sDx2x
#- But watch out! The following "seems" to work, but the answer is not as intended.  
#- The problem is that the first argument is evaluated before being used.  Since 
#- x exists, it fails
x
Dx2xx <- D(x^2, "x")
Dx2xx
eval(Dx2xx)
#-  Something 'tougher':
trig.exp <- expression(sin(cos(x + y^2)))
( D.sc <- D(trig.exp, "x") )
all.equal(D(trig.exp[[1]], "x"), D.sc)
( dxy <- deriv(trig.exp, c("x", "y")) )
y <- 1
eval(dxy)
eval(D.sc)
#-  function returned:
deriv((y ~ sin(cos(x) * y)), c("x","y"), func = TRUE)
#- ??#-  Surely there is an error, since documentation says no lhs! i.e.,
#- "expr: a 'expression' or 'call' or (except 'D') a formula with no lhs."
#-  function with defaulted arguments:
(fx <- deriv(y ~ b0 + b1 * 2^(-x/th), c("b0", "b1", "th"),
             function(b0, b1, th, x = 1:7){} ) )
fx(2, 3, 4)
#-  First derivative
D(expression(x^2), "x")
#-  stopifnot(D(as.name("x"), "x") == 1) #- A way of testing. 
#- This works by coercing "x" to name/symbol, and derivative should be 1.
#- Would fail only if "x" cannot be so coerced. How could this happen??
#-  Higher derivatives showing deriv3
myd3 <- deriv3(y ~ b0 + b1 * 2^(-x/th), c("b0", "b1", "th"),
     c("b0", "b1", "th", "x") )
myd3(2,3,4, x=1:7)
#- check against deriv()
myd3a <- deriv(y ~ b0 + b1 * 2^(-x/th), c("b0", "b1", "th"),
     c("b0", "b1", "th", "x"), hessian=TRUE )
myd3a(2,3,4, x=1:7)
identical(myd3a, myd3) #- Remember to check things!

#-  Higher derivatives:
DD <- function(expr, name, order = 1) {
   if(order < 1) stop("'order' must be >= 1")
   if(order == 1) D(expr, name)
   else DD(D(expr, name), name, order - 1)
}
DD(expression(sin(x^2)), "x", 3)
#-  showing the limits of the internal "simplify()" :
#-  -sin(x^2) * (2 * x) * 2 + ((cos(x^2) * (2 * x) * (2 * x) + sin(x^2) *
#-     2) * (2 * x) + sin(x^2) * (2 * x) * 2)
```

### nlsr

```{r chunk02}
require(nlsr)
dx2xn <- nlsDeriv(~ x^2, "x")
dx2xn
mode(dx2xn)
str(dx2xn)
x <- -1:2
eval(dx2xn) # This is evaluated at -1, 0, 1, 2, BUT result is returned directly,
#-  NOT in "gradient" attribute
firstdn <- dx2xn
str(firstdn)
d2x2xn <- nlsDeriv(firstdn, "x")
d2x2xn
d2x2xnF <- nlsDeriv(firstdn, "x", do_substitute=FALSE)
d2x2xnF # in this case we get the same result
d2x2xnT <- nlsDeriv(firstdn, "x", do_substitute=TRUE)
d2x2xnT # 0 ## WATCH OUT

#- ?? We can iterate the derivatives
nlsDeriv(d2x2xn, "x")

nlsDeriv(x^2, "x")# 0
nlsDeriv(x^2, "x", do_substitute=FALSE)# 0
nlsDeriv(x^2, "x", do_substitute=TRUE) # 2 * x
nlsDeriv(~ x^2, "x") # 2 * x
nlsDeriv(~ x^2, "x", do_substitute=FALSE) # 2 * x
nlsDeriv(~ x^2, "x", do_substitute=TRUE) # 2 * x

#- Some things to check and explain??
#?? firstde <- quote(firstd)
#?? firstde
#?? firstde <- bquote(firstd)
#?? firstde
#?? nlsDeriv(firstde, "x")
d2 <- nlsDeriv(2 * x, "x")
str(d2)
d2
#?? firstc <- as.call(firstd)
#?? nlsDeriv(firstc, "x")
#- Build a function from the expression
#?? fdx2xn<-function(x){eval(dx2xn)}
#?? fdx2xn(1)
#?? fdx2xn(3.21)
#?? fdx2xn(1:5)
```

The tool `codeDeriv` returns an R expression to evaluate the
derivative efficiently.  `fnDeriv` wraps it in a function.
By default the arguments to the function are constructed from
all variables in the
expression.  In the example below this includes `x`.

```{r chunk03}
codeDeriv(parse(text="b0 + b1 * 2^(-x/th)"), c("b0", "b1", "th")) 
#- Include parameters as arguments
fj.1 <- fnDeriv(parse(text="b0 + b1 * 2^(-x/th)"), c("b0", "b1", "th")) 
head(fj.1)
fj.1(1,2,3,4)
#- Get all parameters from the calling environment
fj.2 <- fnDeriv(parse(text="b0 + b1 * 2^(-x/th)"), c("b0", "b1", "th"),
        args = character())
head(fj.2)
b0 <- 1
b1 <- 2
x <- 3
th <- 4 
fj.2()

#- Just use an expression
fje <- codeDeriv(parse(text="b0 + b1 * 2^(-x/th)"), c("b0", "b1", "th"))
eval(fje)

dx2xnf <- fnDeriv(~ x^2, "x") #- Use tilde
dx2xnf <- fnDeriv(expression(x^2), "x") #- or use expression()
dx2xnf
mode(dx2xnf)
str(dx2xnf)
x <- -1:2
#?? eval(dx2xnf) # This is evaluated at -1, 0, 1, 2, BUT result is returned directly,
#-  NOT in "gradient" attribute
# Note that we cannot (easily) differentiate this again.
# firstd <- dx2xnf
# str(firstd)
# d2x2xnf <- try(nlsDeriv(firstd, "x")) #- this APPEARS to work, but WRONG answer 
# str(d2x2xnf)
# d2x2xnf
# eval(d2x2xnf)

# dx2xnfh <- fnDeriv(expression(x^2), "x", hessian=TRUE) #- Try for second derivatives
# dx2xnfh
# mode(dx2xnfh)
# str(dx2xnfh)
# x <- -1
# eval(dx2xnfh) # This is evaluated at -1, 0, 1, 2, BUT result is returned directly,
```


### Deriv

The following examples are drawn from the `example(Deriv)` contained in the `Deriv` package.

```{r chunk04}
require(Deriv)

f <- function(x) x^2
Deriv(f)
#- Should see
#- function (x)
#- 2 * x
#- Now save the derivative
f1 <- Deriv(f)
f1 #- print it
f2 <- Deriv(f1) #- and take second derivative
f2 #- print it

f <- function(x, y) sin(x) * cos(y)
f_ <- Deriv(f)
f_ #- print it
#- Should see
#- function (x, y)
#- c(x = cos(x) * cos(y), y = -(sin(x) * sin(y)))
f_(3, 4)
#- Should see
#-              x         y
#- [1,] 0.6471023 0.1068000

f2 <- Deriv(~ f(x, y^2), "y") 
#- This has a tilde to render the 1st argument as a formula object
#- Also we are substituting in y^2 for y
f2 #- print it
#- -(2 * (y * sin(x) * sin(y^2)))
mode(f2) #- check what type of object it is
arg1 <- ~ f(x,y^2)
mode(arg1) #- check the type
f2a <- Deriv(arg1, "y")
f2a #- and print to see if same as before
#- try evaluation of f using current x and y
x
y
f(x,y^2)
eval(f2a) #- We need x and y defined to do this.

f3 <- Deriv(quote(f(x, y^2)), c("x", "y"), cache.exp=FALSE) #- check cache.exp operation
#- Note that we need to quote or will get evaluation at current x, y values (if they exist)
f3 #- print it
#- c(x = cos(x) * cos(y^2), y = -(2 * (y * sin(x) * sin(y^2))))
f3c <- Deriv(quote(f(x, y^2)), c("x", "y"), cache.exp=TRUE) #- check cache.exp operation
f3c #- print it
#- Now want to evaluate the results
#- First must provide some data
x <- 3
y <- 4
eval(f3c)
#- Should see
#- x         y 
#- 0.9480757 0.3250313 
eval(f3) #- check this also
#- or we can create functions
f3cf <- function(x, y){eval(f3c)}
f3cf(x=1, y=2)
#-          x          y 
#- -0.3531652  2.5473094 
f3f <- function(x,y){eval(f3)}
f3f(x=3, y=4)
#-         x         y 
#- 0.9480757 0.3250313 

#- try an expression
Deriv(expression(sin(x^2) * y), "x")
#- should see
#- expression(2 * (x * y * cos(x^2)))

#- quoted string
Deriv("sin(x^2) * y", "x") # differentiate only by x
#- Should see
#- "2 * (x * y * cos(x^2))"

Deriv("sin(x^2) * y", cache.exp=FALSE) #- differentiate by all variables (here by x and y)
#- Note that default is to differentiate by all variables.
#- Should see
#- "c(x = 2 * (x * y * cos(x^2)), y = sin(x^2))"

#- Compound function example (here abs(x) smoothed near 0)
#- Note that this introduces the possibilty of `if` statements in the code
#- BUT (JN) seems to give back quoted string, so we must parse.
fc <- function(x, h=0.1) if (abs(x) < h) 0.5*h*(x/h)**2 else abs(x)-0.5*h
efc1 <- Deriv("fc(x)", "x", cache.exp=FALSE) 
#- "if (abs(x) < h) x/h else sign(x)"
#- A few checks on the results
efc1

fc1 <- function(x,h=0.1){ eval(parse(text=efc1)) }
fc1
## h=0.1
fc1(1)
fc1(0.001)
fc1(-0.001)
fc1(-10)
fc1(0.001, 1)

#- Example of a first argument that cannot be evaluated in the current environment:
  try(suppressWarnings(rm("xx", "yy"))) #- Make sure there are no objects xx or yy
  Deriv(~ xx^2+yy^2)
#- Should show
#- c(xx = 2 * xx, yy = 2 * yy)
#- ?? What is the meaning / purpose of this construct?
  
#- ?? Is following really AD?  
#- Automatic differentiation (AD), note intermediate variable 'd' assignment
Deriv(~{d <- ((x-m)/s)^2; exp(-0.5*d)}, "x")
# Note that the result we see does NOT match what follows in the example(Deriv) (JN ??)
#{
#   d <- ((x - m)/s)^2
#   .d_x <- 2 * ((x - m)/s^2)
#   -(0.5 * (.d_x * exp(-(0.5 * d))))
#}
#- For some reason the intermediate variable d is NOT included.??


#- Custom derivative rule. Note that this needs explanations??
  myfun <- function(x, y=TRUE) NULL #- do something useful
  dmyfun <- function(x, y=TRUE) NULL #- myfun derivative by x.
  drule[["myfun"]] <- alist(x=dmyfun(x, y), y=NULL) #- y is just a logical
  Deriv(myfun(z^2, FALSE), "z")
  # 2 * (z * dmyfun(z^2, FALSE))

#- Differentiation by list components
  theta <- list(m=0.1, sd=2.) #- Why do we set values??
  x <- names(theta) #- and why these particular names??
  names(x)=rep("theta", length(theta))
  Deriv(~exp(-(x-theta$m)**2/(2*theta$sd)), x, cache.exp=FALSE)
#- Should show the following (but why??)
#- c(theta_m = exp(-((x - theta$m)^2/(2 * theta$sd))) *
#-  (x - theta$m)/theta$sd, theta_sd = 2 * (exp(-((x - theta$m)^2/
#-  (2 * theta$sd))) * (x - theta$m)^2/(2 * theta$sd)^2))
lderiv <-  Deriv(~exp(-(x-theta$m)**2/(2*theta$sd)), x, cache.exp=FALSE)
fld <- function(x){ eval(lderiv)} #- put this in a function
fld(2) #- and evaluate at a value
```

**Deriv** has some design choices that can get the user into trouble. The following
example shows one such problem.

```{r chunk05}
library(Deriv)
rm(x) # ensures x is undefined
Deriv(~ x, "x")  # returns [1] 1 -- clearly a bug!
Deriv(~ x^2, "x")   # returns 2 * x
x <- quote(x^2)
Deriv(x, "x") # returns 2 * x
```

By comparison, **nlsr**

```{r chunk06}
rm(x) # in case it is defined
library(nlsr)
try(nlsDeriv(x, "x")  ) # fails, not a formula
try(nlsDeriv(as.expression("x"), "x")  ) # expression(NULL)
try(nlsDeriv(~x, "x")  ) # 1
try(nlsDeriv(x^2, "x"))   # fails
try(nlsDeriv(~x^2, "x")) # 2 * x
x <- quote(x^2)
try(nlsDeriv(x, "x")) # returns 2 * x
```


### Ryacas

There is at least one other symbolic package for R. Here we look at 
**Ryacas**. 
The structures for using yacas tools do not seem at the time of writing
(2016-10-21) to be suitable for working with nonlinear least squares or
optimization facilities of **R**. Thus, for the moment, we will not pursue
the derivatives available in `Ryacas` beyond the following example 
provided by Gabor Grothendieck.


```{r chunk07, eval = require(Ryacas)}
require(nlsr)
dnlsr <- nlsr::nlsDeriv(~ sin(x+y), "x")
print(dnlsr)
class(dnlsr)
detach("package:nlsr", unload=TRUE)
detach("package:Deriv", unload=TRUE)

## New Ryacas mechanism as of 2019-8-29 from mikl@math.aau.dk (Mikkel Meyer Andersen)

yac_str("D(x) Sin(x+y)") 

# or if an expression is needed:
ex <- yac_expr("D(x) Sin(x+y)")
ex
expression(cos(x + y))
eval(ex, list(x = pi, y = pi/2))

## Previous syntax for Ryacas was
## x <- Sym("x")
## y <- Sym("y")
## dryacas <- deriv(sin(x+y), x)
## print(dryacas)
## class(dryacas)

detach("package:Ryacas", unload=TRUE)
```



## Derivatives and simplifications -- base **R**

See specific notes either in comments or at the end of the section.

The help page for `D` lists the functions for which derivatives are
known:  "The internal code knows about the arithmetic operators `+`,
`-`, `*`, `/` and `^`, and the single-variable functions `exp`, `log`, `sin`, `cos`,
`tan`, `sinh`, `cosh`, `sqrt`, `pnorm`, `dnorm`, `asin`, `acos`, `atan`, `gamma`,
`lgamma`, `digamma` and `trigamma`, as well as `psigamma` for one or two
arguments (but derivative only with respect to the first)."

## Derivatives and simplifications -- package `nlsr`

This package supports the derivatives that `D` supports, as well
as a few others, and users can add their own definitions.  The current
list is
```{r chunk08}
ls(nlsr::sysDerivs)
```

### Derivatives table

Here is a slightly expanded testing of the elements of the `nlsr` derivatives
table.

```{r chunk09}
require(nlsr)
## Try different ways to supply the log function
aDeriv <- nlsDeriv(~ log(x), "x")
class(aDeriv)
aDeriv
aderiv <- try(deriv( ~ log(x), "x"))
class(aderiv)
aderiv
aD <- D(expression(log(x)), "x")
class(aD)
aD
cat("but \n")
try(D( "~ log(x)", "x")) # fails -- gives NA rather than expected answer due to quotes
try(D( ~ log(x), "x"))
interm <- ~ log(x)
interm
class(interm)
interme <- as.expression(interm)
class(interme)
try(D(interme, "x"))
try(deriv(interme, "x"))
try(deriv(interm, "x"))


nlsDeriv(~ log(x, base=3), "x" ) # OK
try(D(expression(log(x, base=3)), "x" )) # fails - only single-argument calls supported
try(deriv(~ log(x, base=3), "x" )) # fails - only single-argument calls supported
try(deriv(expression(log(x, base=3)), "x" )) # fails - only single-argument calls supported
try(deriv3(expression(log(x, base=3)), "x" )) # fails - only single-argument calls supported
fnDeriv(quote(log(x, base=3)), "x" )

nlsDeriv(~ exp(x), "x")
D(expression(exp(x)), "x") # OK
deriv(~exp(x), "x") # OK, but much more complicated
fnDeriv(quote(exp(x)), "x")

nlsDeriv(~ sin(x), "x")
D(expression(sin(x)), "x")
deriv(~sin(x), "x")
fnDeriv(quote(sin(x)), "x")

nlsDeriv(~ cos(x), "x")
D(expression(cos(x)), "x")
deriv(~ cos(x), "x")
fnDeriv(quote(cos(x)), "x")

nlsDeriv(~ tan(x), "x")
D(expression(tan(x)), "x")
deriv(~ tan(x), "x")
fnDeriv(quote(tan(x)), "x")

nlsDeriv(~ sinh(x), "x")
D(expression(sinh(x)), "x")
deriv(~sinh(x), "x")
fnDeriv(quote(sinh(x)), "x")

nlsDeriv(~ cosh(x), "x")
D(expression(cosh(x)), "x")
deriv(~cosh(x), "x")
fnDeriv(quote(cosh(x)), "x")

nlsDeriv(~ sqrt(x), "x")
D(expression(sqrt(x)), "x")
deriv(~sqrt(x), "x")
fnDeriv(quote(sqrt(x)), "x")

nlsDeriv(~ pnorm(q), "q")
D(expression(pnorm(q)), "q")
deriv(~pnorm(q), "q")
fnDeriv(quote(pnorm(q)), "q")

nlsDeriv(~ dnorm(x, mean), "mean")
D(expression(dnorm(x, mean)), "mean")
deriv(~dnorm(x, mean), "mean")
fnDeriv(quote(dnorm(x, mean)), "mean")

nlsDeriv(~ asin(x), "x")
D(expression(asin(x)), "x")
deriv(~asin(x), "x")
fnDeriv(quote(asin(x)), "x")

nlsDeriv(~ acos(x), "x")
D(expression(acos(x)), "x")
deriv(~acos(x), "x")
fnDeriv(quote(acos(x)), "x")

nlsDeriv(~ atan(x), "x")
D(expression(atan(x)), "x")
deriv(~atan(x), "x")
fnDeriv(quote(atan(x)), "x")

nlsDeriv(~ gamma(x), "x")
D(expression(gamma(x)), "x")
deriv(~gamma(x), "x")
fnDeriv(quote(gamma(x)), "x")

nlsDeriv(~ lgamma(x), "x")
D(expression(lgamma(x)), "x")
deriv(~lgamma(x), "x")
fnDeriv(quote(lgamma(x)), "x")

nlsDeriv(~ digamma(x), "x")
D(expression(digamma(x)), "x")
deriv(~digamma(x), "x")
fnDeriv(quote(digamma(x)), "x")

nlsDeriv(~ trigamma(x), "x")
D(expression(trigamma(x)), "x")
deriv(~trigamma(x), "x")
fnDeriv(quote(trigamma(x)), "x")

nlsDeriv(~ psigamma(x, deriv = 5), "x")
D(expression(psigamma(x, deriv = 5)), "x")
deriv(~psigamma(x, deriv = 5), "x")
fnDeriv(quote(psigamma(x, deriv = 5)), "x")

nlsDeriv(~ x*y, "x")
D(expression(x*y), "x")
deriv(~x*y, "x")
fnDeriv(quote(x*y), "x")

nlsDeriv(~ x/y, "x")
D(expression(x/y), "x")
deriv(~x/y, "x")
fnDeriv(quote(x/y), "x")

nlsDeriv(~ x^y, "x")
D(expression(x^y), "x")
deriv(~x^y, "x")
fnDeriv(quote(x^y), "x")

nlsDeriv(~ (x), "x")
D(expression((x)), "x")
deriv(~(x), "x")
fnDeriv(quote((x)), "x")

nlsDeriv(~ +x, "x")
D(expression(+x), "x")
deriv(~ +x, "x")
fnDeriv(quote(+x), "x")

nlsDeriv(~ -x, "x")
D(expression(- x), "x")
deriv(~ -x, "x")
fnDeriv(quote(-x), "x")

nlsDeriv(~ abs(x), "x")
try(D(expression(abs(x)), "x")) # 'abs' not in derivatives table
try(deriv(~ abs(x), "x"))
fnDeriv(quote(abs(x)), "x")

nlsDeriv(~ sign(x), "x")
try(D(expression(sign(x)), "x")) # 'sign' not in derivatives table
try(deriv(~ sign(x), "x"))
fnDeriv(quote(sign(x)), "x")
```

### Notes:

- the base tool `deriv` (and `deriv3`)  and 
`nlsr::codeDeriv` are intended to output an expression to compute a derivative. 
`deriv` generates an expression object, while `codeDeriv` will generate a language object. 
Note that input to `deriv` is of the form of a 
tilde expression with no left hand side, while `codeDeriv` is
more flexible:  quoted expressions, or length-1 expression vectors may also be used. 

- the base tool `D` and `nlsr::nlsDeriv` generate expressions, but `D` requires an
expression, while `nlsDeriv` can handle the expression without a wrapper. ?? Do we need 
to discuss more??

- **nlsr** includes `abs(x)` and `sign(x)` in the derivatives table despite conventional
wisdom that these are not differentiable. However, `abs(x)` clearly has a defined
derivative everywhere except at x = 0, where assigning a value of 0 to the 
derivative is almost certainly acceptable in computations. Similarly for `sign(x)`.

### Simplifying algebraic expressions
 
**nlsr** also includes some tools for simplification of algebraic expressions, extensible by the user.  Currently these
involve the following functions:
```{r chunk10}
ls(nlsr::sysSimplifications)
```

```{r chunk11}
#- Remove ##? to see reproducible error
#- ?? For some reason, if we leave packages attached, we get errors.
#- Here we detach all the non-base packages and then reload nlsr
##? require(nlsr)
##? sessionInfo()
##? ##? nlsSimplify(quote(+(a+b)))
##? nlsSimplify(quote(-5))
```


```{r chunk12}
sessionInfo()
if ("Deriv" %in% loadedNamespaces()){detach("package:Deriv", unload=TRUE)} 
if ("nlsr" %in% loadedNamespaces() ){detach("package:nlsr", unload=TRUE)}
if ("Ryacas" %in% loadedNamespaces() ){detach("package:Ryacas", unload=TRUE)}
require(nlsr)
#- require(Deriv)
#- require(stats)
#- Various simplifications
#- ?? Do we need quote() to stop attempt to evaluate before applying simplification

nlsSimplify(quote(+(a+b)))
nlsSimplify(quote(-5))
nlsSimplify(quote(--(a+b)))

nlsSimplify(quote(exp(log(a+b))))
nlsSimplify(quote(exp(1)))

nlsSimplify(quote(log(exp(a+b))))
nlsSimplify(quote(log(1)))

nlsSimplify(quote(!TRUE))
nlsSimplify(quote(!FALSE))

nlsSimplify(quote((a+b)))

nlsSimplify(quote(a + b + 0))
nlsSimplify(quote(0 + a + b))
nlsSimplify(quote((a+b) + (a+b)))
nlsSimplify(quote(1 + 4))

nlsSimplify(quote(a + b - 0))
nlsSimplify(quote(0 - a - b))
nlsSimplify(quote((a+b) - (a+b)))
nlsSimplify(quote(5 - 3))

nlsSimplify(quote(0*(a+b)))
nlsSimplify(quote((a+b)*0))
nlsSimplify(quote(1L * (a+b)))
nlsSimplify(quote((a+b) * 1))
nlsSimplify(quote((-1)*(a+b)))
nlsSimplify(quote((a+b)*(-1)))
nlsSimplify(quote(2*5))

nlsSimplify(quote((a+b) / 1))
nlsSimplify(quote((a+b) / (-1)))
nlsSimplify(quote(0/(a+b)))
nlsSimplify(quote(1/3))

nlsSimplify(quote((a+b) ^ 1))
nlsSimplify(quote(2^10))

nlsSimplify(quote(log(exp(a), 3)))

nlsSimplify(quote(FALSE && b))
nlsSimplify(quote(a && TRUE))
nlsSimplify(quote(TRUE && b))

nlsSimplify(quote(a || TRUE))
nlsSimplify(quote(FALSE || b))
nlsSimplify(quote(a || FALSE))

nlsSimplify(quote(if (TRUE) a+b))
nlsSimplify(quote(if (FALSE) a+b))

nlsSimplify(quote(if (TRUE) a+b else a*b))
nlsSimplify(quote(if (FALSE) a+b else a*b))
nlsSimplify(quote(if (cond) a+b else a+b))

nlsSimplify(quote(--(a+b)))
nlsSimplify(quote(-(-(a+b))))
```




## Derivatives and simplifications -- package `Deriv`

### Derivatives table

?? To be added


### Simplifications

```{r chunk13}
#- For some reason, if we leave packages attached, we get errors.
#- Here we detach all the non-base packages and then reload nlsr
sessionInfo()
if ("Deriv" %in% loadedNamespaces()){detach("package:Deriv", unload=TRUE)} 
if ("Deriv" %in% loadedNamespaces() ){detach("package:nlsr", unload=TRUE)}
if ("Deriv" %in% loadedNamespaces() ){detach("package:Ryacas", unload=TRUE)}
require(Deriv)
#- Various simplifications
#- ?? Do we need quote() to stop attempt to evaluate before applying simplification?

Simplify(quote(+(a+b)))
Simplify(quote(-5))
Simplify(quote(--(a+b)))

Simplify(quote(exp(log(a+b))))
Simplify(quote(exp(1)))

Simplify(quote(log(exp(a+b))))
Simplify(quote(log(1)))

Simplify(quote(!TRUE))
Simplify(quote(!FALSE))

Simplify(quote((a+b)))

Simplify(quote(a + b + 0))
Simplify(quote(0 + a + b))
Simplify(quote((a+b) + (a+b)))
Simplify(quote(1 + 4))

Simplify(quote(a + b - 0))
Simplify(quote(0 - a - b))
Simplify(quote((a+b) - (a+b)))
Simplify(quote(5 - 3))

Simplify(quote(0*(a+b)))
Simplify(quote((a+b)*0))
Simplify(quote(1L * (a+b)))
Simplify(quote((a+b) * 1))
Simplify(quote((-1)*(a+b)))
Simplify(quote((a+b)*(-1)))
Simplify(quote(2*5))

Simplify(quote((a+b) / 1))
Simplify(quote((a+b) / (-1)))
Simplify(quote(0/(a+b)))
Simplify(quote(1/3))

Simplify(quote((a+b) ^ 1))
Simplify(quote(2^10))

Simplify(quote(log(exp(a), 3)))

Simplify(quote(FALSE && b))
Simplify(quote(a && TRUE))
Simplify(quote(TRUE && b))

Simplify(quote(a || TRUE))
Simplify(quote(FALSE || b))
Simplify(quote(a || FALSE))

Simplify(quote(if (TRUE) a+b))
Simplify(quote(if (FALSE) a+b))

Simplify(quote(if (TRUE) a+b else a*b))
Simplify(quote(if (FALSE) a+b else a*b))
Simplify(quote(if (cond) a+b else a+b))

#- This one is wrong... the double minus is an error, yet it works ??.
Simplify(quote(--(a+b)))
#- By comparison
Simplify(quote(-(-(a+b))))
```

### Comparison with other approaches

?? to be added

### check modelexpr() works with an ssgrfun ??

?? to be added

### test model2rjfun vs model2rjfunx ??

?? to be added

### Need more extensive discussion of Simplify??

?? to be added

## Issues of programming on the language

?? need to explain where Deriv package comes from

One of the key tasks with tools for derivatives is that of taking objects
in one or other form (that is, **R** class) and using it as an input for
a symbolic function. The object may, of course, be an output from another
such function, and this is one of the reasons we need to do such 
transformations.

We also note that the different tools for symbolic derivatives use slightly
different inputs. For example, for the derivative of log(x), we have

```{r chunk14}
require(nlsr)
dlogx <- nlsr::nlsDeriv(~ log(x), "x")
str(dlogx)
print(dlogx)
```

Unfortunately, there are complications when we have an 
expression object, and 
we need to specify that we do NOT execute the *substitute()* function. 
Here we
show how to do this implicitly and with an explicit object.

```{r chunk15}
require(nlsr)
dlogxs <- nlsr::nlsDeriv(expression(log(x)), "x", do_substitute=FALSE)
str(dlogxs)
print(dlogxs)
cat(as.character(dlogxs), "\n")
fne <- expression(log(x))
dlogxe <- nlsr::nlsDeriv(fne, "x", do_substitute=FALSE)
str(dlogxe)
print(dlogxe)


# base R
dblogx <- D(expression(log(x)), "x")
str(dblogx)
print(dblogx)

require(Deriv)
ddlogx <- Deriv::Deriv(expression(log(x)), "x")
str(ddlogx)
print(ddlogx)
cat(as.character(ddlogx), "\n")
ddlogxf <- ~ ddlogx
str(ddlogxf)
```

?? do each example by all methods and by numDeriv and put in dataframe for later
presentation in a table.

Do we want examples in columns or rows. Probably 1 fn per row and work out
a name for the row that is reasonably meaningful. Probably want an index column
as well that is a list of strings. Can we then act on those strings to automate
the whole setup?


```{r chunk16}
require(nlsr)
# require(stats)
# require(Deriv)
# require(Ryacas)

# Various derivatives 

new <- codeDeriv(quote(1 + x + y), c("x", "y"))
old <- deriv(quote(1 + x + y), c("x", "y"))
print(new)
# Following generates a very long line on output of knitr (for markdown)
class(new)
str(new)
as.expression(new)
newf <- function(x, y){
   eval(new)
}
newf(3,5)

print(old)
class(old)
str(old)
oldf <- function(x,y){
    eval(old)
}
oldf(3,5)


```

Unfortunately, the inputs and outputs are not always easily transformed so that the
symbolic derivatives can be found. (?? Need to codify this and provide filters so we
can get things to work nicely.)

As an example, how could we take object **new** and embed it in a function we can then use
in **R**? We can certainly copy and paste the output into a function template, as follows,

```{r chunk17}
fnfromnew <- function(x,y){
    .value <- 1 + x + y
    .grad <- array(0, c(length(.value), 2L), list(NULL, c("x", 
    "y")))
    .grad[, "x"] <- 1
    .grad[, "y"] <- 1
    attr(.value, "gradient") <- .grad
    .value
}

print(fnfromnew(3,5))

```

However, we would ideally like to be able to automate this to generate functions and
gradients for nonlinear least squares and optimization calculations. The same criticism
applies to the object **old**

#### Another issue: 

If we have x and y set such that the function is not admissible, then 
both our old and new functions give a gradient that is seemingly reasonable. While the
gradient of this simple function could be considered to be defined for ANY values of x and
y, I (JN) am sure most users would wish for a warning at the very least in such cases.

```{r chunk18}
x <- NA
y <- Inf
print(eval(new))
print(eval(old))
```

####SafeD

We could define a way to avoid the issue of character vs. expression (and possibly
other classes) as follows:


```{r chunk19}
safeD <- function(obj, var) {
   # safeguarded D() function for symbolic derivs
   if (! is.character(var) ) stop("The variable var MUST be character type")
   if (is.character(obj) ) {
       eobj <- parse(text=obj)
       result <- D(eobj, var)
   } else {
       result <- D(obj, var)
   }
}

lxy2 <- expression(log(x+y^2))
clxy2 <- "log(x+y^2)"
try(print(D(clxy2, "y")))
print(try(D(lxy2, "y")))
print(safeD(clxy2, "y"))
print(safeD(lxy2, "y"))
```


## Indexed parameters or variables

Erin Hodgess on R-help in January 2015 raised the issue of taking the 
derivative of an expression that contains an indexed variable. We
show the example and its resolution, then give an explanation.
Note that indexed parameters in nonlinear regression models is an
open issue for package `nlsr`, largely because there does not seem
to be an agreed mechanism to specify models with indexed parameters.

```{r chunk20}
zzz <- expression(y[3]*r1 + r2)
try(deriv(zzz,c("r1","r2")))
require(nlsr)
try(nlsr::nlsDeriv(zzz, c("r1","r2")))
try(fnDeriv(zzz, c("r1","r2")))
newDeriv(`[`(x,y), stop("no derivative when indexing"))
try(nlsr::nlsDeriv(zzz, c("r1","r2")))
try(nlsr::fnDeriv(zzz, c("r1","r2")))
```

Richard Heiberger pointed out that internally, **R** stores

    y[3]

as

    "["(y,3)

that is, as a function. Duncan Murdoch pointed out the availability of
**nlsr** and the use of newDeriv() to redefine the "[" function for
the purposes of derivatives. 

This is not an ideal resolution, especially as we would like to be able
to get the gradients of functions with respect to vectors of parameters,
noted also by Sergei Sokol in the manual for package **Deriv**. The 
following examples illustrate this.

```{r chunk21}
try(nlsr::nlsDeriv(zzz, "y[3]"))
try(nlsr::nlsDeriv(y3*r1+r2,"y3"))
try(nlsr::nlsDeriv(y[3]*r1+r2,"y[3]"))
```

## References

---
title: "nlsr/nlsr Background, Development, and Discussion"
author: "John C. Nash"
date: "`r Sys.Date()`"
output: 
    pdf_document: 
      keep_md: TRUE
    toc: yes
bibliography: ImproveNLS.bib
vignette: >
  %\VignetteIndexEntry{nlsr Background, Development, Examples and Discussion}
  %\usepackage[utf8]{inputenc}
  %\VignetteEngine{knitr::rmarkdown}
---

---
<!-- Above creates horizontal line -->


```{r setup, echo=FALSE}
rm(list=ls()) # clear workspace for each major section
```

\pagebreak
## Overview and objectives

This extended vignette is to explain and to (partially) record the development 
and testing of **nlsr**, an R package to try to bring
the **R** function nls() up to date and to add capabilities for the
extension of the symbolic and automatic derivative tools in **R**.
As of 2022, it also records the upgrade to **nlsr**.

A particular goal in **nlsr** (and previously **nlsr**) is to attempt, 
wherever possible, to
use analytic or automatic derivatives. The function nls() generally uses a
rather weak forward derivative approximation, though a central difference
approximation is available if one uses advanced options.

A second objective 
is to use a Marquardt stabilization of the Gauss-Newton equations
to avoid the commonly encountered "singular gradient" failure
of nls(). This refers to the loss of rank of the Jacobian at the
parameters for evaluation. The particular stabilization also 
incorporates a very simple trick to avoid very small diagonal
elements of the Jacobian inner product, though in the present
implementations, this is accomplished indirectly.
See the section below **Implementation of method**

Two other objectives have arisen in 2022 in the move to a new **nlsr**:

- to allow for variations in the method of solving the Gauss-Newton
  equations, so that we may more easily test the performance of different
  approaches by changing control parameters to our programs. In practice,
  these changes have shown almost no substantive changes in performance; 
  if a Marquardt stabilization of any sort is used, it seems to provide 
  similar advantages over the simple Gauss-Newton approach of `nls()`.
  
- to employ the `roxygen2` approach to documenting routines. This choice does
  confer the advantage of consolidating the documentation (.Rd) files in the
  source code (.R) files, as well as building the NAMESPACE file more or less
  automatically. On the other hand, it brings some additional syntactic complications
  and the need to remember to `roxygenise()` the package before building and using
  it.

In preparing the **nlsr** package there was a sub-goal to unify, 
or at least render compatible, various
packages in **R** for the estimation or analysis of problems amenable
to nonlinear least squares solution. This was expanded in 2021 with
a Google Summer of Code initiative **Improvements to nls()** in which 
Arkajyoti Bhattacharjee was the contributor. Unfortunately, while we made
some progress, we were NOT able to overcome some of the densely entangled
code sufficiently to make more than limited improvements.

A large part of the work for the `nlsr` package family -- particularly the parts
concerning derivatives and R language structures -- was initially
carried out by Duncan
Murdoch in 2014. Without his input, much of the capability of the package
would not exist, even though there was an earlier 2012 package `nlmrt`
(@jnnlmrt2012). That package was clumsily written, but did show the possibilities
for automatically providing analytic Jacobian information to a nonlinear
regression package.

The `nlsr` package and this vignette are works in progress, and assistance
and examples are welcome. Note that there is similar work on symbolic
derivatives in the 
package Deriv (Andrew Clausen and Serguei Sokol (2015) Symbolic 
Differentiation, version 2.0, https://cran.r-project.org/package=Deriv),
and making the current work "play nicely" with that package is 
desirable. There are also aspects of `nls()` in base R and the package
`minpack.lm` (@minpacklm12) which could be better aligned. Much more
on `nls()` in particular is in process with Arkajyoti Bhattacharjee in
mid 2022.

As a mechanism for highlighting issues that remain to be resolved, I have
put double question marks (??) where I believe attention is needed in this
document.

## 0. Issues remaining to address and TODOS

Issues are related to concerns about one or more of the nonlinear modeling tools
as revealed by tests and examples used in this vignette. I have labeled some of 
these as MINOR, and regard these as issues that could be fixed easily.

### nls() uses different models with different algorithm choices

In Section 7. under Partially Linear Models, `nls()` uses different model forms
according to the choice of `algorithm` in some cases where the model can be 
partially linear. I believe there should at least be a WARNING about this
possibility, though preferably I would like to see this treated as an error
in the code. This is discussed in more detail in the paper "Refactoring nls()"
(@JNABRefactoringNLS22).

### MINOR -- Bounds specification and warnings for nlsLM and nls.lm

`nls()`, `nlsr::nlxb` and `nlsr::nlfb` allow a single value to be given for the
`lower` and `upper` bounds. This single value is expanded to a vector of length 
equal to the number of parameters, but the `minpack.lm` routines are more
fussy. 

Also, while `nls()` and the `nlsr` functions warn of initial starting parameters that
violate specified bounds, the `minpack.lm` routines do not. This is easily
fixable with a `warning()`.

These are illustrated below.

```{r nlsbtsimple, echo=TRUE}
# Bounds Test nlsbtsimple.R (see BT.RES in Nash and Walker-Smith (1987))
rm(list=ls())
bt.res<-function(x){ x }
bt.jac<-function(x){nn <- length(x); JJ <- diag(nn); attr(JJ, "gradient") <- JJ;  JJ}
n <- 4
x<-rep(0,n) ; lower<-rep(NA,n); upper<-lower # to get arrays set
for (i in 1:n) { lower[i]<-1.0*(i-1)*(n-1)/n; upper[i]<-1.0*i*(n+1)/n }
x <-0.5*(lower+upper) # start on mean
xnames<-as.character(1:n) # name our parameters just in case
for (i in 1:n){ xnames[i]<-paste("p",xnames[i],sep='') }
names(x) <- xnames
require(minpack.lm)
require(nlsr)
bsnlf0<-nlfb(start=x, resfn=bt.res, jacfn=bt.jac) # unconstrained
bsnlf0
bsnlm0<-nls.lm(par=x, fn=bt.res, jac=bt.jac) # unconstrained
bsnlm0
bsnlf1<-nlfb(start=x, resfn=bt.res, jacfn=bt.jac, lower=lower, upper=upper)
bsnlf1
bsnlm1<-nls.lm(par=x, fn=bt.res, jac=bt.jac, lower=lower, upper=upper)
bsnlm1

# single value bounds
bsnlf2<-nlfb(start=x, resfn=bt.res, jacfn=bt.jac, lower=0.25, upper=4)
bsnlf2
# nls.lm will NOT expand single value bounds
bsnlm2<-try(nls.lm(par=x, fn=bt.res, jac=bt.jac, lower=0.25, upper=4))
```

For example (from the file `nlsbtsimple.R`):

### MINOR -- nlsLM and nls.lm do not warn of out of bounds initial parameters

```{r nlsLMoob, echo=TRUE}
x<-rep(0,n) # resetting to this puts x out of bounds
cat("lower:"); print(lower)
cat("upper:"); print(upper)
names(x) <- xnames # to ensure names set
obnlm<-nls.lm(par=x, fn=bt.res, jac=bt.jac, lower=lower, upper=upper)
obnlm
```

For nonlinear regression with `minpack.lm::nlsLM` there is also no warning.
The example using file `Hobbsbdformula1.R` in the vignette
**R: Examples of different nonlinear least squares calculations** (file
`NLS-Examples.Rmd`) shows this. However, in this case, the program actually
gets a good answer, despite the failure to warn. In a start from all 1's, 
which is feasible, a poor answer is obtained, as noted in the next section.

### Bounded minimization with minpack.lm

While minpack.lm mentions lower and upper bounds in the manual pages, there are
no examples in the package (that I could find) of their application. For the file
`nlsbtsimple.R`, we get acceptable answers. For the Hobbs problem, from a
start of all 1's, both `nlsLM` and `nls.lm` converge to sums of
squares higher than `nlxb`, `nlfb` or `nls()` using the "port"
algorithm (when the last is able to
get started). Moreover, in one case for the scaled Hobbs problem, the
two `minpack.lm` functions give different results. I have not yet been able to
understand how these routines apply bounds constraints. There is mention in 
\url{https://deepwiki.com/search/minpack_7e65b1cf-b3b1-4896-b00e-6453517b4fa8?mode=fast} 
that the original
MINPACK did NOT cater for bounds constraints on parameters. MINUIT,
which uses the MINPACK ideas, used a transformation of the parameters to 
accomplish this. (Hans Werner Borchers uses the same idea in the the 
code `dfoptim::nmkb`, which he calls a **transfinite transformation**.)

**Comments**: 

1) the transfinite idea is useful in that it may be applicable
quite generally. If a wrapper for unconstrained minimizers could be devised, 
it would enlarge the capability of a number of R tools. 

2) The Hobbs problem, even when scaled, may be unscaled again by such
a transformation
of the parameters. 

I have seen many cases where methods that are generally 
reliable give an occasional unsatisfactory result. However, it would be useful to know 
the precise reasons why or where `minpack.lm` routines are obtaining these results,
since it may be possible to either fix the issues or else provide some warnings
or diagnostics, which hopefully would be of wider application.

### TODOS

- sysDerivs and sysSimplifications are new environments whose parent is emptyenv().
  Why? This should be better explained with motivations.

- R function `optim()` and function `optimr()` in package `optimx` include the control
  `parscale` which is a vector of scaling factors so that optimization is performed
  on `par/parscale` where `par` are the user-supplied parameters. The intent is to
  have the internal scaled parameters of similar general size. The `Hobbssetup`
  script below carries out this scaling explicitly. Providing a `parscale` capability
  for `nlxb()` and `nlfb()` is mostly a matter of effort. At the time of writing
  (August 2022), I feel that the performance of the functions is good and adding 
  `parscale` is not an urgent need.
  
- `nls()` does not insist that the `formula` argument in its call is of the structure
   where the left hand side is "simple", that is, usually a single variable. However,
   it is difficult to find examples where this is not the case, though it is NOT a
   requirement of the nonlinear least squares algorithms. However, to get the right
   features for more complicated modelling formulas, I need collaboration with 
   practitioners.

- Data can be passed to `nls()` using variables already in the working environment
  or in the `data` argument, which can be a dataframe, a list or an environment,
  but not a matrix. Why can a matrix of data not be used?
  
- How can VARPRO / conditional linearity be married to the nlsr algorithms?

- A long-term need is a better, more consistent way to specify formulas for 
  nonlinear models. At the moment `nlsr` does not support indexed parameters,
  and `nls()` does so in an awkward way. The output parameters do not match the
  input specification in that they are not indexed using the traditional square
  brackets, but build the index value into the parameter name.

## 1. Summary of capabilities and functions in `nlsr`

Throughout this exposition, we have chosen to set `trace` variables
to `FALSE` to reduce the volume of output. Changing such values to 
`TRUE` will expand output.

We will illustrate many of the capabilities with the Hobbs weed 
problem (@jncnm79, page 121). This can be set up in a scaled or
unscaled form.

```{r Hobbssetup, echo=TRUE}
## Use the Hobbs Weed problem
weed <- c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443,
          38.558, 50.156, 62.948, 75.995, 91.972)
tt <- 1:12
weeddf <- data.frame(y=weed, tt=tt)
st <- c(b1=1, b2=1, b3=1) # a default starting vector (named!)
## Unscaled model
wmodu <- y ~ b1/(1+b2*exp(-b3*tt))
## Scaled model
wmods <- y ~ 100*b1/(1+10*b2*exp(-0.1*b3*tt))
## We can provide the residual and Jacobian as functions
# Unscaled Hobbs problem
hobbs.res  <-  function(x){ # scaled Hobbs weeds problem -- residual
  # This variant uses looping
  if(length(x) != 3) stop("hobbs.res -- parameter vector n!=3")
  y  <-  c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443, 
           38.558, 50.156, 62.948, 75.995, 91.972)
  tt  <-  1:12
  res  <-  x[1]/(1+x[2]*exp(-x[3]*tt)) - y
}

hobbs.jac  <-  function(x) { # scaled Hobbs weeds problem -- Jacobian
  jj  <-  matrix(0.0, 12, 3)
  tt  <-  1:12
  yy  <-  exp(-x[3]*tt)
  zz  <-  1.0/(1+x[2]*yy)
  jj[tt,1]   <-   zz
  jj[tt,2]   <-   -x[1]*zz*zz*yy
  jj[tt,3]   <-   x[1]*zz*zz*yy*x[2]*tt
  attr(jj, "gradient") <- jj
  jj
}
# Scaled Hobbs problem
shobbs.res  <-  function(x){ # scaled Hobbs weeds problem -- residual
  # This variant uses looping
  if(length(x) != 3) stop("shobbs.res -- parameter vector n!=3")
  y  <-  c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443, 
           38.558, 50.156, 62.948, 75.995, 91.972)
  tt  <-  1:12
  res  <-  100.0*x[1]/(1+x[2]*10.*exp(-0.1*x[3]*tt)) - y
}

shobbs.jac  <-  function(x) { # scaled Hobbs weeds problem -- Jacobian
  jj  <-  matrix(0.0, 12, 3)
  tt  <-  1:12
  yy  <-  exp(-0.1*x[3]*tt)
  zz  <-  100.0/(1+10.*x[2]*yy)
  jj[tt,1]   <-   zz
  jj[tt,2]   <-   -0.1*x[1]*zz*zz*yy
  jj[tt,3]   <-   0.01*x[1]*zz*zz*yy*x[2]*tt
  attr(jj, "gradient") <- jj
  jj
}
```

### nlxb

This is the likely the function most called by users from `nlsr`.

-   Given a nonlinear model expressed as an expression of the form
    `         lhs ~ formula_for_rhs    `
   and a start vector where parameters used in the model formula are named,
   attempts to find the minimum of the residual sum of squares using as
   a default method the Nash variant (Nash, 1979) of the Marquardt algorithm, 
   where the linear sub-problem is solved by a qr method. The process is to 
   develop the residual and Jacobian functions using `model2rjfun`, 
   then call `nlfb`.

```{r exnlxb}
require(nlsr)
# Use Hobbs scaled formula model
anlxb1  <-  try(nlxb(wmods, start=st, data=weeddf))
print(anlxb1)
# summary(anlxb1) # for different output
# Test without starting parameters
anlxb1nostart  <-  try(nlxb(wmodu, data=weeddf))
print(anlxb1nostart)
```

Here we see that the Jacobian at the solution is essentially of rank 1, even 
though there are 3
coefficients. It is therefore not surprising that `nls()`, which does not benefit from the
Levenberg-Marquardt stabilization in solving the nonlinear least squares program fails
for this problem. See the subsection below for `wrapnlsr`. Note that the singular values
of the Jacobian computed via a numerical approximation are more extreme (i.e., nearly
singular) than those determined by analytic derivatives in the preceding solution
`anlxb1`. For this example, there is a clear advantage to the analytic derivatives.

While there is an almost liturgical adherence to setting up models where the dependent 
(or predicted) variable is on the left hand side (LHS) of the model formula and 
the independent (or predictor) variable(s) on the right hand side (RHS), this is  
NOT an actual requirement in most cases, though there are some situations such 
as the `minpack.lm` function `wfct` that do assume this structure. Here is an
example of solving the Hobbs unscaled problem using a 1-sided model formula.

```{r nlxb1side1}
# One-sided unscaled Hobbs weed model formula
wmodu1  <-  ~ b1/(1+b2*exp(-b3*tt)) - y
anlxb11  <-  try(nlxb(wmodu1, start=st, data=weeddf))
print(anlxb11)
```

### model2rjfun, model2ssgrfun, modelexpr

- These functions create functions to evaluate residuals or sums of 
squares at particular parameter locations. `model2rjfun` is the key
call in `nlxb()` to estimate models specified as expressions or 
formulas.

```{r c003}
# st  <-  c(b1=1, b2=1, b3=1)
wrj <- model2rjfun(wmods, st, data=weeddf)
wrj
weedux <- nlxb(wmods, start=st, data=weeddf) 
print(weedux)

wss <- model2ssgrfun(wmods, st, data=weeddf)
print(wss)

# We can get expressions used to calculate these as follows:
wexpr.rj <- modelexpr(wrj) 
print(wexpr.rj)

wexpr.ss <- modelexpr(wss) 
print(wexpr.ss)
```

<!-- ### rjfundoc -->

<!-- This function was removed from the `nlsr` package in late 2020. Back in 2022. -->
<!-- What is its function? Why include or leave out? -->

<!-- ```{r} -->
<!-- rjfundoc <- function(fun, savefile=NULL) { -->
<!--   efun <- environment(fun) # get the environment with data, expression, etc -->
<!--   avn <- all.vars(efun$modelformula) # vars and parameters -->
<!--   pnames <- names(efun$pvec) # assumes that vector is named (normal) -->
<!-- #? what do we do when it is not -- need to name it p1, p2, etc. -->
<!-- # DJM:  the model2rjfun* functions do this; if a user creates fun some other way, -->
<!-- # they'd better do it too! -->
<!--   iprm <- match(pnames, avn) -->
<!--   if (length(iprm)) -->
<!--     notprm <- avn[-iprm] -->
<!--   else -->
<!--     notprm <- avn -->
<!--   funname <- deparse(match.call()[[2]]) -->
<!--   data <- efun$data -->
<!--   # data is an environment, but not all variables are in the top  -->
<!--   # level, so use get() to find them -->
<!--   modeldata <- mget(notprm, envir=data, inherits=TRUE, ifnotfound=list(function(n) NULL)) -->
<!--   notdata <- setdiff(notprm, names(modeldata)) -->
<!--   resids <- fun(efun$pvec) -->
<!--   n <- length(resids) -->
<!--   islengthn <- sapply(modeldata, function(col) length(col) == n) -->
<!--   result <- structure(list(funname=funname, modelformula=efun$modelformula, -->
<!--                            modelexpr=modelexpr(fun), n=n, -->
<!--                            pvec=efun$pvec, data=as.data.frame(modeldata[islengthn]),  -->
<!--                            extradata=modeldata[!islengthn], unknown = notdata), -->
<!--                       class = "rjfundoc") -->
<!--   if (!is.null(savefile)) { -->
<!--     sink(savefile) -->
<!--     print(result) -->
<!--     sink() -->
<!--   } -->
<!--   result -->
<!-- } -->

<!-- print.rjfundoc <- function(x, ...) { -->
<!--   cat("FUNCTION", x$funname, "\n") -->
<!--   cat("Formula:\t") -->
<!--   print(x$modelformula) -->
<!--   cat("Code:\t\t") -->
<!--   print(x$modelexpr) -->
<!--   cat("Parameters:\t", paste(names(x$pvec), collapse=", "), "\n") -->
<!--   cat("Data:\t\t", paste(names(x$data), collapse=", "), "\n") -->
<!--   if (length(x$extradata)) -->
<!--     cat("Extra:\t\t", paste(names(x$extradata), collapse=", "), "\n") -->
<!--   if (length(x$unknown))  -->
<!--     cat("Unknown symbols:\t", paste(x$unknown, collapse=", "), "\n") -->
<!--   cat("\nVALUES\n") -->
<!--   cat("Observations:\t", x$n, "\n") -->
<!--   cat("Parameters:\n") -->
<!--   print(x$pvec) -->
<!--   if (length(x$data)) { -->
<!--     cat("Data (length ", x$n, "):\n", sep="") -->
<!--     print(x$data) -->
<!--   } -->
<!--   if (length(x$extradata)) { -->
<!--     cat("Extra:\n") -->
<!--     print(x$extradata) -->
<!--   } -->
<!--   x -->
<!-- } -->

<!-- wrjdoc <- rjfundoc(wrj) -->
<!-- print(wrjdoc) -->
<!-- ``` -->

### nlfb -- minimize nonlinear least squares residual functions

- Given a nonlinear model expressed as an expression of the form of a function
   that computes the residuals from the model and a start vector `par` , tries
   to minimize the nonlinear sum of squares of these residuals w.r.t. `par`. 
   If `model(par, mydata)` computes
   an a vector of numbers that are presumed to be able to fit data `lhs`, then
   the residual vector is `(model(par,mydata) - lhs)`,
   though traditionally we write the negative of this vector. (Writing it this
   way allows the derivatives of the residuals w.r.t. the parameters `par` to be
   the same as those for `model(par,mydata)`.)  `nlfb` tries to minimize the sum
   of squares of the residuals with respect to the parameters. 
  
- The method takes a parameter `jacfn` which returns the Jacobian matrix of
   derivatives of the residuals w.r.t. the parameters in an attribute `gradient`.
   If `jacfn` is missing, then a numerical approximation to derivatives can be
   used if the control `japprox` is appropriately specified. Valid choices for
   approximations are `jafwd`, `jaback`, `jacentral` and `jand` for forward, 
   backward, central and package `numDeriv` difference methods. There is also
   the choice `SSJac`, which is not necessarily an approximation, but gradient
   code within a selfStart model function. (CAUTION: There is no check that such 
   code is actually present!)
   The Jacobian is stored in the attribute `gradient` of the residual allowing 
   us to combine the computation of the residual 
   and Jacobian in the same code. That is, we can specify the same R function for
   both `resfn` and `jacfn`. Since there are generally common computations, this may
   give a small improvement in efficiency, though it
   does make the setup slightly more complicated. See the example below.
   However, the main reason for this
   choice was to allow `nlxb` to be more easily coded. 

- The start vector preferably uses named parameters (especially if there is an 
   underlying formula). The attempted minimization of the sum of squares
   uses the Nash variant @jn77ima, @jncnm79, of the Marquardt algorithm, where the linear 
   sub-problem is solved by a qr method. We explain how this is done later,
   as well as giving a short discussion of the relative offset convergence
   criterion.

```{r c004}
require(nlsr)

cat("try nlfb\n")
st  <-  c(b1=1, b2=1, b3=1)
ans1 <- nlfb(st, shobbs.res, shobbs.jac, trace=FALSE)
summary(ans1)

## No jacobian function -- use internal approximation
ans1n <- nlfb(st, shobbs.res, trace=FALSE, control=list(japprox="jafwd", watch=FALSE)) # NO jacfn -- tell it fwd approx
print(ans1n)
## difference
coef(ans1)-coef(ans1n)
```

### coef.nlsr 

- extracts and displays the coefficients for a model 
    estimated by nlxb() or nlfb() in the nlsr structured object. 

```{r  c001}
coef(anlxb1) # this is solution of scaled problem from unit start
```

### print.nlsr

- Print summary output (but involving some serious computations!) of
an object of class nlsr from `nlxb` or `nlfb` from package
`nlsr`.

```{r c006}
### From examples above
print(weedux)
print(ans1)
```

### summary.nlsr

- Provide a summary output (but involving some serious computations!) of an object of
class nlsr from nlxb or nlfb from package nlsr. Also available for nls().
Note that this gives a rather lengthy output, so calling it a summary is a bit of a stretch.

```{r c006a}
### From examples above
summary(weedux)
summary(ans1)
```

### wrapnlsr

- Given a nonlinear model expressed as an expression of the form
  `  lhs ~ formula_for_rhs  `
and a start vector where parameters used in the model formula are named, attempts
to find the minimum of the residual sum of squares using the Nash variant 
@jncnm79 of the Marquardt algorithm, where the linear sub-problem is 
solved by a qr method.

A particular purpose of this function is to create the `nls`-style model object
for a problem when the solution has been obtained by `nlsr::nlxb`. `minpack.lm::nlsLM`
creates a structure that is parallel to that from `nls()`.

```{r c009, echo=TRUE}
## The following attempt at Hobbs unscaled with nls() fails!
rm(list=ls()) # clear before starting
weed <- c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443,
          38.558, 50.156, 62.948, 75.995, 91.972)
tt <- 1:12
weeddf <- data.frame(tt, weed)
st <- c(b1=1, b2=1, b3=1) # a default starting vector (named!)
wmodu<- weed ~ b1/(1 + b2 * exp(- b3 * tt))
anls1u  <-  try(nls(wmodu, start=st, trace=FALSE, data=weeddf)) # fails
## But we succeed by calling nlxb first.
library(nlsr)
anlxb1un  <-  try(nlxb(wmodu, start=st, trace=FALSE, data=weeddf))
print(anlxb1un)
st2 <- coef(anlxb1un) # We try to start nls from solution using nlxb
anls2u  <-  try(nls(wmodu, start=st2, trace=FALSE, data=weeddf))
print(anls2u)
## Or we can simply call wrapnlsr FROM THE ORIGINAL start
anls2a  <-  try(wrapnlsr(wmodu, start=st, trace=FALSE, data=weeddf))
summary(anls2a)
# # For comparison could run nlsLM
# require(minpack.lm)
# anlsLM1u  <-  try(nlsLM(wmodu, start=st, trace=FALSE, data=weeddf))
# print(anlsLM1u)
```

### resgr, resss

- For a nonlinear model originally expressed as an expression of the form
    `     lhs ~ formula_for_rhs `
   assume we have a resfn and jacfn that compute the residuals and the 
   Jacobian at a set of parameters. This routine computes the gradient, 
   that is, t(Jacobian) %*% residuals. 

```{r c007, echo=TRUE}
## Use shobbs example -- Scaled Hobbs problem
shobbs.res  <-  function(x){ # scaled Hobbs weeds problem -- residual
  if(length(x) != 3) stop("shobbs.res -- parameter vector n!=3")
  y  <-  c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443, 
           38.558, 50.156, 62.948, 75.995, 91.972)
  tt  <-  1:12
  res  <-  100.0*x[1]/(1+x[2]*10.*exp(-0.1*x[3]*tt)) - y
}

shobbs.jac  <-  function(x) { # scaled Hobbs weeds problem -- Jacobian
  jj  <-  matrix(0.0, 12, 3)
  tt  <-  1:12
  yy  <-  exp(-0.1*x[3]*tt)
  zz  <-  100.0/(1+10.*x[2]*yy)
  jj[tt,1]   <-   zz
  jj[tt,2]   <-   -0.1*x[1]*zz*zz*yy
  jj[tt,3]   <-   0.01*x[1]*zz*zz*yy*x[2]*tt
  attr(jj, "gradient") <- jj
  jj
}
RG <- resgr(st, shobbs.res, shobbs.jac)
RG
SS <- resss(st, shobbs.res)
SS
```

### nlsDeriv, codeDeriv, fnDeriv, newDeriv

These functions are mainly for test and development use.

- Functions **Deriv** and **fnDeriv** are designed as replacements 
for the stats package functions **D** and **deriv**
respectively, though the argument lists do not match exactly.
**newDeriv** allows additional analytic derivative expressions to 
be added. The following is an expanded and commented version of the 
examples from the manual for these functions.

```{r c002}
newDeriv() # a call with no arguments returns a list of available functions
           # for which derivatives are currently defined
newDeriv(sin(x)) # a call with a function that is in the list of available derivatives
                 # returns the derivative expression for that function
nlsDeriv(~ sin(x+y), "x") # partial derivative of this function w.r.t. "x"

## CAUTION !! ##
newDeriv(joe(x)) # but an undefined function returns NULL
newDeriv(joe(x), deriv=log(x^2)) # We can define derivatives, though joe() is meaningless.
nlsDeriv(~ joe(x+z), "x")

# Some examples of usage
f <- function(x) x^2
newDeriv(f(x), 2*x*D(x))
nlsDeriv(~ f(abs(x)), "x")
nlsDeriv(~ x^2, "x")
nlsDeriv(~ (abs(x)^2), "x")

# derivatives of distribution functions
nlsDeriv(~ pnorm(x, sd=2, log = TRUE), "x")
# get evaluation code from a formula
codeDeriv(~ pnorm(x, sd = sd, log = TRUE), "x")
# wrap it in a function call
fnDeriv(~ pnorm(x, sd = sd, log = TRUE), "x")
f <- fnDeriv(~ pnorm(x, sd = sd, log = TRUE), "x", args = alist(x =, sd = 2))
f
f(1)
100*(f(1.01) - f(1))  # Should be close to the gradient
                      # The attached gradient attribute (from f(1.01)) is
                      # meaningless after the subtraction.
```

### nlsSimplify and related functions

- Simplifications to render derivative expresssions more usable.

The related tools are: `newSimplification, sysSimplifications, isFALSE, isZERO, isONE, isMINUSONE, isCALL, findSubexprs, sysDerivs`.

`nlsSimplify` simplifies expressions according to rules specified
by `newSimplification`.

`findSubexprs` finds common subexpressions in an expression vector
so that duplicate computation can be avoided.

```{r c008}
## nlsSimplify
nlsSimplify(quote(a + 0))
nlsSimplify(quote(exp(1)), verbose = TRUE)

nlsSimplify(quote(sqrt(a + b)))  # standard rule
## sysSimplifications
# creates a new environment whose parent is emptyenv()  Why?
str(sysSimplifications)
myrules <- new.env(parent = sysSimplifications)
## newSimplification
newSimplification(sqrt(a), TRUE, a^0.5, simpEnv = myrules)
nlsSimplify(quote(sqrt(a + b)), simpEnv = myrules)

## isFALSE
print(isFALSE(1==2))
print(isFALSE(2==2))

## isZERO
print(isZERO(0))
x <- -0
print(isZERO(x))
x <- 0
print(isZERO(x))
print(isZERO(~(-1)))
print(isZERO("-1"))
print(isZERO(expression(-1)))


## isONE
print(isONE(1))
x <- 1
print(isONE(x))
print(isONE(~(1)))
print(isONE("1"))
print(isONE(expression(1)))


## isMINUSONE
print(isMINUSONE(-1))
x <- -1
print(isMINUSONE(x))
print(isMINUSONE(~(-1)))
print(isMINUSONE("-1"))
print(isMINUSONE(expression(-1)))

## isCALL  ?? don't have good understanding of this
x <- -1
print(isCALL(x,"isMINUSONE"))
print(isCALL(x, quote(isMINUSONE)))
      
## findSubexprs
findSubexprs(expression(x^2, x-y, y^2-x^2))

## sysDerivs
# creates a new environment whose parent is emptyenv()  Why? 
# Where are derivative definitions are stored?
str(sysDerivs)
```


## 2. Analytic versus approximate Jacobians

A key goal of package `nlsr` was to be able to use analytic or symbolic derivatives
for the Jacobian in nonlinear least squares computations, in particular when the model
is specified as a formula or expression. In this `nlsr::nlxb()` has been quite successsful.
It should be pointed out that the principal advantage of using analytic derivatives is 
that we get a more assured measure of the "flatness" of the sum of squares surface
at the termination point. My experience is that there is no particular gain in 
the speed in getting to that termination point.

A disadvantage of the approach is that specifying a "formula" that includes an R
function will (usually) fail. `nls()`, because it defaults to using a derivative
approximation, can accept
formulas that include R functions, and indeed, one of the examples in the manual
page `nls.Rd` is of this type. In package `nlsr` we have introduced the possibility
of using Jacobian approximations via the control element `japprox` which is 
discussed in several places below.

A second goal of including approximations for the Jacobian is to be able to specify
or control the approximation. `nls()`, as shown in Appendix A, has a rather 
complicated code involving both R and C to compute a forward difference approximation
to the Jacobian. In this computation, each parameter is adjusted by its absolute value 
times the square root of the machine precision (double). Call this the `ndstep`. So 
the parameter `delta` is the absolute value of the parameter times approximately 
1.5e-8. If the parameter is zero, then the `delta` is `ndstep`. 
In `nlsr::jafwd()`, I use a `delta` of `ndstep` times the absolute value
of the parameter PLUS the `ndstep`. This avoids the check for a zero parameter.

It is known (https://en.wikipedia.org/wiki/Finite_difference) that the forward (and backward)
difference approximations are not ideal. Central differences and higher approximations such
as those found in the CRAN package `numDeriv` are better, and it is desirable to be able
to specify that these be used.

### Specifying approximations to nlfb

We first look at the `nlfb()` function that uses R functions for the residual and
Jacobian. Borrowing from the mechanism used in `optimx::optimr()`, we can invoke
an approximation by putting the name of an appropriate R function in quotation
marks. In `nlsr` there are four functions `jafwd()`, `jaback`, `jacentral`, and
`jand` for the forward, backward, central and `numDeriv` (default) approximations.
Moreover, the `ndstep` can be set in the `control()` list in the `nlfb()` call.
Its default value is 1e-7. An open question is whether to change to the value
`sqrt(.Machine$double.eps)` to more closely match `nls()`. 

```{r nlfbnumex, echo=TRUE}
## Test with functional spec. of problem
## Default call WITH jacobian function
ans1 <- nlfb(st, resfn=shobbs.res, jacfn=shobbs.jac)
ans1
## No jacobian function -- and no japprox control setting
ans1n <- try(nlfb(st, shobbs.res)) # NO jacfn
ans1n
## Force jafwd approximation
ans1nf <- nlfb(st, shobbs.res, control=list(japprox="jafwd")) # NO jacfn, but specify fwd approx
ans1nf
## Alternative specification
ans1nfa <- nlfb(st, shobbs.res, jacfn="jafwd") # NO jacfn, but specify fwd approx in jacfn
ans1nfa
## Coeff differences from analytic: 
ans1nf$coefficients-ans1$coefficients
## Force jacentral approximation
ans1nc <- nlfb(st, shobbs.res, control=list(japprox="jacentral")) # NO jacfn
ans1nc
## Coeff differences from analytic:
ans1nc$coefficients-ans1$coefficients
## Force jaback approximation
ans1nb <- nlfb(st, shobbs.res, control=list(japprox="jaback")) # NO jacfn
ans1nb
## Coeff differences from analytic:
ans1nb$coefficients-ans1$coefficients
## Force jand approximation
ans1nn <- nlfb(st, shobbs.res, control=list(japprox="jand"), trace=FALSE) # NO jacfn
ans1nn
## Coeff differences from analytic:
ans1nc$coefficients-ans1$coefficients
```

### Specifying Jacobian approximations to nlxb

Since `nlxb()` provides the model as a formula or expression, we need to tell 
it how to get an approximate Jacobian. First, we can specify WHICH approximation
to use by putting the name of the jacobian approximating function in the 
control list element `japprox` in quotation marks. 

The default mechanism for using `nlxb()` is to create a function `trjfn` (by calling
`model2rjfun()` ). To make our work a lot easier, `trjfn` is used as BOTH the residual
and Jacobian function by copying the created Jacobian matrix into the "gradient" attribute
of the returned object from `trjfn`. 

**NOTE:** This requirement that the Jacobian matrix be returned in the "gradient"
attribute of the returned object of the `jacfn` specified in the call to `nlfb()`
is one that users should be careful to observe.

When we specify a Jacobian approximation (via `control$japprox`) to `nlxb()`, the
call to `model2rjfun` is made with `jacobian=FALSE` and the appropriate function for
the approximation is supplied in the subsequent call the `nlfb()` to minimize the
sum of squared objective. The `jacobian` parameter in the call to `model2rjfun()`
defaults to TRUE.

```{r nlxbnumex1, echo=TRUE}
## rm(list=ls()) # clear workspace for each major section
## Use the Hobbs Weed problem
weed <- c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443,
          38.558, 50.156, 62.948, 75.995, 91.972)
tt <- 1:12
weeddf <- data.frame(y=weed, tt=tt)
st1 <- c(b1=1, b2=1, b3=1) # a default starting vector (named!)
wmods <- y ~ 100*b1/(1+10*b2*exp(-0.1*b3*tt)) ## Scaled model
print(wmods)
anlxb1  <-  try(nlxb(wmods, start=st1, data=weeddf))
print(anlxb1)

anlxb1fwd  <-  (nlxb(wmods, start=st1, data=weeddf,
    control=list(japprox="jafwd")))
print(anlxb1fwd)

## fwd - analytic
print(anlxb1fwd$coefficients - anlxb1$coefficients)

anlxb1bak  <- (nlxb(wmods, start=st1, data=weeddf,
    control=list(japprox="jaback")))
print(anlxb1bak)
## back - analytic
print(anlxb1bak$coefficients - anlxb1$coefficients)

anlxb1cen  <- (nlxb(wmods, start=st1, data=weeddf,
    control=list(japprox="jacentral")))
print(anlxb1cen)
## central - analytic
print(anlxb1cen$coefficients - anlxb1$coefficients)

anlxb1nd  <-  (nlxb(wmods, start=st1, data=weeddf,
    control=list(japprox="jand")))
print(anlxb1nd)
## numDeriv - analytic
print(anlxb1nd$coefficients - anlxb1$coefficients)
```

## 3. Weighted nonlinear regression

While the standard approach to nonlinear regression is to minimize the sum of
squared residuals, there are frequently good reasons to **weight** each residual
to scale them according to their importance. That is, we wish to favour those
residuals believed to be more certain. Until 2020, `nlsr` did not provide for
weights that could alter with the parameters of the model. That is, the weights
were pre-specified, at least in `nlxb`. The `resfn` and `jacfn` of `nlfb` could,
of course be specified with almost any loss function that results in a sum of
squared residuals.

With the addition of the possibility of forcing the use of an approximation to
the Jacobian (Section 2 above), formulas that allow the inclusion of functions
(i.e., subprograms) are possible, and these may include weighting.

The following examples illustrate how this works. 

### Static weights


```{r wts1, echo=TRUE}
## weighted nonlinear regression using inverse squared variance of the response
require(minpack.lm)
Treated <- Puromycin[Puromycin$state == "treated", ]
# We want the variance of each "group" of the rate variable 
# for which the conc variable is the same. We first find
# this variance by group using the tapply() function.
var.Treated <- tapply(Treated$rate, Treated$conc, var)
var.Treated <- rep(var.Treated, each = 2)
Pur.wt1nls <- nls(rate ~ (Vm * conc)/(K + conc), data = Treated, 
               start = list(Vm = 200, K = 0.1), weights = 1/var.Treated^2)
Pur.wt1nlm <- nlsLM(rate ~ (Vm * conc)/(K + conc), data = Treated, 
               start = list(Vm = 200, K = 0.1), weights = 1/var.Treated^2)
Pur.wt1nlx <- nlxb(rate ~ (Vm * conc)/(K + conc), data = Treated, 
               start = list(Vm = 200, K = 0.1), weights = 1/var.Treated^2)
rnls <- summary(Pur.wt1nls)$residuals
ssrnls<-as.numeric(crossprod(rnls))
rnlm <- summary(Pur.wt1nlm)$residuals
ssrnlm<-as.numeric(crossprod(rnlm))
rnlx <- Pur.wt1nlx$resid
ssrnlx<-as.numeric(crossprod(rnlx))
cat("Compare nls and nlsLM: ", all.equal(coef(Pur.wt1nls), coef(Pur.wt1nlm)),"\n")
cat("Compare nls and nlsLM: ", all.equal(coef(Pur.wt1nls), coef(Pur.wt1nlx)),"\n")
cat("Sumsquares nls - nlsLM: ", ssrnls-ssrnlm,"\n")
cat("Sumsquares nls - nlxb: ", ssrnls-ssrnlx,"\n")
```

### Dynamic weights via the wfct() function

`minpack.lm` provides an interesting function that allows us to access current
values of various quantities associated with our model. There are five possibilities,
of which two are static weightings -- the name of the response or the predictor variable.
(It seems `wfct` assumes only one such variable.) The other three possibilities are the
current values of the "fitted" model, the residuals as specified by "resid", or the 
"error", which is the square root of the variance of the response variable. The last
possibility requires repetitions of the independent or predictor variable. 

Concerns: I have found that the structure of `wfct` means that examples using it in 
calls to `nlsLM`, `nls` or `nlxb` with fail if they are accessed via `source()` or
if the call is surrounded by a `print()` or `try()`. This remains to be sorted out.

Note that `nlxb` cannot use `fitted` or `resid` in the `wfct` function to specify weights.
`nls()` generates such **functions** as part of the returned object. `nlsr` does have
`predict.nlsr()` that could be used to generate fits and by extension, residuals.
What is possible??

```{r wtsfunction, eval=FALSE}
## minpack.lm::wfct
### Examples from 'nls' doc where wfct used ###
## Weighting by inverse of response 1/y_i:
wtt1nlm<-nlsLM(rate ~ Vm * conc/(K + conc), data = Treated,
              start = c(Vm = 200, K = 0.05), weights = wfct(1/rate))
print(wtt1nlm)

wtt1nls<-nls(rate ~ Vm * conc/(K + conc), data = Treated,
          start = c(Vm = 200, K = 0.05), weights = wfct(1/rate))
print(wtt1nls)

wtt1nlx<-nlxb(rate ~ Vm * conc/(K + conc), data = Treated,
          start = c(Vm = 200, K = 0.05), weights = wfct(1/rate))
print(wtt1nlx)

## Weighting by square root of predictor \sqrt{x_i}:
# ?? why does try() not work
wtt2nlm<-nlsLM(rate ~ Vm * conc/(K + conc), data = Treated,
       start = c(Vm = 200, K = 0.05), weights = wfct(sqrt(conc)))
print(wtt2nlm)

wtt2nls<-nls(rate ~ Vm * conc/(K + conc), data = Treated,
       start = c(Vm = 200, K = 0.05), weights = wfct(sqrt(conc)))
print(wtt2nls)

wtt2nlx<-nlxb(rate ~ Vm * conc/(K + conc), data = Treated,
       start = c(Vm = 200, K = 0.05), weights = wfct(sqrt(conc)))
print(wtt2nlx)

## Weighting by inverse square of fitted values 1/\hat{y_i}^2:
wtt3nlm<-nlsLM(rate ~ Vm * conc/(K + conc), data = Treated,
       start = c(Vm = 200, K = 0.05), weights = wfct(1/fitted^2))
print(wtt3nlm)

wtt3nls<-nls(rate ~ Vm * conc/(K + conc), data = Treated,
       start = c(Vm = 200, K = 0.05), weights = wfct(1/fitted^2))
print(wtt3nls)

# These don't work -- there is no fitted() function available in nlsr
# wtt3nlx<-try(nlxb(rate ~ Vm * conc/(K + conc), data = Treated,
#        start = c(Vm = 200, K = 0.05), weights = wfct(1/fitted^2)))
##  (nlxb(rate ~ Vm * conc/(K + conc), data = Treated,
##        start = c(Vm = 200, K = 0.05), weights = wfct(1/(resid+rate)^2)))
## (wrapnlsr(rate ~ Vm * conc/(K + conc), data = Treated,
##        start = c(Vm = 200, K = 0.05), weights = wfct(1/(resid+rate)^2)))

## Weighting by inverse variance 1/\sigma{y_i}^2:
wtt4nlm<-nlsLM(rate ~ Vm * conc/(K + conc), data = Treated,
       start = c(Vm = 200, K = 0.05), weights = wfct(1/error^2))
print(wtt4nlm)

wtt4nls<-nls(rate ~ Vm * conc/(K + conc), data = Treated,
       start = c(Vm = 200, K = 0.05), weights = wfct(1/error^2))
print(wtt4nls)

wtt4nlx<-nlxb(rate ~ Vm * conc/(K + conc), data = Treated,
       start = c(Vm = 200, K = 0.05), weights = wfct(1/error^2))
print(wtt4nlx)

wtt5nls<-nls(rate ~ Vm * conc/(K + conc), data = Treated,
              start = c(Vm = 200, K = 0.05), weights = wfct(1/resid^2))
print(wtt5nls)
wtt5nlm<-nlsLM(rate ~ Vm * conc/(K + conc), data = Treated,
              start = c(Vm = 200, K = 0.05), weights = wfct(1/resid^2))
print(wtt5nlm)
## Won't work! No resid function available from nlxb.
## wtt5nlx<-nlxb(rate ~ Vm * conc/(K + conc), data = Treated,
##            start = c(Vm = 200, K = 0.05), weights = wfct(1/resid^2))
## print(wtt5nlx)
```


### Weights built into a one-sided model function

In all three formula-based nonlinear model estimators (`nls`, `nlsLM`, `nlxb`), the "formula" can be set so there is no so-called Left Hand Side (LHS). 

```{r nolhs, echo=TRUE}
st<-c(b1=1, b2=1, b3=1)
frm <- ~ b1/(1+b2*exp(-b3*tt))-y
nolhs <- nlxb(formula=frm, data=weeddf, start=st)
nolhs
```

Since we can set up problems in this way, we could, in some cases, build weights 
into the expression on the right hand side. 
For `nlxb`, attempts to develop the analytic Jacobian will generally fail
if the formula includes an R function call. This can
be overcome by specifying a Jacobian approximation in the `control` 
list element `japprox`. 

```{r wts1side}
## weighted nonlinear regression using 1-sided model functions
Treated <- Puromycin[Puromycin$state == "treated", ]
weighted.MM <- function(resp, conc, Vm, K)
{
  ## Purpose: exactly as white book p. 451 -- RHS for nls()
  ##  Weighted version of Michaelis-Menten model
  ## ----------------------------------------------------------
  ## Arguments: 'y', 'x' and the two parameters (see book)
  ## ----------------------------------------------------------
  ## Author: Martin Maechler, Date: 23 Mar 2001
  
  pred <- (Vm * conc)/(K + conc)
  (resp - pred) / sqrt(pred)
}

Pur.wtMMnls <- nls( ~ weighted.MM(rate, conc, Vm, K), data = Treated,
               start = list(Vm = 200, K = 0.1))
print(Pur.wtMMnls)

Pur.wtMMnlm <- nlsLM( ~ weighted.MM(rate, conc, Vm, K), data = Treated,
                    start = list(Vm = 200, K = 0.1))
print(Pur.wtMMnlm)

Pur.wtMMnlx <- nlxb( ~ weighted.MM(rate, conc, Vm, K), data = Treated,
                    start = list(Vm = 200, K = 0.1), control=list(japprox="jafwd"))
print(Pur.wtMMnlx)

# Another approach
## Passing arguments using a list that can not be coerced to a data.frame
lisTreat <- with(Treated, list(conc1 = conc[1], conc.1 = conc[-1], rate = rate))
weighted.MM1 <- function(resp, conc1, conc.1, Vm, K){
  conc <- c(conc1, conc.1)
  pred <- (Vm * conc)/(K + conc)
  (resp - pred) / sqrt(pred)
}
Pur.wtMM1nls <- nls( ~ weighted.MM1(rate, conc1, conc.1, Vm, K),
                data = lisTreat, start = list(Vm = 200, K = 0.1))
print(Pur.wtMM1nls)
# Should we put in comparison of coeffs / sumsquares??
# stopifnot(all.equal(coef(Pur.wt), coef(Pur.wt1)))
Pur.wtMM1nlm <- nlsLM( ~ weighted.MM1(rate, conc1, conc.1, Vm, K),
                     data = lisTreat, start = list(Vm = 200, K = 0.1))
print(Pur.wtMM1nlm)
Pur.wtMM1nlx <- nlxb( ~ weighted.MM1(rate, conc1, conc.1, Vm, K),
                     data = lisTreat, start = list(Vm = 200, K = 0.1), control=list(japprox="jafwd"))
print(Pur.wtMM1nlx)
# yet another approach
## Chambers and Hastie (1992) Statistical Models in S  (p. 537):
## If the value of the right side [of formula] has an attribute called
## 'gradient' this should be a matrix with the number of rows equal
## to the length of the response and one column for each parameter.
weighted.MM.grad <- function(resp, conc1, conc.1, Vm, K) {
  conc <- c(conc1, conc.1)
  K.conc <- K+conc
  dy.dV <- conc/K.conc
  dy.dK <- -Vm*dy.dV/K.conc
  pred <- Vm*dy.dV
  pred.5 <- sqrt(pred)
  dev <- (resp - pred) / pred.5
  Ddev <- -0.5*(resp+pred)/(pred.5*pred)
  attr(dev, "gradient") <- Ddev * cbind(Vm = dy.dV, K = dy.dK)
  dev
}
Pur.wtMM.gradnls <- nls( ~ weighted.MM.grad(rate, conc1, conc.1, Vm, K),
                    data = lisTreat, start = list(Vm = 200, K = 0.1))
print(Pur.wtMM.gradnls)
Pur.wtMM.gradnlm <- nlsLM( ~ weighted.MM.grad(rate, conc1, conc.1, Vm, K),
                         data = lisTreat, start = list(Vm = 200, K = 0.1))
print(Pur.wtMM.gradnlm)
Pur.wtMM.gradnlx <- nlxb( ~ weighted.MM.grad(rate, conc1, conc.1, Vm, K),
                         data = lisTreat, start = list(Vm = 200, K = 0.1),
                         control=list(japprox="jafwd"))
print(Pur.wtMM.gradnlx)
#3 To display the coefficients for comparison:
## rbind(coef(Pur.wtMMnls), coef(Pur.wtMM1nls), coef(Pur.wtMM.gradnls))
## In this example, there seems no advantage to providing the gradient.
## In other cases, there might be.
```

## 4. Relative offset and other convergence criteria

Here we will mostly look at the relative offset convergence criterion (ROCC) 
that was proposed by @BatesWatts81. The primary merit of this test is that it
it compares the estimated progress towards a minimum sum of squares with the
current value. When this is very small, the method terminates. Both `nls()` and
`nlsr` try to use this criterion for terminating the process of minimizing the
objective, though there are minor differences of detail. `nlsr` also includes
the option of turning off the ROCC with the control parameter `rofftest=FALSE`.
`nlsr::nlfb()` also has a small sum of squares test: if the sum of squares is 
less than the fourth power of the machine precision, `.Machine$double.eps`, the
iteration will terminate unless control `smallsstest=FALSE`. In some extreme
problems this may be necessary, and they would likely
run until limits on maximum number of evaluations of the sum of 
squares (control `femax`) or Jacobian (control `jemax`) are exceeded. In other
words, we are dealing with problems at the edge of computability.

There is a weakness in the ROCC, in that it does not work well with problems 
that have a zero sum of squares as the solution, so called zero-residual problems.
In October 2020, I suggested a patch to the `stats::nls()` function to avoid
a zero-divide in the ROCC of `nls()`, using ideas, but not the exact implementation,
already in `nlsr::nlfb()`. Previously, the documentation of `nls()` said:

**Warning**

**The default settings of `nls` generally fail on small-residual problems.**

The `nls` function uses a relative-offset convergence criterion
     that compares the numerical imprecision at the current parameter
     estimates to the residual sum-of-squares.  To avoid a zero-divide in 
     computing the test value, a positive constant `convTestAdd` should 
     be added to the sum-of-squares. This quantity is set via `nls.control()`,
     as in the example below.
  
### Overview of the ROCC test

The ROCC test uses the calculated **reduction** in the sum of squared residuals in the
linearized sub-problem for the latest iteration and compares this to the 
current sum of squared residuals. The comparison is made by division of the reduction
in the sum of squares to the current sum of squares. Clearly if the denominator is 
zero, we have the awkwardness of zero divided by zero. This is overcome if
we add a positive value `scaleOffset` to the denominator. A default value of 0.0
for `scaleOffset` preserves the legacy behaviour of `nls()`.

While I have used ROCC as if there is a single
definition, `nls()` and `nlsr::nffb()` use somewhat different formulas and
scalings. However, I believe they result in essentially similar outcomes once the
`scaleOffset` safeguard is applied. 

### Some implementation ideas for the ROCC

Below in Section 5 we will see that the essential iteration in either a Gauss-Newton or
Marquardt method finds a least squares solution to the linearized problem

$$   A \delta = b $$

where $A$ is either the Jacobian or a modification thereof for a Marquardt statbilization,
and $b$ the appropriate negative of the residuals, possibly augmented with zeros according
to the  needs of the stabilization.

Clearly we can compute the current sum of squared as the cross product $b^T b$, since 
the zeros do not alter this value. It turns out the QR decomposition gives us a 
quite convenient computation for the reduction in this value by the current
linearized sub-problem. This happens to be the sum of squares of the elements of
the vector 

$$     r_{proj} = Q' b $$

where $Q'$ is the transpose of the orthogonal decomposition matrix $Q$ from the 
decomposition.

Let us illustrate this with a linear least squares problem. Note that this problem
uses an explicit column of 1s. Most "regression" calculations assume a constant term
which is included automatically. Furthermore, a linear model with just a single constant
minimizes the sum of squares when that constant is the mean of the dependent variable,
that is the mean of column $b$. This sum of squares is generally called the 
"Total Sum of Squares" and any linear model
with more than a constant will have a smaller some of squares. Here, and in most nonlinear
models, we do not have that guarantee. Hence we will talk of the "overall" sum of squares
for want of a better term, and this is simply the sum of squares of the elements of $b$.
We will judge our progress by this number.

First we create some data for the $A$ and $b$.

```{r lls01}
# QRsolveEx.R -- example of solving least squares with QR
# J C Nash 2020-12-06
# Enter matrix from Compact Numerical Methods for Computers, Ex04
text <- c(563, 262, 461, 221, 1, 305,
658, 291, 473, 222, 1, 342,
676, 294, 513, 221, 1, 331,
749, 302, 516, 218, 1, 339,
834, 320, 540, 217, 1, 354,
973, 350, 596, 218, 1, 369,
1079, 386, 650, 218, 1, 378,
1151, 401, 676, 225, 1, 368,
1324, 446, 769, 228, 1, 405,
1499, 492, 870, 230, 1, 438,
1690, 510, 907, 237, 1, 438,
1735, 534, 932, 235, 1, 451,
1778, 559, 956, 236, 1, 485)
mm<-matrix(text, ncol=6, byrow=TRUE) # byrow is critical!
A <- mm[,1:5] # select the matrix
A # display
b <- mm[,6] # rhs
b
oss<-as.numeric(crossprod(b))
cat("Overall sumsquares of b =",oss,"\n")
cat("(n-1)*variance=",(length(b)-1)*var(b),"= tss = ",as.numeric(crossprod(b-mean(b))),"\n")
Aqr<-qr(A) # compute the QR decomposition of A
QAqr<-qr.Q(Aqr) # extract the Q matrix of this decomposition
RAqr<-qr.R(Aqr) # This is how to get the R matrix
XAqr<-qr.X(Aqr) # And this is the reconstruction of A from the decomposition
cat("max reconstruction error = ", max(abs(XAqr-A)),"\n")
cat("check the orthogonality Q' * Q: ", max(abs(t(QAqr)%*%QAqr-diag(rep(1,dim(A)[2])))),"\n")
cat("note failure of orthogonality of Q * Q': ",max(abs(QAqr%*%t(QAqr)-diag(rep(1,dim(A)[1])))),"\n")
Absol<-qr.solve(A,b, tol=1000*.Machine$double.eps) # solve the linear ls problem
Absol # and display it 
Abres<-qr.resid(Aqr,b)# and get the residual
rss<-as.numeric(crossprod(Abres))
cat("residual sumsquares=",rss,"\n")
Qtb<-as.vector(t(QAqr)%*%b) # Q' * b -- turns out to be reduction in sumsquares
ssQtb<-as.numeric(crossprod(Qtb))
cat("oss-ssQtb = overall sumsquares minus sumsquares Q' * b = ",oss-ssQtb,"\n")
cat("Thus reduction in sumsquares is ", ssQtb,"\n")
Qtr<-as.vector(t(QAqr)%*%Abres) # projection of residuals onto range space of A
Qtr
```

We thus have a quite convenient and available method to compute a relative offset test
criterion if we solve our Gauss-Newton or Marquardt sub-problem with a QR decomposition.

### Other convergence and termination tests

In order to catch runaway computations where some numerical imprecision causes convergence
tests to fail, methods generally check the number of Jacobian or sum of squares (i.e.,
residual) calculations. `nlsr` does this with the `femax` and `jemax` elements of the
`control` list, for which the defaults
are 10000 and 5000 respectively, which is possibly overly loose. `minpack.lm::nlsLM` uses
control list elements `maxfev` and `maxiter` (default 50) which I believe parallel `femax`
and `jemax`. The default 
value of `maxfev` is 100 times the (number of parameters to estimate + 1). `nls()` appears
to have only control element `maxiter` for which the default is 50 also.

While `nls()` and `nlsr` use the ROCC, `minpack.lm` uses a combination of tests:

- control element `ftol` is used in a test that seems to be related to the ROCC,
since termination occurs when both the actual and predicted relative reductions in the 
sum of squares are at most `ftol`.

- when the relative change between two consecutive iterates is at most `ptol` the 
process terminates.

- if the cosine of the angle between result of evaluation of the residual and 
any column of the Jacobian is at most `gtol` in absolute value the process
terminates. Therefore, `gtol` measures the orthogonality desired between the residual
vector and the columns of the Jacobian.

## 5. Implementation of nonlinear least squares methods

This section is a review of approaches to solving the nonlinear least
squares problem that underlies nonlinear regression modelling. In particular,
we look at using a QR decomposition for the Levenberg-Marquardt stabilization
of the solution of the Gauss-Newton equations.

### Gauss-Newton variants

Nonlinear least squares methods are mostly founded on some or
other variant of the Gauss-Newton algorithm. The function we
wish to minimize is the sum of squares of the (nonlinear) 
residuals $r(x)$ where there are $m$ observations (elements of $r$)
and $n$ parameters $x$. Hence the function is

$$    f(x) = \sum_i({r_i}^2)$$

Newton's method starts with an original set of parameters $x[0]$.
At a given iteraion, which could be the first, we want to solve

$$    x[k+1]  =  x[k]  -   H^{-1}  g$$
   
where $H$ is the Hessian and $g$ is the gradient at $x[k]$. We can 
rewrite this as a solution, at each iteration, of 
  
$$    H \delta = -g$$

with

$$    x[k+1]  =  x[k] + \delta$$
         
For the particular sum of squares, the gradient is 

$$    g(x) = 2 * r[k]$$

and 

$$    H(x) = 2 ( J' J  +  \sum_i(r_i * Z_i))$$

where $J$ is the Jacobian (first derivatives of $r$ w.r.t. $x$)
and $Z_i$ is the tensor of second derivatives of $r_i$ w.r.t. $x$).
Note that $J'$ is the transpose of $J$. 

The primary simplification of the Gauss-Newton method is to
assume that the second term above is negligible. As there is
a common factor of 2 on each side of the Newton iteration 
after the simplification of the Hessian, the Gauss-Newton
iteration equation is

$$    (J' J)  \delta = - J' r$$
<!-- `nls()` attempts to minimize a sum of squared residuals by a Gauss-Newton method. If we -->
<!-- compute a Jacobian matrix $J$ and a vector of residuals $r$ from a vector of parameters $x$,  -->
<!-- then we can define a linearized problem  -->

<!-- $$  J^T J  \delta  = - J^T r $$ -->

<!-- This leads to an iteration where, from a set of starting parameters $x_0$, we compute -->

<!-- $$ x_{i+1} = x_i + \delta$$ -->

<!-- This is commonly modified to use a step factor `step` -->

<!-- $$ x_{i+1} = x_i + step * \delta$$ -->

<!-- It is in the mechanisms to choose the size of $step$ and to decide when to terminate the -->
<!-- iteration that Gauss-Newton methods differ. Indeed, though I have tried several times, I -->
<!-- find the very convoluted code behind `nls()` very difficult to decipher. Unfortunately, its authors have now (as far as I am aware) all ceased to maintain the code. -->




This iteration frequently fails. The approximation of the Hessian
by the Jacobian inner-product is one reason, but there is also
the possibility that the sum of squares function is not
"quadratic" enough that the unit step reduces it. @Hartley61
introduced a line search along delta. @Marquardt1963 suggested
replacing $J' J$ with $(J' J + \lambda * D)$ where $D$ is a diagonal
matrix intended to partially approximate the omitted portion of
the Hessian. Choices suggested by Marquardt were $D = I$ (a unit matrix) or
$D$ = (diagonal part of $J' J$). The former approach, when $\lambda$ is
large enough that the iteration is essentially

$$    \delta = - g / \lambda $$

gives a version of the steepest descents algorithm. Using the
diagonal of $J' J$, we have a scaled version of this
(see \url{https://en.wikipedia.org/wiki/Levenberg-Marquardt_algorithm};
<!-- https://en.wikipedia.org/wiki/Levenberg%E2%80%93Marquardt_algorithm -->
@Levenberg1944 predated Marquardt, but the latter seems to
have done the practical work that brought the approach to general attention.)

<!-- @jn77ima found that on low precision machines, it -->
<!-- was common for diagonal elements of $J' J$ to underflow. A very -->
<!-- small modification to solve  -->

<!-- $$    (J' J + lambda * (D + \phi * I)) * \delta = - g $$ -->

<!-- where $\phi$ is a small number avoids most of these conditions. $\phi = 1$ seems  -->
<!-- to work quite well. We note that this modification would likely not have been  -->
<!-- recognized had I not been working on machines with mediochre floating-point -->
<!-- arithmetic -- a little more than 6 decimal digits of precision and no extended -->
<!-- precision. -->

### Choices

Both `nlsr::nlxb()` and `minpack.lm::nlsLM` use a Levenberg-Marquardt stabilization 
of the iteration described above, with `nlsr` using the modification involving the 
$\phi$ control parameter. The complexities of the code in `minpack.lm` are such that I
have relied largely on the documentation
to judge how the iteration is accomplished. `nls()` uses a straightforward 
Hartley variant of the Gauss-Newton iteration,
but rather than form the sum of squares and cross-products, uses a QR decomposition 
of the matrix
$J$ that has been found by a forward difference approximation. The line search used
by `nls()` is a simple back-tracking search using a step reduction factor of 0.5 as
the default stepsize reduction.

@jncnm79 and @jnmws87 solve

$$ (J^T J + \lambda D_x)   \delta  = - J^T r $$

by the Cholesky decomposition. In this $D_x = (D + \phi * I)$ as described above
and $\lambda$ is a number of modest size initially. Clearly
for $\lambda = 0$ we have a Gauss-Newton method. Typically, the sum of squares of 
the residuals calculated at the "new" set of parameters is used as a 
criterion for keeping those parameter values. If so, the size of $\lambda$ is 
reduced. If not, we increase the size of $\lambda$ and compute a new $\delta$. 
Note that a new $J$, the expensive step in each iteration, is NOT required in 
this latter case. 

In 2022, a modification to use $D_y = (\psi * D + \phi * I)$ was introduced,
though the matrix equations are solved via a QR decomposition approach. Within
the code, control parameters `psi`, `phi` and `stepredn` were introduced so that
a variety of Gauss-Newton, Hartley, or Marquardt approaches are available by 
simple control modifications. Experience so far suggests that a Levenberg-Marquardt
stabilization is much more reliable than the Gauss-Newton-Hartley choices, but that
different selections of `psi` and `phi` perform rather similarly.
As for Gauss-Newton methods, the details of how to start, adjust and 
terminate the iteration lead to many variants, increased by these different 
possibilities for specifying $D$. See @jncnm79.

### Using matrix decompositions  

In @jncnm79, the iteration equation was solved as stated. However, this
involves forming the sum of squares and cross products of $J$, a process that
loses some numerical precision. A better way to solve the linear equations is
to apply the QR decomposition to the matrix $J$ itself. However, we still need
to incorporate the $\lambda * I$ or $\lambda * D$ adjustments. This is done by 
adding rows to $J$ that are the square roots of the "pieces". We add 1 row for
each diagonal element of $I$ and each diagonal element of $D$. Note that we
also need to add a zero to the residual vector (the right-hand side) for each
diagonal element.

Various authors (including the present one) have suggested different strategies 
for modification of $\lambda$. In the present approach, we reduce the $lambda$ 
parameter before solution. The initial value is the control element `lamda` (note
the mis-spelling), which has default 1e-4. If the
resulting sum of squares is not reduced, $lambda$ is increased, otherwise we
move to the next iteration. My current opinion is that
a "quick" increase, say a factor of 10, and a "slow" decrease, say a factor
of 0.4, work quite well. However, it is worth checking that `lamda` has
not got too small or underflowed before applying the increase factor. On
the other hand, it is useful to be able to set lamda = 0 along with zero increase
and decrease factors `laminc` and `lamdec` so a Hartley method can be 
evaluated with the program(s) by
setting the control `stepredn`. Regularly, the
current code `nlfb()` uses the line

   `if (lamda<1000*.Machine$double.eps) lamda<-1000*.Machine$double.eps`
   
to ensure we get an increase. However, these possibilities are really for
those of us playing to improve algorithms and not for practitioner use.

**The Levenberg-Marquardt adjustment to the Gauss-Newton approach is the
second major improvement of `nlsr` (and also its predecessor `nlmrt` and 
the package `minpack-lm`) over `nls()`.**


We could implement the methods using the equations above. However, the accumulation of
inner products in $J^T J$ occasions some numerical error, and it is generally both safer
and more efficient to use matrix decompositions. In particular, if we form the QR decomposition
of $J$

$$  Q R = J$$
    
where Q is an orthonormal matrix and R is Right or Upper triangular, we can easily
solve

$$  R \delta = Q^T r$$
for which the solution is also the solution of the Gauss-Newton equations.
But how do we get the Marquardt stabilization?

If we augment $J$ with a square matrix $sqrt(\lambda D)$ whose diagonal elements are the
square roots of $\lambda$ times the diagonal elements of $D$, and augment the vector 
$r$ with $n$ zeros
where $n$ is the column dimension of $J$ and $D$, we achieve our goal. 

Typically we can use $D = 1_n$ (the identity of order $n$), but @Marquardt1963 showed 
that using the 
diagonal elements of $J^T J$ for $D$ results in a useful implicit scaling of the parameters.
@jn77ima pointed out that on computers with limited arithmetic (which now are rare since the 
IEEE 754 standard appeared in 1985), underflow might
cause a problem of very small elements in $D$ and proposed adding $\phi 1_n$ to the diagonals
of $J^T J$ before multiplying by $\lambda$ in the Marquardt stabilization. This avoids some
awkward cases with very little extra overhead. It is accomplished with the QR approach by
appending $sqrt(\phi * \lambda)$ times a unit matrix $I_n$ to the matrix already augmented
matrix. We must also append a further $n$ zeros to the augmented $r$. 


## 6. Fixed parameters

### Motivation for fixed parameters

In finding optimal parameters in nonlinear optimization and nonlinear 
least squares problems, we may wish to fix one or more parameters
while allowing the rest to be adjusted to explore or optimize an objective 
function. A 
lot of the material is drawn from Nash J C (2014) **Nonlinear parameter 
optimization using R tools** Chichester UK: Wiley, in particular chapters
11 and 12.


### Background for fixed parameters

Here are some of the ways fixed parameters may be specified in R packages.

- `nlxb()` in package `nlsr`  (formerly part of defunct package `nlmrt`) uses a
character vector `masked` of quoted parameter names. These parameters will NOT 
be altered by the algorithm. This approach has a simplicity that is attractive, 
but requires an extra argument to calling sequences.

- `nlfb()` in `nlsr` (formerly in `nlmrt` also) uses the vector `maskidx` of 
(integer) indices of the parameters to be masked. 
These parameters will NOT be altered by the algorithm. 
Note that the mechanism here is different 
from that in nlxb which uses the names of the parameters.

- `Rvmmin` and `Rcgmin` use an indicator vector `bdmsk` that has 
1 for each parameter that is "free" or unconstrained, and 0 for 
any parameter that is fixed or MASKED for the duration of the optimization.

Note that the function ``bmchk()`` in package ``optimx`` contains a much more
extensive examination of the bounds on parameters. In particular, it considers
such issues as:

- inadmissible bounds (lower > upper), 

- when to convert a pair of bounds where `upper["parameter"] - lower["parameter"] < tol` to a
fixed or masked parameter (`maskadded`), and 

- whether parameters outside of bounds should be
moved to the nearest bound (`parchanged`). 

It may be useful to use **inadmissible** to refer
to situations where a lower bound is higher than an upper bound 
and **infeasible** where
a parameter is outside the bounds.

From `optimx` the function `optimr()` can call many different "optimizers" (actually 
function minimization methods that may include bounds and possibly masks).
Masks could be specified by setting the lower and upper bounds 
equal for 
the parameters to be fixed. While this may seem to be a simple method for specifying
masks, there can be computational as well as conceptual difficulties.
For example, what happens when the
upper bound is only very slightly greater than the lower bound. Also 
should we stop or declare an error if starting values are NOT on the
fixed value.

Of these choices, my current preference is to use the last one -- setting
lower and upper bounds equal for fixed parameters, and furthermore setting the starting
value to this fixed value, and otherwise declaring an error. The approach 
does not add any special argument for masking, and is relatively obvious
to novice users. However, such users may be tempted to put in narrow 
bounds rather than explicit equalities, and this could have deleterious
consequences.

### Internal structures to specify fixed parameters

`bdmsk` is the internal structure used in `Rcgmin` and `Rvmmin` to handle bounds constraints as well as masks.
There is one element of `bdmsk` for each parameter, and in `Rcgmin` and `Rvmmin`, this is used on input to 
specify parameter i as fixed or masked by setting `bdmsk[i] <- 0`. Free parameters have their `bdmsk` element 1,
but during optimization in the presence of bounds, we can set other values. The full set is as follows

* 1 for a free or unconstrained parameter
* 0 for a masked or fixed parameter
* -0.5 for a parameter that is out of bounds high (> upper bound)
* -1 for a parameter at its upper bound
* -3 for a parameter at its lower bound
* -3.5 for a parameter that is out of bounds low (< lower bound)

Not all these possibilities will be used by all methods that use `bdmsk`.

The -1 and -3 are historical, and arose in the development of BASIC 
codes for @jnmws87 which is now
<!-- Nash J C and Walker-Smith M (1987) -->
<!-- **Nonlinear parameter estimation: and integrated system in BASIC** New York: Dekker.  -->
available for free download
at https://archive.org/details/NLPE87plus. 
In particular, adding 2 gives 1 for an upper bound and -1 for a lower bound, 
simplifying the expression to decide if an optimization trial step will move away from a bound.


### Proposed approaches to fix parameters

Because masks (fixed parameters) reduce the dimension of the 
optimization problem, we can consider
modifying the problem to the lower dimension space. This is Duncan 
Murdoch's suggestion, using 

-  `fn0(par0)` to be the initial user function of the full dimension parameter vector `par0`
-  `fn1(par1)` to be the reduced or internal functin of the reduced dimension vector `par1`
-  `par1 <- forward(par0)`
-  `par0 <- inverse(par1)`

The major advantage of this approach is explicit dimension reduction. The main disadvantage
is the effort of transformation at every step of an optimization.

An alternative is to use the `bdmsk` vector to **mask** 
the optimization search or adjustment vector, 
including gradients and (approximate) Hessian matrices. A 0 element of `bdmsk` "multiplies" any 
adjustment. The principal difficulty is to ensure we do not essentially divide by zero in applying
any inverse Hessian. This approach avoids `forward`, `inverse` and `fn1`. However, it may hide the
reduction in dimension, and caution is necessary in using the function and its derived gradient,
Hessian and derived information.


### Examples of use of fixed parameters

Using `Rvmmin`, we easily minimize the function 

$$  sq(x) = \sum_i{(i - x_i)^2} $$
a simple quadratic. The unconstrained solution has the function zero when the 
parameters are the sequence 1 to the number of parameters. When we fix the first
parameter at 0.3, the smallest we can make the function is 0.49, i.e., $(1 - 0.3)^2$.
In both cases the `convergence` indicator, 2, means that the gradient at the 
suggested solution was very small.

```{r maskRvmmin}
require(optimx)
sq<-function(x){
   nn<-length(x)
   yy<-1:nn
   f<-sum((yy-x)^2)
#   cat("Fv=",f," at ")
#   print(x)
   f
}
sq.g <- function(x){
   nn<-length(x)
   yy<-1:nn
   gg<- 2*(x - yy)
}
xx <- c(.3, 4)
uncans <- Rvmmin(xx, sq, sq.g)
uncans
mybm <- c(0,1) # fix parameter 1
cans <- Rvmmin(xx, sq, sq.g, bdmsk=mybm)
cans
```

Using the same example for `nlsr::nlfb()`, we get the following.

```{r masknlfb, echo=TRUE}
## rm(list=ls()) # clear workspace??
library(nlsr)
sqres<-function(x){
   nn<-length(x)
   yy<-1:nn
   res <- (yy-x)
}
sqjac <- function(x){
   nn<-length(x)
   yy<-1:nn
   JJ <- matrix(-1, nrow=nn, ncol=nn)
   attr(JJ, "gradient") <- JJ ## !! Critical statement
   JJ
}
xx <- c(.3, 4) # repeat for completeness
anlfbu<-nlfb(xx, sqres, sqjac)
anlfbu
anlfbc<-nlfb(xx, sqres, sqjac, lower=c(xx[1], -Inf), upper=c(xx[1], Inf), trace=FALSE)
anlfbc
# Following will warn All parameters are masked
anlfbe<-try(nlfb(xx, sqres, sqjac, lower=c(xx[1], xx[2]), upper=c(xx[1],xx[2]))) 
```

It is difficult to run this example with `nlsr::nlxb()`, as we need to 
provide a formula that spans the "data". Note that the unconstrained 
problem gives a warning when we try to compute a p-value for an 
essentially perfect minimum of the sum of squares.

?? Note that Dec 28 version does NOT work, but Dec 03 version does. 
Issue with changes between 03 and 28, probably in how "formula" is treated.
-- see stats::model.frame -- need to have a version that handles "masked"

```{r masknlxb, echo=TRUE}
nn<-length(xx) # Also length(yy)
if (nn != 2) stop("This example has nn=2 only!")
yy<-1:2
v1 <- c(1,0); v2 <- c(0,1)
anlxbu <- nlxb(yy~v1*p1+v2*p2, start=c(p1=0.3, p2=4) )
anlxbu  
anlxbc <- nlxb(yy~v1*p1+v2*p2, start=c(p1=0.3, p2=4), masked=c("p1") )
anlxbc  
```

We can also use a different example that better illustrates `nlsr::nlxb()`. 

```{r masknlxb2, echo=TRUE}
weed <- c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443,
   38.558, 50.156, 62.948, 75.995, 91.972)
ii <- 1:12
wdf <- data.frame(weed, ii)
weedux <- nlxb(weed~b1/(1+b2*exp(-b3*ii)), start=c(b1=200, b2=50, b3=0.3)) 
weedux
#- Old mechanism of 'nlsr' NO longer works
#- weedcx <- nlxb(weed~b1/(1+b2*exp(-b3*ii)), start=c(b1=200, b2=50, b3=0.3), masked=c("b1")) 
weedcx <- nlxb(weed~b1/(1+b2*exp(-b3*ii)), start=c(b1=200, b2=50, b3=0.3), 
               lower=c(b1=200, b2=0, b3=0), upper=c(b1=200, b2=100, b3=40)) 
weedcx
rfn <- function(bvec, weed=weed, ii=ii){
  res <- rep(NA, length(ii))
  for (i in ii){
    res[i]<- bvec[1]/(1+bvec[2]*exp(-bvec[3]*i))-weed[i]
  }
  res
}
weeduf <- nlfb(start=c(200, 50, 0.3),resfn=rfn,weed=weed, ii=ii, control=list(japprox="jacentral"))
weeduf
#- maskidx method to specify masks no longer works
#- weedcf <- nlfb(start=c(200, 50, 0.3),resfn=rfn,weed=weed, ii=ii, 
#-                maskidx=c(1), control=list(japprox="jacentral"))
weedcf <- nlfb(start=c(200, 50, 0.3),resfn=rfn,weed=weed, ii=ii, lower=c(200, 0, 0), 
               upper=c(200, 100, 40), control=list(japprox="jacentral"))
weedcf
```

## 7. Capabilities added to `nlsr` in the 2022 version

In the review of nonlinear modelling tools starting in December 2020, several aspects
of `nlsr` were modified. 

### Numerical approximations to Jacobians

While a defining aspect of `nlsr` is the ability to develop analytic Jacobian 
functions for `nlfb` to use from the `formula` used in the call to `nlxb`, there
are many models for which analytic derivatives are either impossible or, more
commonly, simply not available through the table of derivatives in the `nlsr`
package. Thus it is important to be able to specify an approximation. 

This has been documented above in "2. Analytic versus approximate Jacobians", 
with examples. Note that the `formula` can use R functions that are not in the
derivatives table if we specify a Jacobian approximation.
  
<!-- Within `nlfb`, an earlier implementation used a forward difference Jacobian  -->
<!-- approximation if the `jacfn` was missing from the call. Now we permit the `control`  -->
<!-- list element `japprox` to specify the name of a function as a character string.  -->
<!-- Moreover, this can be specified for `nlxb()` as well. Four approximations are -->
<!-- supported, `jafwd`, `jaback`, `jacentral`, and `jand`. It is also possible to provide the -->
<!-- name of such a function to the `jacfn` calling argument in quotations, or in the  -->
<!-- control element `japprox`. -->

<!-- ```{r nlfbapprox, echo=TRUE} -->
<!-- ## Use the Hobbs Weed problem -->
<!-- weed <- c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443, -->
<!--           38.558, 50.156, 62.948, 75.995, 91.972) -->
<!-- tt <- 1:12 -->
<!-- weedframe <- data.frame(y=weed, tt=tt) -->
<!-- st <- c(b1=1, b2=1, b3=1) # a default starting vector (named!) -->
<!-- ## We can provide the residual and Jacobian as functions -->
<!-- # Unscaled Hobbs problem -->
<!-- hobbs.res  <-  function(x){ # scaled Hobbs weeds problem -- residual -->
<!--   # This variant uses looping -->
<!--   if(length(x) != 3) stop("hobbs.res -- parameter vector n!=3") -->
<!--   y  <-  c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443,  -->
<!--            38.558, 50.156, 62.948, 75.995, 91.972) -->
<!--   tt  <-  1:12 -->
<!--   res  <-  x[1]/(1+x[2]*exp(-x[3]*tt)) - y -->
<!-- } -->


<!-- hobbs.jac  <-  function(x) { # scaled Hobbs weeds problem -- Jacobian -->
<!--   jj  <-  matrix(0.0, 12, 3) -->
<!--   tt  <-  1:12 -->
<!--   yy  <-  exp(-x[3]*tt) -->
<!--   zz  <-  1.0/(1+x[2]*yy) -->
<!--   jj[tt,1]   <-   zz -->
<!--   jj[tt,2]   <-   -x[1]*zz*zz*yy -->
<!--   jj[tt,3]   <-   x[1]*zz*zz*yy*x[2]*tt -->
<!--   attr(jj, "gradient") <- jj -->
<!--   jj -->
<!-- } -->
<!-- axfail <- try(nlfb(start=st, resfn=hobbs.res, lower=c(0,0,0), -->
<!--                upper=c(2,6,3))) -->
<!-- ax0 <- try(nlfb(start=st, resfn=hobbs.res, jacfn=hobbs.jac)) -->
<!-- ax0 -->
<!-- axcent1 <- try(nlfb(start=st, resfn=hobbs.res, control=list(japprox="jacentral"))) -->
<!-- axcent1 -->
<!-- axfwd2 <- try(nlfb(start=st, resfn=hobbs.res, jacfn="jafwd")) -->
<!-- axfwd2 -->
<!-- ## Note differences in gradient. Calculations are not quite identical. -->
<!-- ``` -->

<!-- - Selfstart model capability -->

<!--   A useful feature of `nls()` is the infrastructure set up to allow supposedly  -->
<!--   "good" starting values to be computed if no `start` calling argument is provided. -->
<!--   (A fall-back is to use the value 1 for each parameter in the model.  -->
<!--   This feature can be used with `nlxb` by leaving `start` -->
<!--   out of the call and specifying in `control` element `japprox` a valid Jacobian -->
<!--   approximation function.  -->
<!--   When a `start` vector of model parameters is not provided, a selfstart method -->
<!--   will be sought by `nlxb`. If no such method is available, those parameters not -->
<!--   set will be initialized to 1. Here I borrowed code quite blatantly from `nls.R` -->
<!--   in R-4.0.3. `nlsLM` appears to follow the same approach. -->

<!--   Unfortunately, the selfStart capability of `nls()` is much more tightly integrated -->
<!--   than I would like. The code to compute initial values of the nonlinear parameters -->
<!--   to be estimated for different models is contained in  -->
<!--   `R-(version)/src/library/stats/R/zzModels.R`. There we find that for many, if not -->
<!--   all of the models, the (approximate) starting values are computed by calls to `nls()` itself. -->

<!--   I have a script, `testselfstart.R` that exercises each of the available selfStart  -->
<!--   models in base R. There are also selfStart models in the CRAN package `nlraa`, but -->
<!--   I have not (yet?) tested these in the three solvers. -->

<!--   Writing the selfStart code is non-trivial, since it must attempt to decode the -->
<!--   formula of the models and apply an approximation to determine starting values for -->
<!--   parameters. I believe a less complicated approach would be better.  -->
  
<!-- - Weights that are dependent on current parameters ??-->

<!--   In the same way that selfstart models can be used in `nlxb`, the explicit specification -->
<!--   of the Jacobian approximation via `control$japprox` allows `nlxb` to run most examples  -->
<!--   of `nls` and 'nlsLM` that use weights that depend on current values of the parameters in the -->
<!--   model. This is discussed below in Section 3. -->

### Self start models

`nls()` and `nlsLM()` allow for self-starting models, but they are not explicitly
part of `nlsr::nlxb()`. `nls()` does not need a `start` argument if the `formula`
contains as its right-hand side (rhs) the name of a selfStart function that is
available in the current search path. 

Such selfStart functions often also include code to compute the Jacobian, 
though this does not seem to be a requirement. We have also noted that some of
the selfStart models -- particularly those in the base-R file 
`./src/library/stats/R/zzModels.R` -- may simply use a crude start and a
different algoithm, such as the 'plinear' option. Thus they are not really
developing an approximate start, but conducting an alternative solution.

While I contemplated using the "no start argument" approach to selfStart models,
the mechanism chosen to use their capabilities is to invoke the `getInitial()`
function to set the starting parameter vector. Moreover, by setting control 
element `japprox` to 'SSJac', we use the Jacobian code available in the selfStart
function.

Here is an example using the Michaelis-Menten model in the `stats` package.

```{r ssmicmen, echo=TRUE}
cat("=== SSmicmen ===\n")
dat <- PurTrt <- Puromycin[ Puromycin$state == "treated", ]
frm <- rate ~ SSmicmen(conc, Vm, K)
strt<-getInitial(frm, data = dat)
print(strt)
fm1x <- nlxb(frm, data = dat, start=strt,
             trace=FALSE, control=list(japprox="SSJac"))
fm1x
```

We see very quick convergence using the approximation generated by the SSmicmen
code.

`nls()`, in the absence of a selfStart model and with no `start` specified,
puts all parameters equal to 1. Since there are situations such as the 
Lanczos multiple exponential model, i.e., 

``` y ~ a * exp(-b*x) + c * exp(-d*x) + e * exp(-f*x)```

where the three terms would be equivalent with parameters `a` through `f` all
at 1, a minor modification to set the parameters so they are all 
different seems more appropriate. While this could be automated, I 
prefer to insist that the user take responsibility for an initial guess
in these cases.

### Models with partially linear parameters

`nls()` offers the choice of solving a problem with partially linear parameters with
a version of the Golub-Pereyra VARPRO algorithm. The choice is specified in by 
`algorithm="plinear"` in the call. (Note that `algorithm="port"` is also possible and
replaces the default Gauss-Newton method with the `nl2sol` code from the Bell
Port library (@PORTlib, @nl2solDGW81). However, "port" does not provide for partially
linear parameters, though it does allow bounds constraints on them.)

Unfortunately, at least in my opinion, when the algorithm is changed to "plinear",
the user must also change the formula that specifies the model to be fitted! This is
illustrated with one of the examples from the manual page of `nls()`.  Note how 
asking for the `plinear` option introduces another parameter into the model.
We can set up the computation with this parameter explicitly included when using
the default algorithm, which I believe is a more transparent choice. However, the
conditional linearity (VARPRO) algorithm is generally more reliable in getting 
the optimal parameters in difficult cases. Ideally, the details of use of the 
"plinear" approach would be hidden from view. Accomplishing that remains an open issue.

```{r partlin1}
## Comment in example is "## using conditional linearity"
DNase1 <- subset(DNase, Run == 1) # data
## using nls with plinear
# Using a formula that explicitly includes the asymptote. (Default algorithm.)
fm2DNase1orig <- nls(density ~ Asym/(1 + exp((xmid - log(conc))/scal)),
                 data = DNase1,
                 start = list(Asym = 10, xmid = 0, scal = 1))
summary(fm2DNase1orig)
# Using conditional linearity. Note the linear paraemter does not appear explicitly.
#  And we must change the starting vector.
fm2DNase1 <- nls(density ~ 1/(1 + exp((xmid - log(conc))/scal)),
                 data = DNase1,
                 start = list(xmid = 0, scal = 1),
                 algorithm = "plinear")
summary(fm2DNase1)
# How to run with "port" algorithm -- NOT RUN
# fm2DNase1port <- nls(density ~ Asym/(1 + exp((xmid - log(conc))/scal)),
#                  data = DNase1,
#                  start = list(Asym = 10, xmid = 0, scal = 1),
#                  algorithm = "port")
# summary(fm2DNase1port)

# require(minpack.lm) # Does NOT offer VARPRO. First example is WRONG. NOT RUN.
# fm2DNase1mA <- nlsLM(density ~ 1/(1 + exp((xmid - log(conc))/scal)),
#                  data = DNase1,
#                  start = list(xmid = 0, scal = 1))
# summary(fm2DNase1mA)
# fm2DNase1mB <- nlsLM(density ~ Asym/(1 + exp((xmid - log(conc))/scal)),
#                  data = DNase1,
#                  start = list(Asym=10, xmid = 0, scal = 1))
# summary(fm2DNase1mB)
# 
# # This gets wrong answer. NOT RUN
# fm2DNase1xA <- nlxb(density ~ 1/(1 + exp((xmid - log(conc))/scal)),
#                  data = DNase1,
#                  start = list(xmid = 0, scal = 1))
# print(fm2DNase1xA)
# Original formula with nlsr::nlxb().
fm2DNase1xB <- nlxb(density ~ Asym/(1 + exp((xmid - log(conc))/scal)),
                 data = DNase1,
                 start = list(Asym=10, xmid = 0, scal = 1))
print(fm2DNase1xB)
```

There are, however, problems where being able to exploit the partial linearity is an
advantage. It would be very nice to be able to use the capability WITHOUT having to 
adjust the model specification.


## 8. Capabilities still missing from `nlsr`

This section last updated 2021-02-19.

### Automatic differentiation of functional models

This is the natural extension of Multi-line expressions. The authors would welcome
collaboration with someone who has expertise in this area. Some progress has been
made with the `autodiffr` package of Changcheng Li (see https://github.com/Non-Contradiction/autodiffr).

At the time of writing, functional models require that `nlxb` be called with the
control `japprox` set to an available approximating function, as discussed above.


<!-- ### Vector parameters (?? nls2 stuff?) -->

<!-- We would like to be able to find the Jacobian or gradient of functions -->
<!-- that have as their parameters a vector, e.g., *prm*. At time of writing -->
<!-- (January 2015, confirmed December 2020) we cannot specify such a vector within **nlsr**.  -->

<!-- ?? is this true? -->

<!-- ### Multi-line function expressions -->

<!-- ?? are these in nls() or nlsLM()?? -->

<!-- Multi-line functions present a challenge. This is in part because -->
<!-- the chain rule may have to be applied backwards (last line first), but -->
<!-- also because there may be structures that are not always differentiable, -->
<!-- such as *if* statements. -->

<!-- Note that the functional approach to nonlinear least squares -- accessible via -->
<!-- function `nlfb` -- does let us prepare residuals and Jacobians of complexity -->
<!-- limited only by the capabilities of **R**. -->

### Indexed parameters

There is an example in nls.Rd where indexed parameters are used. That is, parameters
can be given a subscript e.g., `b[2]`. As far as I can determine, this facility
is not documented, and neither `minpack.lm::nlsLM` nor `nlsr::nlxb` can do this.

Here is the example in the `nls()` documentation.

```{r nls-indexed-ex}
## The muscle dataset in MASS is from an experiment on muscle
## contraction on 21 animals.  The observed variables are Strip
## (identifier of muscle), Conc (Cacl concentration) and Length
## (resulting length of muscle section).
utils::data(muscle, package = "MASS")

## The non linear model considered is
##       Length = alpha + beta*exp(-Conc/theta) + error
## where theta is constant but alpha and beta may vary with Strip.

with(muscle, table(Strip)) # 2, 3 or 4 obs per strip

## We first use the plinear algorithm to fit an overall model,
## ignoring that alpha and beta might vary with Strip.

musc.1 <- nls(Length ~ cbind(1, exp(-Conc/th)), muscle,
              start = list(th = 1), algorithm = "plinear")
summary(musc.1)

## Then we use nls' indexing feature for parameters in non-linear
## models to use the conventional algorithm to fit a model in which
## alpha and beta vary with Strip.  The starting values are provided
## by the previously fitted model.
## Note that with indexed parameters, the starting values must be
## given in a list (with names):
b <- coef(musc.1)
musc.2 <- nls(Length ~ a[Strip] + b[Strip]*exp(-Conc/th), muscle,
              start = list(a = rep(b[2], 21), b = rep(b[3], 21), th = b[1]))
## IGNORE_RDIFF_BEGIN
summary(musc.2)
## IGNORE_RDIFF_END
```

The structure of the call is interesting in that `start` is a **list**
and the elements of each part are NOT equal in length, as can bee seen 
from 

```{r nls-indexed-start}
istart = list(a = rep(b[2], 21), b = rep(b[3], 21), th = b[1])
str(istart)
```


<!-- ### ??nls-profile -- profile of the loss function surface?? -->

<!-- See \url{https://www.rdocumentation.org/packages/stats/versions/3.6.2/topics/profile.nls} -->

<!-- ?? note no TRACE in nls.lm etc. How to get output? -->


<!-- ### Examples of capabilities and weaknesses -->

<!-- The following (rather long) script shows things that work or fail. In particular,  -->
<!-- it shows that the calls needed to get derivatives need to be set up precisely. This -->
<!-- is not unique to R -- the same sort of attention to (syntax specific) detail is  -->
<!-- present in other systems like Macsyma/Maxima. -->

<!-- First we set up the Hobbs weed problem. Initially it seemed a good idea to put the data -->
<!-- WITHIN the residual function so that it would not have to be passed to the function. -->
<!-- In retrospect, this is a bad idea because y and t are needed. To make the use of the -->
<!-- data explicit, it is saved in the variables `tdat` for time and `ydat` for the weed -->
<!-- data. -->

<!-- ```{r hobbsrestest} -->
<!-- # tryhobbsderiv.R -->
<!-- ydat<-c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443, 38.558, 50.156, 62.948, -->
<!--        75.995, 91.972) -->
<!-- tdat<-1:12 -->
<!-- # try setting t and y here -->
<!-- t <- tdat -->
<!-- y <- ydat -->
<!-- # now define a function -->

<!-- hobbs.res<-function(x, t, y){ # Hobbs weeds problem -- residual -->
<!--   # This variant uses looping -->
<!-- #  if(length(x) != 3) stop("hobbs.res -- parameter vector n!=3") -->
<!-- #  if(abs(12*x[3])>50) { -->
<!-- #    res<-rep(Inf,12) -->
<!-- #  } else { -->
<!--     res<-x[1]/(1+x[2]*exp(-x[3]*t)) - y -->
<!-- #  } -->
<!-- } -->
<!-- # test it -->
<!-- start1 <- c(200, 50, .3) -->
<!-- ## residuals at start1 -->
<!-- r1 <- hobbs.res(start1, t=tdat, y=ydat) -->
<!-- print(r1) -->
<!-- ``` -->
<!-- Now we try some of the derivative capabilities of `nlsr`. -->

<!-- ```{r hobbnlsr, error=TRUE, warning=TRUE, messages=TRUE} -->
<!-- ## NOTE: some functions may be seemingly correct for R, but we do not -->
<!-- ## get the result desired, despite no obvious error. Always test. -->
<!-- # Try directly to differentiate the residual vector. r1 is numeric, so this should -->
<!-- # return a vector of zeros in a mathematical sense. In fact it gives an error,  -->
<!-- # since R does not want to differentiate a numeric vector. -->
<!-- Jr1a <- fnDeriv(r1, "x") -->
<!-- # Set up a function containing expression with subscripted parameters x[] -->
<!-- hobbs1 <- function(x, t, y){ res<-x[1]/(1+x[2]*exp(-x[3]*t)) - y } -->
<!-- # test the residuals -->
<!-- print(hobbs1(start1, t=tdat, y=ydat)) -->
<!-- # Alternatively, let us set up t and y -->
<!-- y<-c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443, 38.558, 50.156, 62.948, -->
<!--        75.995, 91.972) -->
<!-- t<-1:12 -->
<!-- # try different calls. Note that we need to include t and y data somehow -->
<!-- print(hobbs1(start1)) -->
<!-- print(hobbs1(start1, t, y)) -->
<!-- print(hobbs1(start1, tdat, ydat)) -->
<!-- # We remove t and y, to ensure we don't get results from their presence -->
<!-- rm(t) -->
<!-- rm(y) -->
<!-- # Set up a function containing an expression with named (with numbers) parameters  -->
<!-- # Note that we need to link these to the values in x -->
<!-- hobbs1m <- function(x, t, y){ -->
<!--   x001 <- x[1] -->
<!--   x002 <- x[2] -->
<!--   x003 <- x[3] -->
<!--   res<-x001/(1+x002*exp(-x003*t)) - y  -->
<!-- } -->
<!-- print(hobbs1m(x=start1, t=tdat, y=ydat)) -->
<!-- # Function with explicit use of the expression()  -->
<!-- hobbs1me <- function(x, t, y){ -->
<!--   x001 <- x[1] -->
<!--   x002 <- x[2] -->
<!--   x003 <- x[3] -->
<!--   expression(x001/(1+x002*exp(-x003*t)) - y)  -->
<!-- } -->
<!-- print(hobbs1me(start1, t=tdat, y=ydat)) -->
<!-- # note failure (because the expression is not evaluated?) -->
<!-- # -->
<!-- # Now try to take derivatives -->
<!-- Jr11m <- fnDeriv(hobbs1m, c("x001", "x002", "x003")) -->
<!-- # fails because expression is INSIDE a function (i.e., closure) -->
<!-- # -->
<!-- # try directly differentiating the expression -->
<!-- Jr11ex <-fnDeriv(expression(x001/(1+x002*exp(-x003*t)) - y) -->
<!--                   , c("x001", "x002", "x003")) -->
<!-- # this seems to "work". Let us display the result -->
<!-- Jr11ex -->
<!-- # Set the values of the parameters by name -->
<!-- x001 <- start1[1] -->
<!-- x002 <- start1[2] -->
<!-- x003 <- start1[3] -->
<!-- # and try to evaluate -->
<!-- print(eval(Jr11ex)) -->
<!-- # we need t and y, so set them -->
<!-- t<-tdat -->
<!-- y<-ydat -->
<!-- print(eval(Jr11ex)) -->
<!-- # But there is still a problem. WHY??? -->
<!-- # -->
<!-- # Let us try it piece by piece (column by column) -->
<!-- resx <- expression(x001/(1+x002*exp(-x003*t)) - y) -->
<!-- res1 <- Deriv(resx, "x001", do_substitute=FALSE) -->
<!-- res1 -->
<!-- col1 <- eval(res1) -->
<!-- res2 <- Deriv(resx, "x002", do_substitute=FALSE) -->
<!-- res2 -->
<!-- col2 <- eval(res2) -->
<!-- res3 <- Deriv(resx, "x003", do_substitute=FALSE) -->
<!-- res3 -->
<!-- col3 <- eval(res3) -->
<!-- hobJac <- cbind(col1, col2, col3) -->
<!-- print(hobJac) -->
<!-- ``` -->

<!-- Clearly there is still some work to do to make the mechanism for getting derivatives -->
<!-- more easily, and with less chance of error.  -->

<!-- ### Jacobians -->

<!-- Jacobians, the matrices of partial derivatives of residuals with respect -->
<!-- to the parameters, have a vector input (the parameters). `nlsr` attempts -->
<!-- to generate the code for this computation. **This is the first key improvement -->
<!-- of `nlsr` over `nls()`.** It will likely be a continuing effort to enlarge the -->
<!-- set of functions and expressions that can be handled and to deal with them  -->
<!-- more efficiently. Also it would be nice to automate the generation of numerical -->
<!-- approximations for those components of the Jacobian for which analytic derivatives -->
<!-- are not available.  -->

<!-- Note that the Jacobian inner product (J^T J) differs from the Hessian of the -->
<!-- sum of squares by a matrix whose elements that are an inner product of the residuals with -->
<!-- their second derivatives with respect to the parameters. This is a summation of m matrices -->
<!-- each involving a residual times its (n by n) partial derivatives with respect to the -->
<!-- parameters. Generally we treat this matrix as "ignorable" on the basis that the residuals -->
<!-- should be "small", but clearly this is not always the case. -->

<!-- ### Termination and convergence tests -->

<!-- The success of many optimization codes often depends on knowing when no more -->
<!-- progress can be made. For the present codes, one simple procedure increases -->
<!-- the damping parameter lamda whenever the sum of squares cannot be  -->
<!-- decreased, with termination when the parameters are not altered by the addition -->
<!-- of delta. While this "works", it does incur unnecessary computational effort. -->

<!-- A better approach is that of @BatesWatts81. Their relative offset criterion  -->
<!-- considers the potential progress that could be made in reducing the current -->
<!-- residuals to the initial sum of squares. This is a sensible and very effective -->
<!-- strategy for most situations, but is dangerous where the problem has very  -->
<!-- small or zero residuals, as in the case of an interpolation problem or nonlinear -->
<!-- equations.  -->

<!-- To illustrate this for zero residual problems, let us set up a simple test. -->

<!-- ```{r smallresid} -->
<!-- ## rm(list=ls()) -->
<!-- ## test small resid case with roffset -->
<!-- tt <- 1:25 -->
<!-- ymod <- 10 * exp(-0.01*tt) + 5 -->
<!-- n <- length(tt) -->
<!-- evec0 <- rep(0, n) -->
<!-- evec1 <- 1e-4*runif(n, -.5, .5) -->
<!-- evec2 <-  1e-1*runif(n, -.5, .5) -->
<!-- y0 <- ymod + evec0 -->
<!-- y1 <- ymod + evec1 -->
<!-- y2 <- ymod + evec2 -->
<!-- mydata <- data.frame(tt, y0, y1, y2) -->
<!-- prm0 <- c(aa=1, bb=1, cc=1) -->
<!-- ``` -->
<!-- We provide three sizes of residuals here, but will only illustrate the consequences of -->
<!-- zero residuals (the problem with y0). Let us run the nonlinear least squares problem -->
<!-- to get the interpolating function, first with `nls()`, then with `nlxb()`. In the -->
<!-- second case we have turned off the trace, as the approach "works" because we have  -->
<!-- taken care in the code to provide for problems with small residuals.  -->

<!-- ```{r nlsfit0} -->
<!-- nlsfit0 <-  try(nls(y0 ~ aa * exp(-bb*tt) + cc, start=prm0, data=mydata, trace=FALSE)) -->
<!-- nlsfit0 -->
<!-- library(nlsr) -->
<!-- nlsrfit0 <- try(nlxb(y0 ~ aa * exp(-bb*tt) + cc, start=prm0, data=mydata, trace=FALSE)) -->
<!-- nlsrfit0 -->
<!-- ``` -->
<!-- We can approximate what `nls()` is doing by running a simple Gauss-Newton method one -->
<!-- step at a time. It is (or was at the time of writing) easier to do this in functional  -->
<!-- form with explicit derivatives. Note that we test these functions!  -->

<!-- ```{r trf} -->
<!-- trf <- function(par, data) { -->
<!--     tt <- data[,"tt"] -->
<!--     res <- par["aa"]*exp(-par["bb"]*tt) + par["cc"] - y0 -->
<!-- } -->
<!-- st1 <- c(b1=1, b2=1, b3=1) # a default starting vector (named!) (repeat in case) -->
<!-- print(trf(st1, data=mydata)) -->
<!-- trj <- function(par, data) { -->
<!--     tt <- data[,"tt"] -->
<!--     m <- dim(data)[1] -->
<!--     JJ <- matrix(NA, nrow=m, ncol=3) -->
<!--     JJ[,1] <- exp(-par["bb"]*tt) -->
<!--     JJ[,2] <- - tt * par["aa"] * exp(-par["bb"]*tt) -->
<!--     JJ[,3] <- 1 -->
<!--     JJ -->
<!-- } -->
<!-- Ja <- trj(st1, data=mydata) -->
<!-- print(Ja) -->

<!-- library(numDeriv) -->
<!-- Jn <- jacobian(trf, st1, data=mydata) -->
<!-- print(Jn) -->
<!-- print(max(abs(Jn-Ja))) -->
<!-- ssf <- function(par, data){ -->
<!--    rr <- trf(par, data) -->
<!--    ss <- crossprod(rr) -->
<!-- } -->
<!-- print(ssf(st1, data=mydata)) -->

<!-- library(numDeriv) -->
<!-- print(jacobian(trf, st1, data=mydata)) -->
<!-- ``` -->

<!-- The traditional way to implement a Gauss Newton method was to form the sum of  -->
<!-- squares and cross-products matrix at in traditional linear regression, using -->
<!-- the Jacobian as the data matrix. The right hand side uses the inner product -->
<!-- of the Jacobian with the negative residuals. This approach increases the  -->
<!-- condition number of the problem, and a more direct QR decomposition of the -->
<!-- Jacobian is now the preferred approach. However, let us try both to ensure we -->
<!-- are getting similar answers. First the QR approach. -->

<!-- ```{r gnjn, eval=FALSE} -->
<!-- gnjn <- function(start, resfn, jacfn = NULL, trace = FALSE,  -->
<!-- 		data=NULL, control=list(), ...){ -->
<!-- # simplified Gauss Newton -->
<!--    offset = 1e6 # for no change in parms -->
<!--    stepred <- 0.5 # start with this as per nls() -->
<!--    par <- start -->
<!--    cat("starting parameters:") -->
<!--    print(par) -->
<!--    res <- resfn(par, data, ...) -->
<!--    ssbest <- as.numeric(crossprod(res)) -->
<!--    cat("initial ss=",ssbest,"\n") -->
<!--    par0 <- par -->
<!--    kres <- 1 -->
<!--    kjac <- 0 -->
<!--    keepon <- TRUE -->
<!--    while (keepon) { -->
<!--       cat("kjac=",kjac,"  kres=",kres,"  SSbest now ", ssbest,"\n") -->
<!--       print(par) -->
<!--       JJ <- jacfn(par, data, ...) -->
<!--       kjac <- kjac + 1 -->
<!--       QJ <- qr(JJ) -->
<!--       delta <- qr.coef(QJ, -res) -->
<!--       ss <- ssbest + offset*offset # force evaluation -->
<!--       step <- 1.0 -->
<!--       if (as.numeric(max(par0+delta)+offset) != as.numeric(max(par0+offset)) ) { -->
<!--          while (ss > ssbest) { -->
<!--            par <- par0+delta * step -->
<!--            res <- resfn(par, data, ...) -->
<!--            ss <- as.numeric(crossprod(res)) -->
<!--            kres <- kres + 1 -->
<!-- ##           cat("step =", step,"  ss=",ss,"\n") -->
<!-- ##           tmp <- readline("continue") -->
<!--            if (ss > ssbest) { -->
<!--               step <- step * stepred -->
<!--            } else { -->
<!--               par0 <- par -->
<!--               ssbest <- ss -->
<!--            } -->
<!--          } # end inner loop -->
<!--          if (kjac >= 4)  {  -->
<!--             keepon = FALSE -->
<!--             cat("artificial stop at kjac=4 -- we only want to check output")  -->
<!--          } -->
<!--       } else { keepon <- FALSE # done } -->
<!--    } # end main iteration -->
<!-- } # seems to need this -->

<!-- } # end gnjne -->

<!-- fitgnjn0 <- gnjn(st, trf, trj, data=mydata) -->
<!-- ## Another way -->
<!-- #- set lamda = 0 in nlxb, fix laminc, lamdec -->
<!-- nlx00 <- try(nlxb(y0 ~ aa * exp(-bb*tt) + cc, start=st, data=mydata, trace=FALSE, -->
<!--                      control=list(lamda=0, laminc=0, lamdec=0, watch=TRUE))) -->
<!-- nlx00 -->

<!-- ``` -->

<!-- The first iteration more or less matches the `nls()` result. The problem is quite -->
<!-- ill-conditioned, and `nls()` is using a numerical approximation to the Jacobian, -->
<!-- so deviations of this magnitude from the iterations are not unexpected. -->

<!-- Let us test with a traditional Gauss-Newton. -->

<!-- ```{r gnjn2, eval=FALSE} -->
<!-- gnjn2 <- function(start, resfn, jacfn = NULL, trace = FALSE,  -->
<!-- 		data=NULL, control=list(), ...){ -->
<!-- # simplified Gauss Newton -->
<!--    offset = 1e6 # for no change in parms -->
<!--    stepred <- 0.5 # start with this as per nls() -->
<!--    par <- start -->
<!--    cat("starting parameters:") -->
<!--    print(par) -->
<!--    res <- resfn(par, data, ...) -->
<!--    ssbest <- as.numeric(crossprod(res)) -->
<!--    cat("initial ss=",ssbest,"\n") -->
<!--    kres <- 1 -->
<!--    kjac <- 0 -->
<!--    par0 <- par -->
<!--    keepon <- TRUE -->
<!--    while (keepon) { -->
<!--       cat("kres=",kres,"  kjac=",kjac,"   SSbest now ", ssbest,"\n") -->
<!--       print(par) -->
<!--       JJ <- jacfn(par, data, ...) -->
<!--       kjac <- kjac + 1 -->
<!--       JTJ <- crossprod (JJ) -->
<!--       JTr <- crossprod (JJ, res) -->
<!--       delta <- - as.vector(solve(JTJ, JTr)) -->
<!--       ss <- ssbest + offset*offset # force evaluation -->
<!--       step <- 1.0 -->
<!--       if (as.numeric(max(par0+delta)+offset) != as.numeric(max(par0+offset)) ) { -->
<!--          while (ss > ssbest) { -->
<!--            par <- par0+delta * step -->
<!--            res <- resfn(par, data, ...) -->
<!--            ss <- as.numeric(crossprod(res)) -->
<!--            kres <- kres + 1 -->
<!-- ##           cat("step =", step,"  ss=",ss,"  best is",ssbest,"\n") -->
<!-- ##           tmp <- readline("continue") -->
<!--            if (ss > ssbest) { -->
<!--               step <- step * stepred -->
<!--            } else { -->
<!--               par0 <- par -->
<!--               ssbest <- ss -->
<!--            } -->
<!--          } # end inner loop -->
<!--          if (kjac >= 4)  {  -->
<!--             keepon = FALSE -->
<!--             cat("artificial stop at kjac=4 -- we only want to check output")  -->
<!--          } -->
<!--       } else { keepon <- FALSE # done } -->
<!--    } # end main iteration -->
<!-- } # seems to need this -->
<!-- } # end gnjn2 -->

<!-- fitgnjn20 <- gnjn2(st, trf, trj, data=mydata) -->

<!-- ``` -->

## 9. Nonlinear equations and other non-modelling problems

Solution of sets of nonlinear equations is generally NOT a problem that
is commonly required for statisticians or data analysts. My experience is that
the occasions where it does arise are when workers try to solve the first order
conditions for optimality of a function, rather than try to optimize the
function. If this function is a sum of squares, then we have a nonlinear
least squares problem, and generally such problems are best approached by
methods of the type discussed in this article, in particular codes
`nlfb` and `nls.lm`. 

Conversely, since our problem is, using the notation already established, equivalent to
residuals equal to zero, namely,

   r(x) = 0 
  
the solution of a nonlinear least squares problem for which the final sum
of squares is zero provides a solution to the nonlinear equations. In my 
experience this is a valid approach to the nonlinear equations problem, 
especially if there is concern that a solution may not exist. However, 
there are methods for nonlinear equations, some of which (e.g., @p-nleqslv) are 
available in R packages, and they may be more appropriate. On the other hand,
if the nonlinear least squares tools are familiar, it may be more human-efficient
to use them, at least as a first try.


\pagebreak
# Appendix A: Providing exogenous data

These examples show dotargs do NOT work for any of nlsr, nls, or minpack.lm.
Use of a dataframe or local (calling) environment objects does work in all.


```{r appx1, cache=FALSE, echo=TRUE}
# NOTE: This is OLD material and not consistent in usage with rest of vignette
library(knitr)
# try different ways of supplying data to R nls stuff
ydata <- c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443, 38.558,
           50.156, 62.948, 75.995, 91.972)
ttdata <- seq_along(ydata) # for testing
mydata <- data.frame(y = ydata, tt = ttdata)
hobsc <- y ~ 100*b1/(1 + 10*b2 * exp(-0.1 * b3 * tt))
ste <- c(b1 = 2, b2 = 5, b3 = 3)

# let's try finding the variables
findmainenv <- function(formula, prm) {
  vn <- all.vars(formula)
  pnames <- names(prm)
  ppos <- match(pnames, vn)
  datvar <- vn[-ppos]
  cat("Data variables:")
  print(datvar)
  cat("Are the variables present in the current working environment?\n")
  for (i in seq_along(datvar)){
    cat(datvar[[i]]," : present=",exists(datvar[[i]]),"\n")
  }
}

findmainenv(hobsc, ste)
y <- ydata
tt <- ttdata
findmainenv(hobsc, ste)
rm(y)
rm(tt)

# ===============================


# let's try finding the variables in dotargs
finddotargs <- function(formula, prm, ...) {
  dots <- list(...)
  cat("dots:")
  print(dots)
  cat("names in dots:")
  dtn <- names(dots)
  print(dtn)
  vn <- all.vars(formula)
  pnames <- names(prm)
  ppos <- match(pnames, vn)
  datvar <- vn[-ppos]
  cat("Data variables:")
  print(datvar)
  cat("Are the variables present in the dot args?\n")
  for (i in seq_along(datvar)){
    dname <- datvar[[i]]
    cat(dname," : present=",(dname %in% dtn),"\n")
  }
}

finddotargs(hobsc, ste, y=ydata, tt=ttdata)
# ===============================

y <- ydata
tt <- ttdata
tryq <- try(nlsquiet <- nls(formula=hobsc, start=ste))
if (class(tryq) != "try-error") {print(nlsquiet)} else {cat("try-error\n")}
#- OK
rm(y); rm(tt)

## this will fail
tdots1<-try(nlsdots <- nls(formula=hobsc, start=ste, y=ydata, tt=ttdata))
# But ...
y <- ydata # put data in globalenv
tt <- ttdata
tdots2<-try(nlsdots <- nls(formula=hobsc, start=ste))
tdots2
rm(y); rm(tt)
## but ...
mydata<-data.frame(y=ydata, tt=ttdata)
tframe <- try(nlsframe <- nls(formula=hobsc, start=ste, data=mydata))
if (class(tframe) != "try-error") {print(nlsframe)} else {cat("try-error\n")}
#- OK

y <- ydata
tt <- ttdata
tquiet <- try(nlsrquiet <- nlxb(formula=hobsc, start=ste))
if ( class(tquiet) != "try-error") {print(nlsrquiet)}
#- OK
rm(y); rm(tt)
test <- try(nlsrdots <- nlxb(formula=hobsc, start=ste, y=ydata, tt=ttdata))
#- Note -- does NOT work -- do we need to specify the present env. in nlfb for y, tt??
if (class(test) != "try-error") { print(nlsrdots) } else {cat("Try error\n") }
test2 <- try(nlsframe <- nls(formula=hobsc, start=ste, data=mydata))
if (class(test) != "try-error") {print(nlsframe) } else {cat("Try error\n") }
#- OK


library(minpack.lm)
y <- ydata
tt <- ttdata
nlsLMquiet <- nlsLM(formula=hobsc, start=ste)
print(nlsLMquiet)
#- OK
rm(y)
rm(tt)
## Dotargs
## Following fails
## tdots <- try(nlsLMdots <- nlsLM(formula=hobsc, start=ste, y=ydata, tt=ttdata))
## but ... 
tdots <- try(nlsLMdots <- nlsLM(formula=hobsc, start=ste, data=mydata))
if (class(tdots) != "try-error") { print(nlsLMdots) } else {cat("try-error\n") }
#-  Note -- does NOT work
## dataframe
tframe <- try(nlsLMframe <- nlsLM(formula=hobsc, start=ste, data=mydata) )
if (class(tdots) != "try-error") {print(nlsLMframe)} else {cat("try-error\n") }
#- does not work

## detach("package:nlsr", unload=TRUE)
## Uses nlmrt here for comparison
## library(nlmrt)
## txq <- try( nlxbquiet <- nlxb(formula=hobsc, start=ste))
## if (class(txq) != "try-error") {print(nlxbquiet)} else { cat("try-error\n")}
#- Note -- does NOT work
## txdots <- try( nlxbdots <- nlxb(formula=hobsc, start=ste, y=y, tt=tt) )
## if (class(txdots) != "try-error") {print(nlxbdots)} else {cat("try-error\n")}
#- Note -- does NOT work
## dataframe
## nlxbframe <- nlxb(formula=hobsc, start=ste, data=mydata)
## print(nlxbframe)
#- OK
```

\pagebreak
# Appendix B: Derivative approximation in nls()

This Appendix could benefit from some examples.

### From nls.R

```
numericDeriv <- function(expr, theta, rho = parent.frame(), dir=1.0)
{
    dir <- rep_len(dir, length(theta))
    val <- .Call(C_numeric_deriv, expr, theta, rho, dir)
    valDim <- dim(val)
    if (!is.null(valDim)) {
        if (valDim[length(valDim)] == 1)
            valDim <- valDim[-length(valDim)]
        if(length(valDim) > 1L)
            dim(attr(val, "gradient")) <- c(valDim,
                                            dim(attr(val, "gradient"))[-1L])
    }
    val
}
```

### From nls.c

```
/*
 *  call to numeric_deriv from R -
 *  .Call("numeric_deriv", expr, theta, rho)
 *  Returns: ans
 */
SEXP
numeric_deriv(SEXP expr, SEXP theta, SEXP rho, SEXP dir)
{
    SEXP ans, gradient, pars;
    double eps = sqrt(DOUBLE_EPS), *rDir;
    int start, i, j, k, lengthTheta = 0;

    if(!isString(theta))
	error(_("'theta' should be of type character"));
    if (isNull(rho)) {
	error(_("use of NULL environment is defunct"));
	rho = R_BaseEnv;
    } else
	if(!isEnvironment(rho))
	    error(_("'rho' should be an environment"));
    PROTECT(dir = coerceVector(dir, REALSXP));
    if(TYPEOF(dir) != REALSXP || LENGTH(dir) != LENGTH(theta))
	error(_("'dir' is not a numeric vector of the correct length"));
    rDir = REAL(dir);

    PROTECT(pars = allocVector(VECSXP, LENGTH(theta)));

    PROTECT(ans = duplicate(eval(expr, rho)));

    if(!isReal(ans)) {
	SEXP temp = coerceVector(ans, REALSXP);
	UNPROTECT(1);
	PROTECT(ans = temp);
    }
    for(i = 0; i < LENGTH(ans); i++) {
	if (!R_FINITE(REAL(ans)[i]))
	    error(_("Missing value or an infinity produced when evaluating the model"));
    }
    const void *vmax = vmaxget();
    for(i = 0; i < LENGTH(theta); i++) {
	const char *name = translateChar(STRING_ELT(theta, i));
	SEXP s_name = install(name);
	SEXP temp = findVar(s_name, rho);
	if(isInteger(temp))
	    error(_("variable '%s' is integer, not numeric"), name);
	if(!isReal(temp))
	    error(_("variable '%s' is not numeric"), name);
	if (MAYBE_SHARED(temp)) /* We'll be modifying the variable, so need to make sure it's unique PR#15849 */
	    defineVar(s_name, temp = duplicate(temp), rho);
	MARK_NOT_MUTABLE(temp);
	SET_VECTOR_ELT(pars, i, temp);
	lengthTheta += LENGTH(VECTOR_ELT(pars, i));
    }
    vmaxset(vmax);
    PROTECT(gradient = allocMatrix(REALSXP, LENGTH(ans), lengthTheta));

    for(i = 0, start = 0; i < LENGTH(theta); i++) {
	for(j = 0; j < LENGTH(VECTOR_ELT(pars, i)); j++, start += LENGTH(ans)) {
	    SEXP ans_del;
	    double origPar, xx, delta;

	    origPar = REAL(VECTOR_ELT(pars, i))[j];
	    xx = fabs(origPar);
	    delta = (xx == 0) ? eps : xx*eps;
	    REAL(VECTOR_ELT(pars, i))[j] += rDir[i] * delta;
	    PROTECT(ans_del = eval(expr, rho));
	    if(!isReal(ans_del)) ans_del = coerceVector(ans_del, REALSXP);
	    UNPROTECT(1);
	    for(k = 0; k < LENGTH(ans); k++) {
		if (!R_FINITE(REAL(ans_del)[k]))
		    error(_("Missing value or an infinity produced when evaluating the model"));
		REAL(gradient)[start + k] =
		    rDir[i] * (REAL(ans_del)[k] - REAL(ans)[k])/delta;
	    }
	    REAL(VECTOR_ELT(pars, i))[j] = origPar;
	}
    }
    setAttrib(ans, install("gradient"), gradient);
    UNPROTECT(4);
    return ans;
}
```

### nlsr::numericDerivR.R

This is a replacement for the nls() function numericDeriv() that is coded all in R.

```{r rndr, code=readLines("../R/numericDerivR.R"), echo=TRUE}
```

Let us compute the Jacobian for the Hobbs problem with this function and its
`nls()` original. I find the mechanism for these functions awkward.

```{r ndx, echo=TRUE}
library(nlsr) # so we have numericDerivR code
# Data for Hobbs problem
weed  <-  c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443, 
            38.558, 50.156, 62.948, 75.995, 91.972) # for testing
tt  <-  seq_along(weed) # for testing
# A simple starting vector -- must have named parameters for nlxb, nls, wrapnlsr.
st  <-  c(b1=1, b2=1, b3=1)
wmodu  <-   weed ~ b1/(1+b2*exp(-b3*tt))
weeddf  <-  data.frame(weed=weed, tt=tt)
weedenv <- list2env(weeddf)
weedenv$b1 <- st[[1]]
weedenv$b2 <- st[[2]]
weedenv$b3 <- st[[3]]
rexpr<-call("-",wmodu[[3]], wmodu[[2]])
r0<-eval(rexpr, weedenv)
cat("Sumsquares at 1,1,1 is ",sum(r0^2),"\n")## Another way
expr <- wmodu
rho <- weedenv
rexpr<-call("-",wmodu[[3]], wmodu[[2]])
res0<-eval(rexpr, rho) # the base residuals
res0
## Try the numericDeriv option
theta<-names(st)
nDnls<-numericDeriv(rexpr, theta, weedenv)
nDnls
nDnlsR<-numericDerivR(rexpr, theta, weedenv)
nDnlsR
```


\pagebreak

# Appendix C: A comparison of nlsr::nlxb with nls and minpack::nlsLM

**R** has several tools for estimating nonlinear models and minimizing sums of squares functions.
Sometimes we talk of **nonlinear regression** and at other times of 
**minimizing a sum of squares function**. Many workers conflate these two tasks.
In this appendix, some of the differences between the tools available in R for these two
computational tasks are highlighted. In particular, we compare the tools from the package
`nlsr` (@nlsr2019manual), particularly function `nlxb()` with those from base-R `nls()` and the `nlsLM` 
function of package `minpack.lm` (@minpacklm12). We also compare how `nlsr:nlfb()` and
`minpack.lm:nls.lm` allow a sum of squares function to be minimized.


### Principal differences

The main differences in the tools relate to the following features:

- the way in which derivative information is computed for the Jacobian of the modelling function
- specification of the model as an R programmatic function is unavailable in `nlsr::nlxb()`
- the use of a Marquardt stabilization for solution of the linearized least squares problem at
each iteration
- details of the criterion used to terminate the iteration
- the structure of the output of the tools
- how models are predicted for new data.

### Derivative information

As detailed above, `nlsr::nlxb()` attempts to use symbolic and algorithmic tools to obtain the derivatives
of the model expression that are needed for the **Jacobian** matrix that is used in creating
a linearized sub-problem at each iteration of an attempted solution of the minimization of the
sum of squared residuals. As discussed in the section "Analytic versus approximate Jacobians" and
using the code in Appendix B, `nls()` and `minpack.lm::nlsLM()` use a very simple forward-difference
approximation for the partial derivatives for the Jacobian. 

Forward difference approximations are less accurate than central differences, and both are subject
to numerical error when the modelling function is "flat", so that there is a large amount of
digit cancellation in the subtraction necessary to compute the derivative approximation.

`minpack.lm::nlsLM` uses the same derivatives as far as I can determine. The loss of information 
compared to the analytic or algorithmic derivatives of `nlsr::nlxb()` is important in that it
can lead to Jacobian matrices that are computationally singular, where `nls()` will stop with
"singular gradient". (It is actually the Jacobian which is singular here, and I will stay with
that terminology.)  `minpack.lm::nlsLM()` may fail to get started if the initial Jacobian is
singular, but is less susceptible in general, as described in the sub-section on Marquardt 
stabilization which follows.

#### Consequences of different derivative computations

While readers might expect that the precise derivative information of `nlsr::nlxb()` would mean
a faster solution, this is quite often not the case. Approximate derivatives may allow faster
approach to the solution by "ironing out" wrinkles in the function surface. In my opinion, the
main advantage of precise derivative information is in testing that we actually have arrived
at a solution. 

There are even some cases where the approximation may be helpful, though users may not realize
the potential danger. Thanks to Karl Schilling for an example of modelling with the function

  `a * (x ^ b)`
  
where `x` is our data and we wish to estimate `a` and `b`. Now the partial derivative of this
function w.r.t. `b` is 

```{r derivexp, eval=TRUE}
partialderiv <- D(expression(a * (x ^ b)),"b")
print(partialderiv)
```

The danger here is that we may have data values `x = 0`, in which case the **derivative** is 
not defined, though the model can still be evaluated. Thus `nlsr::nlxb()` will not compute
a solution, while `nls()` and `minpack.lm::nlsLM()` will generally proceed. A workaround is
to provide a very small value instead of zero for the data, though I find this inelegant.
Another approach is to drop the offending element of the data, though this risks altering
the model estimated. A proper treatment might be to develop the limit of the derivative as 
the data value goes to zero, but finding general software that can detect and deal with 
this is a large project.

#### Timing comparisons

Let us compare timings on the (scaled) Hobbs weed problem. 

```{r timingshobbsmodel}
require(microbenchmark)
## nls on Hobbs scaled model
wmods  <-   weed ~ 100*b1/(1+10*b2*exp(-0.1*b3*tt))
stx<-c(b1=2, b2=5, b3=3)
tnls<-microbenchmark((anls<-nls(wmods, start=stx, data=weeddf)), unit="us")
tnls
## nlsr::nlfb() on Hobbs scaled model
tnlxb<-microbenchmark((anlxb<-nlsr::nlxb(wmods, start=stx, data=weeddf)), unit="us")
tnlxb
## minpack.lm::nlsLM() on Hobbs scaled model
tnlsLM<-microbenchmark((anlsLM<-minpack.lm::nlsLM(start=stx, formula=wmods, data=weeddf)), 
                       unit="us")
tnlsLM
```

### Programmatic modelling functions

A consequence of the symbolic derivative approach in `nlsr::nlxb()` is that it cannot be
applied to a modelling expression that includes an R function i.e., sub-program. 

This limitation could be overcome using appropriate automatic differentiation
code (to provide derivative computations based on transformation of the modelling 
function's programmatic form). The present work-around is to use numerical approximation
by specifying the control element `japprox`.

### Functional expression of residuals and Jacobian

```{r timingnlsrminpackfn}
require(microbenchmark)
## nlsr::nlfb() on Hobbs scaled
tnlfb<-microbenchmark((anlfb<-nlsr::nlfb(start=st1, resfn=shobbs.res, jacfn=shobbs.jac)), unit="us")
tnlfb
## minpack.lm::nls.lm() on Hobbs scaled
tnls.lm<-microbenchmark((anls.lm<-minpack.lm::nls.lm(par=st1, fn=shobbs.res, jac=shobbs.jac)))
tnls.lm
```

<!-- > microbenchmark( (for (i in 1:1e5){x=1}), unit="milliseconds") -->
<!-- Error in match.arg(unit) :  -->
<!--   'arg' should be one of “ns”, “us”, “ms”, “s”, “t”, “hz”, “khz”, “mhz”, “eps”, “f” -->


### Marquardt stabilization

All three of the R functions under consideration try to minimize a sum of squares. If the
model is provided in the form

   `y ~ (some expression)`
   
then the residuals are computed by evaluating the difference between `(some expression)` and `y`.
My own preference, and that of K F Gauss, is to use `(some expression) - y`. This is to avoid
having to be concerned with the negative sign -- the derivative of the residual defined in this
way is the same as the derivative of the modelling function, and we avoid the chance of a sign
error. The Jacobian matrix is made up of elements where element `i, j` is the partial derivative
of residual `i` w.r.t. parameter `j`. 

`nls()` attempts to minimize a sum of squared residuals by a Gauss-Newton method. If we
compute a Jacobian matrix `J` and a vector of residuals `r` from a vector of parameters `x`, 
then we can define a linearized problem 

$$  J^T J  \delta  = - J^T r $$

This leads to an iteration where, from a set of starting parameters `x0`, we compute

$$ x_{i+1} = x_i + \delta$$

This is commonly modified to use a step factor `step`

$$ x_{i+1} = x_i + step * \delta$$

It is in the mechanisms to choose the size of `step` and to decide when to terminate the
iteration that Gauss-Newton methods differ. Indeed, though I have tried several times, I
find the very convoluted code behind `nls()` very difficult to decipher. Unfortunately, its
authors have (at 2018 as far as I am aware) all ceased to maintain the code.

Both `nlsr::nlxb()` and `minpack.lm::nlsLM` use a Levenberg-Marquardt stabilization of the iteration.
(@Marquardt1963, @Levenberg1944), solving

$$ (J^T J + \lambda D)   \delta  = - J^T r $$

where $D$ is some diagonal matrix and lambda is a number of modest size initially. Clearly
for $\lambda = 0$ we have a Gauss-Newton method. Typically, the sum of squares of the residuals
calculated at the "new" set of parameters is used as a criterion for keeping those parameter
values. If so, the size of $\lambda$ is reduced. If not, we increase the size of $\lambda$ and
compute a new $\delta$. Note that a new $J$, the expensive step in each iteration, is NOT
required. 

As for Gauss-Newton methods, the details of how to start, adjust and terminate the iteration
lead to many variants, increased by different possibilities for specifying $D$. See
@jncnm79. There are also a number of ways to solve the stabilized Gauss-Newton equations,
some of which do not require the explicit $J^T J$ matrix. 

### Criterion used to terminate the iteration

`nls()` and `nlsr` use a form of the relative offset convergence criterion, @BatesWatts81. 
`minpack.lm` uses a somewhat different and more complicated set of tests. Unfortunately,
the relative offset criterion as implemented in `nls()` is unsuited to problems where 
the residuals can be zero. As of R 4.1.0, there is a work-around in providing a non-zero
value to the control element `scaleOffset` as documented in the manual page of `nls()`.
See *An illustrative nonlinear regression problem* below.


### Output of the modelling functions

`nls()` and `nlsLM()` return the same solution structure. Let us examine this for one of our 
example results (we will choose one that does NOT have small residuals, so that all
the functions "work").

```{r hiddenexprob1, echo=FALSE}
# Here we set up an example problem with data
# Define independent variable
t0 <- 0:19
t0a<-t0
t0a[1]<-1e-20 # very small value
# Drop first value in vectors
t0t<-t0[-1]
y1 <- 4 * (t0^0.25)
y1t<-y1[-1]
n <- length(t0)
fuzz <- rnorm(n)
range <- max(y1)-min(y1)
## add some "error" to the dependent variable
y1q <- y1 + 0.2*range*fuzz
edta <- data.frame(t0=t0, t0a=t0a, y1=y1, y1q=y1q)
edtat <- data.frame(t0t=t0t, y1t=y1t)

start1 <- c(a=1, b=1)
try(nlsy0t0ax <- nls(formula=y1~a*(t0a^b), start=start1, data=edta, control=nls.control(maxiter=10000)))
nlsry1t0a <- nlxb(formula=y1~a*(t0a^b), start=start1, data=edta)
library(minpack.lm)
nlsLMy1t0a <- nlsLM(formula=y1~a*(t0a^b), start=start1, data=edta)
```

```{r nlsstr}
str(nlsy0t0ax)
```

The `minpack.lm::nlsLM` output has the same structure, which could be revealed by the 
R command `str(nlsLMy1t0a)`.
Note that this structure has a lot of special functions in the sub-list `m`.
By contrast, the `nlsr()` output is much less flamboyant. There are, in fact,
no functions as part of the structure. 

```{r nlsrstr}
str(nlsry1t0a)
```

Which of these approaches is "better" can be debated. My preference is for the results
of optimization computations to be essentially data, including messages, though some
tools within some of my packages will return functions for specific reasons, e.g.,
to return a function from an expression. However, I prefer to use specified functions
such as `predict.nlsr()` below to obtain predictions. I welcome comment and discussion,
as this is not, in my view, a closed topic.


### Prediction

Let us predict our models at the mean of the data.
Because `nlxb()` returns a different structure from that found by `nls()` and 
`nlsLM()` the code for `predict()` for an object from `nlsr` is different. 
`minpack.lm` uses `predict.nls` since the output structure of the modelling step is equivalent to that from `nls()`.

```{r  pred1}
nudta <- colMeans(edta)
predict(nlsy0t0ax, newdata=nudta)
predict(nlsLMy1t0a, newdata=nudta)
predict(nlsry1t0a, newdata=nudta)
```

### An illustrative nonlinear regression problem

So we can illustrate some of the issues, let us create some example data for a seemingly
straightforward computational problem.

```{r exprob1}
# Here we set up an example problem with data
# Define independent variable
t0 <- 0:19
t0a<-t0
t0a[1]<-1e-20 # very small value
# Drop first value in vectors
t0t<-t0[-1]
y1 <- 4 * (t0^0.25)
y1t<-y1[-1]
n <- length(t0)
fuzz <- rnorm(n)
range <- max(y1)-min(y1)
## add some "error" to the dependent variable
y1q <- y1 + 0.2*range*fuzz
edta <- data.frame(t0=t0, t0a=t0a, y1=y1, y1q=y1q)
edtat <- data.frame(t0t=t0t, y1t=y1t)
```

Let us try this example modelling `y0` against `t0`. Note that this is a zero-residual problem,
so `nls()` should complain or fail, which it appears to do by exceeding the iteration limit,
which is not very communicative of the underlying issue. The `nls()` documentation warns

"Warning

Do not use nls on artificial "zero-residual" data."

It goes on to recommend that users add "error" to the data to avoid such problems.
I feel this is a very unsatisfactory kludge. It is NOT due to a genuine mathematical
issue, but due to the relative offset convergence criterion used to terminate the
method. Using the contr

<!-- ```{r smallres} -->
<!-- x <- (1:m) -->
<!-- y <- 2.5 * (x ^ 0.33) -->
<!-- fmla <- y ~ a * (x^b) -->
<!-- edta <- data.frame(x=x, y=y) -->
<!-- library(nlsr) -->
<!-- library(minpack.lm) -->
<!-- enls <- try(nls(fmla, start=c(a=1, b=1), data=edta)) -->
<!-- enlxb <- try(nlxb(fmla, start=c(a=1, b=1), data=edta)) -->
<!-- enlxb -->
<!-- enlsLM <- try(nlsLM(fmla, start=c(a=1, b=1), data=edta)) -->
<!-- enlsLM -->
<!-- ``` -->

Here is the output.

#### nls

```{r extrynls}
cprint <- function(obj){
   # print object if it exists
  sobj<-deparse(substitute(obj))
  if (exists(sobj)) {
      print(obj)
  } else {
      cat(sobj," does not exist\n")
  }
#  return(NULL)
}
start1 <- c(a=1, b=1)
try(nlsy0t0 <- nls(formula=y1~a*(t0^b), start=start1, data=edta))
cprint(nlsy0t0)
# Since this fails to converge, let us increase the maximum iterations
try(nlsy0t0x <- nls(formula=y1~a*(t0^b), start=start1, data=edta,
                    control=nls.control(maxiter=10000)))
cprint(nlsy0t0x)
try(nlsy0t0ax <- nls(formula=y1~a*(t0a^b), start=start1, data=edta, 
                     control=nls.control(maxiter=10000)))
cprint(nlsy0t0ax)
try(nlsy0t0t <- nls(formula=y1t~a*(t0t^b), start=start1, data=edtat))
cprint(nlsy0t0t)
```

#### nlsr

```{r extry1nlsr}
nlsry1t0 <- try(nlxb(formula=y1~a*(t0^b), start=start1, data=edta))
cprint(nlsry1t0)
nlsry1t0a <- nlxb(formula=y1~a*(t0a^b), start=start1, data=edta)
cprint(nlsry1t0a)
nlsry1t0t <- nlxb(formula=y1t~a*(t0t^b), start=start1, data=edtat)
cprint(nlsry1t0t)
```

#### minpack.lm

```{r extry1minlm}
library(minpack.lm)
nlsLMy1t0 <- nlsLM(formula=y1~a*(t0^b), start=start1, data=edta)
nlsLMy1t0
nlsLMy1t0a <- nlsLM(formula=y1~a*(t0a^b), start=start1, data=edta)
nlsLMy1t0a
nlsLMy1t0t <- nlsLM(formula=y1t~a*(t0t^b), start=start1, data=edtat)
nlsLMy1t0t
```

We have seemingly found a workaround for our difficulty, but I caution that initially 
I found very unsatisfactory results when I set the "very small value" to 1.0e-7. The
correct approach is clearly to understand what is going on. Getting computers to provide
that understanding is a serious challenge.


### Problems that are NOT regressions

Some nonlinear least squares problems are NOT nonlinear regressions. That is, we do not
have a formula `y ~ (some function)` to define the problem. This is another reason to
use the residual in the form `(some function) - y` In many cases of interest we have
no `y`.

The Brown and Dennis test problem (@More1981TUO, problem 16) is of this form. Suppose we have `m` observations,
then we create a scaled index `t` which is the "data" for the function. To run the nonlinear least squares
functions that use a formula, we do, however, need a "y" variable. Clearly adding zero to the residual
will not change the problem, so we set the data for "y" as all zeros. Note that `nls()` and `nlsLM()`
need some extra iterations to find the solution to this somewhat nasty problem.

```{r brownden}
m <- 20
t <- seq(1, m) / 5
y <- rep(0,m)
library(nlsr)
library(minpack.lm)

bddata <- data.frame(t=t, y=y)
bdform <- y ~ ((x1 + t * x2 - exp(t))^2 + (x3 + x4 * sin(t) - cos(t))^2)
prm0 <- c(x1=25, x2=5, x3=-5, x4=-1)
fbd <-model2ssgrfun(bdform, prm0, bddata)
cat("initial sumsquares=",as.numeric(crossprod(fbd(prm0))),"\n")
nlsrbd <- nlxb(bdform, start=prm0, data=bddata, trace=FALSE)
nlsrbd

nlsbd10k <- nls(bdform, start=prm0, data=bddata, trace=FALSE, 
                control=nls.control(maxiter=10000))
nlsbd10k

nlsLMbd10k <- nlsLM(bdform, start=prm0, data=bddata, trace=FALSE, 
                    control=nls.lm.control(maxiter=10000, maxfev=10000))
nlsLMbd10k
```

Let us try predicting the "residual" for some new data.

```{r browndenpred}
ndata <- data.frame(t=c(5,6), y=c(0,0))
predict(nlsLMbd10k, newdata=ndata)

# now nls
predict(nlsbd10k, newdata=ndata)

# now nlsr
predict(nlsrbd, newdata=ndata)
```


We could, of course, try setting up a different formula, since the "residuals" can be
computed in any way such that their absolute value is the same. 
Therefore we could try moving the exponential
part of the function for each equation to the left hand side as in `bdf2` below.
```{r brownden2}
bdf2 <-  (x1 + t * x2 - exp(t))^2 ~ - (x3 + x4 * sin(t) - cos(t))^2
```

However, we discover that the parsing of the model formula fails for this formulation.

<!-- ?? Do we need a check on the formula for this?? -->

### A check on the Brown and Dennis calculation via function minimization

We can attack the Brown and Dennis problem by applying nonlinear function minimization
programs to the sum of squared "residuals" as a function of the parameters. The code
below does this. We omit the output for space reasons.

```{r bdcheck, eval=FALSE}
#' Brown and Dennis Function
#'
#' Test function 16 from the More', Garbow and Hillstrom paper.
#'
#' The objective function is the sum of \code{m} functions, each of \code{n}
#' parameters.
#'
#' \itemize{
#'   \item Dimensions: Number of parameters \code{n = 4}, number of summand
#'   functions \code{m >= n}.
#'   \item Minima: \code{f = 85822.2} if \code{m = 20}.
#' }
#'
#' @param m Number of summand functions in the objective function. Should be
#'   equal to or greater than 4.
#' @return A list containing:
#' \itemize{
#'   \item \code{fn} Objective function which calculates the value given input
#'   parameter vector.
#'   \item \code{gr} Gradient function which calculates the gradient vector
#'   given input parameter vector.
#'   \item \code{fg} A function which, given the parameter vector, calculates
#'   both the objective value and gradient, returning a list with members
#'   \code{fn} and \code{gr}, respectively.
#'   \item \code{x0} Standard starting point.
#' }
#' @references
#' More', J. J., Garbow, B. S., & Hillstrom, K. E. (1981).
#' Testing unconstrained optimization software.
#' \emph{ACM Transactions on Mathematical Software (TOMS)}, \emph{7}(1), 17-41.
#' \url{https://doi.org/10.1145/355934.355936}
#'
#' Brown, K. M., & Dennis, J. E. (1971).
#' \emph{New computational algorithms for minimizing a sum of squares of
#' nonlinear functions} (Report No. 71-6).
#' New Haven, CT: Department of Computer Science, Yale University.
#'
#' @examples
#' # Use 10 summand functions
#' fun <- brown_den(m = 10)
#' # Optimize using the standard starting point
#' x0 <- fun$x0
#' res_x0 <- stats::optim(par = x0, fn = fun$fn, gr = fun$gr, method =
#' "L-BFGS-B")
#' # Use your own starting point
#' res <- stats::optim(c(0.1, 0.2, 0.3, 0.4), fun$fn, fun$gr, method =
#' "L-BFGS-B")
#'
#' # Use 20 summand functions
#' fun20 <- brown_den(m = 20)
#' res <- stats::optim(fun20$x0, fun20$fn, fun20$gr, method = "L-BFGS-B")
#' @export
#`
brown_den <- function(m = 20) {
  list(
    fn = function(par) {
      x1 <- par[1]
      x2 <- par[2]
      x3 <- par[3]
      x4 <- par[4]

      ti <- (1:m) * 0.2
      l <- x1 + ti * x2 - exp(ti)
      r <- x3 + x4 * sin(ti) - cos(ti)
      f <- l * l + r * r
      sum(f * f)
    },
    gr = function(par) {
      x1 <- par[1]
      x2 <- par[2]
      x3 <- par[3]
      x4 <- par[4]

      ti <- (1:m) * 0.2
      sinti <- sin(ti)
      l <- x1 + ti * x2 - exp(ti)
      r <- x3 + x4 * sinti - cos(ti)
      f <- l * l + r * r
      lf4 <- 4 * l * f
      rf4 <- 4 * r * f
      c(
        sum(lf4),
        sum(lf4 * ti),
        sum(rf4),
        sum(rf4 * sinti)
      )
    },
    fg = function(par) {
      x1 <- par[1]
      x2 <- par[2]
      x3 <- par[3]
      x4 <- par[4]

      ti <- (1:m) * 0.2
      sinti <- sin(ti)
      l <- x1 + ti * x2 - exp(ti)
      r <- x3 + x4 * sinti - cos(ti)
      f <- l * l + r * r
      lf4 <- 4 * l * f
      rf4 <- 4 * r * f

      fsum <- sum(f * f)
      grad <- c(
        sum(lf4),
        sum(lf4 * ti),
        sum(rf4),
        sum(rf4 * sinti)
      )

      list(
        fn = fsum,
        gr = grad
      )
    },
    x0 = c(25, 5, -5, 1)
  )
}
mbd <- brown_den(m=20)
mbd
mbd$fg(mbd$x0)
bdsolnm <- optim(mbd$x0, mbd$fn, control=list(trace=0))
bdsolnm
bdsolbfgs <- optim(mbd$x0, mbd$fn, method="BFGS", control=list(trace=0))
bdsolbfgs

library(optimx)
methlist <- c("Nelder-Mead","BFGS","Rvmmin","L-BFGS-B","Rcgmin","ucminf")

solo <- opm(mbd$x0, mbd$fn, mbd$gr, method=methlist, control=list(trace=0))
summary(solo, order=value)

## A failure above is generally because a package in the 'methlist' is not installed.
```

\pagebreak
# References

---
title: "Symbolic and analytical derivatives in R"
author: "John C. Nash"
date: "`r Sys.Date()`"
output: 
  pdf_document:
    toc: true
bibliography: ImproveNLS.bib
vignette: >
  %\VignetteIndexEntry{Symbolic and analytical derivatives in R}
  %\VignetteEngine{knitr::rmarkdown}
  %\usepackage[utf8]{inputenc}
---

---
<!-- Above creates horizontal line -->

```{r setup, echo=FALSE}
rm(list=ls()) # clear workspace for each major section
```

\pagebreak


This article is an attempt to catalog and illustrate the various 
capabilities in the **R** statistical computing system to 
perform analytic or symbolic differentiation. There are many traps and pitfalls for
the unwary in doing this, and it is hoped that this rather
long treatment will serve to record these and show how to avoid
them, and how to reliably compute the derivatives desired.
Derivative capabilities of **R** are in the base system (essentially
the functions `D()` and `deriv()`) and in
different packages, namely `nlsr`, `Deriv`, and `Ryacas`. General tools for
approximations to derivatives are found in the package `numDeriv` as
well as `optimx`. Other approximations may be embedded in 
various packages, but not necessarily exported for use in scripts
or packages.

As a way of recording where attention is needed either to this document
or to the functions and methods described, I have put double question marks
in various places.

Note: To distinguish output results (which are prefaced '`## `' by **knitr**, 
I have attempted to put comments in the **R** code with the preface '`#- `'.)


## Available analytic differentiation tools

**R** has a number of tools for finding analytic derivatives.

- **stats**: tools `D()` and `deriv()` (@Rcite)

- **nlsr**: tools (formerly `nlsr`) `nlsDeriv()`, `fnDeriv()`, and the wrapper `model2rjfun` (@nlsr2019manual)

- **Deriv**: tools `Deriv()` (@Deriv-manual)

- **Ryacas**: tools ??  (@Ryacas-manual)

- In 2018, Changcheng Li conducted a Google Summer of Code project to link R to Julia's
Automatic Differentiation tools, resulting in the experimental package `autodiffr`
(see https://github.com/Non-Contradiction/autodiffr).

<!-- - ?? any other packages that give analytic derivatives? -->

## How the tools are used

This is an overview section to give an idea of the capabilities. It is
not intended to be exhaustive, but to give pointers to how the tools can
be used quickly.

An important issue that may cause a lot of difficulty is the iterating of the tools.
That is, we compute a derivative, then want to apply a tool to the derivative to get
a second derivative. In doing so, we need to be careful that the type (class??) of
the quantity output by the tool is passed back into the tool in a form that will
generate a derivative expression. Some examples are presented. 

We also note that the **Deriv** package will give a result in cases when the input
is undefined. This is clearly a bug. There is an example below on the section for
`Deriv`.

### stats (i.e., the base R installation)

`D()`, `deriv()` and `deriv3()`: As `deriv3()` is stated to be the same as `deriv()` but with
argument `hessian=TRUE`, we will for now only consider the first two.

```{r chunk01}
dx2x <- deriv(~ x^2, "x") 
dx2x
mode(dx2x)
str(dx2x)
x <- -1:2
eval(dx2x) # This is evaluated at -1, 0, 1, 2, with the result in the "gradient" attribute
# Note that we cannot (easily) differentiate this again.
firstd <- attr(dx2x,"gradient")
str
#  ... and the following gives an error
d2x2x <- try(deriv(firstd, "x"))
str(d2x2x)
#- Build a function from the expression
fdx2x<-function(x){eval(dx2x)}
fdx2x(1)
fdx2x(3.21)
fdx2x(1:5)
#- # Now try D()
Dx2x <- D(expression(x^2), "x")
Dx2x
x <- -1:2
eval(Dx2x)
# We can differentiate aggain
D2x2x <- D(Dx2x,"x")
D2x2x
eval(D2x2x) #- But we don't get a vector -- could be an issue in gradients/Jacobians
#- Note how we handle an expression stored in a string via parse(text=  ))
sx2 <- "x^2"
sDx2x <- D(parse(text=sx2), "x")
sDx2x
#- But watch out! The following "seems" to work, but the answer is not as intended.  The problem is that the first
# argument is evaluated before being used.  Since 
# x exists, it fails
x
Dx2xx <- D(x^2, "x")
Dx2xx
eval(Dx2xx)
#-  Something 'tougher':
trig.exp <- expression(sin(cos(x + y^2)))
( D.sc <- D(trig.exp, "x") )
all.equal(D(trig.exp[[1]], "x"), D.sc)
( dxy <- deriv(trig.exp, c("x", "y")) )
y <- 1
eval(dxy)
eval(D.sc)
#-  function returned:
deriv((y ~ sin(cos(x) * y)), c("x","y"), func = TRUE)
#- ??#-  Surely there is an error, since documentation says no lhs! i.e.,
#- "expr: a 'expression' or 'call' or (except 'D') a formula with no lhs."
#-  function with defaulted arguments:
(fx <- deriv(y ~ b0 + b1 * 2^(-x/th), c("b0", "b1", "th"),
             function(b0, b1, th, x = 1:7){} ) )
fx(2, 3, 4)
#-  First derivative
D(expression(x^2), "x")
#-  stopifnot(D(as.name("x"), "x") == 1) #- A way of testing. 
#- This works by coercing "x" to name/symbol, and derivative should be 1.
#- Would fail only if "x" cannot be so coerced. How could this happen??
#-  Higher derivatives showing deriv3
myd3 <- deriv3(y ~ b0 + b1 * 2^(-x/th), c("b0", "b1", "th"),
     c("b0", "b1", "th", "x") )
myd3(2,3,4, x=1:7)
#- check against deriv()
myd3a <- deriv(y ~ b0 + b1 * 2^(-x/th), c("b0", "b1", "th"),
     c("b0", "b1", "th", "x"), hessian=TRUE )
myd3a(2,3,4, x=1:7)
identical(myd3a, myd3) #- Remember to check things!

#-  Higher derivatives:
DD <- function(expr, name, order = 1) {
   if(order < 1) stop("'order' must be >= 1")
   if(order == 1) D(expr, name)
   else DD(D(expr, name), name, order - 1)
}
DD(expression(sin(x^2)), "x", 3)
#-  showing the limits of the internal "simplify()" :
#-  -sin(x^2) * (2 * x) * 2 + ((cos(x^2) * (2 * x) * (2 * x) + sin(x^2) *
#-     2) * (2 * x) + sin(x^2) * (2 * x) * 2)
```

### nlsr

```{r chunk02}
library(nlsr)
dx2xn <- nlsDeriv(~ x^2, "x")
dx2xn
mode(dx2xn)
str(dx2xn)
x <- -1:2
eval(dx2xn) # This is evaluated at -1, 0, 1, 2, BUT result is returned directly,
#-  NOT in "gradient" attribute
firstdn <- dx2xn
str(firstdn)
d2x2xn <- nlsDeriv(firstdn, "x")
d2x2xn
d2x2xnF <- nlsDeriv(firstdn, "x", do_substitute=FALSE)
d2x2xnF # in this case we get the same result
d2x2xnT <- nlsDeriv(firstdn, "x", do_substitute=TRUE)
d2x2xnT # 0 ## WATCH OUT

#- ?? We can iterate the derivatives
nlsDeriv(d2x2xn, "x")

nlsDeriv(x^2, "x")# 0
nlsDeriv(x^2, "x", do_substitute=FALSE)# 0
nlsDeriv(x^2, "x", do_substitute=TRUE) # 2 * x
nlsDeriv(~ x^2, "x") # 2 * x
nlsDeriv(~ x^2, "x", do_substitute=FALSE) # 2 * x
nlsDeriv(~ x^2, "x", do_substitute=TRUE) # 2 * x

#?? firstde <- quote(firstd)
#?? firstde
#?? firstde <- bquote(firstd)
#?? firstde
#?? nlsDeriv(firstde, "x")
d2 <- nlsDeriv(2 * x, "x")
str(d2)
d2
#?? firstc <- as.call(firstd)
#?? nlsDeriv(firstc, "x")
#- Build a function from the expression
#?? fdx2xn<-function(x){eval(dx2xn)}
#?? fdx2xn(1)
#?? fdx2xn(3.21)
#?? fdx2xn(1:5)
```

The tool `codeDeriv` returns an R expression to evaluate the
derivative efficiently.  `fnDeriv` wraps it in a function.
By default the arguments to the function are constructed from
all variables in the
expression.  In the example below this includes `x`.

```{r chunk03}
codeDeriv(parse(text="b0 + b1 * 2^(-x/th)"), c("b0", "b1", "th")) 
#- Include parameters as arguments
fj.1 <- fnDeriv(parse(text="b0 + b1 * 2^(-x/th)"), c("b0", "b1", "th")) 
head(fj.1)
fj.1(1,2,3,4)
#- Get all parameters from the calling environment
fj.2 <- fnDeriv(parse(text="b0 + b1 * 2^(-x/th)"), c("b0", "b1", "th"),
        args = character())
head(fj.2)
b0 <- 1
b1 <- 2
x <- 3
th <- 4 
fj.2()

#- Just use an expression
fje <- codeDeriv(parse(text="b0 + b1 * 2^(-x/th)"), c("b0", "b1", "th"))
eval(fje)

dx2xnf <- fnDeriv(~ x^2, "x") #- Use tilde
dx2xnf <- fnDeriv(expression(x^2), "x") #- or use expression()
dx2xnf
mode(dx2xnf)
str(dx2xnf)
x <- -1:2
#?? eval(dx2xnf) # This is evaluated at -1, 0, 1, 2, BUT result is returned directly,
#-  NOT in "gradient" attribute
# Note that we cannot (easily) differentiate this again.
# firstd <- dx2xnf
# str(firstd)
# d2x2xnf <- try(nlsDeriv(firstd, "x")) #- this APPEARS to work, but WRONG answer 
# str(d2x2xnf)
# d2x2xnf
# eval(d2x2xnf)

# dx2xnfh <- fnDeriv(expression(x^2), "x", hessian=TRUE) #- Try for second derivatives
# dx2xnfh
# mode(dx2xnfh)
# str(dx2xnfh)
# x <- -1
# eval(dx2xnfh) # This is evaluated at -1, 0, 1, 2, BUT result is returned directly,
```


### Deriv

The following examples are drawn from the `example(Deriv)` contained in the `Deriv` package.

```{r chunk04}
require(Deriv)

f <- function(x) x^2
Deriv(f)
#- Should see
#- function (x)
#- 2 * x
#- Now save the derivative
f1 <- Deriv(f)
f1 #- print it
f2 <- Deriv(f1) #- and take second derivative
f2 #- print it

f <- function(x, y) sin(x) * cos(y)
f_ <- Deriv(f)
f_ #- print it
#- Should see
#- function (x, y)
#- c(x = cos(x) * cos(y), y = -(sin(x) * sin(y)))
f_(3, 4)
#- Should see
#-              x         y
#- [1,] 0.6471023 0.1068000

f2 <- Deriv(~ f(x, y^2), "y") #- This has a tilde to render the 1st argument as a formula object
#- Also we are substituting in y^2 for y
f2 #- print it
#- -(2 * (y * sin(x) * sin(y^2)))
mode(f2) #- check what type of object it is
arg1 <- ~ f(x,y^2)
mode(arg1) #- check the type
f2a <- Deriv(arg1, "y")
f2a #- and print to see if same as before
#- try evaluation of f using current x and y
x
y
f(x,y^2)
eval(f2a) #- We need x and y defined to do this.

f3 <- Deriv(quote(f(x, y^2)), c("x", "y"), cache.exp=FALSE) #- check cache.exp operation
#- Note that we need to quote or will get evaluation at current x, y values (if they exist)
f3 #- print it
#- c(x = cos(x) * cos(y^2), y = -(2 * (y * sin(x) * sin(y^2))))
f3c <- Deriv(quote(f(x, y^2)), c("x", "y"), cache.exp=TRUE) #- check cache.exp operation
f3c #- print it
#- Now want to evaluate the results
#- First must provide some data
x <- 3
y <- 4
eval(f3c)
#- Should see
#- x         y 
#- 0.9480757 0.3250313 
eval(f3) #- check this also
#- or we can create functions
f3cf <- function(x, y){eval(f3c)}
f3cf(x=1, y=2)
#-          x          y 
#- -0.3531652  2.5473094 
f3f <- function(x,y){eval(f3)}
f3f(x=3, y=4)
#-         x         y 
#- 0.9480757 0.3250313 

#- try an expression
Deriv(expression(sin(x^2) * y), "x")
#- should see
#- expression(2 * (x * y * cos(x^2)))

#- quoted string
Deriv("sin(x^2) * y", "x") # differentiate only by x
#- Should see
#- "2 * (x * y * cos(x^2))"

Deriv("sin(x^2) * y", cache.exp=FALSE) #- differentiate by all variables (here by x and y)
#- Note that default is to differentiate by all variables.
#- Should see
#- "c(x = 2 * (x * y * cos(x^2)), y = sin(x^2))"

#- Compound function example (here abs(x) smoothed near 0)
#- Note that this introduces the possibilty of `if` statements in the code
#- BUT (JN) seems to give back quoted string, so we must parse.
fc <- function(x, h=0.1) if (abs(x) < h) 0.5*h*(x/h)**2 else abs(x)-0.5*h
efc1 <- Deriv("fc(x)", "x", cache.exp=FALSE) 
#- "if (abs(x) < h) x/h else sign(x)"
#- A few checks on the results
efc1

fc1 <- function(x,h=0.1){ eval(parse(text=efc1)) }
fc1
## h=0.1
fc1(1)
fc1(0.001)
fc1(-0.001)
fc1(-10)
fc1(0.001, 1)

#- Example of a first argument that cannot be evaluated in the current environment:
  try(suppressWarnings(rm("xx", "yy"))) #- Make sure there are no objects xx or yy
  Deriv(~ xx^2+yy^2)
#- Should show
#- c(xx = 2 * xx, yy = 2 * yy)
#- ?? What is the meaning / purpose of this construct?
  
#- ?? Is following really AD?  
#- Automatic differentiation (AD), note intermediate variable 'd' assignment
Deriv(~{d <- ((x-m)/s)^2; exp(-0.5*d)}, "x")
# Note that the result we see does NOT match what follows in the example(Deriv) (JN ??)
#{
#   d <- ((x - m)/s)^2
#   .d_x <- 2 * ((x - m)/s^2)
#   -(0.5 * (.d_x * exp(-(0.5 * d))))
#}
#- For some reason the intermediate variable d is NOT included.??


#- Custom derivative rule. Note that this needs explanations??
  myfun <- function(x, y=TRUE) NULL #- do something useful
  dmyfun <- function(x, y=TRUE) NULL #- myfun derivative by x.
  drule[["myfun"]] <- alist(x=dmyfun(x, y), y=NULL) #- y is just a logical
  Deriv(myfun(z^2, FALSE), "z")
  # 2 * (z * dmyfun(z^2, FALSE))

#- Differentiation by list components
  theta <- list(m=0.1, sd=2.) #- Why do we set values??
  x <- names(theta) #- and why these particular names??
  names(x)=rep("theta", length(theta))
  Deriv(~exp(-(x-theta$m)**2/(2*theta$sd)), x, cache.exp=FALSE)
#- Should show the following (but why??)
#- c(theta_m = exp(-((x - theta$m)^2/(2 * theta$sd))) *
#-  (x - theta$m)/theta$sd, theta_sd = 2 * (exp(-((x - theta$m)^2/
#-  (2 * theta$sd))) * (x - theta$m)^2/(2 * theta$sd)^2))
lderiv <-  Deriv(~exp(-(x-theta$m)**2/(2*theta$sd)), x, cache.exp=FALSE)
fld <- function(x){ eval(lderiv)} #- put this in a function
fld(2) #- and evaluate at a value
```

**Deriv** has some design choices that can get the user into trouble. The following
example shows one such problem.

```{r chunk05}
library(Deriv)
rm(x) # ensures x is undefined
Deriv(~ x, "x")  # returns [1] 1 -- clearly a bug!
Deriv(~ x^2, "x")   # returns 2 * x
x <- quote(x^2)
Deriv(x, "x") # returns 2 * x
```

By comparison, **nlsr**

```{r chunk06}
rm(x) # in case it is defined
try(nlsDeriv(x, "x")  ) # fails, not a formula
try(nlsDeriv(as.expression("x"), "x")  ) # expression(NULL)
try(nlsDeriv(~x, "x")  ) # 1
try(nlsDeriv(x^2, "x"))   # fails
try(nlsDeriv(~x^2, "x")) # 2 * x
x <- quote(x^2)
try(nlsDeriv(x, "x")) # returns 2 * x
```


### Ryacas

There is at least one other symbolic package for R. Here we look at 
**Ryacas**. 
The structures for using yacas tools do not seem at the time of writing
(2016-10-21) to be suitable for working with nonlinear least squares or
optimization facilities of **R**. Thus, for the moment, we will not pursue
the derivatives available in `Ryacas` beyond the following example 
provided by Gabor Grothendieck.


```{r chunk07, eval = require(Ryacas)}
dnlsr <- nlsr::nlsDeriv(~ sin(x+y), "x")
print(dnlsr)
class(dnlsr)
detach("package:nlsr", unload=TRUE)
detach("package:Deriv", unload=TRUE)

## New Ryacas mechanism as of 2019-8-29 from mikl@math.aau.dk (Mikkel Meyer Andersen)

yac_str("D(x) Sin(x+y)") 

# or if an expression is needed:
ex <- yac_expr("D(x) Sin(x+y)")
ex
expression(cos(x + y))
eval(ex, list(x = pi, y = pi/2))

## Previous syntax for Ryacas was
## x <- Sym("x")
## y <- Sym("y")
## dryacas <- deriv(sin(x+y), x)
## print(dryacas)
## class(dryacas)

detach("package:Ryacas", unload=TRUE)
```



## Derivatives and simplifications -- base **R**

See specific notes either in comments or at the end of the section.

The help page for `D` lists the functions for which derivatives are
known:  "The internal code knows about the arithmetic operators `+`,
`-`, `*`, `/` and `^`, and the single-variable functions `exp`, `log`, `sin`, `cos`,
`tan`, `sinh`, `cosh`, `sqrt`, `pnorm`, `dnorm`, `asin`, `acos`, `atan`, `gamma`,
`lgamma`, `digamma` and `trigamma`, as well as `psigamma` for one or two
arguments (but derivative only with respect to the first)."

## Derivatives and simplifications -- package `nlsr`

This package supports the derivatives that `D` supports, as well
as a few others, and users can add their own definitions.  The current
list is
```{r chunk08}
ls(nlsr::sysDerivs)
```

### Derivatives table

Here is a slightly expanded testing of the elements of the `nlsr` derivatives
table.

```{r chunk09}
require(nlsr)
## Try different ways to supply the log function
aDeriv <- nlsDeriv(~ log(x), "x")
class(aDeriv)
aDeriv
aderiv <- try(deriv( ~ log(x), "x"))
class(aderiv)
aderiv
aD <- D(expression(log(x)), "x")
class(aD)
aD
cat("but \n")
try(D( "~ log(x)", "x")) # fails -- gives NA rather than expected answer due to quotes
try(D( ~ log(x), "x"))
interm <- ~ log(x)
interm
class(interm)
interme <- as.expression(interm)
class(interme)
try(D(interme, "x"))
try(deriv(interme, "x"))
try(deriv(interm, "x"))


nlsDeriv(~ log(x, base=3), "x" ) # OK
try(D(expression(log(x, base=3)), "x" )) # fails - only single-argument calls supported
try(deriv(~ log(x, base=3), "x" )) # fails - only single-argument calls supported
try(deriv(expression(log(x, base=3)), "x" )) # fails - only single-argument calls supported
try(deriv3(expression(log(x, base=3)), "x" )) # fails - only single-argument calls supported
fnDeriv(quote(log(x, base=3)), "x" )

nlsDeriv(~ exp(x), "x")
D(expression(exp(x)), "x") # OK
deriv(~exp(x), "x") # OK, but much more complicated
fnDeriv(quote(exp(x)), "x")

nlsDeriv(~ sin(x), "x")
D(expression(sin(x)), "x")
deriv(~sin(x), "x")
fnDeriv(quote(sin(x)), "x")

nlsDeriv(~ cos(x), "x")
D(expression(cos(x)), "x")
deriv(~ cos(x), "x")
fnDeriv(quote(cos(x)), "x")

nlsDeriv(~ tan(x), "x")
D(expression(tan(x)), "x")
deriv(~ tan(x), "x")
fnDeriv(quote(tan(x)), "x")

nlsDeriv(~ sinh(x), "x")
D(expression(sinh(x)), "x")
deriv(~sinh(x), "x")
fnDeriv(quote(sinh(x)), "x")

nlsDeriv(~ cosh(x), "x")
D(expression(cosh(x)), "x")
deriv(~cosh(x), "x")
fnDeriv(quote(cosh(x)), "x")

nlsDeriv(~ sqrt(x), "x")
D(expression(sqrt(x)), "x")
deriv(~sqrt(x), "x")
fnDeriv(quote(sqrt(x)), "x")

nlsDeriv(~ pnorm(q), "q")
D(expression(pnorm(q)), "q")
deriv(~pnorm(q), "q")
fnDeriv(quote(pnorm(q)), "q")

nlsDeriv(~ dnorm(x, mean), "mean")
D(expression(dnorm(x, mean)), "mean")
deriv(~dnorm(x, mean), "mean")
fnDeriv(quote(dnorm(x, mean)), "mean")

nlsDeriv(~ asin(x), "x")
D(expression(asin(x)), "x")
deriv(~asin(x), "x")
fnDeriv(quote(asin(x)), "x")

nlsDeriv(~ acos(x), "x")
D(expression(acos(x)), "x")
deriv(~acos(x), "x")
fnDeriv(quote(acos(x)), "x")

nlsDeriv(~ atan(x), "x")
D(expression(atan(x)), "x")
deriv(~atan(x), "x")
fnDeriv(quote(atan(x)), "x")

nlsDeriv(~ gamma(x), "x")
D(expression(gamma(x)), "x")
deriv(~gamma(x), "x")
fnDeriv(quote(gamma(x)), "x")

nlsDeriv(~ lgamma(x), "x")
D(expression(lgamma(x)), "x")
deriv(~lgamma(x), "x")
fnDeriv(quote(lgamma(x)), "x")

nlsDeriv(~ digamma(x), "x")
D(expression(digamma(x)), "x")
deriv(~digamma(x), "x")
fnDeriv(quote(digamma(x)), "x")

nlsDeriv(~ trigamma(x), "x")
D(expression(trigamma(x)), "x")
deriv(~trigamma(x), "x")
fnDeriv(quote(trigamma(x)), "x")

nlsDeriv(~ psigamma(x, deriv = 5), "x")
D(expression(psigamma(x, deriv = 5)), "x")
deriv(~psigamma(x, deriv = 5), "x")
fnDeriv(quote(psigamma(x, deriv = 5)), "x")

nlsDeriv(~ x*y, "x")
D(expression(x*y), "x")
deriv(~x*y, "x")
fnDeriv(quote(x*y), "x")

nlsDeriv(~ x/y, "x")
D(expression(x/y), "x")
deriv(~x/y, "x")
fnDeriv(quote(x/y), "x")

nlsDeriv(~ x^y, "x")
D(expression(x^y), "x")
deriv(~x^y, "x")
fnDeriv(quote(x^y), "x")

nlsDeriv(~ (x), "x")
D(expression((x)), "x")
deriv(~(x), "x")
fnDeriv(quote((x)), "x")

nlsDeriv(~ +x, "x")
D(expression(+x), "x")
deriv(~ +x, "x")
fnDeriv(quote(+x), "x")

nlsDeriv(~ -x, "x")
D(expression(- x), "x")
deriv(~ -x, "x")
fnDeriv(quote(-x), "x")

nlsDeriv(~ abs(x), "x")
try(D(expression(abs(x)), "x")) # 'abs' not in derivatives table
try(deriv(~ abs(x), "x"))
fnDeriv(quote(abs(x)), "x")

nlsDeriv(~ sign(x), "x")
try(D(expression(sign(x)), "x")) # 'sign' not in derivatives table
try(deriv(~ sign(x), "x"))
fnDeriv(quote(sign(x)), "x")
```

### Notes:

- the base tool `deriv` (and `deriv3`)  and 
`nlsr::codeDeriv` are intended to output an expression to compute a derivative. 
`deriv` generates an expression object, while `codeDeriv` will generate a language object. 
Note that input to `deriv` is of the form of a 
tilde expression with no left hand side, while `codeDeriv` is
more flexible:  quoted expressions, or length-1 expression vectors may also be used. 

- the base tool `D` and `nlsr::nlsDeriv` generate expressions, but `D` requires an
expression, while `nlsDeriv` can handle the expression without a wrapper. ?? Do we need 
to discuss more??

- **nlsr** includes `abs(x)` and `sign(x)` in the derivatives table despite conventional
wisdom that these are not differentiable. However, `abs(x)` clearly has a defined
derivative everywhere except at x = 0, where assigning a value of 0 to the 
derivative is almost certainly acceptable in computations. Similarly for `sign(x)`.

### Simplifying algebraic expressions
 
**nlsr** also includes some tools for simplification of algebraic expressions, extensible by the user.  Currently these
involve the following functions:
```{r chunk10}
ls(nlsr::sysSimplifications)
```

```{r chunk11}
#- Remove ##? to see reproducible error
#- ?? For some reason, if we leave packages attached, we get errors.
#- Here we detach all the non-base packages and then reload nlsr
##? sessionInfo()
##? ##? nlsSimplify(quote(+(a+b)))
##? nlsSimplify(quote(-5))
```


```{r chunk12}
#- ?? For some reason, if we leave packages attached, we get errors.
#- Here we detach all the non-base packages and then reload nlsr
sessionInfo()
if ("Deriv" %in% loadedNamespaces()){detach("package:Deriv", unload=TRUE)} 
  #- ?? Do we need to unload too.
if ("nlsr" %in% loadedNamespaces() ){detach("package:nlsr", unload=TRUE)}
if ("Ryacas" %in% loadedNamespaces() ){detach("package:Ryacas", unload=TRUE)}
#- require(Deriv)
#- require(stats)
#- Various simplifications
#- ?? Do we need quote() to stop attempt to evaluate before applying simplification
require(nlsr)
nlsSimplify(quote(+(a+b)))
nlsSimplify(quote(-5))
nlsSimplify(quote(--(a+b)))

nlsSimplify(quote(exp(log(a+b))))
nlsSimplify(quote(exp(1)))

nlsSimplify(quote(log(exp(a+b))))
nlsSimplify(quote(log(1)))

nlsSimplify(quote(!TRUE))
nlsSimplify(quote(!FALSE))

nlsSimplify(quote((a+b)))

nlsSimplify(quote(a + b + 0))
nlsSimplify(quote(0 + a + b))
nlsSimplify(quote((a+b) + (a+b)))
nlsSimplify(quote(1 + 4))

nlsSimplify(quote(a + b - 0))
nlsSimplify(quote(0 - a - b))
nlsSimplify(quote((a+b) - (a+b)))
nlsSimplify(quote(5 - 3))

nlsSimplify(quote(0*(a+b)))
nlsSimplify(quote((a+b)*0))
nlsSimplify(quote(1L * (a+b)))
nlsSimplify(quote((a+b) * 1))
nlsSimplify(quote((-1)*(a+b)))
nlsSimplify(quote((a+b)*(-1)))
nlsSimplify(quote(2*5))

nlsSimplify(quote((a+b) / 1))
nlsSimplify(quote((a+b) / (-1)))
nlsSimplify(quote(0/(a+b)))
nlsSimplify(quote(1/3))

nlsSimplify(quote((a+b) ^ 1))
nlsSimplify(quote(2^10))

nlsSimplify(quote(log(exp(a), 3)))

nlsSimplify(quote(FALSE && b))
nlsSimplify(quote(a && TRUE))
nlsSimplify(quote(TRUE && b))

nlsSimplify(quote(a || TRUE))
nlsSimplify(quote(FALSE || b))
nlsSimplify(quote(a || FALSE))

nlsSimplify(quote(if (TRUE) a+b))
nlsSimplify(quote(if (FALSE) a+b))

nlsSimplify(quote(if (TRUE) a+b else a*b))
nlsSimplify(quote(if (FALSE) a+b else a*b))
nlsSimplify(quote(if (cond) a+b else a+b))

nlsSimplify(quote(--(a+b)))
nlsSimplify(quote(-(-(a+b))))
```




## Derivatives and simplifications -- package `Deriv`

### Derivatives table




### Simplifications

```{r chunk13}
#- ?? For some reason, if we leave packages attached, we get errors.
#- Here we detach all the non-base packages and then reload nlsr
sessionInfo()
if ("Deriv" %in% loadedNamespaces()){detach("package:Deriv", unload=TRUE)} 
  #- ?? Do we need to unload too.
if ("Deriv" %in% loadedNamespaces() ){detach("package:nlsr", unload=TRUE)}
if ("Deriv" %in% loadedNamespaces() ){detach("package:Ryacas", unload=TRUE)}
require(Deriv)
#- Various simplifications
#- ?? Do we need quote() to stop attempt to evaluate before applying simplification

Simplify(quote(+(a+b)))
Simplify(quote(-5))
Simplify(quote(--(a+b)))

Simplify(quote(exp(log(a+b))))
Simplify(quote(exp(1)))

Simplify(quote(log(exp(a+b))))
Simplify(quote(log(1)))

Simplify(quote(!TRUE))
Simplify(quote(!FALSE))

Simplify(quote((a+b)))

Simplify(quote(a + b + 0))
Simplify(quote(0 + a + b))
Simplify(quote((a+b) + (a+b)))
Simplify(quote(1 + 4))

Simplify(quote(a + b - 0))
Simplify(quote(0 - a - b))
Simplify(quote((a+b) - (a+b)))
Simplify(quote(5 - 3))

Simplify(quote(0*(a+b)))
Simplify(quote((a+b)*0))
Simplify(quote(1L * (a+b)))
Simplify(quote((a+b) * 1))
Simplify(quote((-1)*(a+b)))
Simplify(quote((a+b)*(-1)))
Simplify(quote(2*5))

Simplify(quote((a+b) / 1))
Simplify(quote((a+b) / (-1)))
Simplify(quote(0/(a+b)))
Simplify(quote(1/3))

Simplify(quote((a+b) ^ 1))
Simplify(quote(2^10))

Simplify(quote(log(exp(a), 3)))

Simplify(quote(FALSE && b))
Simplify(quote(a && TRUE))
Simplify(quote(TRUE && b))

Simplify(quote(a || TRUE))
Simplify(quote(FALSE || b))
Simplify(quote(a || FALSE))

Simplify(quote(if (TRUE) a+b))
Simplify(quote(if (FALSE) a+b))

Simplify(quote(if (TRUE) a+b else a*b))
Simplify(quote(if (FALSE) a+b else a*b))
Simplify(quote(if (cond) a+b else a+b))

#- This one is wrong... the double minus is an error, yet it works ??.
Simplify(quote(--(a+b)))
#- By comparison
Simplify(quote(-(-(a+b))))
```



### Comparison with other approaches




### check modelexpr() works with an ssgrfun ??

### test model2rjfun vs model2rjfunx ??


### Need more extensive discussion of Simplify??


## Issues of programming on the language

?? need to explain where Deriv package comes from

One of the key tasks with tools for derivatives is that of taking objects
in one or other form (that is, **R** class) and using it as an input for
a symbolic function. The object may, of course, be an output from another
such function, and this is one of the reasons we need to do such 
transformations.

We also note that the different tools for symbolic derivatives use slightly
different inputs. For example, for the derivative of log(x), we have

```{r chunk14}
dlogx <- nlsr::nlsDeriv(~ log(x), "x")
str(dlogx)
print(dlogx)
```

Unfortunately, there are complications when we have an 
expression object, and 
we need to specify that we do NOT execute the *substitute()* function. 
Here we
show how to do this implicitly and with an explicit object.

```{r chunk15}
dlogxs <- nlsr::nlsDeriv(expression(log(x)), "x", do_substitute=FALSE)
str(dlogxs)
print(dlogxs)
cat(as.character(dlogxs), "\n")
fne <- expression(log(x))
dlogxe <- nlsr::nlsDeriv(fne, "x", do_substitute=FALSE)
str(dlogxe)
print(dlogxe)


# base R
dblogx <- D(expression(log(x)), "x")
str(dblogx)
print(dblogx)

require(Deriv)
ddlogx <- Deriv::Deriv(expression(log(x)), "x")
str(ddlogx)
print(ddlogx)
cat(as.character(ddlogx), "\n")
ddlogxf <- ~ ddlogx
str(ddlogxf)
```

??? do each example by all methods and by numDeriv and put in dataframe for later
presentation in a table.

Do we want examples in columns or rows. Probably 1 fn per row and work out
a name for the row that is reasonably meaningful. Probably want an index column
as well that is a list of strings. Can we then act on those strings to automate
the whole setup?


```{r chunk16}
##
# require(stats)
# require(Deriv)
# require(Ryacas)

# Various derivatives 

new <- codeDeriv(quote(1 + x + y), c("x", "y"))
old <- deriv(quote(1 + x + y), c("x", "y"))
print(new)
# Following generates a very long line on output of knitr (for markdown)
class(new)
str(new)
as.expression(new)
newf <- function(x, y){
   eval(new)
}
newf(3,5)

print(old)
class(old)
str(old)
oldf <- function(x,y){
    eval(old)
}
oldf(3,5)


```

Unfortunately, the inputs and outputs are not always easily transformed so that the
symbolic derivatives can be found. (?? Need to codify this and provide filters so we
can get things to work nicely.)

As an example, how could we take object **new** and embed it in a function we can then use
in **R**? We can certainly copy and paste the output into a function template, as follows,

```{r chunk17}
fnfromnew <- function(x,y){
    .value <- 1 + x + y
    .grad <- array(0, c(length(.value), 2L), list(NULL, c("x", 
    "y")))
    .grad[, "x"] <- 1
    .grad[, "y"] <- 1
    attr(.value, "gradient") <- .grad
    .value
}

print(fnfromnew(3,5))

```

However, we would ideally like to be able to automate this to generate functions and
gradients for nonlinear least squares and optimization calculations. The same criticism
applies to the object **old**

####Another issue: 

If we have x and y set such that the function is not admissible, then 
both our old and new functions give a gradient that is seemingly reasonable. While the
gradient of this simple function could be considered to be defined for ANY values of x and
y, I (JN) am sure most users would wish for a warning at the very least in such cases.

```{r chunk18}
x <- NA
y <- Inf
print(eval(new))
print(eval(old))
```

####SafeD

We could define a way to avoid the issue of character vs. expression (and possibly
other classes) as follows:


```{r chunk19}
safeD <- function(obj, var) {
   # safeguarded D() function for symbolic derivs
   if (! is.character(var) ) stop("The variable var MUST be character type")
   if (is.character(obj) ) {
       eobj <- parse(text=obj)
       result <- D(eobj, var)
   } else {
       result <- D(obj, var)
   }
}

lxy2 <- expression(log(x+y^2))
clxy2 <- "log(x+y^2)"
try(print(D(clxy2, "y")))
print(try(D(lxy2, "y")))
print(safeD(clxy2, "y"))
print(safeD(lxy2, "y"))
```


## Indexed parameters or variables

Erin Hodgess on R-help in January 2015 raised the issue of taking the 
derivative of an expression that contains an indexed variable. We
show the example and its resolution, then give an explanation.

```{r chunk20}
zzz <- expression(y[3]*r1 + r2)
try(deriv(zzz,c("r1","r2")))
##
try(nlsr::nlsDeriv(zzz, c("r1","r2")))
try(fnDeriv(zzz, c("r1","r2")))
newDeriv(`[`(x,y), stop("no derivative when indexing"))
try(nlsr::nlsDeriv(zzz, c("r1","r2")))
try(nlsr::fnDeriv(zzz, c("r1","r2")))
```

Richard Heiberger pointed out that internally, **R** stores

    y[3]

as

    "["(y,3)

that is, as a function. Duncan Murdoch pointed out the availability of
**nlsr** and the use of newDeriv() to redefine the "[" function for
the purposes of derivatives. 

This is not an ideal resolution, especially as we would like to be able
to get the gradients of functions with respect to vectors of parameters,
noted also by Sergei Sokol in the manual for package **Deriv**. The 
following examples illustrate this.

```{r chunk21}
try(nlsr::nlsDeriv(zzz, "y[3]"))
try(nlsr::nlsDeriv(y3*r1+r2,"y3"))
try(nlsr::nlsDeriv(y[3]*r1+r2,"y[3]"))
```

<!-- =========================================================================== -->

\pagebreak

# Appendix D: A comparison of nlsr::nlxb with nls and minpack::nlsLM

**R** has several tools for estimating nonlinear models and minimizing sums of squares functions.
Sometimes we talk of **nonlinear regression** and at other times of 
**minimizing a sum of squares function**. Many workers conflate these two tasks.
In this appendix, some of the differences between the tools available in R for these two
computational tasks are highlighted. In particular, we compare the tools from the package
`nlsr` (@nlsr2019manual), particularly function `nlxb()` with those from base-R `nls()` and the `nlsLM` 
function of package `minpack.lm` (@minpacklm12). We also compare how `nlsr:nlfb()` and
`minpack.lm:nls.lm` allow a sum of squares function to be minimized.


## Principal differences

The main differences in the tools relate to the following features:

- the way in which derivative information is computed for the Jacobian of the modelling function
- specification of the model as an R programmatic function is unavailable in `nlsr::nlxb()`
- the use of a Marquardt stabilization for solution of the linearized least squares problem at
each iteration
- details of the criterion used to terminate the iteration
- the structure of the output of the tools
- how models are predicted for new data.

### Derivative information

As detailed above, `nlsr::nlxb()` attempts to use symbolic and algorithmic tools to obtain the derivatives
of the model expression that are needed for the **Jacobian** matrix that is used in creating
a linearized sub-problem at each iteration of an attempted solution of the minimization of the
sum of squared residuals. As discussed in the section "Analytic versus approximate Jacobians" and
using the code in Appendix B, `nls()` and `minpack.lm::nlsLM()` use a very simple forward-difference
approximation for the partial derivatives for the Jacobian. 

Forward difference approximations are less accurate than central differences, and both are subject
to numerical error when the modelling function is "flat", so that there is a large amount of
digit cancellation in the subtraction necessary to compute the derivative approximation.

`minpack.lm::nlsLM` uses the same derivatives as far as I can determine. The loss of information 
compared to the analytic or algorithmic derivatives of `nlsr::nlxb()` is important in that it
can lead to Jacobian matrices that are computationally singular, where `nls()` will stop with
"singular gradient". (It is actually the Jacobian which is singular here, and I will stay with
that terminology.)  `minpack.lm::nlsLM()` may fail to get started if the initial Jacobian is
singular, but is less susceptible in general, as described in the sub-section on Marquardt 
stabilization which follows.

#### Consequences of different derivative computations

While readers might expect that the precise derivative information of `nlsr::nlxb()` would mean
a faster solution, this is quite often not the case. Approximate derivatives may allow faster
approach to the solution by "ironing out" wrinkles in the function surface. In my opinion, the
main advantage of precise derivative information is in testing that we actually have arrived
at a solution. 

There are even some cases where the approximation may be helpful, though users may not realize
the potential danger. Thanks to Karl Schilling for an example of modelling with the function

  `a * (x ^ b)`
  
where `x` is our data and we wish to estimate `a` and `b`. Now the partial derivative of this
function w.r.t. `b` is 

```{r derivexp, eval=TRUE}
partialderiv <- D(expression(a * (x ^ b)),"b")
print(partialderiv)
```

The danger here is that we may have data values `x = 0`, in which case the **derivative** is 
not defined, though the model can still be evaluated. Thus `nlsr::nlxb()` will not compute
a solution, while `nls()` and `minpack.lm::nlsLM()` will generally proceed. A workaround is
to provide a very small value instead of zero for the data, though I find this inelegant.
Another approach is to drop the offending element of the data, though this risks altering
the model estimated. A proper treatment might be to develop the limit of the derivative as 
the data value goes to zero, but finding general software that can detect and deal with 
this is a large project.

#### Timing comparisons

Let us compare timings on the (scaled) Hobbs weed problem introduced in Section ??. 

```{r timingshobbsmodel}
require(microbenchmark)
## nls on Hobbs scaled model
weed <- c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443,
          38.558, 50.156, 62.948, 75.995, 91.972)
tt <- 1:12
weeddf <- data.frame(y=weed, tt=tt)
wmods <- y ~ 100*b1/(1+10*b2*exp(-0.1*b3*tt))
stx<-c(b1=2, b2=5, b3=3)
tnls<-microbenchmark((anls<-nls(wmods, start=stx, data=weeddf)), unit="us")
tnls
## nlsr::nlfb() on Hobbs scaled model
tnlxb<-microbenchmark((anlxb<-nlsr::nlxb(wmods, start=stx, data=weeddf)), unit="us")
tnlxb
## minpack.lm::nlsLM() on Hobbs scaled model
tnlsLM<-microbenchmark((anlsLM<-minpack.lm::nlsLM(start=stx, formula=wmods, data=weeddf)), 
                       unit="us")
tnlsLM
```

### Programmatic modelling functions

A consequence of the symbolic derivative approach in `nlsr::nlxb()` is that it cannot be
applied to a modelling expression that includes an R function i.e., sub-program. 

This limitation could be overcome if there were appropriate automatic differentiation
code (to provide derivative computations based on transformation of the modelling 
function's programmatic form) or a mechanism to specify a form of numerical approximation.
As of December 2020, it seems more likely that the latter approach will be realized first,
and it is one the near-term development goals. 


### Functional expression of residuals and Jacobian


```{r timingnlsrminpackfn}
require(microbenchmark)
## nlsr::nlfb() on Hobbs scaled
# Scaled Hobbs problem
shobbs.res  <-  function(x){ # scaled Hobbs weeds problem -- residual
  # This variant uses looping
  if(length(x) != 3) stop("shobbs.res -- parameter vector n!=3")
  y  <-  c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443, 
           38.558, 50.156, 62.948, 75.995, 91.972)
  tt  <-  1:12
  res  <-  100.0*x[1]/(1+x[2]*10.*exp(-0.1*x[3]*tt)) - y
}

shobbs.jac  <-  function(x) { # scaled Hobbs weeds problem -- Jacobian
  jj  <-  matrix(0.0, 12, 3)
  tt  <-  1:12
  yy  <-  exp(-0.1*x[3]*tt)
  zz  <-  100.0/(1+10.*x[2]*yy)
  jj[tt,1]   <-   zz
  jj[tt,2]   <-   -0.1*x[1]*zz*zz*yy
  jj[tt,3]   <-   0.01*x[1]*zz*zz*yy*x[2]*tt
  attr(jj, "gradient") <- jj
  jj
}

st1 <- c(b1=1, b2=1, b3=1)
tnlfb<-microbenchmark((anlfb<-nlsr::nlfb(start=st1, resfn=shobbs.res, jacfn=shobbs.jac)), unit="us")
tnlfb
## minpack.lm::nls.lm() on Hobbs scaled
tnls.lm<-microbenchmark((anls.lm<-minpack.lm::nls.lm(par=st1, fn=shobbs.res, jac=shobbs.jac)))
tnls.lm
```
<!-- > microbenchmark( (for (i in 1:1e5){x=1}), unit="milliseconds") -->
<!-- Error in match.arg(unit) :  -->
<!--   'arg' should be one of “ns”, “us”, “ms”, “s”, “t”, “hz”, “khz”, “mhz”, “eps”, “f” -->


### Marquardt stabilization

All three of the R functions under consideration try to minimize a sum of squares. If the
model is provided in the form

   `y ~ (some expression)`
   
then the residuals are computed by evaluating the difference between `(some expression)` and `y`.
My own preference, and that of K F Gauss, is to use `(some expression) - y`. This is to avoid
having to be concerned with the negative sign -- the derivative of the residual defined in this
way is the same as the derivative of the modelling function, and we avoid the chance of a sign
error. The Jacobian matrix is made up of elements where element `i, j` is the partial derivative
of residual `i` w.r.t. parameter `j`. 

`nls()` attempts to minimize a sum of squared residuals by a Gauss-Newton method. If we
compute a Jacobian matrix `J` and a vector of residuals `r` from a vector of parameters `x`, 
then we can define a linearized problem 

$$  J^T J  \delta  = - J^T r $$

This leads to an iteration where, from a set of starting parameters `x0`, we compute

$$ x_{i+1} = x_i + \delta$$

This is commonly modified to use a step factor `step`

$$ x_{i+1} = x_i + step * \delta$$

It is in the mechanisms to choose the size of `step` and to decide when to terminate the
iteration that Gauss-Newton methods differ. Indeed, though I have tried several times, I
find the very convoluted code behind `nls()` very difficult to decipher. Unfortunately, its
authors have (at 2018 as far as I am aware) all ceased to maintain the code.

Both `nlsr::nlxb()` and `minpack.lm::nlsLM` use a Levenberg-Marquardt stabilization of the iteration.
(@Marquardt1963, @Levenberg1944), solving

$$ (J^T J + \lambda D)   \delta  = - J^T r $$

where $D$ is some diagonal matrix and lambda is a number of modest size initially. Clearly
for $\lambda = 0$ we have a Gauss-Newton method. Typically, the sum of squares of the residuals
calculated at the "new" set of parameters is used as a criterion for keeping those parameter
values. If so, the size of $\lambda$ is reduced. If not, we increase the size of $\lambda$ and
compute a new $\delta$. Note that a new $J$, the expensive step in each iteration, is NOT
required. 

As for Gauss-Newton methods, the details of how to start, adjust and terminate the iteration
lead to many variants, increased by different possibilities for specifying $D$. See
@jncnm79. There are also a number of ways to solve the stabilized Gauss-Newton equations,
some of which do not require the explicit $J^T J$ matrix. 

### Criterion used to terminate the iteration

`nls()` and `nlsr` use a form of the relative offset convergence criterion, @BatesWatts81. 
`minpack.lm` uses a somewhat different and more complicated set of tests. Unfortunately,
the relative offset criterion as implemented in `nls()` is unsuited to problems where 
the residuals can be zero. There are ways to work around the difficulties, and `nlsr`
has used one approach. See *An illustrative nonlinear regression problem* below.


### Output of the modelling functions

`nls()` and `nlsLM()` return the same solution structure. Let us examine this for one of our 
example results (we will choose one that does NOT have small residuals, so that all
the functions "work").

```{r hiddenexprob1, echo=FALSE}
# Here we set up an example problem with data
# Define independent variable
t0 <- 0:19
t0a<-t0
t0a[1]<-1e-20 # very small value
# Drop first value in vectors
t0t<-t0[-1]
y1 <- 4 * (t0^0.25)
y1t<-y1[-1]
n <- length(t0)
fuzz <- rnorm(n)
range <- max(y1)-min(y1)
## add some "error" to the dependent variable
y1q <- y1 + 0.2*range*fuzz
edta <- data.frame(t0=t0, t0a=t0a, y1=y1, y1q=y1q)
edtat <- data.frame(t0t=t0t, y1t=y1t)

start1 <- c(a=1, b=1)
try(nlsy0t0ax <- nls(formula=y1~a*(t0a^b), start=start1, data=edta, control=nls.control(maxiter=10000)))
nlsry1t0a <- nlxb(formula=y1~a*(t0a^b), start=start1, data=edta)
library(minpack.lm)
nlsLMy1t0a <- nlsLM(formula=y1~a*(t0a^b), start=start1, data=edta)
```

```{r nlsstr}
str(nlsy0t0ax)
```
The `minpack.lm::nlsLM` output has the same structure, which could be revealed by the 
R command `str(nlsLMy1t0a)`.
Note that this structure has a lot of special functions in the sub-list `m`.
By contrast, the `nlsr()` output is much less flamboyant. There are, in fact,
no functions as part of the structure. 

```{r nlsrstr}
str(nlsry1t0a)
```



Which of these approaches is "better" can be debated. My preference is for the results
of optimization computations to be essentially data, including messages, though some
tools within some of my packages will return functions for specific reasons, e.g.,
to return a function from an expression. However, I prefer to use specified functions
such as `predict.nlsr()` below to obtain predictions. I welcome comment and discussion,
as this is not, in my view, a closed topic.


### Prediction

Let us predict our models at the mean of the data.
Because `nlxb()` returns a different structure from that found by `nls()` and 
`nlsLM()` the code for `predict()` for an object from `nlsr` is different. 
`minpack.lm` uses `predict.nls` since the output structure of the modelling step is equivalent to that from `nls()`.

```{r  pred1}
nudta <- colMeans(edta)
predict(nlsy0t0ax, newdata=nudta)
predict(nlsLMy1t0a, newdata=nudta)
predict(nlsry1t0a, newdata=nudta)
```



## An illustrative nonlinear regression problem

So we can illustrate some of the issues, let us create some example data for a seemingly
straightforward computational problem.

```{r exprob1}
# Here we set up an example problem with data
# Define independent variable
t0 <- 0:19
t0a<-t0
t0a[1]<-1e-20 # very small value
# Drop first value in vectors
t0t<-t0[-1]
y1 <- 4 * (t0^0.25)
y1t<-y1[-1]
n <- length(t0)
fuzz <- rnorm(n)
range <- max(y1)-min(y1)
## add some "error" to the dependent variable
y1q <- y1 + 0.2*range*fuzz
edta <- data.frame(t0=t0, t0a=t0a, y1=y1, y1q=y1q)
edtat <- data.frame(t0t=t0t, y1t=y1t)
```

Let us try this example modelling `y0` against `t0`. Note that this is a zero-residual problem,
so `nls()` should complain or fail, which it appears to do but by exceeding the iteration limit,
which is not very communicative of the underlying issue. The `nls()` documentation warns

"Warning

Do not use nls on artificial "zero-residual" data."

It goes on to recommend that users add "error" to the data to avoid such problems.
I feel this is a very unsatisfactory kludge. It is NOT due to a genuine mathematical
issue, but due to the relative offset convergence criterion used to terminate the
method. In October 2020, I suggested
a patch for nls() to R-core that it seems will become part of base R eventually. This patch
allows the user to specify a parameter, tentatively named `convTestAdd` with a zero default value, 
in `nls.control()`. (This allows existing examples using `nsl()` to function without change.) 
A small positive value for this control parameter avoids a zero divided by 
zero issue in the relative offset convergence test in `nls()` used to terminate iterations. 
This adjustment to the convergence test has been in `nlsr` since its creation.

<!-- ```{r smallres} -->
<!-- x <- (1:m) -->
<!-- y <- 2.5 * (x ^ 0.33) -->
<!-- fmla <- y ~ a * (x^b) -->
<!-- edta <- data.frame(x=x, y=y) -->
<!-- library(nlsr) -->
<!-- library(minpack.lm) -->
<!-- enls <- try(nls(fmla, start=c(a=1, b=1), data=edta)) -->
<!-- enlxb <- try(nlxb(fmla, start=c(a=1, b=1), data=edta)) -->
<!-- enlxb -->
<!-- enlsLM <- try(nlsLM(fmla, start=c(a=1, b=1), data=edta)) -->
<!-- enlsLM -->
<!-- ``` -->

Here is the output.

### nls

```{r extrynls}
cprint <- function(obj){
   # print object if it exists
  sobj<-deparse(substitute(obj))
  if (exists(sobj)) {
      print(obj)
  } else {
      cat(sobj," does not exist\n")
  }
#  return(NULL)
}
start1 <- c(a=1, b=1)
try(nlsy0t0 <- nls(formula=y1~a*(t0^b), start=start1, data=edta))
cprint(nlsy0t0)
# Since this fails to converge, let us increase the maximum iterations
try(nlsy0t0x <- nls(formula=y1~a*(t0^b), start=start1, data=edta,
                    control=nls.control(maxiter=10000)))
cprint(nlsy0t0x)
try(nlsy0t0ax <- nls(formula=y1~a*(t0a^b), start=start1, data=edta, 
                     control=nls.control(maxiter=10000)))
cprint(nlsy0t0ax)
try(nlsy0t0t <- nls(formula=y1t~a*(t0t^b), start=start1, data=edtat))
cprint(nlsy0t0t)
```

### nlsr

```{r extry1nlsr}
nlsry1t0 <- try(nlxb(formula=y1~a*(t0^b), start=start1, data=edta))
cprint(nlsry1t0)
nlsry1t0a <- nlxb(formula=y1~a*(t0a^b), start=start1, data=edta)
cprint(nlsry1t0a)
nlsry1t0t <- nlxb(formula=y1t~a*(t0t^b), start=start1, data=edtat)
cprint(nlsry1t0t)
```
### minpack.lm

```{r extry1minlm}
library(minpack.lm)
nlsLMy1t0 <- nlsLM(formula=y1~a*(t0^b), start=start1, data=edta)
nlsLMy1t0
nlsLMy1t0a <- nlsLM(formula=y1~a*(t0a^b), start=start1, data=edta)
nlsLMy1t0a
nlsLMy1t0t <- nlsLM(formula=y1t~a*(t0t^b), start=start1, data=edtat)
nlsLMy1t0t
```

We have seemingly found a workaround for our difficulty, but I caution that initially 
I found very unsatisfactory results when I set the "very small value" to 1.0e-7. The
correct approach is clearly to understand what is going on. Getting computers to provide
that understanding is a serious challenge.


### Problems that are NOT regressions

Some nonlinear least squares problems are NOT nonlinear regressions. That is, we do not
have a formula `y ~ (some function)` to define the problem. This is another reason to
use the residual in the form `(some function) - y` In many cases of interest we have
no `y`.

The Brown and Dennis test problem (@More1981TUO, problem 16) is of this form. Suppose we have `m` observations,
then we create a scaled index `t` which is the "data" for the function. To run the nonlinear least squares
functions that use a formula, we do, however, need a "y" variable. Clearly adding zero to the residual
will not change the problem, so we set the data for "y" as all zeros. Note that `nls()` and `nlsLM()`
need some extra iterations to find the solution to this somewhat nasty problem.

```{r brownden}
m <- 20
t <- seq(1, m) / 5
y <- rep(0,m)
library(nlsr)
library(minpack.lm)

bddata <- data.frame(t=t, y=y)
bdform <- y ~ ((x1 + t * x2 - exp(t))^2 + (x3 + x4 * sin(t) - cos(t))^2)
prm0 <- c(x1=25, x2=5, x3=-5, x4=-1)
fbd <-model2ssgrfun(bdform, prm0, bddata)
cat("initial sumsquares=",as.numeric(crossprod(fbd(prm0))),"\n")
nlsrbd <- nlxb(bdform, start=prm0, data=bddata, trace=FALSE)
nlsrbd

nlsbd10k <- nls(bdform, start=prm0, data=bddata, trace=FALSE, 
                control=nls.control(maxiter=10000))
nlsbd10k

nlsLMbd10k <- nlsLM(bdform, start=prm0, data=bddata, trace=FALSE, 
                    control=nls.lm.control(maxiter=10000, maxfev=10000))
nlsLMbd10k
```

Let us try predicting the "residual" for some new data.

```{r browndenpred}
ndata <- data.frame(t=c(5,6), y=c(0,0))
predict(nlsLMbd10k, newdata=ndata)

# now nls
predict(nlsbd10k, newdata=ndata)

# now nlsr
predict(nlsrbd, newdata=ndata)


```


We could, of course, try setting up a different formula, since the "residuals" can be
computed in any way such that their absolute value is the same. 
Therefore we could try moving the exponential
part of the function for each equation to the left hand side as in `bdf2` below.
```{r brownden2}
bdf2 <-  (x1 + t * x2 - exp(t))^2 ~ - (x3 + x4 * sin(t) - cos(t))^2
```

However, we discover that the parsing of the model formula fails for this formulation.

<!-- ?? Do we need a check on the formula for this?? -->

### A check on the Brown and Dennis calculation via function minimization

We can attack the Brown and Dennis problem by applying nonlinear function minimization
programs to the sum of squared "residuals" as a function of the parameters. The code
below does this. We omit the output for space reasons.

```{r bdcheck, eval=FALSE}
#' Brown and Dennis Function
#'
#' Test function 16 from the More', Garbow and Hillstrom paper.
#'
#' The objective function is the sum of \code{m} functions, each of \code{n}
#' parameters.
#'
#' \itemize{
#'   \item Dimensions: Number of parameters \code{n = 4}, number of summand
#'   functions \code{m >= n}.
#'   \item Minima: \code{f = 85822.2} if \code{m = 20}.
#' }
#'
#' @param m Number of summand functions in the objective function. Should be
#'   equal to or greater than 4.
#' @return A list containing:
#' \itemize{
#'   \item \code{fn} Objective function which calculates the value given input
#'   parameter vector.
#'   \item \code{gr} Gradient function which calculates the gradient vector
#'   given input parameter vector.
#'   \item \code{fg} A function which, given the parameter vector, calculates
#'   both the objective value and gradient, returning a list with members
#'   \code{fn} and \code{gr}, respectively.
#'   \item \code{x0} Standard starting point.
#' }
#' @references
#' More', J. J., Garbow, B. S., & Hillstrom, K. E. (1981).
#' Testing unconstrained optimization software.
#' \emph{ACM Transactions on Mathematical Software (TOMS)}, \emph{7}(1), 17-41.
#' \url{https://doi.org/10.1145/355934.355936}
#'
#' Brown, K. M., & Dennis, J. E. (1971).
#' \emph{New computational algorithms for minimizing a sum of squares of
#' nonlinear functions} (Report No. 71-6).
#' New Haven, CT: Department of Computer Science, Yale University.
#'
#' @examples
#' # Use 10 summand functions
#' fun <- brown_den(m = 10)
#' # Optimize using the standard starting point
#' x0 <- fun$x0
#' res_x0 <- stats::optim(par = x0, fn = fun$fn, gr = fun$gr, method =
#' "L-BFGS-B")
#' # Use your own starting point
#' res <- stats::optim(c(0.1, 0.2, 0.3, 0.4), fun$fn, fun$gr, method =
#' "L-BFGS-B")
#'
#' # Use 20 summand functions
#' fun20 <- brown_den(m = 20)
#' res <- stats::optim(fun20$x0, fun20$fn, fun20$gr, method = "L-BFGS-B")
#' @export
#`
brown_den <- function(m = 20) {
  list(
    fn = function(par) {
      x1 <- par[1]
      x2 <- par[2]
      x3 <- par[3]
      x4 <- par[4]

      ti <- (1:m) * 0.2
      l <- x1 + ti * x2 - exp(ti)
      r <- x3 + x4 * sin(ti) - cos(ti)
      f <- l * l + r * r
      sum(f * f)
    },
    gr = function(par) {
      x1 <- par[1]
      x2 <- par[2]
      x3 <- par[3]
      x4 <- par[4]

      ti <- (1:m) * 0.2
      sinti <- sin(ti)
      l <- x1 + ti * x2 - exp(ti)
      r <- x3 + x4 * sinti - cos(ti)
      f <- l * l + r * r
      lf4 <- 4 * l * f
      rf4 <- 4 * r * f
      c(
        sum(lf4),
        sum(lf4 * ti),
        sum(rf4),
        sum(rf4 * sinti)
      )
    },
    fg = function(par) {
      x1 <- par[1]
      x2 <- par[2]
      x3 <- par[3]
      x4 <- par[4]

      ti <- (1:m) * 0.2
      sinti <- sin(ti)
      l <- x1 + ti * x2 - exp(ti)
      r <- x3 + x4 * sinti - cos(ti)
      f <- l * l + r * r
      lf4 <- 4 * l * f
      rf4 <- 4 * r * f

      fsum <- sum(f * f)
      grad <- c(
        sum(lf4),
        sum(lf4 * ti),
        sum(rf4),
        sum(rf4 * sinti)
      )

      list(
        fn = fsum,
        gr = grad
      )
    },
    x0 = c(25, 5, -5, 1)
  )
}
mbd <- brown_den(m=20)
mbd
mbd$fg(mbd$x0)
bdsolnm <- optim(mbd$x0, mbd$fn, control=list(trace=0))
bdsolnm
bdsolbfgs <- optim(mbd$x0, mbd$fn, method="BFGS", control=list(trace=0))
bdsolbfgs

library(optimx)
methlist <- c("Nelder-Mead","BFGS","Rvmmin","L-BFGS-B","Rcgmin","ucminf")

solo <- opm(mbd$x0, mbd$fn, mbd$gr, method=methlist, control=list(trace=0))
summary(solo, order=value)

## A failure above is generally because a package in the 'methlist' is not installed.

```

\pagebreak
# References

---
title: "Specifying and Using Fixed Parameters in Nonlinear Estimation"
author: "John C. Nash"
date: "`r Sys.Date()`"
output: rmarkdown::pdf_document
bibliography: ImproveNLS.bib
vignette: >
  %\VignetteIndexEntry{Specifying Fixed Parameters}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


# Motivation

In finding optimal parameters in nonlinear optimization and nonlinear 
least squares problems, we frequently wish to fix one or more parameters
while allowing the rest to be adjusted to explore or optimize an objective 
function. 

This vignette discusses some ideas about specifying the fixed parameters. A 
lot of the material is drawn from Nash J C (2014) **Nonlinear parameter 
optimization using R tools** Chichester UK: Wiley, in particular chapters
11 and 12. There is, however, additional material concerning ways to
manage extensible models, as well as some update to the package `nlsr`. 
The algorithm has been marginally altered to allow for
different sub-variants to be used, and mechanisms
for specifying parameter constraints and providing Jacobian
approximations have been changed.


## Background

Here are some of the ways fixed parameters may be specified in R packages.

Function `nlxb()` in package `nlsr` has argument `masked`:

|   Character vector of quoted parameter names. These parameters will NOT be 
|   altered by the algorithm.
   
This approach has a simplicity that is attractive, but introduces an extra argument to 
calling sequences. (This approach was previously in defunct package `nlmrt`.)

Simlarly, function `nlfb()` in `nlsr` has argument `maskidx`:

|   Vector of indices of the parameters to be masked. These parameters will NOT 
|   be altered by the algorithm. Note that the mechanism here is different from  
|   that in nlxb which uses the names of the parameters.

From `Rvmmin` and `Rcgmin` in package `optimx` the argument `bdmsk`:

|   An indicator vector, having 1 for each parameter that is "free" or 
|   unconstrained, and 0 for any parameter that is fixed or MASKED for the 
|   duration of the optimization.

Note that the function ``bmchk()`` in package `optimx` contains a much more
extensive examination of the bounds on parameters. In particular, it considers 
the issues of inadmissible bounds (lower > upper), when to convert a pair of bounds
where upper["parameter"] - lower["parameter"] < tol to a
fixed or masked parameter (`maskadded`) and whether parameters outside of bounds should be
moved to the nearest bound (`parchanged`). It may be useful to use **inadmissible** 
to refer to situations where a lower bound is higher than an upper bound and
**infeasible** where a parameter value, especially in a given starting vector, 
is outside the bounds.

Further in package `optimx`, the function `optimr()` can call many different 
"optimizers" (actually 
function minimization methods that may include bounds and possibly masks).
These may be specified by setting the lower and upper bounds equal for 
the parameters to be fixed. This seems a simple method for specifying
masks, but does pose some issues. For example, what happens when the
upper bound is only very slightly greater than the lower bound? Also 
should we stop or declare an error if starting values are NOT on the
fixed value?

Of these methods, my preference is now to use the last one -- setting
lower and upper bounds equal, and furthermore requiring the starting
value of such a parameter to this fixed value, otherwise declaring an error. 
The approach 
does not add any special argument for masking, and is relatively obvious
to novice users. However, such users may be tempted to put in narrow 
bounds rather than explicit equalities, and this could have deleterious
consequences.

In the revision to package `nlsr`, package `nlsr`, I have stopped using
`masked` in `nlxb()` and `maskidx` in `nlfb()` (though the latter is a
returned value). This is because I feel the use of equal lower and upper
bounds is a better approach. Moreover, though it is not documented, it 
appears to "mostly work" for the base R function `nls()` with the `algorithm="port"`
option and with `minpack.lm::nlsLM()`.

### Internal structures

`bdmsk` is the internal structure used in `Rcgmin`, `Rvmmin` and `nlfb` to handle bounds 
constraints as well as masks.
There is one element of `bdmsk` for each parameter, and in `Rcgmin` and `Rvmmin`, 
this is used on input to 
specify parameter i as fixed or masked by setting `bdmsk[i] <- 0`. Free 
parameters have their `bdmsk` element `1`,
but during optimization in the presence of bounds, we can set other values. 
The full set is as follows

* 1 for a free or unconstrained parameter
* 0 for a masked or fixed parameter
* -0.5 for a parameter that is out of bounds high (> upper bound)
* -1 for a parameter at its upper bound
* -3 for a parameter at its lower bound
* -3.5 for a parameter that is out of bounds low (< lower bound)

Not all these possibilities will be used by all methods that use `bdmsk`.

The -1 and -3 are historical, and arose in the development of BASIC 
codes for @jnmws87 (This is now available for free download
from archive.org. (https://archive.org/details/NLPE87plus). 
In particular, adding 2 to the `bdmsk` element gives 1 for an upper bound 
and -1 for a lower bound, 
simplifying the expression to decide if an optimization trial step 
will move away from a bound.


## Proposed algorithmic approaches

Because masks (fixed parameters) reduce the dimensionality of the 
optimization problem, we can consider
modifying the problem to the lower dimension space. This is Duncan 
Murdoch's suggestion, using 

*  `fn0(par0)` to be the initial user function of the full dimension 
    parameter vector `par0`
*  `fn1(par1)` to be the reduced or internal functin of the reduced dimension 
    vector `par1`
*  `par1 <- forward(par0)`
*  `par0 <- inverse(par1)`

The major advantage of this approach is explicit dimension reduction. The main disadvantage
is the effort of transformation at every step of an optimization.

An alternative is to use the `bdmsk` vector to **mask** 
the optimization search or adjustment vector, 
including gradients and (approximate) Hessian or Jacobian matrices. A 0 element of `bdmsk`
"multiplies" any 
adjustment. The principal difficulty is to ensure we do not essentially divide by 
zero in applying
any inverse Hessian. This approach avoids `forward`, `inverse` and `fn1`. However, 
it may hide the
reduction in dimension, and caution is necessary in using the function and its 
derived gradient,
Hessian and derived information.


## Examples of use

### For optimx

```{r opx}
require(optimx)
sq<-function(x){
   nn<-length(x)
   yy<-1:nn
   f<-sum((yy-x)^2)
   f
}
sq.g <- function(x){
   nn<-length(x)
   yy<-1:nn
   gg<- 2*(x - yy)
}
xx <- c(.3, 4)
uncans <- Rvmmin(xx, sq, sq.g)
proptimr(uncans)
mybm <- c(0,1) # fix parameter 1
cans <- Rvmmin(xx, sq, sq.g, bdmsk=mybm)
proptimr(cans)
require(nlsr)
weed <- c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443,
   38.558, 50.156, 62.948, 75.995, 91.972)
ii <- 1:12
wdf <- data.frame(weed, ii)
weedux <- nlxb(weed~b1/(1+b2*exp(-b3*ii)), start=c(b1=200, b2=50, b3=0.3)) 
weedux
weedcx <- nlxb(weed~b1/(1+b2*exp(-b3*ii)), start=c(b1=200, b2=50, b3=0.3), masked=c("b1")) 
weedcx
rfn <- function(bvec, weed=weed, ii=ii){
  res <- rep(NA, length(ii))
  for (i in ii){
    res[i]<- bvec[1]/(1+bvec[2]*exp(-bvec[3]*i))-weed[i]
  }
  res
}
weeduf <- nlfb(start=c(200, 50, 0.3),resfn=rfn,weed=weed, ii=ii,
               control=list(japprox="jacentral"))
weeduf
weedcf <- nlfb(start=c(200, 50, 0.3),resfn=rfn,weed=weed, ii=ii, lower=c(200, 0, 0),
               upper=c(200, 100,100), control=list(japprox="jacentral"))
weedcf
```

### An extensible bell-curve model

Package `nlraa` has a selfStart model `SSbell` (@ArchMiguez2013) of which the formula is

$$ y \approx ymax * exp(a *(x - xc)^2 + b*(x-xc)^3) $$

This is essentially the Gaussian bell curve with an additional cubic element in 
the exponential
function. If we **fix** $b = 0$, then we have the usual Gaussian, and we can use 
the standard
deviation $sigma$ of the variable $x$ with $xc$ equal to its mean and our 
parameters will be given approximately by 

$$ ymax = max(y)$$
$$ a = -0.5/sigma^2$$
$$ xc = mean(y) $$

We illustrate this in the following example.

```{r bellx, code=readLines("./BellX.R"), echo=TRUE}
```


# References

\name{nlsr.package}
\alias{nlsr.package}
\title{nlsr-package
 
Tools for solving nonlinear least squares problems
 
The package provides some tools related to using the Nash variant
of Marquardt's algorithm for nonlinear least squares. Jacobians can
usually be developed by automatic or symbolic derivatives.}
\usage{
nlsr.package()
}
\description{
nlsr-package
 
Tools for solving nonlinear least squares problems
 
The package provides some tools related to using the Nash variant
of Marquardt's algorithm for nonlinear least squares. Jacobians can
usually be developed by automatic or symbolic derivatives.
}
\details{
This package includes methods for solving nonlinear least squares problems
specified by a modeling expression and given a starting vector of named 
paramters. Note: You must provide an expression of the form
    lhs ~ rhsexpression
so that the residual expression 
    rhsexpression - lhs
can be computed. The expression can be enclosed in quotes, and this seems to give
fewer difficulties with R. Data variables must already be defined, either within 
the parent environment or else in the dot-arguments. Other symbolic elements in
the modeling expression must be standard functions or else parameters that are 
named in the start vector.

The main functions in \code{nlsr} are:

nlfb   Nash variant of the Marquardt procedure for nonlinear least squares,
   with bounds constraints, using a residual and optionally Jacobian
   described as \code{R} functions.
   
nlxb  Nash variant of the Marquardt procedure for nonlinear least squares,
  	with bounds constraints, using an expression to describe the residual via
  	an \code{R} modeling expression. The Jacobian is computed via symbolic
  	differentiation.
  	
wrapnlsr  Uses \code{nlxb} to solve nonlinear least squares then calls 
   \code{nls()} to create an object of type nls. \code{nlsr} is an alias
for \code{wrapnlsr}
   
model2rjfun  returns a function with header \code{function(prm)}, which
   evaluates the residuals (and if jacobian is TRUE the Jacobian matrix) 
   of the model at \code{prm}.  The residuals are defined to be the 
   right hand side of \code{modelformula} minus the left hand side.
   
model2ssgrfun  returns a function with header \code{function(prm)}, which
    evaluates the sum of squared residuals (and if gradient is \code{TRUE} the
    gradient vector) of the model at \code{prm}.
    
modelexpr  returns the expression used to calculate the vector of 
        residuals (and possibly the Jacobian) used in the previous functions.
}
\references{
Nash, J. C. (1979, 1990) _Compact Numerical Methods for Computers.
    Linear Algebra and Function Minimisation._ Adam Hilger./Institute
    of Physics Publications

  Nash, J. C. (2014) _Nonlinear Parameter Optimization Using R Tools._
    Wiley
}
\author{
John C Nash and Duncan Murdoch
}
\keyword{nls}
\keyword{nonlinear&least&squares}

\name{pnls}
\alias{pnls}
\title{pnls}
\usage{
pnls(x)
}
\arguments{
\item{x}{an nls() result object from nls() or nlsLM()}
}
\value{
none
}
\description{
Compact display of specified \code{nls} object \code{x}
}
\author{
J C Nash 2014-7-16, 2023-5-8 nashjc _at_ uottawa.ca
}

\name{pnlslm}
\alias{pnlslm}
\title{pnlslm}
\usage{
pnlslm(x)
}
\arguments{
\item{x}{an nls() result object from minpack.lm::nls.lm()}
}
\value{
none
}
\description{
Compact display of specified \code{nls.lm} object \code{x}.
This code returns the iteration count in a different variable
from that of \code{nls} objects.
}
\author{
J C Nash 2014-7-16, 2023-5-8 nashjc _at_ uottawa.ca
}

\name{nlsSimplify}
\alias{nlsSimplify}
\title{nlsSimplify}
\usage{
nlsSimplify(expr, simpEnv = sysSimplifications, verbose = FALSE)
}
\arguments{
\item{expr}{An expression represented in a variety of ways. See Details.}

\item{simpEnv}{The environment in which simplifications are stored.}

\item{verbose}{If \code{TRUE}, then diagnostic output will be printed as simplifications are recognized.}
}
\description{
Try to simplify an expression.
}
\details{
The \code{expr} can be an expression vector or other language object.
If it
is a complex expression (e.g. \code{(a + b) + c}), then simplifications
will be applied recursively.  Simplifications are applied 
from the database created by \code{\link{newSimplification}},
and the new expression (or call...) is returned.

This function is mainly for internal use by \code{\link{nlsDeriv}}.
}
\examples{
nlsSimplify(quote(x + 0 + y*1), verbose = TRUE)
}
\keyword{internal}

\name{newDeriv}
\alias{newDeriv}
\title{newDeriv}
\usage{
newDeriv (expr, deriv, derivEnv = sysDerivs)
}
\arguments{
\item{expr}{An expression represented as a quoted expression.}

\item{deriv}{The derivative.  See Details below. }

\item{derivEnv}{The environment in which to store the setting.}
}
\description{
Define a new derivative expression.
}
\details{
Both \code{expr} and \code{deriv} are treated
as unevaluated expressions.

\code{deriv} should include the sum of derivatives with respect to
all variables similar to a \dQuote{total derivative} using
\code{\link{D}} for the differential.  See the examples
below.

This is mainly intended for internal use.
}
\examples{
newDeriv(sin(x), cos(x)*D(x))
newDeriv(x*y, x*D(y) + D(x)*y)

nlsDeriv(quote(sin(x)*sin(y)), "x")
}
\keyword{internal}

\name{predict.nlsr}
\alias{predict.nlsr}
\title{predict.nlsr}
\usage{
\method{predict}{nlsr}(object = NULL, newdata = list(), ...)
}
\arguments{
\item{object}{an object of class \code{nlsr}}

\item{newdata}{a dataframe containing columns that match the original dataframe
used to estimate the nonlinear model in the \code{nlsr} object}

\item{...}{additional data needed to evaluate the modeling functions
Default FALSE}
}
\description{
prepare and display predictions from an \code{nlsr} model
}
\author{
J C Nash 2014-7-16   nashjc _at_ uottawa.ca
}

\name{nlfb}
\alias{nlfb}
\title{nlfb: nonlinear least squares modeling by functions}
\usage{
nlfb(
  start,
  resfn,
  jacfn = NULL,
  trace = FALSE,
  lower = -Inf,
  upper = Inf,
  weights = NULL,
  data = NULL,
  ctrlcopy = FALSE,
  control = list(),
  ...
)
}
\arguments{
\item{start}{a numeric vector with all elements present
e.g., start=c(b1=200, b2=50, b3=0.3) 
The start vector for this \code{nlfb}, unlike \code{nlxb}, does
not need to be named.}

\item{resfn}{A function that evaluates the residual vector for 
computing the elements of the sum of squares function at the set of 
parameters \code{start}. Where this function is created by actions on 
a formula or expression in \code{nlxb}, this residual vector will be 
created by evaluation of the 'model - data', rather than 
the conventional 'data - model' approach. The sum of squares is the same.}

\item{jacfn}{A function that evaluates the Jacobian of the sum of squares 
function, that is, the matrix of partial derivatives of the residuals 
with respect to each of the parameters. If NULL (default), uses an 
approximation.

The Jacobian MUST be returned as the attribute "gradient" of this function,
allowing \code{jacfn} to have the same name and be the same code block
as \code{resfn}, which may permit some efficiencies of computation.
}

\item{trace}{TRUE for console output during execution}

\item{lower}{a vector of lower bounds on the parameters. 
If a single number, this will be applied to all. Default \code{-Inf}.}

\item{upper}{a vector of upper bounds on the parameters. If a single number, 
this will be applied to all parameters. Default \code{Inf}.}

\item{weights}{A vector of fixed weights or a function producing one.
See the Details below.}

\item{data}{a data frame of variables used by resfn and jacfn to compute the
required residuals and Jacobian.}

\item{ctrlcopy}{If TRUE use control supplied as is.
           This avoids reprocessing controls.}

\item{control}{a list of control parameters. See nlsr.control().}

\item{...}{additional data needed to evaluate the modeling functions}
}
\value{
{
  A list of the following items: \describe{
  \item{coefficients}{A named vector giving the parameter values at
         the supposed solution.}
  \item{ssquares}{The sum of squared residuals at this set of parameters.}
  \item{resid}{The weighted residual vector at the returned parameters.}
  \item{jacobian}{The jacobian matrix (partial derivatives of residuals w.r.t.
         the parameters) at the returned parameters.}
  \item{feval}{The number of residual evaluations (sum of squares
         computations) used.}
  \item{jeval}{The number of Jacobian evaluations used.}
  \item{weights0}{The weights argument as specified.}
  \item{weights}{The weights vector at the final fit.}
}            
}
}
\description{
A simplified and hopefully robust alternative to finding
the nonlinear least squares minimizer that causes
'formula' to give a minimal residual sum of squares.
}
\details{
nlfb is particularly intended to allow for the
resolution of very ill-conditioned or else near
zero-residual problems for which the regular nls()
function is ill-suited. 

This variant uses a qr solution without forming the sum
of squares and cross products t(J)%*%J

Neither this function nor \code{nlxb} have provision for parameter
scaling (as in the \code{parscale} control of \code{optim} and
package \code{optimx}). This would be more tedious than difficult to
introduce, but does not seem to be a priority feature to add.

The \code{weights} argument can be a vector of fixed weights, in which
case the objective function that will be 
minimized is the sum of squares where each residual is multiplied by the 
square root of the corresponding weight. Default \code{NULL} implies 
unit weights.  \code{weights} may alternatively be a function with header \code{function(parms, resids)} to compute such a vector.
}
\examples{
library(nlsr)
# Scaled Hobbs problem
shobbs.res  <-  function(x){ # scaled Hobbs weeds problem -- residual
  # This variant uses looping
  if(length(x) != 3) stop("shobbs.res -- parameter vector n!=3")
  y  <-  c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443,
           38.558, 50.156, 62.948, 75.995, 91.972)
  tt  <-  1:12
  res  <-  100.0*x[1]/(1+x[2]*10.*exp(-0.1*x[3]*tt)) - y
}
shobbs.jac  <-  function(x) { # scaled Hobbs weeds problem -- Jacobian
  jj  <-  matrix(0.0, 12, 3)
  tt  <-  1:12
  yy  <-  exp(-0.1*x[3]*tt)
  zz  <-  100.0/(1+10.*x[2]*yy)
  jj[tt,1]   <-   zz
  jj[tt,2]   <-   -0.1*x[1]*zz*zz*yy
  jj[tt,3]   <-   0.01*x[1]*zz*zz*yy*x[2]*tt
  attr(jj, "gradient") <- jj
  jj
}
st <- c(b1=2, b2=1, b3=1) # a default starting vector (named!)
# Default controls, standard Nash-Marquardt algorithm
anlf0 <- nlfb(start=st, resfn=shobbs.res, jacfn=shobbs.jac,
               trace=TRUE, control=list(prtlvl=1))
anlf0

# Hartley with step reduction factor of .2
anlf0h <- nlfb(start=st, resfn=shobbs.res, jacfn=shobbs.jac,
              trace=TRUE, control=list(prtlvl=1, lamda=0, laminc=1.0,
              lamdec=1.0, phi=0, stepredn=0.2))
anlf0h

anlf1bm <- nlfb(start=st, resfn=shobbs.res, jacfn=shobbs.jac, lower=c(2,0,0),
                upper=c(2,6,3), trace=TRUE, control=list(prtlvl=1))
anlf1bm
cat("backtrack using stepredn=0.2\n")
anlf1bmbt <- nlfb(start=st, resfn=shobbs.res, jacfn=shobbs.jac, lower=c(2,0,0),
                upper=c(2,6,3), trace=TRUE, control=list(stepredn=0.2, prtlvl=1))
anlf1bmbt
## Short output
pshort(anlf1bm)
anlf2bm <- nlfb(start=st, resfn=shobbs.res, jacfn=shobbs.jac, lower=c(2,0,0),
                upper=c(2,6,9), trace=TRUE, control=list(prtlvl=1))
anlf2bm
cat("backtrack using stepredn=0.2\n")

anlf2bmbt <- nlfb(start=st, resfn=shobbs.res, jacfn=shobbs.jac, lower=c(2,0,0),
                upper=c(2,6,9), trace=TRUE, control=list(stepredn=0.2, prtlvl=1))
anlf2bmbt
## Short output
pshort(anlf2bm)
}
\author{
J C Nash 2014-7-16   nashjc _at_ uottawa.ca
}

\name{nvec}
\alias{nvec}
\title{nvec}
\description{
Compact display of a specified named vector
}
\usage{
nvec(vec)
}
\arguments{
\item{vec}{a named vector of parameters}
}
\value{
none (Note that we may want to change this.)
}
\author{
J C Nash 2014-7-16   nashjc _at_ uottawa.ca
}

\name{jand}
\alias{jand}
\title{jand}
\usage{
jand(pars, resfn = NULL, bdmsk = NULL, resbest = NULL, ndstep = 1e-07, ...)
}
\arguments{
\item{pars}{a named numeric vector of parameters to the model}

\item{resfn}{a function to compute a vector of residuals}

\item{bdmsk}{Vector defining bounds and masks. Default is \code{NULL}}

\item{resbest}{If supplied, a vector of the residuals at the parameters
\code{pars} to save re-evaluation.}

\item{ndstep}{A tolerance used to alter parameters to compute numerical
approximations to derivatives. Default \code{1e-7}.}

\item{\dots}{Extra information needed to compute the residuals}
}
\description{
approximate Jacobian via numDeriv::jacobian
}
\author{
J C Nash 2014-7-16   nashjc _at_ uottawa.ca
}

\name{pshort}
\alias{pshort}
\title{pshort}
\description{
To provide a 1-line summary of an object of class \code{nlsr}.
}
\usage{
pshort(x)
}
\arguments{
\item{x}{an object of class \code{nlsr}}
}
\author{
J C Nash 2022-11-22   nashjc _at_ uottawa.ca
}

\name{model2rjfun}
\alias{model2rjfun}
\alias{model2ssgrfun}
\alias{modelexpr}
\alias{SSmod2rjfun}
\title{model2rjfun}
\usage{
model2rjfun(modelformula, pvec, data = NULL, jacobian = TRUE, testresult = TRUE, ...)
SSmod2rjfun(modelformula, pvec, data = NULL, jacobian = TRUE, testresult = TRUE, ...)
model2ssgrfun(modelformula, pvec, data = NULL, gradient = TRUE, 
              testresult = TRUE, ...)
modelexpr(fun)
}
\arguments{
\item{modelformula}{A formula describing a nonlinear regression model.}

\item{pvec}{A vector of parameters.}

\item{data}{A dataframe, list or environment holding data used in the calculation.}

\item{jacobian}{Whether to compute the Jacobian matrix.}

\item{testresult}{Whether to test the function by evaluating it at \code{pvec}.}

\item{gradient}{Whether to compute the gradient vector.}

\item{fun}{A function produced by one of \code{model2rjfun} or \code{model2ssgrfun}.}

\item{\dots}{Dot arguments, that is, arguments that may be supplied by \code{name = value}
  to supply information needed to compute specific quantities in the model.}
}
  
\details{
If \code{pvec} does not have names, the parameters will have names
generated in the form \samp{p_<n>}, e.g. \code{p_1, p_2}.  Names that appear in
\code{pvec} will be taken to be parameters of the model.  

The \code{data} argument may be a dataframe, list or environment, or \code{NULL}.
If it is not an environment, one will be constructed using the components
of \code{data} with parent environment set to be
the environment of \code{modelformula}.  

\code{SSmod2rjfun} returns a function with header \code{function(prm)}, which
evaluates the residuals (and if \code{jacobian} is \code{TRUE} the
Jacobian matrix) of the selfStart model (the rhs is used) at \code{prm}.  
The residuals are defined to be the right hand side of \code{modelformula} 
minus the left hand side. Note that the selfStart model used in the model
formula must be available (i.e., loaded). If this function is called from 
\code{nlxb()} then the \code{control} element \code{japprox} must be 
set to value \code{SSJac}. 
}

\description{
These functions create functions to evaluate residuals or sums of squares at 
particular parameter locations.
}

\value{
\code{model2rjfun} returns a function with header \code{function(prm)}, which
evaluates the residuals (and if \code{jacobian} is \code{TRUE} the
Jacobian matrix) of the model at \code{prm}.  The residuals are defined to be
the right hand side of \code{modelformula} minus the left hand side.

\code{model2ssgrfun} returns a function with header \code{function(prm)}, which
evaluates the sum of squared residuals (and if \code{gradient} is \code{TRUE} the
gradient vector) of the model at \code{prm}. 

\code{modelexpr} returns the expression used to calculate the vector of 
residuals (and possibly the Jacobian) used in the previous functions.
}

\keyword{
   nonlinear
}

\examples{
# We do not appear to have an example for modelexpr. See nlsr-devdoc.Rmd for one.

y <- c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443, 38.558,
       50.156, 62.948, 75.995, 91.972)

tt <- seq_along(y)  # for testing

mydata <- data.frame(y = y, tt = tt)
f <- y ~ b1/(1 + b2 * exp(-1 * b3 * tt))
p <- c(b1 = 1, b2 = 1, b3 = 1)
rjfn <- model2rjfun(f, p, data = mydata)
rjfn(p)
rjfnnoj <- model2rjfun(f, p, data = mydata, jacobian=FALSE)
rjfnnoj(p)

myexp <- modelexpr(rjfn)
cat("myexp:"); print(myexp)

ssgrfn <- model2ssgrfun(f, p, data = mydata)
ssgrfn(p)

ssgrfnnoj <- model2ssgrfun(f, p, data = mydata, gradient=FALSE)
ssgrfnnoj(p)
}
\seealso{
\code{\link{nls}}
}
\author{
John Nash and Duncan Murdoch
}

\name{summary.nlsr}
\alias{summary.nlsr}
\title{summary.nlsr}
\usage{
\method{summary}{nlsr}(object, ...)
}
\arguments{
\item{object}{an object of class \code{nlsr}}

\item{...}{additional data needed to evaluate the modeling functions}
}
\description{
prepare display result of \code{nlsr} computations - NOT compact output
}
\details{
The set of possible controls to set is as follows
}
\author{
J C Nash 2014-7-16   nashjc _at_ uottawa.ca
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SSlogisJN.R
\name{SSlogisJN}
\alias{SSlogisJN}
\title{Alternative self start for three-parameter logistic function SSlogis}
\usage{
SSlogisJN(input, Asym, xmid, scal)
}
\arguments{
\item{input}{input vector (input)}

\item{Asym}{asymptotic value for large values of x}

\item{xmid}{a numeric parameter representing the x value at the inflection point of 
the curve. The value of SSlogisJN will be Asym/2 at xmid.}

\item{scal}{numeric scale parameter on the input axis}
}
\description{
Self starter for a 3-parameter logistic function.

The equation for this function is:

\deqn{f(input) = Asym/(1 + exp((xmid - input)/scal))}

The approach of the function SSlogis() in base R uses a different
algorithm and returns the actual solution rather than starting
parameters, so runs a complete set of iterations, only to try to
repeat from the solution with the standard algorithm.
}
\references{
  Ratkowsky, David A. (1983) Nonlinear Regression Modeling, A Unified
      Practical Approach, Dekker: New York, section 8.3.2
}
\examples{
{
## require(ggplot2)
require(nlsr)
set.seed(1234)
x <- seq(0, 20, .2)
y <- SSlogisJN(x, 5, 10, .5) + rnorm(length(x), 0, 0.15)
frm<-y ~ SSlogisJN(x, Asym, xmid, scal)
dat <- data.frame(x = x, y = y)
strt<-getInitial(frm, dat)
cat("Proposed start:\n"); print(strt)
fit <- nlxb(frm, strt, data = dat, control=list(japprox="SSJac"))
print(fit)
## plot
## ggplot(data = dat, aes(x = x, y = y)) + 
##   geom_point() + 
##  geom_line(aes(y = fitted(fit)))
}
}

\name{isCALL}
\alias{isCALL}
\title{isCALL
 
Test if argument is a particular call}
\usage{
isCALL(x, name)
}
\arguments{
\item{x}{object to be tested}

\item{name}{The function to test for.}
}
\description{
Test if \code{x} is a call to the function given by \code{name}.

Used in \code{\link{newSimplification}} definitions.
}
\examples{
x <- quote(mean(1:10))
isCALL(x, "mean")
isCALL(x, "sd")
}
\keyword{internal}

\name{newSimplification}
\alias{newSimplification}
\title{newSimplification}
\usage{
newSimplification(expr, test, simplification, do_eval = FALSE, 
                       simpEnv = sysSimplifications)
}
\arguments{
\item{expr}{An expression to simplify.}

\item{test}{An expression to evaluate whether the simplification
should be applied to \code{expr}.}

\item{simplification}{An equivalent but simpler version of the expression.}

\item{do_eval}{Whether the simplification should be evaluated
or stored as-is.}

\item{simpEnv}{The environment in which the simplification is stored.}
}
\description{
Define a new simplification expression
}
\value{
If \code{expr} is missing, list all the functions with known 
simplifications.

If \code{test} is missing, list all the known simplifications for
\code{expr}.

Otherwise, add the new simplification to the list of possible simplifications.
}
\examples{
# The unary + operation can always be safely removed:
newSimplification(+a, TRUE, a)

# The unary - operation can be absorbed into numeric values:
newSimplification(-a, is.numeric(a), -a, do_eval = TRUE)

# Adding zero to anything can be skipped:
newSimplification(a + b, isZERO(b), a)
}
\keyword{internal}

\name{nlsDeriv}
\alias{nlsDeriv}
\alias{codeDeriv}
\alias{fnDeriv}
\title{nlsDeriv
  Functions to take symbolic derivatives.}
\usage{
nlsDeriv(expr, name, derivEnv = sysDerivs, do_substitute = FALSE, verbose = FALSE, ...)

codeDeriv(expr, namevec, hessian = FALSE, derivEnv = sysDerivs, 
          do_substitute = FALSE, verbose = FALSE, ...)

fnDeriv(expr, namevec, args = all.vars(expr), env = environment(expr), 
          do_substitute = FALSE, verbose = FALSE, ...)
}
\arguments{
\item{expr}{An expression represented in a variety of ways. See Details.}

\item{name}{The name of the variable with respect to which the derivative will be computed.}

\item{derivEnv}{The environment in which derivatives are stored.}

\item{do_substitute}{If \code{TRUE}, use \code{\link{substitute}} to get the expression passed as
\code{expr}, otherwise evaluate it.}

\item{verbose}{If \code{TRUE}, then diagnostic output will be printed as derivatives
and simplifications are recognized.}

\item{...}{Additional parameters which will be passed to \code{codeDeriv}
from \code{fnDeriv}, and to \code{nlsSimplify} from 
\code{nlsDeriv} and \code{codeDeriv}.}

\item{namevec}{Character vector giving the variable names with respect to 
which the derivatives will be taken.}

\item{hessian}{Logical indicator of whether the 2nd derivatives should also be computed.}

\item{args}{Desired arguments for the function.  See Details below.}

\item{env}{The environment to be attached to the created function.
If \code{NULL}, the caller's frame is used.}
}
\description{
Compute derivatives of simple expressions symbolically, allowing user-specified derivatives.
}
\details{   
 Functions \code{nlsDeriv} and \code{codeDeriv} are designed as replacements 
 for the \pkg{stats} package functions \code{\link{D}} and \code{\link{deriv}}
 respectively, though the argument lists do not match exactly.
 
 The \code{nlsDeriv} function computes a symbolic derivative of an expression
 or language object.  Known derivatives are stored in
 \code{derivEnv}; the default \code{sysDerivs} contains expressions for
 all of the derivatives recognized by \code{\link{deriv}}, but in
 addition allows differentiation with respect to any parameter
 where it makes sense.  It also allows the derivative of \code{abs}
 and \code{sign}, using an arbitrary choice of 0 at the discontinuities.
 
 The \code{codeDeriv} function computes
 an expression for efficient calculation of the expression value together
 with its gradient and optionally the Hessian matrix.
 
 The \code{fnDeriv} function wraps the \code{codeDeriv} result
 in a function.  If the \code{args} are given as a character
 vector (the default), the arguments will have those names,
 with no default values.  Alternatively, a custom argument list with default values can
 be created using \code{\link{alist}}; see the example below.
 
 The \code{expr} argument will be converted to a
 language object using \code{\link{dex}} (but note
 the different default for \code{do_substitute}).  
 Normally it should be a formula with no left
 hand side, e.g. \code{ ~ x^2 }, or an expression vector
 e.g. \code{ expression(x, x^2, x^3) }, or a language
 object e.g. \code{quote(x^2)}.  In \code{codeDeriv} and
 \code{fnDeriv} the expression vector must be of length 1.
 
 The \code{newDeriv} function is used to define a new derivative.
 The \code{expr} argument should match the header of the function as a
 call to it (e.g. as in the help pages), and the \code{deriv} argument
 should be an expression giving the derivative, including calls to
 \code{D(arg)}, which will not be evaluated, but will be substituted
 with partial derivatives of that argument with respect to \code{name}.
 See the examples below.  
 
 If \code{expr} or \code{deriv} is missing in a call to
 \code{newDeriv()}, it will return the currently saved derivative
 record from \code{derivEnv}.  If \code{name} is missing in a call to
 \code{nlsDeriv} with a function call, it will print a message describing
 the derivative formula and return \code{NULL}.
 
 To handle functions which act differently if a parameter is
 missing, code the default value of that parameter to \code{.MissingVal},
 and give a derivative that is conditional on \code{missing()}
 applied to that parameter.  See the derivatives of \code{"-"} and \code{"+"} 
 in the file \code{derivs.R} for an example.
}
 
\value{
 If \code{expr} is an expression vector, \code{nlsDeriv} and \code{nlsSimplify}
 return expression vectors containing the response.  
 For formulas or language objects, a language object is returned.
 
 \code{codeDeriv} always returns a language object.
 
 \code{fnDeriv} returns a closure (i.e. a function).
 
 \code{nlsDeriv} returns the symbolic derivative of the expression.
 
 \code{newDeriv} with \code{expr} and \code{deriv} specified is
 called for the side effect of recording the derivative in \code{derivEnv}.
 If \code{expr} is missing, it will return the list of names of functions
 for which derivatives are recorded.  If \code{deriv} is missing, it
 will return its record for the specified function.
}
\note{
 \code{newDeriv(expr, deriv, ...)} will issue a warning
 if a different definition for the derivative exists
 in the derivative table.
}
\author{
  Duncan Murdoch
}
\seealso{\code{\link{deriv}}}
 
\examples{
 nlsDeriv(~ sin(x+y), "x")

 f <- function(x) x^2
 newDeriv(f(x), 2*x*D(x))
 nlsDeriv(~ f(abs(x)), "x")
 
 nlsDeriv(~ pnorm(x, sd=2, log = TRUE), "x")
 fnDeriv(~ pnorm(x, sd = sd, log = TRUE), "x")
 f <- fnDeriv(~ pnorm(x, sd = sd, log = TRUE), "x", args = alist(x =, sd = 2))
 f
 f(1)
 100*(f(1.01) - f(1))  # Should be close to the gradient
 
       # The attached gradient attribute (from f(1.01)) is
       # meaningless after the subtraction.
       
 # Multiple point example
 xvals <- c(1, 3, 4.123)
 print(f(xvals))
 # Getting a hessian matrix
 f2 <- ~ (x-2)^3*y - y^2
 mydf2 <- fnDeriv(f2, c("x","y"), hessian=TRUE)
 # display the resulting function
 print(mydf2)
 x <- c(1, 2)
 y <- c(0.5, 0.1)
 evalmydf2 <- mydf2(x, y)
 print(evalmydf2)
 # the first index of the hessian attribute is the point at which we want the hessian
 hmat1 <- as.matrix(attr(evalmydf2,"hessian")[1,,])
 print(hmat1)
 hmat2 <- as.matrix(attr(evalmydf2,"hessian")[2,,])
 print(hmat2)
}
\keyword{
   jacobian
}

\name{nlsr.control}
\alias{nlsr.control}
\title{nlsr.control}
\description{
Set and provide defaults of controls for package \code{nlsr}
}
\usage{
nlsr.control(control)
}
\arguments{
\item{control}{A list of controls. 
       If missing, the defaults are provided. See below.
       If a named control is provided,  e.g., via a call
       ctrllist<- nlsr.control(japprox="jand"), then
       that value is substituted for the default of the control
       in the FULL list of controls that is returned.

NOTE: at 2022-6-17 there is NO CHECK FOR VALIDITY

The set of possible controls to set is as follows, and is returned.}
}
\value{
\item{femax}{INTEGER limit on the number of evaluations of residual function
                          Default  10000.}
                          
\item{japprox}{CHARACTER name of the Jacobian approximation to use
           Default NULL, since we try to use analytic gradient}
                 
\item{jemax}{INTEGER limit on the number of evaluations of the Jacobian
           Default 5000}
                          
\item{lamda}{REAL initial value of the Marquardt parameter
           Default 0.0001 
           Note: mis-spelling as in JNMWS, kept as historical serendipity.}
           
\item{lamdec}{ REAL multiplier used to REDUCE \code{lambda} (0 < \code{lamdec} < \code{laminc})
           Default 4, so \code{lamda <- lamda * (lamdec/laminc)} }
                           
\item{laminc}{ REAL multiplier to INCREASE \code{lambda} (1 < \code{laminc}
           Default 10}

\item{nbtlim}{ if stepredn > 0, then maximum number of backtrack loops
           (in addition to default evaluation); Default 6}

\item{ndstep}{ REAL  stepsize for numerical Jacobian approximation
           Default 1e-7}
                          
\item{offset}{ REAL A value used to test for numerical equality, i.e. \code{a} and
                  \code{b} are taken equal if \code{(a + offset) == (b + offset)}
                          Default 100.}
                          
\item{phi}{REAL Factor used to add unit Marquardt stabilization matrix in Nash
              modification of LM method.  Default 1}
                          
\item{prtlvl}{ INTEGER The higher the value, the more intermediate output is provided.
           Default 0}
                          
\item{psi}{REAL Factor used to add scaled Marquardt stabilization matrix in Nash
              modification of LM method.  Default 0} 
                          
\item{rofftest}{ LOGICAL If TRUE, perform (safeguarded) relative offset convergence test
                          Default TRUE}
                          
\item{scaleOffset}{ a positive constant to be added to the denominator sum-of-squares in
              the relative offset convergence criteria. Default 0}

\item{smallsstest}{ LOGICAL. If TRUE tests sum of squares and terminates if very small.
                          Default TRUE}
                          
\item{stepredn}{REAL Factor used to reduce the stepsize in a Gauss-Newton algorithm (Hartley's
       		method). 0 means NO backtrack. Default 0}
                          
\item{watch}{LOGICAL to provide a pause at the end of each iteration for user to monitor
           progress.   Default FALSE}
}
\author{
J C Nash 2014-7-16 revised 2022-11-22   nashjc _at_ uottawa.ca
}

\name{wrapnlsr}
\alias{wrapnlsr}
\title{wrapnlsr}
\description{
Provides class nls solution to a nonlinear least squares solution 
using the Nash Marquardt tools.
}
\usage{
wrapnlsr(formula = NULL,  data = NULL,  start = NULL,  control = NULL,
    trace = FALSE, subset = NULL, lower = -Inf, upper = Inf,  weights = NULL,
    ...)                           
}
\arguments{
\item{formula}{The modeling formula. Looks like 'y~b1/(1+b2*exp(-b3*T))'}

\item{data}{a data frame containing data for variables
used in the formula that are NOT the parameters. This
data may also be defined in the parent frame i.e.,
'global' to this function}

\item{start}{MUST be a named vector with all elements present
e.g., start=c(b1=200, b2=50, b3=0.3)}

\item{control}{a list of control parameters. See nlsr.control().}

\item{trace}{TRUE for console output during execution (default FALSE)}

\item{subset}{an optional vector specifying a subset of observations 
to be used in the fitting process. NOT used currently by nlxb()
or nlfb() and will throw an error if present and not NULL.}

\item{lower}{a vector of lower bounds on the parameters. 
If a single number, this will be applied to all parameters
Default \code{-Inf}.}

\item{upper}{a vector of upper bounds on the parameters. If a single number, 
this will be applied to all parameters. Default \code{Inf}.}

\item{weights}{A vector of (usually fixed) weights. The objective function that will be 
minimized is the sum of squares where each residual is multiplied by the 
square root of the corresponding weight. Default \code{NULL} implies 
unit weights.}

\item{...}{additional data needed to evaluate the modeling functions}
}
\value{
A solution object of type \code{nls}
}
\examples{
library(nlsr)
cat("kvanderpoel.R test of wrapnlsr\n")
x<-c(1,3,5,7)
y<-c(37.98,11.68,3.65,3.93)
pks28<-data.frame(x=x,y=y)
fit0<-try(nls(y~(a+b*exp(1)^(-c*x)), data=pks28, start=c(a=0,b=1,c=1), 
              trace=TRUE))
print(fit0)
fit1<-nlxb(y~(a+b*exp(-c*x)), data=pks28, start=c(a=0,b=1,c=1), trace = TRUE)
print(fit1) 
cat("\n\n or better\n")
fit2<-wrapnlsr(y~(a+b*exp(-c*x)), data=pks28, start=c(a=0,b=1,c=1), 
               lower=-Inf, upper=Inf, trace = TRUE)
fit2

weed <- c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443,
          38.558, 50.156, 62.948, 75.995, 91.972)
tt <- 1:12
weeddf <- data.frame(tt, weed)
hobbsu <- weed ~ b1/(1+b2*exp(-b3*tt))
st2 <- c(b1=200, b2=50, b3=0.3)
wts <- 0.5^tt # a straight scaling comes via wts <- rep(0.01, 12)
lo <- c(200, 0, 0)
up <- c(1000, 1000, 1000)
whuw2 <-  try(wrapnlsr(start=st2, formula=hobbsu, data=weeddf, subset=2:11,
                  weights=wts, trace=TRUE, lower=lo, upper=up))
summary(whuw2)
deviance(whuw2)
whuw2a <-  try(nlsr(start=st2, formula=hobbsu, data=weeddf, subset=2:11,
                  weights=wts, trace=TRUE, lower=lo, upper=up))
summary(whuw2a)
deviance(whuw2a)
}

\name{findSubexprs}
\alias{findSubexprs}
\title{findSubexprs}
\usage{
findSubexprs(expr, simplify = FALSE, tag = ".expr", verbose = FALSE, ...)
}
\arguments{
\item{expr}{An expression represented in a variety of ways. See Details.}

\item{simplify}{Whether to simplify.}

\item{tag}{to be attached to the returned object(s)}

\item{verbose}{If \code{TRUE}, then diagnostic output will be printed as derivatives
and simplifications are recognized.}

\item{...}{Additional arguments, passed to \code{\link{nlsSimplify}}}.
}
\description{
Find common sub-expressions in \code{expr}.
}
\details{
If \code{expr} contains repeated subexpressions, this function
returns an expression to evaluate them just once and store them
in temporary variables with names starting with the \code{tag}.

This function is used internally by \code{\link{codeDeriv}}.
}
\examples{
findSubexprs(expression(x^2 + x^2), tag = ".example")
}
\keyword{internal}

\name{rawres}
\alias{rawres}
\title{rawres}
\description{
Prepare and display raw residuals of \code{nlsr} computations
NOTE: we use model - data form i.e., rhs - lhs
}
\usage{
rawres(object = NULL, data = parent.frame(), ...)
}
\arguments{
\item{object}{an object of class \code{nlsr}}

\item{data}{a data frame with the data for which fits are wanted}

\item{...}{additional data needed to evaluate the modeling functions}
}
\value{
A vector of the raw residuals
}
\author{
J C Nash 2014-7-16 revised 2022-11-22   nashjc _at_ uottawa.ca
}

\name{residuals.nlsr}
\alias{residuals.nlsr}
\title{residuals.nlsr}
\usage{
\method{residuals}{nlsr}(object, ...)
}
\arguments{
\item{object}{an object of class \code{nlsr}}

\item{...}{additional data needed to evaluate the modeling functions}
}
\description{
prepare and display result of \code{nlsr} computations
}
\author{
J C Nash nashjc _at_ uottawa.ca
}

\docType{data}
\name{sysDerivs}
\alias{sysDerivs}
\alias{sysSimplifications}
\title{Internal Environments}
\format{
\code{environment}s holding derivatives and simplification rules.
}
\usage{
sysDerivs
sysSimplifications
}
\description{
These are the default collection of derivatives and 
simplifications used internally by
\code{\link{nlsDeriv}} and \code{\link{nlsSimplify}}.
}
\keyword{internal}

\name{prt}
\alias{prt}
\title{prt}
\description{
To display the calling name of \code{x} and print
the object with the print.nlsr() function.
}
\usage{
prt(x)
}
\arguments{
\item{x}{an object of class \code{nlsr}}
}
\author{
J C Nash 2022-11-22   nashjc _at_ uottawa.ca
}

\name{fitted.nlsr}
\alias{fitted.nlsr}
\title{fitted.nlsr}
\usage{
\method{fitted}{nlsr}(object = NULL, data = parent.frame(), ...)
}
\arguments{
\item{object}{an object of class \code{nlsr}}

\item{data}{a data frame with the data for which fits are wanted.}

\item{...}{additional data needed to evaluate the modeling functions
Default FALSE}
}
\description{
prepare and display fits of \code{nlsr} computations
}
\author{
J C Nash 2014-7-16 revised 2022-11-22  nashjc _at_ uottawa.ca
}

\name{isZERO}
\alias{isZERO}
\alias{isONE}
\alias{isMINUSONE}
\title{Test for constants}
\usage{
isZERO(x)
isONE(x)
isMINUSONE(x)
}
\arguments{
\item{x}{object to be tested}
}
\description{
Test for the values \code{0}, \code{1} or \code{-1}.

Used in \code{\link{newSimplification}} definitions.
}
\value{
Returns \code{TRUE} if the argument is the appropriate scalar value.
}
\examples{
isZERO(0)
x <- quote(0*1)
isZERO(x) # This is `*`(0, 1), not a value
isZERO(eval(x))
x <- quote(-1)
isMINUSONE(x) # This is `-`(1), not a value
isMINUSONE(eval(x))
}
\keyword{internal}

\name{resss}
\alias{resss}
\title{resss}
\usage{
resss(prm, resfn, ...)
}
\arguments{
\item{prm}{a named numeric vector of parameters to the model}

\item{resfn}{a function to compute a vector of residuals}

\item{...}{Extra information needed to compute the residuals}
}
\description{
compute the sum of squares from \code{resfn} at parameters \code{prm}
}
\author{
J C Nash 2014-7-16   nashjc _at_ uottawa.ca
}

\name{dex}
\alias{dex}
\title{dex}
\usage{
dex(x, do_substitute = NA, verbose = FALSE)
}
\arguments{
\item{x}{An expression represented in a variety of ways.
See Details.}

\item{do_substitute}{Whether to use the expression passed as \code{x}, or
to evaluate it and use its value.}

\item{verbose}{Print messages describing the process.}
}
\description{
Calculate expression for derivative calculations.
Mainly for internal use, \code{dex()} converts 
its input to an expression suitable for use
in \code{\link{nlsDeriv}} and related functions.
}
\section{Details}{

   If \code{do_substitute} is \code{NA}, the following 
   rules are used:
     
     \itemize{
        \item An attempt is made to evaluate \code{x}.  If that fails,
         the expression is used.
     
        \item If the evaluation succeeds and the value is a character
          vector, it is parsed.
     
        \item If the value is not a character vector and the
         expression is a single name, the value is used.
     }
     Otherwise, the expression is used.
     
     Once the expression is determined it may be simplified,
     by extracting the language object from a length-one
     expression vector, or the right-hand-side from a 
     formula.
     
     Normally a warning will be issued if \code{x} is a formula
     containing a left-hand side.  To suppress this, 
     wrap the formula in \code{expression()}, or pass it 
     as a character string to be parsed.
}

\section{value}{

     An expression or language object suitable as input
     to \code{\link{nlsDeriv}} and related functions.
}

\examples{
   aa <- dex(~ x^2)
   aa
   str(aa)
   bb <- dex(expression(x^2))
   bb
   str(bb)
   cc <- dex("x^2")
   cc
   str(cc)
   
}
\author{
Duncan Murdoch
}
\keyword{internal}

\name{print.nlsr}
\alias{print.nlsr}
\title{print.nlsr}
\usage{
\method{print}{nlsr}(x, ...)
}
\arguments{
\item{x}{an object of class \code{nlsr}}

\item{...}{additional data needed to evaluate the modeling functions
Default FALSE}
}
\description{
prepare and display result of \code{nlsr} computations
}
\details{
The set of possible controls to set is as follows
}
\author{
J C Nash 2014-7-16   nashjc _at_ uottawa.ca
}

\name{numericDerivR}
\alias{numericDerivR}
\title{numericDerivR: numerically evaluates the gradient of an expression. All in R}
\usage{
numericDerivR(
  expr,
  theta,
  rho = parent.frame(),
  dir = 1,
  eps = .Machine$double.eps^(1/if (central) 3 else 2),
  central = FALSE
)
}
\arguments{
\item{expr}{expression or call to be differentiated. Should evaluate to a numeric vector.}

\item{theta}{character vector of names of numeric variables used in expr.}

\item{rho}{environment containing all the variables needed to evaluate expr.}

\item{dir}{numeric vector of directions, typically with values in -1, 1 to use for the finite 
differences; will be recycled to the length of theta.}

\item{eps}{a positive number, to be used as unit step size hh for the approximate 
numerical derivative (f(x+h)-f(x))/h (f(x+h)-f(x))/h or the central version, see central.}

\item{central}{logical indicating if central divided differences should be computed, 
i.e., (f(x+h) - f(x-h)) / 2h (f(x+h)-f(x-h))/2h. These are typically more accurate but 
need more evaluations of f()f().}
}
\value{
The value of eval(expr, envir = rho) plus a matrix attribute "gradient". The columns of 
this matrix are the derivatives of the value with respect to the variables listed in theta.
}
\description{
This version is all in R to replace the C version in package stats
}
\examples{
ex <- expression(a/(1+b*exp(-tt*c)) - weed)
weed <- c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443,
              38.558, 50.156, 62.948, 75.995, 91.972)
tt <- 1:12
a <- 200; b <- 50; c <- 0.3
dhobb <- numericDerivR(ex, theta=c("a", "b", "c"))
print(dhobb)
# exf <- ~ a/(1+b*exp(-tt*c)) - weed
# Note that a formula doesn't work
# dh1 <-  try(numericDerivR(exf, theta=c("a", "b", "c")))
}

\name{pctrl}
\alias{pctrl}
\title{pctrl}
\usage{
pctrl(control)
}
\arguments{
\item{control}{a control list}
}
\value{
none
}
\description{
Compact display of specified \code{control} vector for
package \code{nlsr}.
}
\author{
J C Nash 2014-7-16   nashjc _at_ uottawa.ca
}

\name{nlsrSS}
\alias{nlsrSS}
\title{nlsrSS - solve selfStart nonlinear least squares with nlsr package}
\usage{
nlsrSS(formula, data)
}
\arguments{
\item{formula}{a model formula incorporating a selfStart function in the right hand side}

\item{data}{a data frame with named columns that allow evaluation of the \code{formula}}
}
\value{
A solution object of class \code{nlsr}.

List of solution elements
  
\item{resid}{weighted residuals at the proposed solution}
\item{jacobian}{Jacobian matrix at the proposed solution}
\item{feval}{residual function evaluations used to reach solution from starting parameters}
\item{jeval}{Jacobian function (or approximation) evaluations used}
\item{coefficients}{a named vector of proposed solution parameters}
\item{ssquares}{weighted sum of squared residuals (often the deviance)}
\item{lower}{lower bounds on parameters}
\item{upper}{upper bounds on parameters}
\item{maskidx}{vector if indices of fixed (masked) parameters}
\item{weights}{specified weights on observations}
\item{formula}{the modeling formula}
\item{resfn}{the residual function (unweighted) based on the formula}

}
\description{
This function uses the \code{getInitial()} function to estimate starting parameters for
a Gauss-Newton iteration, then calls \code{nlsr::nlxb()} appropriately to find a solution
to the required nonlinear least squares problem. 
}
\author{J C Nash 2022-9-14   nashjc _at_ uottawa.ca}

\name{nlxb}
\alias{nlxb}
\title{nlxb: nonlinear least squares modeling by formula}
\usage{
nlxb(
  formula,
  data = parent.frame(),
  start,
  trace = FALSE,
  lower = NULL,
  upper = NULL,
  weights = NULL,
  control = list(),
  ...
)
}
\arguments{
\item{formula}{The modeling formula. Looks like 'y~b1/(1+b2*exp(-b3*T))'}

\item{data}{a data frame containing data for variables
used in the formula that are NOT the parameters. This
data may also be defined in the parent frame i.e.,
'global' to this function}

\item{start}{MUST be a named vector with all elements present
e.g., start=c(b1=200, b2=50, b3=0.3)}

\item{trace}{TRUE for console output during execution}

\item{lower}{a vector of lower bounds on the parameters. 
If a single number, this will be applied to all parameters
Default \code{NULL}.}

\item{upper}{a vector of upper bounds on the parameters. If a single number, 
this will be applied to all parameters. Default \code{NULL}.}

\item{weights}{A vector of fixed weights or a function or formula
producing one.  See the Details below.}

\item{control}{a list of control parameters. See nlsr.control().}

\item{...}{additional data needed to evaluate the modeling functions}
}
\value{
list of solution elements
  \item{resid}{weighted residuals at the proposed solution}
  \item{jacobian}{Jacobian matrix at the proposed solution}
  \item{feval   }{residual function evaluations used to reach solution from starting parameters}
  \item{jeval   }{Jacobian function (or approximation) evaluations used}
  \item{coefficients}{a named vector of proposed solution parameters}
  \item{ssquares}{weighted sum of squared residuals (often the deviance)}
  \item{lower   }{lower bounds on parameters}
  \item{upper   }{upper bounds on parameters}
  \item{maskidx }{vector if indices of fixed (masked) parameters}
  \item{weights0}{weights specified in function call}
  \item{weights }{weights at the final solution}
  \item{formula }{the modeling formula}
  \item{resfn   }{the residual function (unweighted) based on the formula}
}
\description{
A simplified and hopefully robust alternative to finding
the nonlinear least squares minimizer that causes
'formula' to give a minimal residual sum of squares.
}
\details{
nlxb is particularly intended to allow for the
resolution of very ill-conditioned or else near
zero-residual problems for which the regular nls()
function is ill-suited. 

This variant uses a qr solution without forming the sum
of squares and cross products t(J)%*%J

Neither this function nor \code{nlfb} have provision for parameter
scaling (as in the \code{parscale} control of \code{optim} and
package \code{optimx}). This would be more tedious than difficult to
introduce, but does not seem to be a priority feature to add.


  There are many controls, and some of them are important for \code{nlxb}.
  In particular, if the derivatives needed for developing the Jacobian are
  NOT in the derivatives table, then we must supply code elsewhere as 
  specified by the control \code{japprox}. This was originally just for
  numerical approximations, with the character strings "jafwd", "jaback",
  "jacentral" and "jand" leading to the use of a forward, backward, central
  or package \code{numDeriv} approximation. However, it is also possible to
  use code embedded in the residual function created using the \code{formula}.
  This is particularly useful for \code{selfStart} models, and we use the
  character string "SSJac" to point to such Jacobian code. Note how the
  starting parameter vector is found using the \code{getInitial} function
  from the \code{stats} package as in an example below.
  
The \code{weights} argument can be a vector of fixed weights, in which
case the objective function that will be 
minimized is the sum of squares where each residual is multiplied by the 
square root of the corresponding weight. Default \code{NULL} implies 
unit weights.  

\code{weights} may alternatively be a function with header \code{function(parms, resids)} to compute such a vector, or a formula
whose right hand side gives an expression for the weights.  Variables
in the expression may include the following:
\describe{
\item{A variable named \code{resid}}{The current residuals.}
\item{A variable named \code{fitted}}{The right hand side of the model formula.}
\item{Parameters}{The parameters of the model.}
\item{Data}{Values from \code{data}.}
\item{Vars}{Variables in the environment of the formula.}
}
}
\examples{
library(nlsr)
weed <- c(5.308, 7.24, 9.638, 12.866, 17.069, 23.192, 31.443,
        38.558, 50.156, 62.948, 75.995, 91.972)
tt <- 1:12
weeddf <- data.frame(tt, weed)
frm <- 
wmodu <- weed ~ b1/(1+b2*exp(-b3*tt)) # Unscaled
## nls from unit start FAILS
start1<-c(b1=1, b2=1, b3=1)
hunls1 <- try(nls(wmodu, data=weeddf, start=start1, trace=FALSE))
if (! inherits(hunls1, "try-error")) print(hunls1) ## else cat("Failure -- try-error\n")
## nlxb from unit start
hunlx1 <- try(nlxb(wmodu, data=weeddf, start=c(b1=1, b2=1, b3=1), trace=FALSE))
if (! inherits(hunlx1, "try-error")) print(hunlx1)

st2h<-c(b1=185, b2=10, b3=.3)
#' hunls2 <- try(nls(wmodu, data=weeddf, start=st2h, trace=FALSE))
if (! inherits(hunls1, "try-error")) print(hunls1) ## else cat("Failure -- try-error\n")
## nlxb from unit start
hunlx1 <- try(nlxb(wmodu, data=weeddf, start=st2h, trace=FALSE))
if (! inherits(hunlx1, "try-error")) print(hunlx1)

# Functional models need to use a Jacobian approximation or external calculation.
# For example, the SSlogis() selfStart model from \code{stats} package.

# nls() needs NO starting value
hSSnls<-try(nls(weed~SSlogis(tt, Asym, xmid, scal), data=weeddf))
summary(hSSnls)
# We need to get the start for nlxb explicitly
stSS <- getInitial(weed ~ SSlogis(tt, Asym, xmid, scal), data=weeddf)
hSSnlx<-try(nlxb(weed~SSlogis(tt, Asym, xmid, scal), data=weeddf, start=stSS))
hSSnlx

# nls() can only bound parameters with algorithm="port"
#   and minpack.lm is unreliable in imposing bounds, but nlsr copes fine.
lo<-c(0, 0, 0)
up<-c(190, 10, 2) # Note: start must be admissible.
bnls0<-try(nls(wmodu, data=weeddf, start=st2h,
         lower=lo, upper=up)) # should complain and fail
 
bnls<-try(nls(wmodu, data=weeddf, start=st2h,
         lower=lo, upper=up, algorith="port"))
summary(bnls)
bnlx<-try(nlxb(wmodu, data=weeddf, start=st2h, lower=lo, upper=up))
bnlx

# nlxb() can also MASK (fix) parameters. The mechanism of maskidx from nls
# is NO LONGER USED. Instead we set upper and lower parameters equal for
# the masked parameters. The start value MUST be equal to this fixed value.
lo<-c(190, 0, 0) # mask first parameter
up<-c(190, 10, 2)
strt <- c(b1=190, b2=1, b3=1)
mnlx<-try(nlxb(wmodu, start=strt, data=weeddf, 
         lower=lo, upper=up))
mnlx
mnls<-try(nls(wmodu, data=weeddf, start=strt,
         lower=lo, upper=up, algorith="port"))
summary(mnls)

# Try first parameter masked and see if we get SEs 
lo<-c(200, 0, 0) # mask first parameter
up<-c(100, 10, 5)
strt <- c(b1=200, b2=1, b3=1)
mnlx<-try(nlxb(wmodu, start=strt, data=weeddf, 
         lower=lo, upper=up))
mnlx
mnls<-try(nls(wmodu, data=weeddf, start=strt,
         lower=lo, upper=up, algorith="port"))
summary(mnls) 

# Try with weights on the observations
mnlx<-try(nlxb(wmodu, start=strt, data=weeddf, 
               weights = ~ 1/weed))
mnlx

}
\author{
J C Nash 2014-7-16   nashjc _at_ uottawa.ca
}

\name{resgr}
\alias{resgr}
\title{resgr}
\description{
Computes the gradient of the sum of squares function for nonlinear least
squares where \code{resfn} and \code{jacfn} supply the residuals and Jacobian
}
\usage{
resgr(prm, resfn, jacfn, ...)
}
\arguments{
\item{prm}{a numeric vector of parameters to the model}

\item{resfn}{a function to compute a vector of residuals}

\item{jacfn}{a function to compute the Jacobian of the sum of squares. If
    the value is quoted, then the function is assumed to be a numerical
    approximation. Currently one of \code{"jafwd"}, \code{"jaback"}, 
    \code{"jacentral"}, or  \code{"jand"}.}

\item{...}{Extra information needed to compute the residuals}
}
\details{
Does NOT (yet) handle calling of code built into selfStart models. That
is, codes that in \code{nlxb} employ control \code{japprox="SSJac"}.
}
\value{
The main object returned is the numeric vector of residuals computed at \code{prm}
by means of the function \code{resfn}. 
There are \code{Jacobian} and \code{gradient} attributes giving the Jacobian
(matrix of 1st partial derivatives whose row i contains the partial derivative
of the i'th residual w.r.t. each of the parameters) and the gradient of the
sum of squared residuals w.r.t. each of the parameters. Moreover, the Jacobian
is repeated within the \code{gradient} attribute of the Jacobian. This somewhat
bizarre structure is present for compatibility with \code{nls()} and some other
legacy functions, as well as to simplify the call to \code{nlfb()}.
}
\author{
J C Nash 2014-7-16 revised 2022-11-22   nashjc _at_ uottawa.ca
}
